=== modified file '.bzrignore'
--- .bzrignore	2012-05-17 01:55:58 +0000
+++ .bzrignore	2010-09-30 06:24:20 +0000
@@ -3,7 +3,6 @@
 *.8mandos
 confdir
 keydir
-statedir
 man
 plugin-runner
 plugins.d/askpass-fifo

=== modified file 'DBUS-API'
--- DBUS-API	2019-02-10 04:20:26 +0000
+++ DBUS-API	2011-10-02 19:18:24 +0000
@@ -13,25 +13,34 @@
   | Path                  | Object            |
   |-----------------------+-------------------|
   | "/"                   | The Mandos Server |
-
-  (To get a list of paths to client objects, use the standard D-Bus
-  org.freedesktop.DBus.ObjectManager interface, which the server
-  object supports.)
-
-
+  | "/clients/CLIENTNAME" | Mandos Client     |
+
+  
 * Mandos Server Interface:
   Interface name: "se.recompile.Mandos"
   
 ** Methods:
+*** GetAllClients() → (ao: Clients)
+    Returns an array of all client D-Bus object paths
+   
+*** GetAllClientsWithProperties() → (a{oa{sv}}: ClientProperties)
+    Returns an array of all clients and all their properties
+   
 *** RemoveClient(o: ObjectPath) → nothing
     Removes a client
    
 ** Signals:
-*** ClientNotFound(s: KeyID, s: Address)
-    A client connected from Address using KeyID, but was
-    rejected because it was not found in the server.  The key ID
+*** ClientAdded(o: ObjectPath)
+    A new client was added.
+   
+*** ClientNotFound(s: Fingerprint, s: Address)
+    A client connected from Address using Fingerprint, but was
+    rejected because it was not found in the server.  The fingerprint
     is represented as a string of hexadecimal digits.  The address is
     an IPv4 or IPv6 address in its normal string format.
+   
+*** ClientRemoved(o: ObjectPath, s: Name)
+    A client named Name on ObjectPath was removed.
 
 
 * Mandos Client Interface:
@@ -46,6 +55,20 @@
     Assert that this client has been checked and found to be alive.
     This will restart the timeout before disabling this client.  See
     also the "LastCheckedOK" property.
+    
+*** Disable() → nothing
+    Disable this client.  See also the "Enabled" property.
+    
+*** Enable() → nothing
+    Enable this client.  See also the "Enabled" property.
+    
+*** StartChecker() → nothing
+    Start a new checker for this client, if none is currently
+    running.  See also the "CheckerRunning" property.
+    
+*** StopChecker() → nothing
+    Abort a running checker process for this client, if any.  See also
+    the "CheckerRunning" property.
 
 ** Properties
    
@@ -66,30 +89,30 @@
    | Expires (f)             | s    | Read       | N/A                 |
    | ExtendedTimeout (a)     | t    | Read/Write | extended_timeout    |
    | Fingerprint             | s    | Read       | fingerprint         |
-   | KeyID                   | s    | Read       | key_id              |
    | Host                    | s    | Read/Write | host                |
    | Interval (a)            | t    | Read/Write | interval            |
    | LastApprovalRequest (g) | s    | Read       | N/A                 |
    | LastCheckedOK (h)       | s    | Read/Write | N/A                 |
-   | LastCheckerStatus (i)   | n    | Read       | N/A                 |
-   | LastEnabled (j)         | s    | Read       | N/A                 |
+   | LastEnabled (i)         | s    | Read       | N/A                 |
    | Name                    | s    | Read       | (Section name)      |
-   | Secret (k)              | ay   | Write      | secret (or secfile) |
+   | ObjectPath              | o    | Read       | N/A                 |
+   | Secret (j)              | ay   | Write      | secret (or secfile) |
    | Timeout (a)             | t    | Read/Write | timeout             |
    
    a) Represented as milliseconds.
    
    b) An approval is currently pending.
    
-   c) Changing this property can either start a new checker or abort a
-      running one.
+   c) Setting this property is equivalent to calling StartChecker() or
+      StopChecker().
    
    d) The creation time of this client object, as an RFC 3339 string.
    
-   e) Changing this property enables or disables a client.
+   e) Setting this property is equivalent to calling Enable() or
+      Disable().
    
    f) The date and time this client will be disabled, as an RFC 3339
-      string, or an empty string if this is not scheduled.
+      string, or an empty string if this has not happened.
    
    g) The date and time of the last approval request, as an RFC 3339
       string, or an empty string if this has not happened.
@@ -101,13 +124,10 @@
       always use an empty string when setting this property, to allow
       for possible future expansion.
    
-   i) The exit status of the last checker, -1 if it did not exit
-      cleanly, -2 if a checker has not yet returned.
-   
-   j) The date and time this client was last enabled, as an RFC 3339
+   i) The date and time this client was last enabled, as an RFC 3339
       string, or an empty string if this has not happened.
    
-   k) A raw byte array, not hexadecimal digits.
+   j) A raw byte array, not hexadecimal digits.
 
 ** Signals
 *** CheckerCompleted(n: Exitcode, x: Waitstatus, s: Command)
@@ -126,30 +146,32 @@
     milliseconds, depending on ApprovedByDefault.  Approve() can now
     usefully be called on this client object.
     
+*** PropertyChanged(s: Property, v: Value)
+    The Property on this client has changed to Value.
+    
 *** Rejected(s: Reason)
     This client was not given its secret for a specified Reason.
 
 * Copyright
 
-    Copyright © 2010-2019 Teddy Hogeborn
-    Copyright © 2010-2019 Björn Påhlsson
+    Copyright © 2010-2011 Teddy Hogeborn
+    Copyright © 2010-2011 Björn Påhlsson
   
 ** License:
-
-   This file is part of Mandos.
-
-   Mandos is free software: you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-
-   Mandos is distributed in the hope that it will be useful, but
+   
+   This program is free software: you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the
+   License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program.  If not, see
+   <http://www.gnu.org/licenses/>.
 
 
 #+STARTUP: showall

=== modified file 'INSTALL'
--- INSTALL	2019-02-09 23:23:26 +0000
+++ INSTALL	2011-03-08 19:09:03 +0000
@@ -4,16 +4,15 @@
   
 ** Operating System
    
-   Debian 8.0 "jessie" or Ubuntu 15.10 "Wily Werewolf" (or later).
+   Debian 6.0 "squeeze" or Ubuntu 10.10 "Maverick Meerkat".
    
    This is mostly for the support scripts which make sure that the
    client is installed and started in the initial RAM disk environment
-   and that the initial RAM file system image file is automatically
-   made unreadable.  The server and client programs themselves *could*
-   be run in other distributions, but they *are* specific to GNU/Linux
-   systems, and are not written with portabillity to other Unixes in
-   mind.
-   
+   and that the initrd.img file is automatically made unreadable.  The
+   server and client programs themselves *could* be run in other
+   distributions, but they *are* specific to GNU/Linux systems, and
+   are not written with portabillity to other Unixes in mind.
+  
 ** Libraries
    
    The following libraries and packages are needed.  (It is possible
@@ -28,7 +27,7 @@
     + DocBook 4.5         http://www.docbook.org/
       Note: DocBook 5.0 is not compatible.
     + DocBook XSL stylesheets 1.71.0
-                         http://wiki.docbook.org/DocBookXslStylesheets
+                   http://wiki.docbook.org/topic/DocBookXslStylesheets
     
     Package names:
     docbook docbook-xsl
@@ -36,43 +35,34 @@
     To build just the documentation, run the command "make doc".  Then
     the manual page "mandos.8", for example, can be read by running
     "man -l mandos.8".
-    
+   
 *** Mandos Server
-    + GnuTLS 3.3          https://www.gnutls.org/
-      (but not 3.6.0 or later, until 3.6.6, which works)
+    + GnuTLS 2.4          http://www.gnu.org/software/gnutls/
     + Avahi 0.6.16        http://www.avahi.org/
-    + Python 2.7          https://www.python.org/
-    + dbus-python 0.82.4 https://dbus.freedesktop.org/doc/dbus-python/
-    + PyGObject 3.7.1     https://wiki.gnome.org/Projects/PyGObject
-    + pkg-config https://www.freedesktop.org/wiki/Software/pkg-config/
-    + Urwid 1.0.1         http://urwid.org/
-      (Only needed by the "mandos-monitor" tool.)
+    + Python 2.6          http://www.python.org/
+    + Python-GnuTLS 1.1.5 http://pypi.python.org/pypi/python-gnutls/
+    + dbus-python 0.82.4  http://dbus.freedesktop.org/doc/dbus-python/
+    + PyGObject 2.14.2    http://library.gnome.org/devel/pygobject/
+    + Urwid 0.9.8.3       http://excess.org/urwid/
     
     Strongly recommended:
-    + fping 2.4b2-to-ipv6      http://www.fping.org/
-    + ssh-keyscan from OpenSSH http://www.openssh.com/
+    + fping 2.4b2-to-ipv6 http://www.fping.com/
     
     Package names:
-    avahi-daemon python python-dbus python-gi python-urwid pkg-config
-    fping ssh-client
-    
+    python-gnutls avahi-daemon python python-avahi python-dbus
+    python-gobject python-urwid
+   
 *** Mandos Client
-    + GNU C Library 2.16 https://gnu.org/software/libc/
     + initramfs-tools 0.85i
-                        https://tracker.debian.org/pkg/initramfs-tools
-    + GnuTLS 3.3        https://www.gnutls.org/
-      (but not 3.6.0 or later, until 3.6.6 which works)
-    + Avahi 0.6.16      http://www.avahi.org/
-    + GnuPG 1.4.9       https://www.gnupg.org/
-    + GPGME 1.1.6       https://www.gnupg.org/related_software/gpgme/
-    + pkg-config https://www.freedesktop.org/wiki/Software/pkg-config/
-    
-    Strongly recommended:
-    + OpenSSH           http://www.openssh.com/
+                  http://packages.qa.debian.org/i/initramfs-tools.html
+    + GnuTLS 2.4          http://www.gnu.org/software/gnutls/
+    + Avahi 0.6.16        http://www.avahi.org/
+    + GnuPG 1.4.9         http://www.gnupg.org/
+    + GPGME 1.1.6         http://www.gnupg.org/related_software/gpgme/
     
     Package names:
-    initramfs-tools libgnutls-dev gnutls-bin libavahi-core-dev gnupg
-    libgpgme11-dev pkg-config ssh
+    initramfs-tools libgnutls-dev libavahi-core-dev gnupg
+    libgpgme11-dev
 
 * Installing the Mandos server
   
@@ -108,11 +98,15 @@
      and append this to the file "/etc/mandos/clients.conf" *on the
      server computer*.
   
-  4. Configure the client to use any special configuration needed for
-     your local system.  Note: This is not necessary if the server is
-     present on the same wired local network as the client.  If you do
-     make changes to /etc/mandos/plugin-runner.conf, the initrd.img
-     file must be updated, possibly using the following command:
+  4. Configure the client to use the correct network interface.  The
+     interface to use is automatically chosen at boot, and if this
+     needs to be adjusted, it will be necessary to edit
+     /etc/initramfs-tools/initramfs.conf to change the DEVICE setting
+     there.  Alternatively, the file /etc/mandos/plugin-runner.conf
+     can be edited to add a "--device" parameter for the
+     mandos-client(8) plugin.  Please note: If any of those files are
+     changed, the initrd.img file must be updated, possibly using the
+     following command:
      
 	# update-initramfs -k all -u
   
@@ -125,23 +119,19 @@
      
 	# /usr/lib/mandos/plugins.d/mandos-client \
 		--pubkey=/etc/keys/mandos/pubkey.txt \
-		--seckey=/etc/keys/mandos/seckey.txt \
-		--tls-privkey=/etc/keys/mandos/tls-privkey.pem \
-		--tls-pubkey=/etc/keys/mandos/tls-pubkey.pem; echo
+		--seckey=/etc/keys/mandos/seckey.txt; echo
      
      This command should retrieve the password from the server,
      decrypt it, and output it to standard output.
      
      After this, the client computer should be able to reboot without
      needing a password entered on the console, as long as it does not
-     take more than five minutes to reboot.
+     take more than an hour to reboot.
 
 * Further customizations
   
   You may want to tighten or loosen the timeouts in the server
   configuration files; see mandos.conf(5) and mandos-clients.conf(5).
-  If IPsec is not used and SSH is not installed, it is suggested that
-  a more cryptographically secure checker program is used and
-  configured, since, without IPsec, ping packets can be faked.
-
-#+STARTUP: showall
+  If IPsec is not used, it is suggested that a more cryptographically
+  secure checker program is used and configured, since without IPsec
+  ping packets can be faked.

=== modified file 'Makefile'
--- Makefile	2019-02-11 06:31:42 +0000
+++ Makefile	2011-10-05 16:56:06 +0000
@@ -1,35 +1,18 @@
-WARN:=-O -Wall -Wextra -Wdouble-promotion -Wformat=2 -Winit-self \
-	-Wmissing-include-dirs -Wswitch-default -Wswitch-enum \
-	-Wunused -Wuninitialized -Wstrict-overflow=5 \
-	-Wsuggest-attribute=pure -Wsuggest-attribute=const \
-	-Wsuggest-attribute=noreturn -Wfloat-equal -Wundef -Wshadow \
+WARN=-O -Wall -Wformat=2 -Winit-self -Wmissing-include-dirs \
+	-Wswitch-default -Wswitch-enum -Wunused-parameter \
+	-Wstrict-aliasing=1 -Wextra -Wfloat-equal -Wundef -Wshadow \
 	-Wunsafe-loop-optimizations -Wpointer-arith \
 	-Wbad-function-cast -Wcast-qual -Wcast-align -Wwrite-strings \
-	-Wconversion -Wlogical-op -Waggregate-return \
-	-Wstrict-prototypes -Wold-style-definition \
-	-Wmissing-format-attribute -Wnormalized=nfc -Wpacked \
-	-Wredundant-decls -Wnested-externs -Winline -Wvla \
-	-Wvolatile-register-var -Woverlength-strings
-#DEBUG:=-ggdb3 -fsanitize=address 
-# For info about _FORTIFY_SOURCE, see feature_test_macros(7)
-# and <https://gcc.gnu.org/ml/gcc-patches/2004-09/msg02055.html>.
-FORTIFY:=-D_FORTIFY_SOURCE=2 -fstack-protector-all -fPIC
-# <https://developerblog.redhat.com/2014/10/16/gcc-undefined-behavior-sanitizer-ubsan/>
-ALL_SANITIZE_OPTIONS:=-fsanitize=leak -fsanitize=undefined \
-        -fsanitize=shift -fsanitize=integer-divide-by-zero \
-        -fsanitize=unreachable -fsanitize=vla-bound -fsanitize=null \
-        -fsanitize=return -fsanitize=signed-integer-overflow \
-        -fsanitize=bounds -fsanitize=alignment \
-        -fsanitize=object-size -fsanitize=float-divide-by-zero \
-        -fsanitize=float-cast-overflow -fsanitize=nonnull-attribute \
-        -fsanitize=returns-nonnull-attribute -fsanitize=bool \
-        -fsanitize=enum
-# Check which sanitizing options can be used
-SANITIZE:=$(foreach option,$(ALL_SANITIZE_OPTIONS),$(shell \
-	echo 'int main(){}' | $(CC) --language=c $(option) /dev/stdin \
-	-o /dev/null >/dev/null 2>&1 && echo $(option)))
-LINK_FORTIFY_LD:=-z relro -z now
-LINK_FORTIFY:=
+	-Wconversion -Wstrict-prototypes -Wold-style-definition \
+	-Wpacked -Wnested-externs -Winline -Wvolatile-register-var
+#	-Wunreachable-code
+#DEBUG=-ggdb3
+# For info about _FORTIFY_SOURCE, see
+# <http://www.kernel.org/doc/man-pages/online/pages/man7/feature_test_macros.7.html>
+# and <http://gcc.gnu.org/ml/gcc-patches/2004-09/msg02055.html>.
+FORTIFY=-D_FORTIFY_SOURCE=2 -fstack-protector-all -fPIC
+LINK_FORTIFY_LD=-z relro -z now
+LINK_FORTIFY=
 
 # If BROKEN_PIE is set, do not build with -pie
 ifndef BROKEN_PIE
@@ -37,60 +20,41 @@
 LINK_FORTIFY += -pie
 endif
 #COVERAGE=--coverage
-OPTIMIZE:=-Os -fno-strict-aliasing
-LANGUAGE:=-std=gnu11
-htmldir:=man
-version:=1.8.3
-SED:=sed
-
-USER:=$(firstword $(subst :, ,$(shell getent passwd _mandos || getent passwd nobody || echo 65534)))
-GROUP:=$(firstword $(subst :, ,$(shell getent group _mandos || getent group nogroup || echo 65534)))
+OPTIMIZE=-Os
+LANGUAGE=-std=gnu99
+htmldir=man
+version=1.3.1
+SED=sed
 
 ## Use these settings for a traditional /usr/local install
-# PREFIX:=$(DESTDIR)/usr/local
-# CONFDIR:=$(DESTDIR)/etc/mandos
-# KEYDIR:=$(DESTDIR)/etc/mandos/keys
-# MANDIR:=$(PREFIX)/man
-# INITRAMFSTOOLS:=$(DESTDIR)/etc/initramfs-tools
-# STATEDIR:=$(DESTDIR)/var/lib/mandos
-# LIBDIR:=$(PREFIX)/lib
+# PREFIX=$(DESTDIR)/usr/local
+# CONFDIR=$(DESTDIR)/etc/mandos
+# KEYDIR=$(DESTDIR)/etc/mandos/keys
+# MANDIR=$(PREFIX)/man
+# INITRAMFSTOOLS=$(DESTDIR)/etc/initramfs-tools
 ##
 
 ## These settings are for a package-type install
-PREFIX:=$(DESTDIR)/usr
-CONFDIR:=$(DESTDIR)/etc/mandos
-KEYDIR:=$(DESTDIR)/etc/keys/mandos
-MANDIR:=$(PREFIX)/share/man
-INITRAMFSTOOLS:=$(DESTDIR)/usr/share/initramfs-tools
-STATEDIR:=$(DESTDIR)/var/lib/mandos
-LIBDIR:=$(shell \
-	for d in \
-	"/usr/lib/`dpkg-architecture -qDEB_HOST_MULTIARCH 2>/dev/null`" \
-	"`rpm --eval='%{_libdir}' 2>/dev/null`" /usr/lib; do \
-		if [ -d "$$d" -a "$$d" = "$${d%/}" ]; then \
-			echo "$(DESTDIR)$$d"; \
-			break; \
-		fi; \
-	done)
+PREFIX=$(DESTDIR)/usr
+CONFDIR=$(DESTDIR)/etc/mandos
+KEYDIR=$(DESTDIR)/etc/keys/mandos
+MANDIR=$(PREFIX)/share/man
+INITRAMFSTOOLS=$(DESTDIR)/usr/share/initramfs-tools
 ##
 
-SYSTEMD:=$(DESTDIR)$(shell pkg-config systemd --variable=systemdsystemunitdir)
-TMPFILES:=$(DESTDIR)$(shell pkg-config systemd --variable=tmpfilesdir)
-
-GNUTLS_CFLAGS:=$(shell pkg-config --cflags-only-I gnutls)
-GNUTLS_LIBS:=$(shell pkg-config --libs gnutls)
-AVAHI_CFLAGS:=$(shell pkg-config --cflags-only-I avahi-core)
-AVAHI_LIBS:=$(shell pkg-config --libs avahi-core)
-GPGME_CFLAGS:=$(shell gpgme-config --cflags; getconf LFS_CFLAGS)
-GPGME_LIBS:=$(shell gpgme-config --libs; getconf LFS_LIBS; \
+GNUTLS_CFLAGS=$(shell pkg-config --cflags-only-I gnutls)
+GNUTLS_LIBS=$(shell pkg-config --libs gnutls)
+AVAHI_CFLAGS=$(shell pkg-config --cflags-only-I avahi-core)
+AVAHI_LIBS=$(shell pkg-config --libs avahi-core)
+GPGME_CFLAGS=$(shell gpgme-config --cflags; getconf LFS_CFLAGS)
+GPGME_LIBS=$(shell gpgme-config --libs; getconf LFS_LIBS; \
 	getconf LFS_LDFLAGS)
-LIBNL3_CFLAGS:=$(shell pkg-config --cflags-only-I libnl-route-3.0)
-LIBNL3_LIBS:=$(shell pkg-config --libs libnl-route-3.0)
 
 # Do not change these two
-CFLAGS+=$(WARN) $(DEBUG) $(FORTIFY) $(SANITIZE) $(COVERAGE) \
-	$(OPTIMIZE) $(LANGUAGE) -DVERSION='"$(version)"'
-LDFLAGS+=-Xlinker --as-needed $(COVERAGE) $(LINK_FORTIFY) $(foreach flag,$(LINK_FORTIFY_LD),-Xlinker $(flag))
+CFLAGS=$(WARN) $(DEBUG) $(FORTIFY) $(COVERAGE) $(OPTIMIZE) \
+	$(LANGUAGE) $(GNUTLS_CFLAGS) $(AVAHI_CFLAGS) $(GPGME_CFLAGS) \
+	-DVERSION='"$(version)"'
+LDFLAGS=-Xlinker --as-needed $(COVERAGE) $(LINK_FORTIFY) $(foreach flag,$(LINK_FORTIFY_LD),-Xlinker $(flag))
 
 # Commands to format a DocBook <refentry> document into a manual page
 DOCBOOKTOMAN=$(strip cd $(dir $<); xsltproc --nonet --xinclude \
@@ -99,12 +63,11 @@
 	--param make.single.year.ranges		1 \
 	--param man.output.quietly		1 \
 	--param man.authors.section.enabled	0 \
-	/usr/share/xml/docbook/stylesheet/nwalsh/manpages/docbook.xsl \
+	 /usr/share/xml/docbook/stylesheet/nwalsh/manpages/docbook.xsl \
 	$(notdir $<); \
-	if locale --all 2>/dev/null | grep --regexp='^en_US\.utf8$$' \
-	&& type man 2>/dev/null; then LANG=en_US.UTF-8 MANWIDTH=80 \
-	man --warnings --encoding=UTF-8 --local-file $(notdir $@); \
-	fi >/dev/null)
+	$(MANPOST) $(notdir $@))
+# DocBook-to-man post-processing to fix a '\n' escape bug
+MANPOST=$(SED) --in-place --expression='s,\\\\en,\\en,g;s,\\n,\\en,g'
 
 DOCBOOKTOHTML=$(strip xsltproc --nonet --xinclude \
 	--param make.year.ranges		1 \
@@ -116,25 +79,24 @@
 	/usr/share/xml/docbook/stylesheet/nwalsh/xhtml/docbook.xsl \
 	$<; $(HTMLPOST) $@)
 # Fix citerefentry links
-HTMLPOST:=$(SED) --in-place \
+HTMLPOST=$(SED) --in-place \
 	--expression='s/\(<a class="citerefentry" href="\)\("><span class="citerefentry"><span class="refentrytitle">\)\([^<]*\)\(<\/span>(\)\([^)]*\)\()<\/span><\/a>\)/\1\3.\5\2\3\4\5\6/g'
 
-PLUGINS:=plugins.d/password-prompt plugins.d/mandos-client \
+PLUGINS=plugins.d/password-prompt plugins.d/mandos-client \
 	plugins.d/usplash plugins.d/splashy plugins.d/askpass-fifo \
 	plugins.d/plymouth
-PLUGIN_HELPERS:=plugin-helpers/mandos-client-iprouteadddel
-CPROGS:=plugin-runner $(PLUGINS) $(PLUGIN_HELPERS)
-PROGS:=mandos mandos-keygen mandos-ctl mandos-monitor $(CPROGS)
-DOCS:=mandos.8 mandos-keygen.8 mandos-monitor.8 mandos-ctl.8 \
+CPROGS=plugin-runner $(PLUGINS)
+PROGS=mandos mandos-keygen mandos-ctl mandos-monitor $(CPROGS)
+DOCS=mandos.8 mandos-keygen.8 mandos-monitor.8 mandos-ctl.8 \
 	mandos.conf.5 mandos-clients.conf.5 plugin-runner.8mandos \
 	plugins.d/mandos-client.8mandos \
 	plugins.d/password-prompt.8mandos plugins.d/usplash.8mandos \
 	plugins.d/splashy.8mandos plugins.d/askpass-fifo.8mandos \
 	plugins.d/plymouth.8mandos intro.8mandos
 
-htmldocs:=$(addsuffix .xhtml,$(DOCS))
+htmldocs=$(addsuffix .xhtml,$(DOCS))
 
-objects:=$(addsuffix .o,$(CPROGS))
+objects=$(addsuffix .o,$(CPROGS))
 
 all: $(PROGS) mandos.lsm
 
@@ -252,24 +214,13 @@
 		--expression='s/\(mandos_\)[0-9.]\+\(\.orig\.tar\.gz\)/\1$(version)\2/' \
 		$@)
 
-# Need to add the GnuTLS, Avahi and GPGME libraries, and can't use
-# -fsanitize=leak because GnuTLS and GPGME both leak memory.
 plugins.d/mandos-client: plugins.d/mandos-client.c
-	$(CC) $(filter-out -fsanitize=leak,$(CFLAGS)) $(strip\
-	) $(GNUTLS_CFLAGS) $(AVAHI_CFLAGS) $(GPGME_CFLAGS) $(strip\
-		) $(CPPFLAGS) $(LDFLAGS) $(TARGET_ARCH) $^ $(strip\
-		) -lrt $(GNUTLS_LIBS) $(AVAHI_LIBS) $(strip\
+	$(LINK.c) $^ -lrt $(GNUTLS_LIBS) $(AVAHI_LIBS) $(strip\
 		) $(GPGME_LIBS) $(LOADLIBES) $(LDLIBS) -o $@
 
-plugin-helpers/mandos-client-iprouteadddel: plugin-helpers/mandos-client-iprouteadddel.c
-	$(LINK.c) $(LIBNL3_CFLAGS) $^ $(LIBNL3_LIBS) $(strip\
-		) $(LOADLIBES) $(LDLIBS) -o $@
-
-.PHONY : all doc html clean distclean mostlyclean maintainer-clean \
-	check run-client run-server install install-html \
-	install-server install-client-nokey install-client uninstall \
-	uninstall-server uninstall-client purge purge-server \
-	purge-client
+.PHONY : all doc html clean distclean run-client run-server install \
+	install-server install-client uninstall uninstall-server \
+	uninstall-client purge purge-server purge-client
 
 clean:
 	-rm --force $(CPROGS) $(objects) $(htmldocs) $(DOCS) core
@@ -277,57 +228,53 @@
 distclean: clean
 mostlyclean: clean
 maintainer-clean: clean
-	-rm --force --recursive keydir confdir statedir
+	-rm --force --recursive keydir confdir
 
 check:	all
 	./mandos --check
-	./mandos-ctl --check
 
 # Run the client with a local config and key
-run-client: all keydir/seckey.txt keydir/pubkey.txt keydir/tls-privkey.pem keydir/tls-pubkey.pem
+run-client: all keydir/seckey.txt keydir/pubkey.txt
 	@echo "###################################################################"
 	@echo "# The following error messages are harmless and can be safely     #"
-	@echo "# ignored:                                                        #"
-	@echo "# From plugin-runner: setgid: Operation not permitted             #"
-	@echo "#                     setuid: Operation not permitted             #"
+	@echo "# ignored.  The messages are caused by not running as root, but   #"
+	@echo "# you should NOT run \"make run-client\" as root unless you also    #"
+	@echo "# unpacked and compiled Mandos as root, which is NOT recommended. #"
+	@echo "# From plugin-runner: setuid: Operation not permitted             #"
 	@echo "# From askpass-fifo:  mkfifo: Permission denied                   #"
-	@echo "# From mandos-client:                                             #"
-	@echo "#             Failed to raise privileges: Operation not permitted #"
-	@echo "#             Warning: network hook \"*\" exited with status *      #"
-	@echo "#                                                                 #"
-	@echo "# (The messages are caused by not running as root, but you should #"
-	@echo "# NOT run \"make run-client\" as root unless you also unpacked and  #"
-	@echo "# compiled Mandos as root, which is also NOT recommended.)        #"
+	@echo "# From mandos-client: setuid: Operation not permitted             #"
+	@echo "#                     seteuid: Operation not permitted            #"
+	@echo "#                     klogctl: Operation not permitted            #"
 	@echo "###################################################################"
-# We set GNOME_KEYRING_CONTROL to block pam_gnome_keyring
 	./plugin-runner --plugin-dir=plugins.d \
-		--plugin-helper-dir=plugin-helpers \
 		--config-file=plugin-runner.conf \
-		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt,--tls-privkey=keydir/tls-privkey.pem,--tls-pubkey=keydir/tls-pubkey.pem,--network-hook-dir=network-hooks.d \
-		--env-for=mandos-client:GNOME_KEYRING_CONTROL= \
+		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt \
 		$(CLIENTARGS)
 
 # Used by run-client
-keydir/seckey.txt keydir/pubkey.txt keydir/tls-privkey.pem keydir/tls-pubkey.pem: mandos-keygen
+keydir/seckey.txt keydir/pubkey.txt: mandos-keygen
 	install --directory keydir
 	./mandos-keygen --dir keydir --force
 
 # Run the server with a local config
-run-server: confdir/mandos.conf confdir/clients.conf statedir
-	./mandos --debug --no-dbus --configdir=confdir \
-		--statedir=statedir $(SERVERARGS)
+run-server: confdir/mandos.conf confdir/clients.conf
+	@echo "#################################################################"
+	@echo "# NOTE: Please IGNORE the error about \"Could not open file      #"
+	@echo "# u'/var/run/mandos.pid'\" -  it is harmless and is caused by    #"
+	@echo "# the server not running as root.  Do NOT run \"make run-server\" #"
+	@echo "# server as root if you didn't also unpack and compile it thus. #"
+	@echo "#################################################################"
+	./mandos --debug --no-dbus --configdir=confdir $(SERVERARGS)
 
 # Used by run-server
 confdir/mandos.conf: mandos.conf
 	install --directory confdir
 	install --mode=u=rw,go=r $^ $@
-confdir/clients.conf: clients.conf keydir/seckey.txt keydir/tls-pubkey.pem
+confdir/clients.conf: clients.conf keydir/seckey.txt
 	install --directory confdir
 	install --mode=u=rw $< $@
 # Add a client password
-	./mandos-keygen --dir keydir --password --no-ssh >> $@
-statedir:
-	install --directory statedir
+	./mandos-keygen --dir keydir --password >> $@
 
 install: install-server install-client-nokey
 
@@ -338,16 +285,6 @@
 
 install-server: doc
 	install --directory $(CONFDIR)
-	if install --directory --mode=u=rwx --owner=$(USER) \
-		--group=$(GROUP) $(STATEDIR); then \
-		:; \
-	elif install --directory --mode=u=rwx $(STATEDIR); then \
-		chown -- $(USER):$(GROUP) $(STATEDIR) || :; \
-	fi
-	if [ "$(TMPFILES)" != "$(DESTDIR)" -a -d "$(TMPFILES)" ]; then \
-		install --mode=u=rw,go=r tmpfiles.d-mandos.conf \
-			$(TMPFILES)/mandos.conf; \
-	fi
 	install --mode=u=rwx,go=rx mandos $(PREFIX)/sbin/mandos
 	install --mode=u=rwx,go=rx --target-directory=$(PREFIX)/sbin \
 		mandos-ctl
@@ -361,9 +298,6 @@
 		$(DESTDIR)/etc/dbus-1/system.d/mandos.conf
 	install --mode=u=rwx,go=rx init.d-mandos \
 		$(DESTDIR)/etc/init.d/mandos
-	if [ "$(SYSTEMD)" != "$(DESTDIR)" -a -d "$(SYSTEMD)" ]; then \
-		install --mode=u=rw,go=r mandos.service $(SYSTEMD); \
-	fi
 	install --mode=u=rw,go=r default-mandos \
 		$(DESTDIR)/etc/default/mandos
 	if [ -z $(DESTDIR) ]; then \
@@ -379,56 +313,43 @@
 		> $(MANDIR)/man5/mandos.conf.5.gz
 	gzip --best --to-stdout mandos-clients.conf.5 \
 		> $(MANDIR)/man5/mandos-clients.conf.5.gz
-	gzip --best --to-stdout intro.8mandos \
-		> $(MANDIR)/man8/intro.8mandos.gz
 
 install-client-nokey: all doc
-	install --directory $(LIBDIR)/mandos $(CONFDIR)
+	install --directory $(PREFIX)/lib/mandos $(CONFDIR)
 	install --directory --mode=u=rwx $(KEYDIR) \
-		$(LIBDIR)/mandos/plugins.d \
-		$(LIBDIR)/mandos/plugin-helpers
-	if [ "$(CONFDIR)" != "$(LIBDIR)/mandos" ]; then \
+		$(PREFIX)/lib/mandos/plugins.d
+	if [ "$(CONFDIR)" != "$(PREFIX)/lib/mandos" ]; then \
 		install --mode=u=rwx \
-			--directory "$(CONFDIR)/plugins.d" \
-			"$(CONFDIR)/plugin-helpers"; \
+			--directory "$(CONFDIR)/plugins.d"; \
 	fi
-	install --mode=u=rwx,go=rx --directory \
-		"$(CONFDIR)/network-hooks.d"
-	install --mode=u=rwx,go=rx \
-		--target-directory=$(LIBDIR)/mandos plugin-runner
-	install --mode=u=rwx,go=rx \
-		--target-directory=$(LIBDIR)/mandos mandos-to-cryptroot-unlock
+	install --mode=u=rwx,go=rx \
+		--target-directory=$(PREFIX)/lib/mandos plugin-runner
 	install --mode=u=rwx,go=rx --target-directory=$(PREFIX)/sbin \
 		mandos-keygen
 	install --mode=u=rwx,go=rx \
-		--target-directory=$(LIBDIR)/mandos/plugins.d \
+		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
 		plugins.d/password-prompt
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(LIBDIR)/mandos/plugins.d \
+		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
 		plugins.d/mandos-client
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(LIBDIR)/mandos/plugins.d \
+		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
 		plugins.d/usplash
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(LIBDIR)/mandos/plugins.d \
+		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
 		plugins.d/splashy
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(LIBDIR)/mandos/plugins.d \
+		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
 		plugins.d/askpass-fifo
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(LIBDIR)/mandos/plugins.d \
+		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
 		plugins.d/plymouth
-	install --mode=u=rwx,go=rx \
-		--target-directory=$(LIBDIR)/mandos/plugin-helpers \
-		plugin-helpers/mandos-client-iprouteadddel
 	install initramfs-tools-hook \
 		$(INITRAMFSTOOLS)/hooks/mandos
-	install --mode=u=rw,go=r initramfs-tools-conf \
-		$(INITRAMFSTOOLS)/conf.d/mandos-conf
+	install --mode=u=rw,go=r initramfs-tools-hook-conf \
+		$(INITRAMFSTOOLS)/conf-hooks.d/mandos
 	install initramfs-tools-script \
 		$(INITRAMFSTOOLS)/scripts/init-premount/mandos
-	install initramfs-tools-script-stop \
-		$(INITRAMFSTOOLS)/scripts/local-premount/mandos
 	install --mode=u=rw,go=r plugin-runner.conf $(CONFDIR)
 	gzip --best --to-stdout mandos-keygen.8 \
 		> $(MANDIR)/man8/mandos-keygen.8.gz
@@ -473,13 +394,13 @@
 	! grep --regexp='^ *[^ #].*keyscript=[^,=]*/mandos/' \
 		$(DESTDIR)/etc/crypttab
 	-rm --force $(PREFIX)/sbin/mandos-keygen \
-		$(LIBDIR)/mandos/plugin-runner \
-		$(LIBDIR)/mandos/plugins.d/password-prompt \
-		$(LIBDIR)/mandos/plugins.d/mandos-client \
-		$(LIBDIR)/mandos/plugins.d/usplash \
-		$(LIBDIR)/mandos/plugins.d/splashy \
-		$(LIBDIR)/mandos/plugins.d/askpass-fifo \
-		$(LIBDIR)/mandos/plugins.d/plymouth \
+		$(PREFIX)/lib/mandos/plugin-runner \
+		$(PREFIX)/lib/mandos/plugins.d/password-prompt \
+		$(PREFIX)/lib/mandos/plugins.d/mandos-client \
+		$(PREFIX)/lib/mandos/plugins.d/usplash \
+		$(PREFIX)/lib/mandos/plugins.d/splashy \
+		$(PREFIX)/lib/mandos/plugins.d/askpass-fifo \
+		$(PREFIX)/lib/mandos/plugins.d/plymouth \
 		$(INITRAMFSTOOLS)/hooks/mandos \
 		$(INITRAMFSTOOLS)/conf-hooks.d/mandos \
 		$(INITRAMFSTOOLS)/scripts/init-premount/mandos \
@@ -491,8 +412,8 @@
 		$(MANDIR)/man8/splashy.8mandos.gz \
 		$(MANDIR)/man8/askpass-fifo.8mandos.gz \
 		$(MANDIR)/man8/plymouth.8mandos.gz \
-	-rmdir $(LIBDIR)/mandos/plugins.d $(CONFDIR)/plugins.d \
-		 $(LIBDIR)/mandos $(CONFDIR) $(KEYDIR)
+	-rmdir $(PREFIX)/lib/mandos/plugins.d $(CONFDIR)/plugins.d \
+		 $(PREFIX)/lib/mandos $(CONFDIR) $(KEYDIR)
 	update-initramfs -k all -u
 
 purge: purge-server purge-client
@@ -502,14 +423,11 @@
 		$(DESTDIR)/etc/dbus-1/system.d/mandos.conf
 		$(DESTDIR)/etc/default/mandos \
 		$(DESTDIR)/etc/init.d/mandos \
-		$(SYSTEMD)/mandos.service \
-		$(DESTDIR)/run/mandos.pid \
 		$(DESTDIR)/var/run/mandos.pid
 	-rmdir $(CONFDIR)
 
 purge-client: uninstall-client
-	-shred --remove $(KEYDIR)/seckey.txt $(KEYDIR)/tls-privkey.pem
+	-shred --remove $(KEYDIR)/seckey.txt
 	-rm --force $(CONFDIR)/plugin-runner.conf \
-		$(KEYDIR)/pubkey.txt $(KEYDIR)/seckey.txt \
-		$(KEYDIR)/tls-pubkey.txt $(KEYDIR)/tls-privkey.txt
+		$(KEYDIR)/pubkey.txt $(KEYDIR)/seckey.txt
 	-rmdir $(KEYDIR) $(CONFDIR)/plugins.d $(CONFDIR)

=== modified file 'NEWS'
--- NEWS	2019-02-11 06:31:42 +0000
+++ NEWS	2011-07-27 17:58:27 +0000
@@ -1,360 +1,6 @@
 This NEWS file records noteworthy changes, very tersely.
 See the manual for detailed information.
 
-Version 1.8.3 (2019-02-11)
-* No user-visible changes.
-
-Version 1.8.2 (2019-02-10)
-* Client
-** In mandos-keygen, ignore failures to remove files in some cases.
-
-Version 1.8.1 (2019-02-10)
-* Client
-** Only generate TLS keys using GnuTLS' certtool, of sufficient
-   version.  Key generation of TLS keys will not happen until a
-   version of GnuTLS is installed with support for raw public keys.
-** Remove any bad keys created by 1.8.0 and openssl.
-* Server
-** On installation, edit clients.conf and remove the same bad key ID
-   which was erroneously reported by all 1.8.0 clients.  Also do not
-   trust this key ID in the server.
-
-Version 1.8.0 (2019-02-10)
-* Client
-** Use new TLS keys for server communication and identification.
-   With GnuTLS 3.6 or later, OpenPGP keys are no longer supported.
-   The client can now use the new "raw public keys" (RFC 7250) API
-   instead, using GnuTLS 3.6.6.  Please note: This *requires* new key
-   IDs to be added to server's client.conf file.
-** New --tls-privkey and --tls-pubkey options to load TLS key files.
-   If GnuTLS is too old, these options do nothing.
-* Server
-** Supports either old or new GnuTLS.
-   The server now supports using GnuTLS 3.6.6 and clients connecting
-   with "raw public keys" as identification.  The server will read
-   both fingerprints and key IDs from clients.conf file, and will use
-   either one or the other, depending on what is supported by GnuTLS
-   on the system.  Please note: both are *not* supported at once; if
-   one type is supported by GnuTLS, all values of the other type from
-   clients.conf are ignored.
-
-Version 1.7.20 (2018-08-19)
-* Client
-** Fix: Adapt to the Debian cryptsetup package 2.0.3 or later.
-   Important: in that version or later, the plugins "askpass-fifo",
-   "password-prompt", and "plymouth" will no longer be run, since they
-   would conflict with what cryptsetup is doing.  Other plugins, such
-   as mandos-client and any user-supplied plugins, will still run.
-** Better error message if failing to decrypt secret data
-** Check for (and report) any key import failure from GPGME
-** Better error message if self-signature verification fails
-** Set system clock if not set; required by GnuPG for key import
-** When debugging plugin-runner, it will now show starting plugins
-
-Version 1.7.19 (2018-02-22)
-* Client
-** Do not print "unlink(...): No such file or directory".
-** Bug fixes: Fix file descriptor leaks.
-** Bug fix: Don't use leak sanitizer with leaking libraries.
-
-Version 1.7.18 (2018-02-12)
-* Client
-** Bug fix: Revert faulty fix for a nonexistent bug in the
-   plugin-runner
-
-Version 1.7.17 (2018-02-10)
-* Client
-** Bug fix: Fix a memory leak in the plugin-runner
-** Bug fix: Fix memory leaks in the plymouth plugin
-
-Version 1.7.16 (2017-08-20)
-* Client
-** Bug fix: ignore "resumedev" entries in initramfs' cryptroot file
-** Bug fix in plymouth plugin: fix memory leak, avoid warning output
-
-Version 1.7.15 (2017-02-23)
-* Server
-** Bug fix: Respect the mandos.conf "zeroconf" and "restore" options
-* Client
-** Bug fix in mandos-keygen: Handle backslashes in passphrases
-
-Version 1.7.14 (2017-01-25)
-* Server
-** Use "Requisite" instead of "RequisiteOverridable" in systemd
-   service file.
-
-Version 1.7.13 (2016-10-08)
-* Client
-** Minor bug fix: Don't ask for passphrase or fail when generating
-   keys using GnuPG 2.1 in a chrooted environment.
-
-Version 1.7.12 (2016-10-05)
-* Client
-** Bug fix: Don't crash after exit() when using DH parameters file
-
-Version 1.7.11 (2016-10-01)
-* Client
-** Security fix: Don't compile with AddressSanitizer
-* Server
-** Bug fix: Find GnuTLS library when gnutls28-dev is not installed
-** Bug fix: Include "Expires" and "Last Checker Status" in mandos-ctl
-   verbose output
-** New option for mandos-ctl: --dump-json
-
-Version 1.7.10 (2016-06-23)
-* Client
-** Security fix: restrict permissions of /etc/mandos/plugin-helpers
-* Server
-** Bug fix: Make the --interface flag work with Python 2.7 when "cc"
-   is not installed
-
-Version 1.7.9 (2016-06-22)
-* Client
-** Do not include intro(8mandos) man page
-
-Version 1.7.8 (2016-06-21)
-* Client
-** Include intro(8mandos) man page
-** mandos-keygen: Use ECDSA SSH keys by default
-** Bug fix: Work with GnuPG 2 when booting (Debian bug #819982)
-   by copying /usr/bin/gpg-agent into initramfs
-* Server
-** Bug fix: Work with GnuPG 2 (don't use --no-use-agent option)
-** Bug fix: Make the --interface option work when using Python 2.7
-   by trying harder to find SO_BINDTODEVICE
-
-Version 1.7.7 (2016-03-19)
-* Client
-** Fix bug in Plymouth client, broken since 1.7.2
-
-Version 1.7.6 (2016-03-13)
-* Server
-** Fix bug where stopping server would time out
-** Make server runnable with Python 3
-
-Version 1.7.5 (2016-03-08)
-* Server
-** Fix security restrictions in systemd service file.
-** Work around bug where stopping server would time out
-
-Version 1.7.4 (2016-03-05)
-* Client
-** Bug fix: Tolerate errors from configure_networking (Debian Bug
-   #816513)
-** Compilation: Only use sanitizing options which work with the
-   compiler used when building.  This should fix compilation with GCC
-   4.9 on mips, mipsel, and s390x.
-* Server
-** Add extra security restrictions in systemd service file.
-
-Version 1.7.3 (2016-02-29)
-* Client
-** Bug fix: Remove new type of keyring directory user by GnuPG 2.1.
-** Bug fix: Remove "nonnull" attribute from a function argument, which
-   would otherwise generate a spurious runtime warning.
-
-Version 1.7.2 (2016-02-28)
-* Server
-** Stop using python-gnutls library; it was not updated to GnuTLS 3.3.
-** Bug fix: Only send D-Bus signal ClientRemoved if using D-Bus.
-** Use GnuPG 2 if available.
-* Client
-** Compile with various sanitizing flags.
-
-Version 1.7.1 (2015-10-24)
-* Client
-** Bug fix: Can now really find Mandos server even if the server has
-   an IPv6 address on a network other than the one which the Mandos
-   server is on.
-
-Version 1.7.0 (2015-08-10)
-* Server
-** Bug fix: Handle local Zeroconf service name collisions better.
-** Bug fix: Finally fix "ERROR: Child process vanished" bug.
-** Bug fix: Fix systemd service file to start server correctly.
-** Bug fix: Be compatible with old 2048-bit DSA keys.
-** The D-Bus API now provides the standard D-Bus ObjectManager
-   interface, and deprecates older functionality.  See the DBUS-API
-   file for the currently recommended API.  Note: the original API
-   still works, but is deprecated.
-* Client
-** Can now find Mandos server even if the server has an IPv6 address
-   on a network without IPv6 Router Advertisment (like if the Mandos
-   client itself is the router, or there is an IPv6 router advertising
-   a network other than the one which the Mandos server is on.)
-** Use a better value than 1024 for the default number of DH bits.
-   This better value is either provided by a DH parameters file (see
-   below) or an appropriate number of DH bits is determined based on
-   the PGP key.
-** Bug fix: mandos-keygen now generates correct output for the
-   "Checker" variable even if the SSH server on the Mandos client has
-   multiple SSH key types.
-** Can now use pre-generated Diffie-Hellman parameters from a file.
-
-Version 1.6.9 (2014-10-05)
-* Server
-** Changed to emit standard D-Bus signal when D-Bus properties change.
-   (The old signal is still emitted too, but marked as deprecated.)
-
-Version 1.6.8 (2014-08-06)
-* Client
-** Bug fix: mandos-keygen now generates working SSH checker commands.
-* Server
-** Bug fix: "mandos-monitor" now really redraws screen on Ctrl-L.
-** Now requires Python 2.7.
-
-Version 1.6.7 (2014-07-17)
-* Client
-** Bug fix: Now compatible with GPGME 1.5.0.
-** Bug fix: Fixed minor memory leaks.
-* Server
-** "mandos-monitor" now has verbose logging, toggleable with "v".
-
-Version 1.6.6 (2014-07-13)
-* Client
-** If client host has an SSH server, "mandos-keygen --password" now
-   outputs "checker" option which uses "ssh-keyscan"; this is more
-   secure than the default "fping" checker.
-** Bug fix: allow "." in network hook names, to match documentation.
-** Better error messages.
-* Server
-** New --no-zeroconf option.
-** Bug fix: Fix --servicename option, broken since 1.6.4.
-** Bug fix: Fix --socket option work for --socket=0.
-
-Version 1.6.5 (2014-05-11)
-* Client
-** Work around bug in GnuPG <http://bugs.g10code.com/gnupg/issue1622>
-** Give better error messages when run without sufficient privileges
-** Only warn if workaround for Debian bug #633582 was necessary and
-   failed, not if it failed and was unnecessary.
-
-Version 1.6.4 (2014-02-16)
-* Server
-** Very minor fix to self-test code.
-
-Version 1.6.3 (2014-01-21)
-* Server
-** Add systemd support.
-** For PID file, fall back to /var/run if /run does not exist.
-* Client
-** Moved files from /usr/lib/mandos to whatever the architecture
-   specifies, like /usr/lib/x86_64-linux-gnu/mandos or
-   /usr/lib64/mandos.
-
-Version 1.6.2 (2013-10-24)
-* Server
-** PID file moved from /var/run to /run.
-** Bug fix: Handle long secrets when saving client state.
-** Bug fix: Use more magic in the GnuTLS priority string to handle
-   both old DSA/ELG 2048-bit keys and new RSA/RSA 4096-bit keys.
-* Client
-** mandos-keygen: Bug fix: now generate RSA keys which GnuTLS can use.
-   		  Bug fix: Output passphrase prompts even when
-   		  redirecting standard output.
-
-Version 1.6.1 (2013-10-13)
-* Server
-** All client options for time intervals now also take an RFC 3339
-   duration.  The same for all options to mandos-ctl.
-** Bug fix: Handle fast checkers (like ":") correctly.
-** Bug fix: Don't print output from checkers when running in
-   foreground.
-** Bug fix: Do not fail when client is removed from clients.conf but
-   saved settings remain.
-** Bug fix: mandos-monitor now displays standout (reverse video) again
-   using new version of Urwid.
-** Bug fix: Make boolean options work from the config file again.
-** Bug fix: Make --no-ipv6 work again.
-** New default priority string to be slightly more compatible with
-   older versions of GnuTLS.
-* Client
-** Bug fix: Fix bashism in mandos-keygen.
-** Default key and subkey types are now RSA and RSA, respectively.
-   Also, new default key size is 4096 bits.
-
-Version 1.6.0 (2012-06-18)
-* Server
-** Takes new --foreground option
-** Init script supports new "status" action.
-* Client
-** Now uses all interfaces by default; the --interface option can
-   still be used to restrict it, and the argument to --interface (as
-   well as the $DEVICE environment variable for the network hooks) is
-   now a comma-separated list of interfaces to use.
-
-Version 1.5.5 (2012-06-01)
-* Server
-** Server takes new --socket option
-
-Version 1.5.4 (2012-05-20)
-* Server
-** Bug fix: Regression fix: Make non-zero approval timeout values work.
-** Bug fix: Regression fix: Allow changing the Timeout D-Bus property.
-** Fall back to not bind to an interface if an invalid interface name
-   is given.
-** Removed support for undocumented feature of using plain "%%s" in
-   "checker" client option.
-** Old D-Bus interface are now marked as deprecated.
-** mandos-monitor: Bug fix: show approval timers correctly.
-** mandos-ctl: Show "Extended Timeout" correctly, not as milliseconds.
-
-Version 1.5.3 (2012-01-15)
-* Server
-** Add D-Bus property se.recompile.Client.LastCheckerStatus and use it
-   in mandos-monitor.
-* Client
-** Fix bugs in the example "bridge" network hook.
-
-Version 1.5.2 (2012-01-08)
-* Server
-** Removed D-Bus signal se.recompile.Mandos.NewRequest() added in
-   1.5.0.  It was buggy and was of questionable utility.
-
-Version 1.5.1 (2012-01-01)
-* Server
-** Include intro(8mandos) manual page, missing since migration from
-   README file in version 1.4.0.
-
-Version 1.5.0 (2012-01-01)
-* Client
-** Network hooks.  The Mandos client can now run custom scripts to take
-   up a network interface before the client is run.  Three example
-   scripts are provided: "wireless", "openvpn", and "bridge".
-   To facilitate this, the client now prefers network interfaces which
-   are up (if any) over all other interfaces.
-* Server
-** Persistent state.  Client state is now saved between server
-   restarts.
-** clients.conf file can now contain "enabled" setting for clients.
-** Bug fix: Fix rare crash bug.
-** Bug fix: Send corrent D-Bus type in PropertyChanged for
-   "ApprovalDelay", "ApprovalDuration", "Timeout", and
-   "ExtendedTimeout".
-** mandos-ctl: Bare numbers as arguments are taken to be milliseconds.
-** Bug fix: mandos-ctl --secret option now works.
-** New D-Bus signal: se.recompile.Mandos.NewRequest(s).
-
-Version 1.4.1 (2011-10-15)
-* Server
-** Make D-Bus properties settable again, and handle checkers
-   for disabled clients correctly.
-* Miscellaneous fixes to "pedantic" Lintian warnings
-
-Version 1.4.0 (2011-10-09)
-* README file migrated to manual page intro(8mandos).
-* Client:
-** Fixed warning about "rmdir: Directory not empty".
-* Server:
-** Default values changed: timeout 5 minutes, interval 2 minutes.
-** Clients gets an expiration extension when receiving a password,
-   controlled by new "extended_timeout" setting.
-** New domain name: "fukt.bsnet.se" changes to "recompile.se".  This
-   also affects the D-Bus bus and interface names (old names still
-   work).  Users should start using the new names immediately.
-** New D-Bus Client object properties "Expires" and "ExtendedTimeout";
-   see DBUS-API for details.
-
 Version 1.3.1 (2011-07-27)
 * Client:
 ** Client now retries all Mandos servers periodically.

=== modified file 'README'
--- README	2016-03-23 07:11:22 +0000
+++ README	2011-10-05 16:00:56 +0000
@@ -1,8 +1,7 @@
-Please see: https://www.recompile.se/mandos/man/intro.8mandos
+Please see: http://www.recompile.se/mandos/man/intro.8mandos
 
 This information previously in this file has been moved to the
-intro(8mandos) manual page.  Go to the above URL, or install the
-Mandos server and run this command:
+intro(8mandos) manual page.  Go to the above URL, or run this command:
 
 	man 8mandos intro
 

=== modified file 'TODO'
--- TODO	2019-02-13 08:45:09 +0000
+++ TODO	2011-09-26 19:36:18 +0000
@@ -1,105 +1,99 @@
 -*- org -*-
 
-* Testing
-** python-nemu
+* Use _attribute_((nonnull)) wherever possible.
+
+* [[http://www.undeadly.org/cgi?action=article&sid=20110530221728][OpenBSD]]
 
 * mandos-applet
 
 * mandos-client
-** TODO A --server option which only adds to the server list.
-   (Unlike --connect, which implicitly disables zeroconf.)
 ** TODO [#B] Use capabilities instead of seteuid().
-   https://forums.grsecurity.net/viewtopic.php?f=7&t=2522
+** TODO [#B] Use struct sockaddr_storage instead of a union
 ** TODO [#B] Use getaddrinfo(hints=AI_NUMERICHOST) instead of inet_pton()
-** TODO [#C] Make start_mandos_communication() take "struct server".
-** TODO [#C] --interfaces=regex,eth*,noregex (bridge-utils-interfaces(5))
-** TODO [#A] Detect partial writes to stdout and exit with EX_TEMPFAIL
-** TODO [#B] Use reallocarray() with GNU LibC 2.29 or later.
+** TODO [#B] Use getnameinfo(serv=NULL, NI_NUMERICHOST) instead of inet_ntop()
+** TODO [#B] Prefer /run/tmp over /tmp, if it exists
 
 * splashy
 ** TODO [#B] use scandir(3) instead of readdir(3)
-** TODO [#A] Detect partial writes to stdout and exit with EX_TEMPFAIL
 
-* usplash (Deprecated)
-** TODO [#B] Make it work again
+* usplash
+** TODO [#A] Make it work again
 ** TODO [#B] use scandir(3) instead of readdir(3)
-** TODO [#A] Detect partial writes to stdout and exit with EX_TEMPFAIL
+** TODO Use [[info:libc:Argz%20Functions][argz_extract]]
 
 * askpass-fifo
-** TODO [#A] Detect partial writes to stdout and exit with EX_TEMPFAIL
+** TODO [#B] Drop privileges after opening FIFO.
 
 * password-prompt
 ** TODO [#B] lock stdin (with flock()?)
-** TODO [#A] Detect partial writes to stdout and exit with EX_TEMPFAIL
 
 * plymouth
-** TODO [#A] Detect partial writes to stdout and exit with EX_TEMPFAIL
-** TODO [#B] Use reallocarray() with GNU LibC 2.29 or later.
 
 * TODO [#B] passdev
 
 * plugin-runner
 ** TODO handle printing for errors for plugins
-*** Hook up stderr of plugins, buffer them, and prepend "Mandos Plugin [plugin name]"
+*** Hook up stderr of plugins, buffer them, and prepend mandos pluig [plugin name]
+** TODO [#B] use scandir(3) instead of readdir(3)
 ** TODO [#C] use same file name rules as run-parts(8)
 ** kernel command line option for debug info
-** TODO [#A] Restart plugins which exit with EX_TEMPFAIL
+** TODO [#B] Use openat()
 
 * mandos (server)
-** TODO [#B] --notify-command
-   This would allow the mandos.service to use
-   --notify-command="systemd-notify --pid --ready"
-** TODO [#B] python-systemd
-*** import systemd.daemon; systemd.daemon.notify()
 ** TODO [#B] Log level							  :BUGS:
+** TODO Persistent state						  :BUGS:
+   /var/lib/mandos/*
 *** TODO /etc/mandos/clients.d/*.conf
     Watch this directory and add/remove/update clients?
 ** TODO [#C] config for TXT record
-** TODO Log level dbus option
-   SetLogLevel D-Bus call
+** TODO Log level option
+   syslogger.setLevel(logging.WARNING)
+   + SetLogLevel D-Bus call
+** TODO Implement --foreground						  :BUGS:
+   [[info:standards:Option%20Table][Table of Long Options]]
+** TODO Implement --socket
+   [[info:standards:Option%20Table][Table of Long Options]]
+** TODO Date+time on console log messages				  :BUGS:
+   Is this the default?
 ** TODO [#C] DBusServiceObjectUsingSuper
 ** TODO [#B] Global enable/disable flag
-** TODO [#B] By-client countdown on number of secrets given
-** D-Bus Client method NeedsPassword(50) - Timeout, default disapprove
+** TODO [#B] By-client countdown on secrets given
+** TODO [#B] Support RFC 3339 time duration syntax
+** More D-Bus methods
+*** NeedsPassword(50) - Timeout, default disapprove
     + SetPass(u"gazonk", True) -> Approval, persistent
     + Approve(False) -> Close client connection immediately
 ** TODO [#C] python-parsedatetime
+** TODO [#C] systemd/launchd
+   http://0pointer.de/blog/projects/systemd.html
 ** TODO Separate logging logic to own object
-** TODO [#B] Limit approval_delay to max gnutls/tls timeout value
+** TODO make clients to a dict!
+** TODO [#A] Limit approval_delay to max gnutls/tls timeout value
 ** TODO [#B] break the wait on approval_delay if connection dies
 ** TODO Generate Client.runtime_expansions from client options + extra
 ** TODO Allow %%(checker)s as a runtime expansion
-** TODO D-Bus AddClient() method on server object
-** TODO Use org.freedesktop.DBus.Method.NoReply annotation on async methods. :2:
-** TODO Save state periodically to recover better from hard shutdowns
-** TODO CheckerCompleted method, deprecate CheckedOK
-** TODO Secret Service API?
-   https://standards.freedesktop.org/secret-service/
-** TODO Remove D-Bus interfaces with old domain name			  :2:
-** TODO Remove old string_to_delta format				  :2:
-** TODO http://0pointer.de/blog/projects/stateless.html
-*** File in /usr/lib/sysusers.d to create user+group "_mandos"
-** TODO Error handling on error parsing config files
-** TODO init.d script error handling
-** TODO D-Bus server properties; address, port, interface, etc.		  :2:
-** Python 3								  :2:
-*** TODO [#C] In Python 3.3, use shlex.quote() instead of re.escape()
+** TODO Use python-tlslite?
+
+* mandos.xml
+** Add mandos contact info in manual pages
 
 * mandos-ctl
 *** Handle "no D-Bus server" and/or "no Mandos server found" better
-** TODO Remove old string_to_delta format				  :2:
+*** [#B] --dump option
+** TODO Support RFC 3339 time duration syntax
 
 * TODO mandos-dispatch
   Listens for specified D-Bus signals and spawns shell commands with
   arguments.
 
 * mandos-monitor
-** TODO --servicename						       :BUGS:
 ** TODO help should be toggleable
 ** Urwid client data displayer
    Better view of client data in the listing
 *** Properties popup
-** Print a nice "We are sorry" message, save stack trace to log.
+** Nicer crashes. Stack traces Messes up shell.
+*** Print a nice "We are sorry" message, save stack trace to log.
+** Show timeout countdown for approval
 
 * mandos-keygen
 ** TODO "--secfile" option
@@ -107,19 +101,21 @@
 ** TODO [#B] "--test" option
    For testing decryption before rebooting.
 
+* Makefile
+** TODO [#C] Implement DEB_BUILD_OPTIONS
+   http://www.debian.org/doc/debian-policy/ch-source.html#s-debianrules-options
+
 * Package
 ** /usr/share/initramfs-tools/hooks/mandos
 *** TODO [#C] use same file name rules as run-parts(8)
 *** TODO [#C] Do not install in initrd.img if configured not to.
     Use "/etc/initramfs-tools/hooksconf.d/mandos"?
-** TODO [#C] $(pkg-config --variable=completionsdir bash-completion)
+** TODO [#C] /etc/bash_completion.d/mandos
    From XML sources directly?
 
 * Side Stuff
-** TODO Locate which package moves the other bin/sh when busybox is deactivated
-** TODO contact owner of package, and ask them to have that shell static in position regardless of busybox
-
-* [[http://www.undeadly.org/cgi?action=article&sid=20110530221728][OpenBSD]]
+** TODO Locate which packet move the other bin/sh when busy box is deactivated
+** TODO contact owner of packet, and ask them to have that shell static in position regardless of busybox
 
 
 #+STARTUP: showall

=== removed file 'bugs.xml'
--- bugs.xml	2016-03-05 21:42:56 +0000
+++ bugs.xml	1970-01-01 00:00:00 +0000
@@ -1,11 +0,0 @@
-<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE legalnotice PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
-	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd">
-<para>
-  Please report bugs to the Mandos development mailing list:
-  <email>mandos-dev@recompile.se</email> (subscription required).
-  Note that this list is public.  The developers can be reached
-  privately at <email>mandos@recompile.se</email> (OpenPGP key
-  fingerprint <literal>153A 37F1 0BBA 0435 987F 2C4A 7223 2973 CA34
-  C2C4</literal> for encrypted mail).
-</para>

=== modified file 'clients.conf'
--- clients.conf	2019-02-09 23:34:15 +0000
+++ clients.conf	2011-09-19 09:42:55 +0000
@@ -4,19 +4,19 @@
 
 # How long until a client is disabled and not be allowed to get the
 # data this server holds.
-;timeout = PT5M
+;timeout = 5m
 
 # How often to run the checker to confirm that a client is still up.
 # Note: a new checker will not be started if an old one is still
 # running.  The server will wait for a checker to complete until the
 # above "timeout" occurs, at which time the client will be disabled,
 # and any running checker killed.
-;interval = PT2M
+;interval = 2m
 
 # Extended timeout is an added timeout that is given once after a
 # password has been sent sucessfully to a client.  This allows for
 # additional delays caused by file system checks and quota checks.
-;extended_timeout = PT15M
+;extended_timeout = 15m
 
 # What command to run as "the checker".
 ;checker = fping -q -- %%(host)s
@@ -25,22 +25,16 @@
 ;approved_by_default = True
 
 # How long to wait for approval.
-;approval_delay = PT0S
+;approval_delay = 0s
 
 # How long one approval will last.
-;approval_duration = PT1S
-
-# Whether this client is enabled by default
-;enabled = True
+;approval_duration = 1s
 
 
 ;####
 ;# Example client
 ;[foo]
 ;
-;# TLS public key ID
-;key_id = f33fcbed11ed5e03073f6a55b86ffe92af0e24c045fb6e3b40547b3dc0c030ed
-;
 ;# OpenPGP key fingerprint
 ;fingerprint =  7788 2722 5BA7 DE53 9C5A  7CFA 59CF F7CD BD9A 5920
 ;
@@ -71,23 +65,20 @@
 ;####
 ;# Another example client, named "bar".
 ;[bar]
-;# The key ID is not space or case sensitive
-;key_id = F33FCBED11ED5E03073F6A55B86FFE92 AF0E24C045FB6E3B40547B3DC0C030ED
-;
 ;# The fingerprint is not space or case sensitive
 ;fingerprint = 3e393aeaefb84c7e89e2f547b3a107558fca3a27
 ;
 ;# If "secret" is not specified, a file can be read for the data.
-;secfile = /etc/keys/mandos/bar-secret.bin
+;secfile = /etc/mandos/bar-secret.bin
 ;
 ;# An IP address for host is also fine, if the checker accepts it.
 ;host = 192.0.2.3
 ;
 ;# Parameters from the [DEFAULT] section can be overridden per client.
-;interval = PT1M
+;interval = 1m
 ;
 ;# This client requires manual approval before it receives its secret.
 ;approved_by_default = False
 ;# Require approval within 30 seconds.
-;approval_delay = PT30S
+;approval_delay = 30s
 ;####

=== modified file 'common.ent'
--- common.ent	2019-02-11 06:31:42 +0000
+++ common.ent	2011-07-27 17:58:27 +0000
@@ -1,3 +1,3 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!-- Edited by Makefile -->
-<!ENTITY version "1.8.3">
+<!ENTITY version "1.3.1">

=== modified file 'debian/changelog'
--- debian/changelog	2019-02-11 11:52:07 +0000
+++ debian/changelog	2011-07-27 17:58:27 +0000
@@ -1,585 +1,3 @@
-mandos (1.8.3-2) unstable; urgency=medium
-
-  * debian/rules (override_dh_shlibdeps-arch): New; conditionally edit
-    debian/control before running dh_shlibdeps.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Mon, 11 Feb 2019 12:49:57 +0100
-
-mandos (1.8.3-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/watch: Make the ".orig" file name suffix non-optional;
-    otherwise uscan thinks that ".orig" is part of the version number.
-  * debian/control (Build-Depends): Changed GnuTLS dependencies; move
-    3.6.6 alternative to first in list, and remove dependencies on the
-    virtual package "gnutls-dev", since we need the version restrictions.
-    (Package: mandos/Depends): Remove dependency on libgnutls28-dev
-    package.
-    (Package: mandos/Suggests): New; set to "libc6-dev, c-compiler". (Used
-    to find value of "SO_BINDTODEVICE").
-    (Package: mandos-client/Depends): Don't depend on openssl anymore;
-    instead depend on either a gnutls-bin (>= 3.6.6) (in which case TLS
-    key generation will work), or on libgnutls30 (<< 3.6.0) (in which case
-    TLS key generation will not be needed).
-
- -- Teddy Hogeborn <teddy@recompile.se>  Mon, 11 Feb 2019 07:30:32 +0100
-
-mandos (1.8.2-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/mandos-client.postinst (create_keys): Ignore failure to remove
-                                                 bad keys.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 10 Feb 2019 11:44:56 +0100
-
-mandos (1.8.1-1) unstable; urgency=high
-
-  * New upstream release.
-  * debian/mandos-client.postinst (create_keys): Remove any bad keys
-    created by 1.8.0-1. Only create TLS keys if certtool succeeds.
-  * debian/mandos.postinst (configure): Remove any bad keys from
-    clients.conf, and inform the user if any were found.
-  * debian/mandos.templates (mandos/removed_bad_key_ids): New message.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 10 Feb 2019 10:00:21 +0100
-
-mandos (1.8.0-1) unstable; urgency=medium
-
-  * New upstream release.
-  * Fix "(tries to) use GnuTLS OpenPGP support" by using raw public keys
-    when available (Closes: #879538)
-  * Fix "mandos : Depends: libgnutls30 (< 3.6.0) but 3.6.5-2 is to be
-    installed" by now also allowing GnuTLS >= 3.6.6 (Closes: #916673)
-  * debian/control (Standards-Version): Update to "4.3.0".
-    (Package: mandos-client/Depends): Change from "cryptsetup" to
-    "cryptsetup (<< 2:2.0.3-1) | cryptsetup-initramfs".  Add "debconf (>=
-    1.5.5) | debconf-2.0".
-    (Source: mandos/Build-Depends): Also allow libgnutls30 (>= 3.6.6).
-    (Package: mandos/Depends): - '' - and add debconf (>= 1.5.5) |
-    debconf-2.0".
-    (Package: mandos/Description): Alter description to match new design.
-    (Package: mandos-client/Description): - '' -
-    (Package: mandos-client/Depends): Move "gnutls-bin | openssl" to here
-    from "Recommends".
-  * debian/mandos-client.README.Debian: Add --tls-privkey and --tls-pubkey
-    options to test command.
-  * debian/mandos-client.postinst (create_key): Renamed to "create_keys"
-    - all callers changed - and also create TLS key files.  Show notice if
-    new TLS key files were created.
-  * debian/mandos-client.postrm (purge): Also remove TLS key files.
-  * debian/mandos-client.lintian-overrides: Override warnings.
-  * debian/mandos-client.templates: New.
-  * debian/mandos.lintian-overrides: Override warnings.
-  * debian/mandos.postinst (configure): If GnuTLS 3.6.6 or later is
-    detected, show an important notice (once) about the new key_id option
-    required in clients.conf.
-  * debian/mandos.templates: New.
-  * debian/copyright: Update copyright year to 2019.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 10 Feb 2019 05:52:49 +0100
-
-mandos (1.7.20-1) unstable; urgency=medium
-
-  * New upstream release.
-  * Fix "[tethys] mandos-client: Mandos client fails while booting but
-    works from chroot into unpacked initramfs" by setting system clock if
-    necessary (Closes: #894495)
-  * Fix "initramfs boot script assumes internal cryptsetup implementation
-    details and is now broken" by only using documented
-    interfaces (Closes: #904899)
-  * debian/mandos-client.dirs: Add
-    "usr/share/initramfs-tools/scripts/local-premount" and
-    "usr/share/initramfs-tools/conf.d", and remove
-    "usr/share/initramfs-tools/conf-hooks.d".
-  * debian/control (mandos-client/Depends): Add "(>= 0.99)" to dependency
-    on "initramfs-tools".
-  * debian/control (Source: mandos/Rules-Requires-Root): New; set to
-    "binary-targets".
-    (Standards-Version): Update to "4.2.0".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 19 Aug 2018 22:14:04 +0200
-
-mandos (1.7.19-1) unstable; urgency=medium
-
-  * New upstream release.
-  * Fix "fails with "LeakSanitizer has encountered a fatal error"" by not
-    using LeakSanitizer in affected binary (Closes: #886595)
-
- -- Teddy Hogeborn <teddy@recompile.se>  Thu, 22 Feb 2018 19:47:59 +0100
-
-mandos (1.7.18-1) unstable; urgency=medium
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Mon, 12 Feb 2018 16:00:11 +0100
-
-mandos (1.7.17-1) unstable; urgency=medium
-
-  * New upstream release.
-  * Fix "fails with "LeakSanitizer has encountered a fatal error""
-    by fixing memory leak in plugin-runner (Closes: #886595)
-  * debian/control (Build-Depends): Also depend on "libgnutls28-dev (<<
-    3.6.0) | libgnutls30 (<< 3.6.0)".
-    (Package: mandos/Depends): - '' -
-  * debian/compat: Change to "10".
-  * debian/watch (version): Change to "4".
-    (opts/pgpsigurlmangle): Remove.
-    (opts/pgpmode): New; set to "auto".
-    (URL): Change to "https://ftp.recompile.se/pub/@PACKAGE@/@PACKAGE@
-    @ANY_VERSION@(?:\.orig)?@ARCHIVE_EXT@".
-  * debian/copyright: Update copyright year to 2018.
-  * debian/rules: Support the "noopt" and "parallel" DEB_BUILD_OPTIONS.
-    (override_dh_fixperms-arch): Use the DEB_HOST_MULTIARCH
-    variable directly instead of shelling out to "dpkg-architecture".
-  * debian/control (Standards-Version): Update to "4.1.3".
-    (Build-Depends): Change version of debhelper dependency to ">= 10".
-  * debian/mandos.lintian-overrides
-    (init.d-script-needs-depends-on-lsb-base): Change line number to "46".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sat, 10 Feb 2018 19:09:50 +0100
-
-mandos (1.7.16-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/copyright (License): Use program name explicitly.
-    (Format): Use https in URL.
-  * debian/control (Priority): Change from "extra" to "optional".
-    (Standards-Version): Update to "4.0.1".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 20 Aug 2017 21:05:26 +0200
-
-mandos (1.7.15-1) unstable; urgency=medium
-
-  * New upstream release.
-  * Upstream release fixes "Seems not to be honoring zeroconf option at
-    mandos.conf" (Closes: #855589)
-  * debian/mandos.lintian-overrides (mandos): Add new line
-    "init.d-script-needs-depends-on-lsb-base etc/init.d/mandos (line 49)".
-  * debian/copyright: Update copyright year to 2017.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Thu, 23 Feb 2017 21:29:36 +0100
-
-mandos (1.7.14-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/mandos-client.postinst (create_key): Stop GPG agent after
-    running mandos-keygen.
-  * debian/control (Package: mandos/Depends): Add "systemd-sysv | lsb-base
-    (>= 3.0-6)", change "gnupg" to "gnupg2 | gnupg", and change
-    "libgpgme11-dev" to "libgpgme-dev | libgpgme11-dev".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Wed, 25 Jan 2017 20:36:03 +0100
-
-mandos (1.7.13-1) unstable; urgency=medium
-
-  * New upstream release.
-  * Fix "fails to install noninteractively" by using the "%no-protection"
-    statement in the GnuPG batch parameter file. (Closes: #840001)
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sat, 08 Oct 2016 06:31:07 +0200
-
-mandos (1.7.12-1) unstable; urgency=medium
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Wed, 05 Oct 2016 22:06:55 +0200
-
-mandos (1.7.11-1) unstable; urgency=high
-
-  * New upstream release.
-  * debian/control (Source: mandos/Vcs-Bzr): Change to use HTTPS.
-    (Vcs-Browser): - '' -
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sat, 01 Oct 2016 16:20:48 +0200
-
-mandos (1.7.10-1) unstable; urgency=high
-
-  * New upstream release.
-  * debian/rules (override_dh_fixperms-arch): Also exclude
-    "etc/mandos/plugin-helpers" from changes by dh_fixperms.
-  * debian/mandos-client.postinst: Fix the permissions of
-    "/etc/mandos/plugin-helpers" for those systems which had a fresh
-    install of an older version.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Thu, 23 Jun 2016 22:00:29 +0200
-
-mandos (1.7.9-1) unstable; urgency=medium
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Wed, 22 Jun 2016 07:30:12 +0200
-
-mandos (1.7.8-1) unstable; urgency=medium
-
-  * New upstream release.
-  * Fix "bad gpgme_op_decrypt: GPGME: Decryption failed." by copying
-    /usr/bin/gpg-agent into initramfs (Closes: #819982)
-  * debian/control (Homepage): Change URL to use HTTPS.
-    (Standards-Version): Update to 3.9.8.
-  * debian/copyright (Source): Change URL to HTTPS.
-  * debian/mandos-client.README.Debian: Change wording to match updated
-    capabilities.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Tue, 21 Jun 2016 21:36:10 +0200
-
-mandos (1.7.7-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/mandos-client.postinst (configure): If older version, fix
-    permissions on plugin helper directory.  Also fix permissions on
-    plugin helper local override directory (/etc/mandos/plugin-helpers),
-    but only if not listed by "dpkg-statoverride".
-  * debian/rules (override_dh_fixperms-arch): Exclude plugin helper
-    directory from dh_fixperms.
-  * debian/mandos.postinst (configure): Fix state directory permissions,
-    but only if not listed by "dpkg-statoverride".
-  * debian/mandos-client.lintian-overrides: Do not warn about permissions
-    on plugin helper directory.
-  * debian/mandos.dirs (usr/lib/tmpfiles.d): Added.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sat, 19 Mar 2016 22:58:49 +0100
-
-mandos (1.7.6-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/control (Source: mandos/Build-Depends-Indep): Remove
-    "python-avahi".
-    (Source: mandos/Build-Depends-Indep): Change "python-gi |
-    python-gobject" to "python-gi"; i.e. remove "python-gobject".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 13 Mar 2016 22:58:23 +0100
-
-mandos (1.7.5-1) unstable; urgency=high
-
-  * New upstream release.
-  * debian/mandos.postinst (configure): If old version was 1.7.4-1 or
-    1.7.4-1~bpo8+1, fix situation where clients.pickle file is owned by
-    root.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Tue, 08 Mar 2016 01:09:55 +0100
-
-mandos (1.7.4-1) unstable; urgency=medium
-
-  * New upstream release.
-  * initramfs-tools-script: Fix "Call to configure_network in initramfs
-    script broken due to set -e" by surrounding call by "set +x" and "set
-    -e" (Closes: #816513)
-  * debian/control: (Source: mandos/Build-Depends-Indep): Change
-    "python-gobject | python-gi" to "python-gi | python-gobject"
-    (Package: mandos/Depends): - '' -
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sat, 05 Mar 2016 23:10:07 +0100
-
-mandos (1.7.3-1) unstable; urgency=medium
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Mon, 29 Feb 2016 22:26:38 +0100
-
-mandos (1.7.2-1) unstable; urgency=medium
-
-  * New upstream release.
-  * Fix "Uses unneeded and obsolete version specific python packages"
-    by removing version-specific dependencies (Closes: #811159)
-  * debian/control (Source: mandos/Build-Depends): Add (>= 3.3.0) to
-    "libgnutls28-dev" and "gnutls-dev".
-    (Source: mandos/Build-Depends-Indep): Remove "python2.7-gnutls",
-    "python2.7", "python2.7-dbus", "python2.7-avahi", and
-    "python2.7-gobject"; replace with "python (>= 2.7), python (<< 3)",
-    "python-dbus", "python-avahi", "python-gobject | python-gi".
-    (Package: mandos/Depends): Remove "python-gnutls" and
-    "python2.7-gnutls", add "libgnutls28-dev (>= 3.3.0) | libgnutls30 (>=
-    3.3.0)".  Add "python (<< 3)".  Remove "python2.7-dbus",
-    "python2.7-avahi", "python2.7-gobject", and "python2.7-urwid".
-    Replace "python-gobject" with "python-gobject | python-gi" and "gnupg
-    (<< 2)" with "gnupg".
-    (Package: mandos-client/Depends): Replace
-    "gnupg (<< 2)" with "gnupg".
-    (Source: mandos/Standards-Version): Change to 3.9.7.
-  * debian/copyright (Copyright): Update copyright year.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 28 Feb 2016 16:09:01 +0100
-
-mandos (1.7.1-2) unstable; urgency=medium
-
-  * debian/control (Package: mandos/Depends): Fix "Please drop versioned
-    dependency on initscripts package" by removing initscripts dependency
-    (Closes: #804967)
-  * debian/rules (override_dh_fixperms) Fix "FTBFS when built with
-    dpkg-buildpackage -A (No such file or directory)" by splitting into
-    "override_dh_fixperms-arch" and "override_dh_fixperms-indep".
-    (Closes: #806073)
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sat, 05 Dec 2015 02:27:40 +0100
-
-mandos (1.7.1-1) unstable; urgency=medium
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sat, 24 Oct 2015 19:43:40 +0200
-
-mandos (1.7.0-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/control (Standards-Version): Updated to "3.9.6".
-    (Build-Depends): Add "libnl-route-3-dev".
-    (Package: mandos-client/Recommends): Added "gnutls-bin | openssl" for
-    the generating of DH parameters.
-  * debian/mandos-client.README.Debian: Update example command line to use
-    new MANDOSPLUGINHELPERDIR environment variable.  Also document the new
-    dhparams.pem file.
-  * debian/mandos-client.postinst: Create DH parameters file.
-  * debian/mandos.prerm: Don't run init script, use only invoke-rc.d.
-  * debian/mandos-client.postinst: Don't use absolute paths to commands.
-  * debian/mandos-client.postrm: Don't use absolute paths to commands.
-    Also remove dhparams.pem file.
-  * debian/copyright (Copyright): Update copyright year.
-  * Upstream changed systemd service file to implicitly be of
-    "Type=dbus". (Closes: #786845)
-
- -- Teddy Hogeborn <teddy@recompile.se>  Mon, 10 Aug 2015 22:00:29 +0200
-
-mandos (1.6.9-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/control (Build-Depends): Fix "still uses GnutLS 2.x" by
-    changing from "libgnutls-dev" to "libgnutls28-dev | gnutls-dev"
-    (Closes: #762349)
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 05 Oct 2014 22:05:06 +0200
-
-mandos (1.6.8-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/control (Source: mandos/Build-Depends-Indep): Since upstream
-    now requires Python 2.7, depend on exactly the python2.7 package and
-    all the Python 2.7 versions of the python modules.
-    (Package: mandos/Depends): - '' - but still depend on python (>=2.7)
-    and the generic versions of the Python modules; this is for mandos-ctl
-    and mandos-monitor, both of which are compatible with Python 3, and
-    use #!/usr/bin/python.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Wed, 06 Aug 2014 22:55:24 +0200
-
-mandos (1.6.7-1) unstable; urgency=medium
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Thu, 17 Jul 2014 05:22:45 +0200
-
-mandos (1.6.6-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/mandos.postinst: Fix typo in comment.
-  * debian/control (mandos/Recommends): Changed to "ssh-client | fping".
-    (mandos-client/Recommends): New; set to "ssh".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 13 Jul 2014 22:49:21 +0200
-
-mandos (1.6.5-3) unstable; urgency=medium
-
-  * debian/control (mandos-client/Depends): Add "dpkg-dev (>=1.16.0)";
-    initramfs-tools-hook runs "dpkg-architecture -qDEB_HOST_MULTIARCH".
-    (Closes: #750221)
-
- -- Teddy Hogeborn <teddy@recompile.se>  Fri, 06 Jun 2014 04:27:15 +0200
-
-mandos (1.6.5-2) unstable; urgency=medium
-
-  * debian/rules (override_dh_auto_test-arch): New; does nothing.  Fixes
-    FTBFS for build-indep.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Tue, 13 May 2014 08:08:31 +0200
-
-mandos (1.6.5-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/copyright: Change year to "2014".
-  * debian/control (Build-Depends, Build-Depends-Indep): Moved build
-    dependencies of "mandos" package to "Build-Depends-Indep".
-  * debian/upstream/signing-key.asc: New; upstream source public key.
-  * debian/control (Standards-Version): Updated to "3.9.5".
-  * debian/control (mandos/Depends): Remove the dependency on
-    "avahi-daemon (>= 0.6.31-3) | systemd-sysv".  It is unnecessary
-    since we have a workaround in debian/mandos.postinst anyway.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 11 May 2014 22:16:33 +0200
-
-mandos (1.6.4-1) unstable; urgency=medium
-
-  * New upstream release.
-  * debian/control (Build-Depends): Add Python dependencies to
-                                    successfully run self-tests.
-  * debian/copyright: GPLv3 now has its own license file - use it.
-  * debian/watch: Set PGP signature URL.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 16 Feb 2014 14:09:25 +0100
-
-mandos (1.6.3-1) unstable; urgency=low
-
-  * New upstream release.
-  * debian/control (Build-Depends): Added "systemd".
-  * debian/mandos.dirs (lib/systemd/system): New.
-  * debian/mandos-client.README.Debian: Refer to architecture libdir.
-  * debian/control (mandos/Depends): Add "avahi-daemon (>= 0.6.31-3) |
-                                     systemd-sysv".
-  * debian/mandos.postinst: If avahi-daemon is version 0.6.31-2 or older,
-                            edit /etc/init.d script headers Required-Start
-                            and Required-Stop to have "avahi" instead of
-                            "avahi-daemon", before insserv(8) sees it.
-  * debian/mandos-client.lintian-overrides: Libdir changes.
-  * debian/rules (override_dh_fixperms): - '' -
-
- -- Teddy Hogeborn <teddy@recompile.se>  Tue, 21 Jan 2014 22:01:30 +0100
-
-mandos (1.6.2-1) unstable; urgency=low
-
-  * New upstream release.
-  * debian/compat: Changed to "9".
-  * debian/control (Build-Depends): Changed debhelper version to (>= 9).
-    (Standards-Version): Updated to "3.9.4".
-    (DM-Upload-Allowed): Removed.
-    (mandos/Depends): Add "initscripts (>= 2.88dsf-13.3)" to be able to
-                      use the "/run" directory (for mandos.pid).
-  * debian/copyright (Copyright): Update year.
-  * Fix "Mandos/gnutls fails to establish connection, "an algorithm that
-    is not enabled was negotiated"" fixed by upstream. (Closes: #702120)
-
- -- Teddy Hogeborn <teddy@recompile.se>  Thu, 24 Oct 2013 22:33:40 +0200
-
-mandos (1.6.1-1) unstable; urgency=low
-
-  * New upstream release.
-  * debian/control (mandos/Depends): No longer depends on
-                                     python-gnupginterface, but does
-                                     depend on gnupg (<< 2).
-    (Build-Depends): Depend on debhelper 8.9.7 for using "override-*-arch"
-                     and "override-*-indep" targets in debian/rules.
-  * debian/mandos-client.README: Update Linux documentation link.
-  * debian/rules: Completely rewritten to use debhelper v7.
-  * initramfs-tools-hook: Bug fix: Make sure the right version of GnuPG is
-    copied into the initramfs image.  Always assume that GPGME is used to
-    avoid searching for it since the path might not be /usr/lib.  Thanks
-    to Félix Sipma <felix+debian@gueux.org> for the initial bug report,
-    and also thanks to Dick Middleton <dick@lingbrae.com> for some more
-    debugging. (Closes:  #721903)
-  * Fix "bashism in /bin/sh script" fixed by upstream. (Closes: #690639)
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 13 Oct 2013 19:03:23 +0200
-
-mandos (1.6.0-1) unstable; urgency=low
-
-  * New upstream release.
-  * debian/copyright (Copyright): Join the two lines to a single line.
-  * debian/mandos-client.README.Debian: Update to refer to the new
-    location of the example network hooks, and the new feature of using
-    all network interfaces.
-  * debian/mandos-client.docs (network-hooks.d): Removed.
-  * debian/mandos-client.examples (network-hooks.d): New.
-  * debian/rules (binary-common): Added "dh_installexamples".
-    (binary-common/dh_fixperms): Exclude new location of
-    "network-hooks.d".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Mon, 18 Jun 2012 00:15:23 +0200
-
-mandos (1.5.5-1) unstable; urgency=low
-
-  * New upstream release.
-  * debian/copyright (Format): Updated to
-    "http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/".
-  * debian/control (Build-Depends): Removed "man, locales-all".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Fri, 01 Jun 2012 20:30:41 +0200
-
-mandos (1.5.4-1) unstable; urgency=low
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 20 May 2012 15:38:34 +0200
-
-mandos (1.5.3-1.2) unstable; urgency=low
-
-  * Non-maintainer upload.
-  * Set Architecture to linux-any.  (Closes: #647670)
-
- -- Robert Millan <rmh@debian.org>  Sun, 22 Apr 2012 16:22:01 +0200
-
-mandos (1.5.3-1.1) unstable; urgency=low
-
-  * Non-maintainer upload.
-  * Fix "mandos FTBFS on buildds": add build-dependency on locales-all and
-    pass LC_ALL to dh_auto_build to make sure we have and use the en_US.UTF-8
-    locale for manpage creation.
-    (Closes: #656178)
-
- -- gregor herrmann <gregoa@debian.org>  Tue, 31 Jan 2012 17:56:05 +0100
-
-mandos (1.5.3-1) unstable; urgency=low
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 15 Jan 2012 22:05:54 +0100
-
-mandos (1.5.2-1) unstable; urgency=low
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 08 Jan 2012 11:17:20 +0100
-
-mandos (1.5.1-1) unstable; urgency=low
-
-  * New upstream release.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 01 Jan 2012 21:53:31 +0100
-
-mandos (1.5.0-1) unstable; urgency=low
-
-  * New upstream release.
-  * debian/control (mandos-client/Depends): Added "initramfs-tools".
-  * debian/mandos-client.README.Debian: Corrected mail address and adjust
-                                        wording.
-  * debian/rules (binary-common): Exclude new nework-hooks.d directory
-                                  from dh_fixperms.
-  * debian/mandos-client.README.Debian: Document network hook facility.
-  * debian/mandos-client.docs (network-hooks.d): Added.
-  * debian/mandos.dirs (var/lib/mandos): Added.
-  * debian/mandos.postinst: Fix ownership of /var/lib/mandos.
-  * debian/control (mandos/Depends): Added "python-gnupginterface".
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 01 Jan 2012 05:58:11 +0100
-
-mandos (1.4.1-1) unstable; urgency=low
-
-  * New upstream release.
-  * debian/control (Build-Depends): Added "man".
-  * debian/control (Conflicts): Changed to "Breaks:".
-  * debian/copyright: Updated format.
-  * debian/mandos-client.postinst: Use "set -e" instead of "#!/bin/sh -e".
-  * debian/mandos-client.postrm: - '' -
-  * debian/mandos.postinst: - '' -
-  * debian/mandos.prerm: Consistent magic.
-
- -- Björn Påhlsson <belorn@fukt.bsnet.se>  Sat, 15 Oct 2011 18:18:52 +0200
-
-mandos (1.4.0-1) unstable; urgency=low
-
-  * New upstream release.
-  * Fix "FTBFS with binutils-gold": Added "-Xlinker --as-needed" to
-    LDFLAGS in Makefile. (Closes: #632145)
-  * Fix "/run transition: uses obsolete /dev/.initramfs": Try both old and
-    new PID file locations. (Closes: #643554)
-  * debian/source/local-options: New; contains "--single-debian-patch".
-  * debian/control (Standards-Version): Upgraded to "3.9.2".
-    (DM-Upload-Allowed): New; set to "yes".
-  * debian/control: Changed domain from "fukt.bsnet.se" to "recompile.se".
-  * debian/copyright: - '' -
-  * debian/mandos-client.README.Debian: - '' -
-  * debian/mandos.README.Debian: - '' -
-  * debian/watch: - '' -
-  * debian/control (mandos/Description): Fix language to placate lintian.
-
- -- Teddy Hogeborn <teddy@recompile.se>  Sun, 09 Oct 2011 19:15:08 +0200
-
 mandos (1.3.1-1) unstable; urgency=low
 
   * New upstream release.

=== modified file 'debian/compat'
--- debian/compat	2018-02-06 20:03:50 +0000
+++ debian/compat	2008-09-17 00:34:09 +0000
@@ -1,1 +1,1 @@
-10
+7

=== modified file 'debian/control'
--- debian/control	2019-02-11 06:14:29 +0000
+++ debian/control	2011-10-05 17:03:36 +0000
@@ -1,32 +1,24 @@
 Source: mandos
 Section: admin
-Priority: optional
+Priority: extra
 Maintainer: Mandos Maintainers <mandos@recompile.se>
 Uploaders: Teddy Hogeborn <teddy@recompile.se>,
 	   Björn Påhlsson <belorn@recompile.se>
-Build-Depends: debhelper (>= 10), docbook-xml, docbook-xsl,
-	libavahi-core-dev, libgpgme-dev | libgpgme11-dev,
-	libgnutls28-dev (>= 3.3.0),
-	libgnutls28-dev (>= 3.6.6) | libgnutls28-dev (<< 3.6.0),
-	xsltproc, pkg-config, libnl-route-3-dev
-Build-Depends-Indep: systemd, python (>= 2.7), python (<< 3),
-	python-dbus, python-gi
-Standards-Version: 4.3.0
-Vcs-Bzr: https://ftp.recompile.se/pub/mandos/trunk
-Vcs-Browser: https://bzr.recompile.se/loggerhead/mandos/trunk/files
-Homepage: https://www.recompile.se/mandos
-Rules-Requires-Root: binary-targets
+Build-Depends: debhelper (>= 7), docbook-xml, docbook-xsl,
+	libavahi-core-dev, libgpgme11-dev, libgnutls-dev, xsltproc,
+	pkg-config
+Standards-Version: 3.9.2
+Vcs-Bzr: http://ftp.recompile.se/pub/mandos/trunk
+Vcs-Browser: http://bzr.recompile.se/loggerhead/mandos/trunk/files
+Homepage: http://www.recompile.se/mandos
+DM-Upload-Allowed: yes
 
 Package: mandos
 Architecture: all
-Depends: ${misc:Depends}, python (>= 2.7), python (<< 3),
-	libgnutls30 (>= 3.3.0),
-	libgnutls30 (>= 3.6.6) | libgnutls30 (<< 3.6.0),
-	python-dbus, python-gi, avahi-daemon, adduser, python-urwid,
-	gnupg2 | gnupg, systemd-sysv | lsb-base (>= 3.0-6),
-	debconf (>= 1.5.5) | debconf-2.0
-Recommends: ssh-client | fping
-Suggests: libc6-dev | libc-dev, c-compiler
+Depends: ${misc:Depends}, python (>=2.6), python-gnutls, python-dbus,
+	python-avahi, python-gobject, avahi-daemon, adduser,
+	python-urwid, python (>=2.7) | python-argparse
+Recommends: fping
 Description: server giving encrypted passwords to Mandos clients
  This is the server part of the Mandos system, which allows
  computers to have encrypted root file systems and at the
@@ -35,22 +27,18 @@
  The computers run a small client program in the initial RAM
  disk environment which will communicate with a server over a
  network.  All network communication is encrypted using TLS.
- The clients are identified by the server using a TLS public
+ The clients are identified by the server using an OpenPGP
  key; each client has one unique to it.  The server sends the
  clients an encrypted password.  The encrypted password is
- decrypted by the clients using an OpenPGP key, and the
+ decrypted by the clients using the same OpenPGP key, and the
  password is then used to unlock the root file system,
  whereupon the computers can continue booting normally.
 
 Package: mandos-client
-Architecture: linux-any
-Depends: ${shlibs:Depends}, ${misc:Depends}, adduser,
-	cryptsetup (<< 2:2.0.3-1) | cryptsetup-initramfs,
-	initramfs-tools (>= 0.99), dpkg-dev (>=1.16.0),
-	gnutls-bin (>= 3.6.6) | libgnutls30 (<< 3.6.0),
-	debconf (>= 1.5.5) | debconf-2.0
-Recommends: ssh
-Breaks: dropbear (<= 0.53.1-1)
+Architecture: any
+Depends: ${shlibs:Depends}, ${misc:Depends}, adduser, cryptsetup,
+	gnupg (<< 2)
+Conflicts: dropbear (<= 0.52-5)
 Enhances: cryptsetup
 Description: do unattended reboots with an encrypted root file system
  This is the client part of the Mandos system, which allows
@@ -60,9 +48,9 @@
  The computers run a small client program in the initial RAM
  disk environment which will communicate with a server over a
  network.  All network communication is encrypted using TLS.
- The clients are identified by the server using a TLS public
+ The clients are identified by the server using an OpenPGP
  key; each client has one unique to it.  The server sends the
  clients an encrypted password.  The encrypted password is
- decrypted by the clients using an OpenPGP key, and the
+ decrypted by the clients using the same OpenPGP key, and the
  password is then used to unlock the root file system,
  whereupon the computers can continue booting normally.

=== modified file 'debian/copyright'
--- debian/copyright	2019-02-10 04:20:26 +0000
+++ debian/copyright	2011-10-05 16:00:56 +0000
@@ -1,26 +1,26 @@
-Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
+Format-Specification:
+ http://wiki.debian.org/Proposals/CopyrightFormat?action=recall&rev=233
 Upstream-Name: Mandos
-Upstream-Contact: Mandos <mandos@recompile.se>
-Source: <https://www.recompile.se/mandos>
+Upstream-Maintainer: Mandos Maintainers <mandos@recompile.se>
+Upstream-Source: <http://www.recompile.se/mandos>
 
 Files: *
-Copyright: Copyright © 2008-2019 Teddy Hogeborn
-	   Copyright © 2008-2019 Björn Påhlsson
+Copyright: Copyright © 2008-2011 Teddy Hogeborn
+Copyright: Copyright © 2008-2011 Björn Påhlsson
 License: GPL-3+
-    This file is part of Mandos.
-    .
-    Mandos is free software: you can redistribute it and/or modify it
-    under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-    .
-    Mandos is distributed in the hope that it will be useful, but
-    WITHOUT ANY WARRANTY; without even the implied warranty of
+    This program is free software: you can redistribute it and/or
+    modify it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of the
+    License, or (at your option) any later version.
+    .
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.
     .
     You should have received a copy of the GNU General Public License
-    along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+    along with this program.  If not, see
+    <http://www.gnu.org/licenses/>.
     .
     On Debian systems, the complete text of the GNU General Public
-    License can be found in "/usr/share/common-licenses/GPL-3".
+    License can be found in "/usr/share/common-licenses/GPL".

=== modified file 'debian/mandos-client.README.Debian'
--- debian/mandos-client.README.Debian	2019-02-09 23:23:26 +0000
+++ debian/mandos-client.README.Debian	2011-10-05 16:00:56 +0000
@@ -1,7 +1,3 @@
-This file documents the next steps to take after installation of the
-Debian package, and also contain some notes specific to the Debian
-packaging which are not also in the manual.
-
 * Adding a Client Password to the Server
   
   The server must be given a password to give back to the client on
@@ -20,14 +16,9 @@
   is possible to verify that the correct password will be received by
   this client by running the command, on the client:
   
-	MANDOSPLUGINHELPERDIR=/usr/lib/$(dpkg-architecture \
-	-qDEB_HOST_MULTIARCH)/mandos/plugin-helpers \
-	/usr/lib/$(dpkg-architecture -qDEB_HOST_MULTIARCH \
-	)/mandos/plugins.d/mandos-client \
+	/usr/lib/mandos/plugins.d/mandos-client \
 		--pubkey=/etc/keys/mandos/pubkey.txt \
-		--seckey=/etc/keys/mandos/seckey.txt \
-		--tls-privkey=/etc/keys/mandos/tls-privkey.pem \
-		--tls-pubkey=/etc/keys/mandos/tls-pubkey.pem; echo
+		--seckey=/etc/keys/mandos/seckey.txt; echo
   
   This command should retrieve the password from the server, decrypt
   it, and output it to standard output.  There it can be verified to
@@ -41,31 +32,25 @@
 
 * Specifying a Client Network Interface
   
-  At boot time the network interfaces to use will by default be
-  automatically detected.  If this should result in incorrect
-  interfaces, edit the DEVICE setting in the
-  "/etc/initramfs-tools/initramfs.conf" file.  (The default setting is
-  empty, meaning it will autodetect the interfaces.)  *If* the DEVICE
-  setting is changed, it will be necessary to update the initrd image
-  by running this command:
+  At boot time the network interface to use will by default be
+  automatically detected.  If should result in an incorrect interface,
+  edit the DEVICE setting in the "/etc/initramfs-tools/initramfs.conf"
+  file.  (The default setting is empty, meaning to autodetect the
+  interface.)  *If* the DEVICE setting is changed, it will be
+  necessary to update the initrd image by running the command
   
 	update-initramfs -k all -u
   
-  The device can also be overridden at boot time on the Linux kernel
+  The device can be overridden at boot time on the Linux kernel
   command line using the sixth colon-separated field of the "ip="
   option; for exact syntax, read the documentation in the file
-  "/usr/share/doc/linux-doc-*/Documentation/filesystems/nfs/nfsroot.txt",
+  "/usr/share/doc/linux-doc-*/Documentation/filesystems/nfsroot.txt",
   available in the "linux-doc-*" package.
   
-  Note that since the network interfaces are used in the initial RAM
-  disk environment, the network interfaces *must* exist at that stage.
-  Thus, an interface can *not* be a pseudo-interface such as "br0" or
-  "tun0"; instead, only real interfaces (such as "eth0") can be used.
-  This can be overcome by writing a "network hook" program to create
-  an interface (see mandos-client(8mandos)) and placing it in
-  "/etc/mandos/network-hooks.d", from where it will be copied into the
-  initial RAM disk.  Example network hook scripts can be found in
-  "/usr/share/doc/mandos-client/examples/network-hooks.d".
+  Note that since this network interface is used in the initial RAM
+  disk environment, the network interface *must* exist at that stage.
+  Thus, the interface can *not* be a pseudo-interface such as "br0" or
+  "tun0"; instead, only real interface (such as "eth0") can be used.
 
 * User-Supplied Plugins
   
@@ -92,20 +77,11 @@
   "mandos=connect:<IP_ADDRESS>:<PORT_NUMBER>" on the kernel command
   line.
   
-  For very advanced users, it is possible to specify simply
+  For very advanced users, it it possible to specify simply
   "mandos=connect" on the kernel command line to make the system only
   set up the network (using the data in the "ip=" option) and not pass
   any extra "--connect" options to mandos-client at boot.  For this to
   work, "--options-for=mandos-client:--connect=<ADDRESS>:<PORT>" needs
   to be manually added to the file "/etc/mandos/plugin-runner.conf".
 
-* Diffie-Hellman Parameters
-
-  On installation, a file with Diffie-Hellman parameters,
-  /etc/keys/mandos/dhparams.pem, will be generated and automatically
-  installed into the initital RAM disk image and also used by the
-  Mandos Client on boot.  If different parameters are needed for
-  policy or other reasons, simply replace the existing dhparams.pem
-  file and update the initital RAM disk image.
-
- -- Teddy Hogeborn <teddy@recompile.se>, Sat,  9 Feb 2019 15:08:04 +0100
+ -- Teddy Hogeborn <teddy@recompile.se>, Wed,  5 Oct 2011 17:50:22 +0200

=== modified file 'debian/mandos-client.dirs'
--- debian/mandos-client.dirs	2018-08-19 14:32:00 +0000
+++ debian/mandos-client.dirs	2009-02-07 04:50:39 +0000
@@ -1,6 +1,5 @@
 usr/share/man/man8
 usr/sbin
 usr/share/initramfs-tools/hooks
-usr/share/initramfs-tools/conf.d
+usr/share/initramfs-tools/conf-hooks.d
 usr/share/initramfs-tools/scripts/init-premount
-usr/share/initramfs-tools/scripts/local-premount

=== removed file 'debian/mandos-client.examples'
--- debian/mandos-client.examples	2012-06-01 21:48:12 +0000
+++ debian/mandos-client.examples	1970-01-01 00:00:00 +0000
@@ -1,1 +0,0 @@
-network-hooks.d

=== modified file 'debian/mandos-client.lintian-overrides'
--- debian/mandos-client.lintian-overrides	2019-02-10 03:50:20 +0000
+++ debian/mandos-client.lintian-overrides	2010-10-13 06:12:52 +0000
@@ -2,14 +2,12 @@
 #
 mandos-client binary: non-standard-dir-perm etc/keys/mandos/ 0700 != 0755
 
-# The directory /usr/lib/<arch>/mandos/plugins.d contains setuid
-# binaries which are not meant to be run outside an initial RAM disk
+# The directory /usr/lib/mandos/plugins.d contains setuid binaries
+# which are not meant to be run outside an initial RAM disk
 # environment (except for test purposes).  It would be insecure to
 # allow anyone to run them.
 #
-mandos-client binary: non-standard-dir-perm usr/lib/*/mandos/plugins.d/ 0700 != 0755
-# Likewise for helper executables for plugins
-mandos-client binary: non-standard-dir-perm usr/lib/*/mandos/plugin-helpers/ 0700 != 0755
+mandos-client binary: non-standard-dir-perm usr/lib/mandos/plugins.d/ 0700 != 0755
 
 # These binaries must be setuid root, since they need root powers, but
 # are started by plugin-runner(8mandos), which runs all plugins as
@@ -17,27 +15,14 @@
 # system, but in an initial RAM disk environment.  Here they are
 # protected from non-root access by the directory permissions, above.
 #
-mandos-client binary: setuid-binary usr/lib/*/mandos/plugins.d/mandos-client 4755 root/root
-mandos-client binary: setuid-binary usr/lib/*/mandos/plugins.d/askpass-fifo 4755 root/root
-mandos-client binary: setuid-binary usr/lib/*/mandos/plugins.d/splashy 4755 root/root
-mandos-client binary: setuid-binary usr/lib/*/mandos/plugins.d/usplash 4755 root/root
-mandos-client binary: setuid-binary usr/lib/*/mandos/plugins.d/plymouth 4755 root/root
+mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/mandos-client 4755 root/root
+mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/askpass-fifo 4755 root/root
+mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/splashy 4755 root/root
+mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/usplash 4755 root/root
+mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/plymouth 4755 root/root
 
 # The directory /etc/mandos/plugins.d can be used by local system
 # administrators to place plugins in, overriding and complementing
-# /usr/lib/<arch>/mandos/plugins.d, and must be likewise protected.
+# /usr/lib/mandos/plugins.d, and must be likewise protected.
 #
 mandos-client binary: non-standard-dir-perm etc/mandos/plugins.d/ 0700 != 0755
-# Likewise for plugin-helpers directory
-mandos-client binary: non-standard-dir-perm etc/mandos/plugin-helpers/ 0700 != 0755
-
-# The debconf templates is only used for displaying information
-# detected in the postinst, not for saving answers to questions, so we
-# don't need a .config file.
-mandos-client binary: no-debconf-config
-
-# The notice displayed from the postinst script really is critical
-mandos-client binary: postinst-uses-db-input
-
-# It is a really long line
-mandos-client binary: manpage-has-errors-from-man usr/share/man/man8/plugin-runner.8mandos.gz *: warning *: can't break line

=== modified file 'debian/mandos-client.postinst'
--- debian/mandos-client.postinst	2019-02-10 10:39:26 +0000
+++ debian/mandos-client.postinst	2009-05-24 23:36:15 +0000
@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/sh -e
 # This script can be called in the following ways:
 #
 # After the package was installed:
@@ -15,14 +15,12 @@
 # If prerm fails during replacement due to conflict:
 #       <postinst> abort-remove in-favour <new-package> <version>
 
-. /usr/share/debconf/confmodule
-
-set -e
-
 # Update the initial RAM file system image
 update_initramfs()
 {
-    update-initramfs -u -k all
+    if [ -x /usr/sbin/update-initramfs ]; then
+	update-initramfs -u -k all
+    fi
     
     if dpkg --compare-versions "$2" lt-nl "1.0.10-1"; then
         # Make old initrd.img files unreadable too, in case they were
@@ -52,123 +50,22 @@
     fi
 }
 
-# Create client key pairs
-create_keys(){
-    # If the OpenPGP key files do not exist, generate all keys using
-    # mandos-keygen
-    if ! [ -r /etc/keys/mandos/pubkey.txt \
-	      -a -r /etc/keys/mandos/seckey.txt ]; then
+# Create client key pair
+create_key(){
+    if [ -r /etc/keys/mandos/pubkey.txt \
+	-a -r /etc/keys/mandos/seckey.txt ]; then
+	return 0
+    fi
+    if [ -x /usr/sbin/mandos-keygen ]; then
 	mandos-keygen
-	gpg-connect-agent KILLAGENT /bye || :
-	return 0
-    fi
-
-    # Remove any bad TLS keys by 1.8.0-1
-    if dpkg --compare-versions "$2" eq "1.8.0-1" \
-       || dpkg --compare-versions "$2" eq "1.8.0-1~bpo9+1"; then
-	# Is the key bad?
-	if ! certtool --password='' \
-	     --load-privkey=/etc/keys/mandos/tls-privkey.pem \
-	     --outfile=/dev/null --pubkey-info --no-text \
-	     2>/dev/null; then
-	    shred --remove -- /etc/keys/mandos/tls-privkey.pem \
-		  2>/dev/null || :
-	    rm --force -- /etc/keys/mandos/tls-pubkey.pem
-	fi
-    fi
-
-    # If the TLS keys already exists, do nothing
-    if [ -r /etc/keys/mandos/tls-privkey.pem \
-	    -a -r /etc/keys/mandos/tls-pubkey.pem ]; then
-	return 0
-    fi
-
-    # Try to create the TLS keys
-
-    TLS_PRIVKEYTMP="`mktemp -t mandos-client-privkey.XXXXXXXXXX`"
-
-    if certtool --generate-privkey --password='' \
-		--outfile "$TLS_PRIVKEYTMP" --sec-param ultra \
-		--key-type=ed25519 --pkcs8 --no-text 2>/dev/null; then
-
-	local umask=$(umask)
-	umask 077
-	cp --archive "$TLS_PRIVKEYTMP" /etc/keys/mandos/tls-privkey.pem
-	shred --remove -- "$TLS_PRIVKEYTMP" 2>/dev/null || :
-
-	# First try certtool from GnuTLS
-	if ! certtool --password='' \
-	     --load-privkey=/etc/keys/mandos/tls-privkey.pem \
-	     --outfile=/etc/keys/mandos/tls-pubkey.pem --pubkey-info \
-	     --no-text 2>/dev/null; then
-	    # Otherwise try OpenSSL
-	    if ! openssl pkey -in /etc/keys/mandos/tls-privkey.pem \
-		 -out /etc/keys/mandos/tls-pubkey.pem -pubout; then
-		rm --force /etc/keys/mandos/tls-pubkey.pem
-		# None of the commands succeded; give up
-		umask $umask
-		return 1
-	    fi
-	fi
-	umask $umask
-
-	key_id=$(mandos-keygen --passfile=/dev/null \
-		     | grep --regexp="^key_id[ =]")
-
-	db_version 2.0
-	db_fset mandos-client/key_id seen false
-	db_reset mandos-client/key_id
-	db_subst mandos-client/key_id key_id $key_id
-	db_input critical mandos-client/key_id || true
-	db_go
-	db_stop
-    else
-	shred --remove -- "$TLS_PRIVKEYTMP" 2>/dev/null || :
-    fi
-}
-
-create_dh_params(){
-    if [ -r /etc/keys/mandos/dhparams.pem ]; then
-	return 0
-    fi
-    # Create a Diffe-Hellman parameters file
-    DHFILE="`mktemp -t mandos-client-dh-parameters.XXXXXXXXXX.pem`"
-    # First try certtool from GnuTLS
-    if ! certtool --generate-dh-params --sec-param high \
-	 --outfile "$DHFILE"; then
-	# Otherwise try OpenSSL
-	if ! openssl genpkey -genparam -algorithm DH -out "$DHFILE" \
-	     -pkeyopt dh_paramgen_prime_len:3072; then
-	    # None of the commands succeded; give up
-	    rm -- "$DHFILE"
-	    return 1
-	fi
-    fi
-    sed --in-place --expression='0,/^-----BEGIN DH PARAMETERS-----$/d' \
-	"$DHFILE"
-    sed --in-place --expression='1i-----BEGIN DH PARAMETERS-----' \
-	    "$DHFILE"
-    cp --archive "$DHFILE" /etc/keys/mandos/dhparams.pem
-    rm -- "$DHFILE"
+    fi
 }
 
 case "$1" in
     configure)
 	add_mandos_user "$@"
-	create_keys "$@"
-	create_dh_params "$@" || :
+	create_key "$@"
 	update_initramfs "$@"
-	if dpkg --compare-versions "$2" lt-nl "1.7.10-1"; then
-	    PLUGINHELPERDIR=/usr/lib/$(dpkg-architecture -qDEB_HOST_MULTIARCH 2>/dev/null)/mandos/plugin-helpers
-	    if ! dpkg-statoverride --list "$PLUGINHELPERDIR" \
-		 >/dev/null 2>&1; then
-		chmod u=rwx,go= -- "$PLUGINHELPERDIR"
-	    fi
-	    if ! dpkg-statoverride --list /etc/mandos/plugin-helpers \
-		 >/dev/null 2>&1; then
-		chmod u=rwx,go= -- /etc/mandos/plugin-helpers
-	    fi
-	fi
 	;;
     abort-upgrade|abort-deconfigure|abort-remove)
 	;;

=== modified file 'debian/mandos-client.postrm'
--- debian/mandos-client.postrm	2019-02-09 23:23:26 +0000
+++ debian/mandos-client.postrm	2011-02-27 17:00:41 +0000
@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/sh -e
 # This script can be called in the following ways:
 #
 # After the package was removed:
@@ -26,12 +26,13 @@
 # If preinst fails during upgrade:
 #       <new-postrm> abort-upgrade <old-version>
 
-set -e
 
 # Update the initial RAM file system image
 update_initramfs()
 {
-    update-initramfs -u -k all || :
+    if [ -x /usr/sbin/update-initramfs ]; then
+	update-initramfs -u -k all
+    fi
 }
 
 case "$1" in
@@ -43,10 +44,7 @@
 	shred --remove /etc/keys/mandos/seckey.txt 2>/dev/null || :
 	rm --force /etc/mandos/plugin-runner.conf \
 	    /etc/keys/mandos/pubkey.txt \
-	    /etc/keys/mandos/seckey.txt \
-	    /etc/keys/mandos/tls-privkey.pem \
-	    /etc/keys/mandos/tls-pubkey.pem \
-	    /etc/keys/mandos/dhparams.pem 2>/dev/null
+	    /etc/keys/mandos/seckey.txt 2>/dev/null
 	update_initramfs
 	;;
     upgrade|failed-upgrade|disappear|abort-install|abort-upgrade)

=== removed file 'debian/mandos-client.templates'
--- debian/mandos-client.templates	2019-02-10 03:50:20 +0000
+++ debian/mandos-client.templates	1970-01-01 00:00:00 +0000
@@ -1,10 +0,0 @@
-Template: mandos-client/key_id
-Type: note
-Description: New client option "${key_id}" is REQUIRED on server
- A new "key_id" client option is REQUIRED in the server's clients.conf file, otherwise this computer most likely will not reboot unattended.  This option:
- .
- ${key_id}
- .
- must be added (all on one line!) on the Mandos server host, in the file /etc/mandos/clients.conf, right before the "fingerprint" option for this Mandos client.  You must edit that file on that server and add this option.
- .
- With GnuTLS 3.6.6, Mandos has been forced to stop using OpenPGP keys as TLS session keys.  A new TLS key pair has been generated and will be used as identification, but the key ID of the public key needs to be added to the server, since this will now be used to identify the client to the server.

=== modified file 'debian/mandos.dirs'
--- debian/mandos.dirs	2016-03-19 12:10:15 +0000
+++ debian/mandos.dirs	2010-09-15 17:33:14 +0000
@@ -4,6 +4,3 @@
 etc/default
 etc/dbus-1/system.d
 usr/sbin
-var/lib/mandos
-lib/systemd/system
-usr/lib/tmpfiles.d

=== modified file 'debian/mandos.lintian-overrides'
--- debian/mandos.lintian-overrides	2019-02-10 03:50:20 +0000
+++ debian/mandos.lintian-overrides	2008-10-01 15:29:01 +0000
@@ -2,12 +2,3 @@
 # it, so it must be kept unreadable for non-root users.
 #
 mandos binary: non-standard-file-perm etc/mandos/clients.conf 0600 != 0644
-mandos: init.d-script-needs-depends-on-lsb-base etc/init.d/mandos (line 46)
-
-# The debconf templates is only used for displaying information
-# detected in the postinst, not for saving answers to questions, so we
-# don't need a .config file.
-mandos binary: no-debconf-config
-
-# The notice displayed from the postinst script really is critical
-mandos binary: postinst-uses-db-input

=== modified file 'debian/mandos.postinst'
--- debian/mandos.postinst	2019-02-10 08:41:14 +0000
+++ debian/mandos.postinst	2009-05-24 23:28:04 +0000
@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/sh -e
 # This script can be called in the following ways:
 #
 # After the package was installed:
@@ -15,10 +15,6 @@
 # If prerm fails during replacement due to conflict:
 #       <postinst> abort-remove in-favour <new-package> <version>
 
-. /usr/share/debconf/confmodule
-
-set -e
-
 case "$1" in
     configure)
 	# Rename old "mandos" user and group
@@ -36,73 +32,18 @@
 		--home /nonexistent --no-create-home --group \
 		--disabled-password --gecos "Mandos password system" \
 		_mandos
-	elif dpkg --compare-versions "$2" eq 1.7.4-1 \
-		|| dpkg --compare-versions "$2" eq "1.7.4-1~bpo8+1"
-	then
-	    start=no
-	    if ! [ -f /var/lib/mandos/clients.pickle ]; then
-		invoke-rc.d mandos stop
-		start=yes
-	    fi
-	    chown _mandos:_mandos /var/lib/mandos/clients.pickle \
-		  2>/dev/null || :
-	    if [ "$start" = yes ]; then
-		invoke-rc.d mandos start
-	    fi
-	fi
-	if ! dpkg-statoverride --list "/var/lib/mandos" >/dev/null \
-	     2>&1; then
-	    chown _mandos:_mandos /var/lib/mandos
-	    chmod u=rwx,go= /var/lib/mandos
-	fi
-
-	if dpkg --compare-versions "$2" eq "1.8.0-1" \
-		|| dpkg --compare-versions "$2" eq "1.8.0-1~bpo9+1"; then
-	    if grep --quiet --regexp='^[[:space:]]*key_id[[:space:]]*=[[:space:]]*[Ee]3[Bb]0[Cc]44298[Ff][Cc]1[Cc]149[Aa][Ff][Bb][Ff]4[Cc]8996[Ff][Bb]92427[Aa][Ee]41[Ee]4649[Bb]934[Cc][Aa]495991[Bb]7852[Bb]855[[:space:]]*$' /etc/mandos/clients.conf; then
-		sed --in-place \
-		    --expression='/^[[:space:]]*key_id[[:space:]]*=[[:space:]]*[Ee]3[Bb]0[Cc]44298[Ff][Cc]1[Cc]149[Aa][Ff][Bb][Ff]4[Cc]8996[Ff][Bb]92427[Aa][Ee]41[Ee]4649[Bb]934[Cc][Aa]495991[Bb]7852[Bb]855[[:space:]]*$/d' \
-		    /etc/mandos/clients.conf
-		invoke-rc.d mandos restart
-		db_version 2.0
-		db_fset mandos/removed_bad_key_ids seen false
-		db_reset mandos/removed_bad_key_ids
-		db_input critical mandos/removed_bad_key_ids || true
-		db_go
-		db_stop
-	    fi
-	fi
-
-	gnutls_version=$(dpkg-query --showformat='${Version}' \
-				    --show libgnutls30 \
-				    2>/dev/null || :)
-	if [ -n "$gnutls_version" ] \
-	       && dpkg --compare-versions $gnutls_version ge 3.6.6; then
-	    db_version 2.0
-	    db_input critical mandos/key_id || true
-	    db_go
-	    db_stop
 	fi
 	;;
-    
+
     abort-upgrade|abort-deconfigure|abort-remove)
 	;;
-    
+
     *)
 	echo "$0 called with unknown argument '$1'" 1>&2
 	exit 1
 	;;
 esac
 
-# Avahi version 0.6.31-2 and older provides "avahi" (instead of
-# "avahi-daemon") in its /etc/init.d script header.  To make
-# insserv(8) happy, we edit our /etc/init.d script header to contain
-# the correct string before the code added by dh_installinit calls
-# update.rc-d, which calls insserv.
-avahi_version="`dpkg-query --showformat='${Version}' --show avahi-daemon`"
-if dpkg --compare-versions "$avahi_version" le 0.6.31-2; then
-    sed --in-place --expression='/^### BEGIN INIT INFO$/,/^### END INIT INFO$/s/^\(# Required-\(Stop\|Start\):.*avahi\)-daemon\>/\1/g' /etc/init.d/mandos
-fi
-
 #DEBHELPER#
 
 exit 0

=== modified file 'debian/mandos.prerm'
--- debian/mandos.prerm	2015-07-12 01:57:54 +0000
+++ debian/mandos.prerm	2009-01-18 00:16:57 +0000
@@ -1,4 +1,4 @@
-#!/bin/sh
+#! /bin/sh
 # prerm script for mandos
 #
 # see: dh_installdeb(1)
@@ -17,7 +17,13 @@
 
 case "$1" in
     remove|deconfigure)
-	invoke-rc.d mandos stop || :
+	if [ -x /etc/init.d/mandos ]; then
+		if [ -x /usr/sbin/invoke-rc.d ]; then
+			invoke-rc.d mandos stop
+		else
+			/etc/init.d/mandos stop
+		fi
+	fi
 	;;
     upgrade|failed-upgrade)
         ;;

=== removed file 'debian/mandos.templates'
--- debian/mandos.templates	2019-02-10 08:41:14 +0000
+++ debian/mandos.templates	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-Template: mandos/key_id
-Type: note
-Description: New client option "key_id" is REQUIRED on server
- A new "key_id" client option is REQUIRED in the clients.conf file, otherwise the client most likely will not reboot unattended.  This option:
- .
- key_id = <HEXSTRING>
- .
- must be added in the file /etc/mandos/clients.conf, right before the "fingerprint" option, for each Mandos client.  You must edit that file and add this option for all clients.  To see the correct key ID for each client, run this command (on each client):
- .
- mandos-keygen -F/dev/null|grep ^key_id
- .
- Note: the client must all also be using GnuTLS 3.6.6 or later; the server cannot serve passwords for both old and new clients!
- .
- Rationale: With GnuTLS 3.6.6, Mandos has been forced to stop using OpenPGP keys as TLS session keys.  A new TLS key pair will be generated on each client and will be used as identification, but the key ID of the public key needs to be added to this server, since this will now be used to identify the client to the server.
-
-Template: mandos/removed_bad_key_ids
-Type: note
-Description: Bad key IDs have been removed from clients.conf
- Bad key IDs, which were reported by a bug in Mandos client 1.8.0, have been removed from /etc/mandos/clients.conf

=== modified file 'debian/rules'
--- debian/rules	2019-02-13 08:45:09 +0000
+++ debian/rules	2010-09-09 18:16:14 +0000
@@ -1,61 +1,106 @@
 #!/usr/bin/make -f
-
-ifeq (,$(filter noopt,$(DEB_BUILD_OPTIONS)))
-    MAKEFLAGS += OPTIMIZE=-O0
-endif
-
-ifneq (,$(filter parallel=%,$(DEB_BUILD_OPTIONS)))
-    NUMJOBS = $(patsubst parallel=%,%,$(filter parallel=%,$(DEB_BUILD_OPTIONS)))
-    MAKEFLAGS += -j$(NUMJOBS)
-endif
-
-%:
-	dh $@
-
-override_dh_auto_build-arch:
-	LC_ALL=en_US.utf8 dh_auto_build -- all doc
-
-override_dh_auto_build-indep:
-	LC_ALL=en_US.utf8 dh_auto_build -- doc
-
-override_dh_installinit-indep:
+# Sample debian/rules that uses debhelper.
+#
+# This file was originally written by Joey Hess and Craig Small.
+# As a special exception, when this file is copied by dh-make into a
+# dh-make output file, you may use that output file without restriction.
+# This special exception was added by Craig Small in version 0.37 of dh-make.
+#
+# Modified to make a template file for a multi-binary package with separated
+# build-arch and build-indep targets  by Bill Allombert 2001
+
+# Uncomment this to turn on verbose mode.
+#export DH_VERBOSE=1
+
+# This has to be exported to make some magic below work.
+export DH_OPTIONS
+
+# -pie was broken briefly on the mips and mipsel architectures, see
+# <http://bugs.debian.org/526961>
+BINUTILS_V := $(shell dpkg-query --showformat='$${Version}' \
+			--show binutils)
+ifeq (yes,$(shell dpkg --compare-versions $(BINUTILS_V) lt 2.20-3 \
+		&& dpkg --compare-versions $(BINUTILS_V) ge 2.19.1-1 \
+		&& echo yes))
+  ifneq (,$(strip $(findstring :$(DEB_HOST_ARCH):,:mips:mipsel:) \
+		$(findstring :$(DEB_BUILD_ARCH):,:mips:mipsel:)))
+    BROKEN_PIE := yes
+    export BROKEN_PIE
+  endif
+endif
+
+configure: configure-stamp
+configure-stamp:
+	dh_testdir
+	touch configure-stamp
+
+build: build-arch build-indep
+
+build-arch: build-arch-stamp
+build-arch-stamp: configure-stamp
+	dh_auto_build -- all doc
+	touch $@
+
+build-indep: build-indep-stamp
+build-indep-stamp: configure-stamp
+	dh_auto_build -- doc
+	touch $@
+
+clean:
+	dh_testdir
+	dh_testroot
+	rm -f build-arch-stamp build-indep-stamp configure-stamp
+	dh_auto_clean
+	dh_clean
+
+install: install-indep install-arch
+install-indep:
+	dh_testdir
+	dh_testroot
+	dh_prep
+	dh_installdirs --indep
+	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos install-server
+	dh_lintian
 	dh_installinit --onlyscripts \
 		--update-rcd-params="defaults 25 15"
-
-override_dh_auto_install-indep:
-	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos install-server
-
-override_dh_auto_install-arch:
-	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos-client \
-		install-client-nokey
-
-override_dh_fixperms-arch:
+	dh_install --indep
+
+install-arch:
+	dh_testdir
+	dh_testroot
+	dh_prep
+	dh_installdirs --same-arch
+	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos-client install-client-nokey
+	dh_lintian
+	dh_install --same-arch
+
+binary-common:
+	dh_testdir
+	dh_testroot
+	dh_installchangelogs
+	dh_installdocs
+	dh_link
+	dh_strip
+	dh_compress
 	dh_fixperms --exclude etc/keys/mandos \
+		--exclude etc/mandos/clients.conf \
 		--exclude etc/mandos/plugins.d \
-		--exclude etc/mandos/plugin-helpers \
-		--exclude usr/lib/$(DEB_HOST_MULTIARCH)/mandos/plugins.d \
-		--exclude usr/lib/$(DEB_HOST_MULTIARCH)/mandos/plugin-helpers \
-		--exclude usr/share/doc/mandos-client/examples/network-hooks.d
-	chmod --recursive g-w -- \
-	"$(CURDIR)/debian/mandos-client/usr/share/doc/mandos-client/examples/network-hooks.d"
-
-override_dh_fixperms-indep:
-	dh_fixperms --exclude etc/mandos/clients.conf
-
-override_dh_auto_test-arch: ;
-
-#bpo## dpkg-shlibdeps sees the "libgnutls28-dev (>= 3.6.6) |
-#bpo## libgnutls28-dev (<< 3.6.0)," in the build-dependencies not as two
-#bpo## alternatives, but as an absolute dependency on libgnutls30 >= 3.6.6.
-#bpo## So we have to do this ugly hack to hide this build dependency if we
-#bpo## compiled with libgnutls30 << 3.6.0.
-#bpo#override_dh_shlibdeps-arch:
-#bpo#	-gnutls_version=$$(dpkg-query --showformat='$${Version}' \
-#bpo#		--show libgnutls30); \
-#bpo#	dpkg --compare-versions $$gnutls_version lt 3.6.0 \
-#bpo#		&& { cp --archive debian/control debian/control.orig; sed --expression='s/libgnutls28-dev (>= 3\.6\.6) |//' debian/control; }
-#bpo#	dh_shlibdeps
-#bpo#	-gnutls_version=$$(dpkg-query --showformat='$${Version}' \
-#bpo#		--show libgnutls30); \
-#bpo#	dpkg --compare-versions $$gnutls_version lt 3.6.0 \
-#bpo#		&& mv debian/control.orig debian/control
+		--exclude usr/lib/mandos/plugins.d
+	dh_installdeb
+	dh_shlibdeps
+	dh_gencontrol
+	dh_md5sums
+	dh_builddeb
+
+# Build architecture independant packages using the common target.
+binary-indep: build-indep install-indep
+	$(MAKE) -f debian/rules DH_OPTIONS=--indep binary-common
+
+# Build architecture dependant packages using the common target.
+binary-arch: build-arch install-arch
+	$(MAKE) -f debian/rules DH_OPTIONS=--same-arch binary-common
+
+binary: binary-arch binary-indep
+
+.PHONY: build clean binary-indep binary-arch binary install \
+		install-indep install-arch configure

=== removed directory 'debian/upstream'
=== removed file 'debian/upstream/signing-key.asc'
--- debian/upstream/signing-key.asc	2014-03-28 22:32:21 +0000
+++ debian/upstream/signing-key.asc	1970-01-01 00:00:00 +0000
@@ -1,52 +0,0 @@
------BEGIN PGP PUBLIC KEY BLOCK-----
-Version: GnuPG v1.4.12 (GNU/Linux)
-
-mQINBFJQOFYBEACoWsEGlOxVWFUAxOxdd3GDLaqEKkKihJwLp102Ks7JKMd9friR
-7+OZuo3U0gdqLU9q1jPJn36J1QbaUTOvcaKtZp+QpUoYJ2OaGtlOY5ML8LSoC0rZ
-MIzGYTtvriwpU/YplLNGPl/90KsB2VqjrY1l1he5M8zziWDlPdJxwg8GFvmPWoif
-6oo+1iCswL5IdQ6c5MVO53zYu0cgyUSazLsVD5Xzy59lefgtaDydahJpPycf5aEQ
-DAoC9fZt2mgG3FLIUCZdXIhZdOJGCMdjLThBnJXYgGbG4rbGLNlI4W/uA5aqa4ME
-WYSAcCyX3ucKY/LkXRtC+z5s05e7tZ3Z+uAJy1eDsbhDXgZERye7a/zPWx1tAlzQ
-E80Oltjh1uXWjQORyx99a0jK87zjm49YjhYw1ZN6Z0HfSaws4Yj2QOzp9t4B3l7f
-DIUYoWBfHW7mseQeQ+t3TwQU5gjFCNu7oDeATqi5A5MksXN0+BcksterbGRBhEyp
-CybIEyrZE033jIs407Ool4Kv10cnjc8oy609BXex/dxwcvVr2vQHle4NPUZd+Xhg
-zC+9Z4jFwE0M/EPvtyieA/DWQse+TZ5itDGMYDub/GJfv1U61ANOgPIbTEF7iSa9
-5nWmq7zyUy/txmABka842Kt0Vp6ayoKcF8EIXCaDrVfPnXj+JlKf3c2u6wARAQAB
-tCxNYW5kb3MgTWFpbnRhaW5lciBUZWFtIDxtYW5kb3NAcmVjb21waWxlLnNlPokC
-PQQTAQgAJwUCUlA4VgIbAwUJCWYBgAULCQgHAwUVCgkICwUWAgMBAAIeAQIXgAAK
-CRByIylzyjTCxETXEACi56jCV9lJNSBbTp0Iet4X/i7Mx0Z8UkFFa3l7o0i4jFQj
-CIBrWECDlcxqZziii2dgh7L0ma93vB3rfjfCWeYLcEQw43MFBsd4dHuobrLXTcqU
-7n0Zmc8BsXwk5B25CnEYgvlbWX9BCYtxHGRcZzQrqOFjCMKatq0EIIVuWaz5yuCU
-V2rEgnr+veTd/rBOE9ez6Ju6xH11Teob5G7pMM0YPKHtZG/J3rvWPw4BDM6Tc60B
-G0sTDZNgkrGWxuB8YaLIwVWzliQK/17Jv/0alajyA3cWLWMkcK9Yhi/einzdMRoD
-IjnbtoHcSC9g6i0VGelwnMpHlFTwXriXEBSttULarK3iKE4tOv9nxMAEwicqlw14
-X96PPgz6ogtJG7FiwZfy1CQ81Uby+YuIhHZx3ZEyR1TFq70e98EgCuvjMZWMIhSy
-gB5Vssfq7c2lXQjltV3ujhK1PD+7/iHlL7t4QRxPDN8fbMS2VPfAdtnWS1K48d5J
-D/jP1LrGWS81HIaX8GFVLVw+jSQEu9cn3TFiZxK/4MMsITmlouJdtZWmQ/otMSMl
-wiCCZp3dGpRXMmaqR1N8V0nMKshM8mci7bD92ubd/t6cR/G6l+VIp45WyFONvtde
-F3ccfmfrKJuroMDfHxPxMf58EroAuWwzJKCPRH4JmwDvSSQoIIAGNL1lOKp6wrkC
-DQRSUDhWARAAzN7pbpAu7XLNPODotV/N+JaCFvNAIqTcr9PrbhxiKFCDs9/IExwP
-sGENL9GZd1DfoGEgxQ8j3l8VGw9VSeUoN7uMY2NwwbXTilAFkn/S8xnr2zQDRZ+n
-EeFSq9MMFxj4Kt1TqVYDbO8vmFfOT3gRCRHeJ+pn4yJeSPau/ndNrmbQ1/Z6vaUG
-yfo931ottx7SXZwkHA6jJVFT9rbHTyx9tzOqMKDJiMrx8qKaHpE9B45oHNR0WJJJ
-75zoDVuOZ6wAxXZuqBFu2lKPqDTZeawfzcu5qplrm1RPgSOjz6w1A41HBLqGe+v9
-7Twx5wfNMgnKC0V2wUe0xR6hQHQlyZoCwcGyrasiu+v/joZ1p66SSWKjs+LGjoe4
-Lwh6VjZU2x+irVBjcgIoRWf3k4JAef0nGYsm0cFjAnwXac+/CYxVt+7Y4+HG8wPB
-oUNZkW+bvdHQouxEClxnccIEgX/AkriJTYDQ4q3tkl2HVE/51R4pdQVLer2a5ov+
-Jwk44DdqzYstMsvqu+iD48hXzADg6HFvofkpct15h463pMaJf99uVVM9ZDNQ6B34
-l3tPX9ZDkIDl6n9dE2Jkaxx1yNVhPXpeMf4EzL+CEdUErVStB66lUkw+tNkuZyVT
-ZTQel9h0196+CNSiqAaL8+ZZdbjKKfzlcB4Qnd897XzMfsFQ7mzJ9QsAEQEAAYkC
-JQQYAQgADwUCUlA4VgIbDAUJCWYBgAAKCRByIylzyjTCxFmlEACBTOg5NqX63d8D
-mwk4smlFPppQBIduxZaMG9HsLcPi3VKTG9Zg6WI6rEdr/4MnoINsudLsEbrQLgRH
-2q1Zs+HqIIP5H2/sYHmswyokYB10zKB6gNUUg/GSlcAcrelsHVKx5B8kccWGT5gk
-Wo/X0BGMUTOvQ6lJ6YNo1idcQ2ZjsyfZoz3G8JS7/EXN//jAZf+017yj8WsAS7hw
-JRFMy7VET4g00JcBoNOAMP7PkozimZ2OwwsggJSYWkR1RaU2tKR1VmDF8R6UxuEd
-BJzwFmz+wNC1Kq+FoSaRNsrKEmzLnfV9unDnF2z7Lc4LqOysXdzOk9zTBPur0gd2
-Lh5H/g5rTAMQBARqXfvIwiTtrBGgil8JW8e4Bc0LQUuHAE7x9gMRil+OtkQrCRk9
-0LWXVS+K0tvvruE4EDtCGiS5046+BEI3aYsp4hNzjHADq0TJeCYjNg9kY0CjxcEq
-cfuMoUbQ0MkARGuBbykCdlylfTrkxrj/dPhr49lctY3H+Pj6F4fMDM4TP6UTGA8k
-993RRNYhkDWSxIp6G7RJpBZobHN+eHQ3r8A4tWdYb4Fvd2lvwEDjUFT9uD6WAff4
-8A1hM2uSy91UYBOPrIjqYdRFKJc9rThYdXH2T6SiRMYtZMrEKhqPffB/i9mqVBlD
-6vKRsaQikZujRdP9Dkf0mLmJ7LANWw==
-=9Noe
------END PGP PUBLIC KEY BLOCK-----

=== modified file 'debian/watch'
--- debian/watch	2019-02-11 05:15:24 +0000
+++ debian/watch	2011-10-05 16:00:56 +0000
@@ -1,3 +1,2 @@
-version=4
-opts=pgpmode=auto \
- https://ftp.recompile.se/pub/@PACKAGE@/@PACKAGE@@ANY_VERSION@\.orig@ARCHIVE_EXT@
+version=3
+ftp://ftp.recompile.se/pub/mandos/mandos[-_]([^\s]+?)(?:\.orig)?\.tar\.(?:gz|bz2|7z|xz)

=== modified file 'init.d-mandos'
--- init.d-mandos	2018-02-10 13:23:58 +0000
+++ init.d-mandos	2011-10-05 16:00:56 +0000
@@ -1,16 +1,19 @@
 #! /bin/sh
 ### BEGIN INIT INFO
 # Provides:          mandos
-# Required-Start:    $remote_fs $syslog avahi-daemon
-# Required-Stop:     $remote_fs $syslog avahi-daemon
+# Required-Start:    $remote_fs $syslog avahi
+# Required-Stop:     $remote_fs $syslog avahi
 # Default-Start:     2 3 4 5
 # Default-Stop:      0 1 6
 # Short-Description: Mandos server
-# Description:       Server of encrypted passwords to Mandos clients
+# Description:       Gives encrypted passwords to Mandos clients
 ### END INIT INFO
 
 # Author: Teddy Hogeborn <teddy@recompile.se>
 # Author: Björn Påhlsson <belorn@recompile.se>
+#
+# Please remove the "Author" lines above and replace them
+# with your own name if you copy and modify this script.
 
 # Do NOT "set -e"
 
@@ -20,11 +23,7 @@
 NAME=mandos
 DAEMON=/usr/sbin/$NAME
 DAEMON_ARGS=""
-if [ -d /run/. ]; then
-    PIDFILE=/run/$NAME.pid
-else
-    PIDFILE=/var/run/$NAME.pid
-fi
+PIDFILE=/var/run/$NAME.pid
 SCRIPTNAME=/etc/init.d/$NAME
 
 # Exit if the package is not installed
@@ -41,8 +40,7 @@
 . /lib/init/vars.sh
 
 # Define LSB log_* functions.
-# Depend on lsb-base (>= 3.2-14) to ensure that this file is present
-# and status_of_proc is working.
+# Depend on lsb-base (>= 3.0-6) to ensure that this file is present.
 . /lib/lsb/init-functions
 
 #
@@ -120,9 +118,6 @@
 		2) [ "$VERBOSE" != no ] && log_end_msg 1 ;;
 	esac
 	;;
-  status)
-	status_of_proc "$DAEMON" "$NAME" -p "$PIDFILE" && exit 0 || exit $?
-	;;
   #reload|force-reload)
 	#
 	# If do_reload() is not implemented then leave this commented out
@@ -149,14 +144,14 @@
 		esac
 		;;
 	  *)
-		# Failed to stop
+	  	# Failed to stop
 		log_end_msg 1
 		;;
 	esac
 	;;
   *)
 	#echo "Usage: $SCRIPTNAME {start|stop|restart|reload|force-reload}" >&2
-	echo "Usage: $SCRIPTNAME {start|stop|status|restart|force-reload}" >&2
+	echo "Usage: $SCRIPTNAME {start|stop|restart|force-reload}" >&2
 	exit 3
 	;;
 esac

=== removed file 'initramfs-tools-conf'
--- initramfs-tools-conf	2018-08-19 14:06:55 +0000
+++ initramfs-tools-conf	1970-01-01 00:00:00 +0000
@@ -1,17 +0,0 @@
-# -*- shell-script -*-
-
-# Since the initramfs image will contain key files, we need to
-# restrict permissions on it by setting UMASK here.
-# 
-# The proper place to set UMASK is (according to
-# /etc/cryptsetup-initramfs/conf-hook), in
-# /etc/initramfs-tools/initramfs.conf, which we shouldn't edit.  The
-# corresponding directory for drop-in files from packages is
-# /usr/share/initramfs-tools/conf.d, and this file will be installed
-# there as "mandos-conf".
-# 
-# This setting of UMASK will have unfortunate unintended side effects
-# on the files *inside* the initramfs, but these are later fixed by
-# "initramfs-tools-hook", installed as
-# "/usr/share/initramfs-tools/hooks/mandos".
-UMASK=0027

=== modified file 'initramfs-tools-hook'
--- initramfs-tools-hook	2018-08-19 14:06:55 +0000
+++ initramfs-tools-hook	2011-10-05 16:56:06 +0000
@@ -3,7 +3,7 @@
 # This script will be run by 'mkinitramfs' when it creates the image.
 # Its job is to decide which files to install, then install them into
 # the staging area, where the initramfs is being created.  This
-# happens when a new 'linux-image' package is installed, or when an
+# happens when a new 'linux-image' package is installed, or when the
 # administrator runs 'update-initramfs' by hand to update an initramfs
 # image.
 
@@ -29,15 +29,13 @@
 
 . /usr/share/initramfs-tools/hook-functions
 
-for d in /usr/lib \
-    "/usr/lib/`dpkg-architecture -qDEB_HOST_MULTIARCH 2>/dev/null`" \
-    "`rpm --eval='%{_libdir}' 2>/dev/null`" /usr/local/lib; do
-    if [ -d "$d"/mandos ]; then
-	libdir="$d"
+for d in /usr /usr/local; do
+    if [ -d "$d"/lib/mandos ]; then
+	prefix="$d"
 	break
     fi
 done
-if [ -z "$libdir" ]; then
+if [ -z "$prefix" ]; then
     # Mandos not found
     exit 1
 fi
@@ -70,25 +68,20 @@
 CONFDIR="/conf/conf.d/mandos"
 MANDOSDIR="/lib/mandos"
 PLUGINDIR="${MANDOSDIR}/plugins.d"
-PLUGINHELPERDIR="${MANDOSDIR}/plugin-helpers"
-HOOKDIR="${MANDOSDIR}/network-hooks.d"
 
 # Make directories
 install --directory --mode=u=rwx,go=rx "${DESTDIR}${CONFDIR}" \
-	"${DESTDIR}${MANDOSDIR}" "${DESTDIR}${HOOKDIR}"
+	"${DESTDIR}${MANDOSDIR}"
 install --owner=${mandos_user} --group=${mandos_group} --directory \
-	--mode=u=rwx "${DESTDIR}${PLUGINDIR}" \
-	"${DESTDIR}${PLUGINHELPERDIR}"
-
-copy_exec "$libdir"/mandos/mandos-to-cryptroot-unlock "${MANDOSDIR}"
+    --mode=u=rwx "${DESTDIR}${PLUGINDIR}"
 
 # Copy the Mandos plugin runner
-copy_exec "$libdir"/mandos/plugin-runner "${MANDOSDIR}"
+copy_exec "$prefix"/lib/mandos/plugin-runner "${MANDOSDIR}"
 
 # Copy the plugins
 
 # Copy the packaged plugins
-for file in "$libdir"/mandos/plugins.d/*; do
+for file in "$prefix"/lib/mandos/plugins.d/*; do
     base="`basename \"$file\"`"
     # Is this plugin overridden?
     if [ -e "/etc/mandos/plugins.d/$base" ]; then
@@ -102,21 +95,6 @@
     esac
 done
 
-# Copy the packaged plugin helpers
-for file in "$libdir"/mandos/plugin-helpers/*; do
-    base="`basename \"$file\"`"
-    # Is this plugin overridden?
-    if [ -e "/etc/mandos/plugin-helpers/$base" ]; then
-	continue
-    fi
-    case "$base" in
-	*~|.*|\#*\#|*.dpkg-old|*.dpkg-bak|*.dpkg-new|*.dpkg-divert)
-	    : ;;
-	"*") : ;;
-	*) copy_exec "$file" "${PLUGINHELPERDIR}" ;;
-    esac
-done
-
 # Copy any user-supplied plugins
 for file in /etc/mandos/plugins.d/*; do
     base="`basename \"$file\"`"
@@ -128,78 +106,12 @@
     esac
 done
 
-# Copy any user-supplied plugin helpers
-for file in /etc/mandos/plugin-helpers/*; do
-    base="`basename \"$file\"`"
-    case "$base" in
-	*~|.*|\#*\#|*.dpkg-old|*.dpkg-bak|*.dpkg-new|*.dpkg-divert)
-	    : ;;
-	"*") : ;;
-	*) copy_exec "$file" "${PLUGINHELPERDIR}" ;;
-    esac
-done
-
-# Get DEVICE from initramfs.conf and other files
-. /etc/initramfs-tools/initramfs.conf
-for conf in /etc/initramfs-tools/conf.d/*; do
-    if [ -n `basename \"$conf\" | grep '^[[:alnum:]][[:alnum:]\._-]*$' \
-	| grep -v '\.dpkg-.*$'` ]; then
-	[ -f "${conf}" ] && . "${conf}"
-    fi
-done
-export DEVICE
-
-# Copy network hooks
-for hook in /etc/mandos/network-hooks.d/*; do
-    case "`basename \"$hook\"`" in
-	"*") continue ;;
-	*[!A-Za-z0-9_.-]*) continue ;;
-	*) test -d "$hook" || copy_exec "$hook" "${HOOKDIR}" ;;
-    esac
-    if [ -x "$hook" ]; then
-        # Copy any files needed by the network hook
-	MANDOSNETHOOKDIR=/etc/mandos/network-hooks.d MODE=files \
-	    VERBOSITY=0 "$hook" files | while read -r file target; do
-	    if [ ! -e "${file}" ]; then
-		echo "WARNING: file ${file} not found, requested by Mandos network hook '${hook##*/}'" >&2
-	    fi
-	    if [ -z "${target}" ]; then
-		copy_exec "$file"
-	    else
-		copy_exec "$file" "$target"
-	    fi
-	done
-        # Copy and load any modules needed by the network hook
-	MANDOSNETHOOKDIR=/etc/mandos/network-hooks.d MODE=modules \
-	    VERBOSITY=0 "$hook" modules | while read -r module; do
-	    force_load "$module"
-	done
-    fi
-done
-
-# GPGME needs GnuPG
-gpg=/usr/bin/gpg
-libgpgme11_version="`dpkg-query --showformat='${Version}' --show libgpgme11`"
-if dpkg --compare-versions "$libgpgme11_version" ge 1.5.0-0.1; then
-    if [ -e /usr/bin/gpgconf ]; then
-	if [ ! -e "${DESTDIR}/usr/bin/gpgconf" ]; then
-	    copy_exec /usr/bin/gpgconf
-	fi
-	gpg="`/usr/bin/gpgconf|sed --quiet --expression='s/^gpg:[^:]*://p'`"
-	gpgagent="`/usr/bin/gpgconf|sed --quiet --expression='s/^gpg-agent:[^:]*://p'`"
-	# Newer versions of GnuPG 2 requires the gpg-agent binary
-	if [ -e "$gpgagent" ] && [ ! -e "${DESTDIR}$gpgagent" ]; then
-	    copy_exec "$gpgagent"
-	fi
-    fi
-elif dpkg --compare-versions "$libgpgme11_version" ge 1.4.1-0.1; then
-    gpg=/usr/bin/gpg2
-fi
-if [ ! -e "${DESTDIR}$gpg" ]; then
-    copy_exec "$gpg"
-fi
-unset gpg
-unset libgpgme11_version
+# GPGME needs /usr/bin/gpg
+if [ ! -e "${DESTDIR}/usr/bin/gpg" \
+    -a -n "`ls \"${DESTDIR}\"/usr/lib/libgpgme.so* \
+		2>/dev/null`" ]; then
+    copy_exec /usr/bin/gpg
+fi
 
 # Config files
 for file in /etc/mandos/plugin-runner.conf; do
@@ -224,24 +136,10 @@
     if [ -d "$file" ]; then
 	continue
     fi
-    case "$file" in
-	*~|.*|\#*\#|*.dpkg-old|*.dpkg-bak|*.dpkg-new|*.dpkg-divert)
-	    : ;;
-	"*") : ;;
-	*)
-	    cp --archive --sparse=always "$file" \
-	       "${DESTDIR}${CONFDIR}"
-	    chown ${mandos_user}:${mandos_group} \
-		  "${DESTDIR}${CONFDIR}/`basename \"$file\"`"
-	    ;;
-    esac
+    cp --archive --sparse=always "$file" "${DESTDIR}${CONFDIR}"
+    chown ${mandos_user}:${mandos_group} \
+	"${DESTDIR}${CONFDIR}/`basename \"$file\"`"
 done
-# Use Diffie-Hellman parameters file if available
-if [ -e "${DESTDIR}${CONFDIR}"/dhparams.pem ]; then
-    sed --in-place \
-	--expression="1i--options-for=mandos-client:--dh-params=${CONFDIR}/dhparams.pem" \
-	"${DESTDIR}/${CONFDIR}/plugin-runner.conf"
-fi
 
 # /lib/mandos/plugin-runner will drop priviliges, but needs access to
 # its plugin directory and its config file.  However, since almost all
@@ -252,8 +150,8 @@
 # initrd; it is intended to affect the initrd.img file itself, since
 # it now contains secret key files.  There is, however, no other way
 # to set the permission of the initrd.img file without a race
-# condition.  This umask is set by "initramfs-tools-conf", installed
-# as "/usr/share/initramfs-tools/conf.d/mandos-conf".)
+# condition.  This umask is set by "initramfs-tools-hook-conf",
+# installed as "/usr/share/initramfs-tools/conf-hooks.d/mandos".)
 # 
 for full in "${MANDOSDIR}" "${CONFDIR}"; do
     while [ "$full" != "/" ]; do
@@ -271,7 +169,7 @@
 done
 for dir in "${DESTDIR}"/lib* "${DESTDIR}"/usr/lib*; do
     if [ -d "$dir" ]; then
-	find "$dir" \! -perm -u+rw,g+r -prune -or \! -type l -print0 \
-	    | xargs --null --no-run-if-empty chmod a+rX --
+	find "$dir" \! -perm -u+rw,g+r -prune -or -print0 \
+	    | xargs --null --no-run-if-empty chmod a+rX
     fi
 done

=== added file 'initramfs-tools-hook-conf'
--- initramfs-tools-hook-conf	1970-01-01 00:00:00 +0000
+++ initramfs-tools-hook-conf	2009-05-17 00:50:09 +0000
@@ -0,0 +1,13 @@
+# -*- shell-script -*-
+
+# if mkinitramfs is started by mkinitramfs-kpkg, mkinitramfs-kpkg has
+# already touched the initrd file with umask 022 before we had a
+# chance to affect it.  We cannot allow a readable initrd file,
+# therefore we must fix this now.
+if [ -e "${outfile}" ] \
+    && [ `stat --format=%s "${outfile}"` -eq 0 ]; then
+    rm "${outfile}"
+    (umask 027; touch "${outfile}")
+fi
+
+UMASK=027

=== modified file 'initramfs-tools-script'
--- initramfs-tools-script	2018-08-19 01:35:11 +0000
+++ initramfs-tools-script	2011-07-16 00:29:19 +0000
@@ -51,10 +51,13 @@
 
 chmod a=rwxt /tmp
 
+test -r /conf/conf.d/cryptroot
+test -w /conf/conf.d
+
 # Get DEVICE from /conf/initramfs.conf and other files
 . /conf/initramfs.conf
 for conf in /conf/conf.d/*; do
-    [ -f "${conf}" ] && . "${conf}"
+    [ -f ${conf} ] && . ${conf}
 done
 if [ -e /conf/param.conf ]; then
     . /conf/param.conf
@@ -91,9 +94,7 @@
 # If we are connecting directly, run "configure_networking" (from
 # /scripts/functions); it needs IPOPTS and DEVICE
 if [ "${connect+set}" = set ]; then
-    set +e			# Required by library functions
     configure_networking
-    set -e
     if [ -n "$connect" ]; then
 	cat <<-EOF >>/conf/conf.d/mandos/plugin-runner.conf
 	
@@ -102,80 +103,50 @@
     fi
 fi
 
-if [ -r /conf/conf.d/cryptroot ]; then
-    test -w /conf/conf.d
-
-    # Do not replace cryptroot file unless we need to.
-    replace_cryptroot=no
-
-    # Our keyscript
-    mandos=/lib/mandos/plugin-runner
-    test -x "$mandos"
-
-    # parse /conf/conf.d/cryptroot.  Format:
-    # target=sda2_crypt,source=/dev/sda2,rootdev,key=none,keyscript=/foo/bar/baz
-    # Is the root device specially marked?
-    changeall=yes
-    while read -r options; do
-	case "$options" in
-	    rootdev,*|*,rootdev,*|*,rootdev)
-		# If the root device is specially marked, don't change all
-		# lines in crypttab by default.
-		changeall=no
+# Do not replace cryptroot file unless we need to.
+replace_cryptroot=no
+
+# Our keyscript
+mandos=/lib/mandos/plugin-runner
+test -x "$mandos"
+
+# parse /conf/conf.d/cryptroot.  Format:
+# target=sda2_crypt,source=/dev/sda2,key=none,keyscript=/foo/bar/baz
+exec 3>/conf/conf.d/cryptroot.mandos
+while read options; do
+    newopts=""
+    # Split option line on commas
+    old_ifs="$IFS"
+    IFS="$IFS,"
+    for opt in $options; do
+	# Find the keyscript option, if any
+	case "$opt" in
+	    keyscript=*)
+		keyscript="${opt#keyscript=}"
+		newopts="$newopts,$opt"
+		;;
+	    "") : ;;
+	    *)
+		newopts="$newopts,$opt"
 		;;
 	esac
-    done < /conf/conf.d/cryptroot
-
-    exec 3>/conf/conf.d/cryptroot.mandos
-    while read -r options; do
-	newopts=""
-	keyscript=""
-	changethis="$changeall"
-	# Split option line on commas
-	old_ifs="$IFS"
-	IFS="$IFS,"
-	for opt in $options; do
-	    # Find the keyscript option, if any
-	    case "$opt" in
-		keyscript=*)
-		    keyscript="${opt#keyscript=}"
-		    newopts="$newopts,$opt"
-		    ;;
-		"") : ;;
-		# Always use Mandos on the root device, if marked
-		rootdev)
-		    changethis=yes
-		    newopts="$newopts,$opt"
-		    ;;
-		# Don't use Mandos on resume device, if marked
-		resumedev)
-		    changethis=no
-		    newopts="$newopts,$opt"
-		    ;;
-		*)
-		    newopts="$newopts,$opt"
-		    ;;
-	    esac
-	done
-	IFS="$old_ifs"
-	unset old_ifs
-	# If there was no keyscript option, add one.
-	if [ "$changethis" = yes ] && [ -z "$keyscript" ]; then
-	    replace_cryptroot=yes
-	    newopts="$newopts,keyscript=$mandos"
-	fi
-	newopts="${newopts#,}"
-	echo "$newopts" >&3
-    done < /conf/conf.d/cryptroot
-    exec 3>&-
-
-    # If we need to, replace the old cryptroot file with the new file.
-    if [ "$replace_cryptroot" = yes ]; then
-	mv /conf/conf.d/cryptroot /conf/conf.d/cryptroot.mandos-old
-	mv /conf/conf.d/cryptroot.mandos /conf/conf.d/cryptroot
-    else
-	rm -f /conf/conf.d/cryptroot.mandos
+    done
+    IFS="$old_ifs"
+    unset old_ifs
+    # If there was no keyscript option, add one.
+    if [ -z "$keyscript" ]; then
+	replace_cryptroot=yes
+	newopts="$newopts,keyscript=$mandos"
     fi
-elif [ -x /usr/bin/cryptroot-unlock ]; then
-    setsid /lib/mandos/mandos-to-cryptroot-unlock &
+    newopts="${newopts#,}"
+    echo "$newopts" >&3
+done < /conf/conf.d/cryptroot
+exec 3>&-
+
+# If we need to, replace the old cryptroot file with the new file.
+if [ "$replace_cryptroot" = yes ]; then
+    mv /conf/conf.d/cryptroot /conf/conf.d/cryptroot.mandos-old
+    mv /conf/conf.d/cryptroot.mandos /conf/conf.d/cryptroot
+else
+    rm /conf/conf.d/cryptroot.mandos
 fi

=== removed file 'initramfs-tools-script-stop'
--- initramfs-tools-script-stop	2018-08-19 14:58:40 +0000
+++ initramfs-tools-script-stop	1970-01-01 00:00:00 +0000
@@ -1,65 +0,0 @@
-#!/bin/sh -e
-# 
-# Script to wait for plugin-runner to exit before continuing boot
-# 
-# Copyright © 2018 Teddy Hogeborn
-# Copyright © 2018 Björn Påhlsson
-# 
-# This file is part of Mandos.
-# 
-# Mandos is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-# 
-#     Mandos is distributed in the hope that it will be useful, but
-#     WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-# 
-# You should have received a copy of the GNU General Public License
-# along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
-# 
-# Contact the authors at <mandos@recompile.se>.
-# 
-# This script will run in the initrd environment at boot and remove
-# the file keeping the dummy plugin running, forcing plugin-runner to
-# exit if it is still running.
-
-# This script should be installed as
-# "/usr/share/initramfs-tools/scripts/local-premount/mandos" which will
-# eventually be "/scripts/local-premount/mandos" in the initrd.img
-# file.
-
-PREREQ=""
-prereqs()
-{
-    echo "$PREREQ"
-}
-
-case $1 in
-prereqs)
-	prereqs
-	exit 0
-	;;
-esac
-
-. /scripts/functions
-
-pid=$(cat /run/mandos-plugin-runner.pid 2>/dev/null)
-
-# If the dummy plugin is running, removing this file should force the
-# dummy plugin to exit successfully, thereby making plugin-runner shut
-# down all its other plugins and then exit itself.
-rm -f /run/mandos-keep-running >/dev/null 2>&1
-
-# Wait for exit of plugin-runner, if still running
-if [ -n "$pid" ]; then
-    while :; do
-	case "$(readlink /proc/"$pid"/exe 2>/dev/null)" in
-	    */plugin-runner) sleep 1;;
-	    *) break;;
-	esac
-    done
-    rm -f /run/mandos-plugin-runner.pid >/dev/null 2>&1
-fi

=== removed file 'initramfs-unpack'
--- initramfs-unpack	2018-02-08 10:23:55 +0000
+++ initramfs-unpack	1970-01-01 00:00:00 +0000
@@ -1,69 +0,0 @@
-#!/bin/bash
-# 
-# Initramfs unpacker - unpacks initramfs images into /tmp
-# 
-# Copyright © 2013-2018 Teddy Hogeborn
-# Copyright © 2013-2018 Björn Påhlsson
-# 
-# This file is part of Mandos.
-#
-# Mandos is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-#     Mandos is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Mandos.  If not, see
-# <http://www.gnu.org/licenses/>.
-# 
-# Contact the authors at <mandos@recompile.se>.
-
-cpio="cpio --extract --make-directories --unconditional --preserve-modification-time"
-
-if [ -z "$*" ]; then
-    set -- /boot/initrd.img-*
-fi
-
-for imgfile in "$@"; do
-    if ! [ -f "$imgfile" ]; then
-	echo "Error: Not an existing file: $imgfile" >&2
-	continue
-    fi
-    imgdir="${TMPDIR:-/tmp}/${imgfile##*/}"
-    if [ -d "$imgdir" ]; then
-	rm --recursive -- "$imgdir"
-    fi
-    mkdir --parents "$imgdir"
-    # Does this image contain microcode?
-    if $cpio --quiet --list --file="$imgfile" >/dev/null 2>&1; then
-	# Number of bytes to skip to get to the compressed archive
-	skip=$(($(LANG=C $cpio --io-size=1 --list --file="$imgfile" 2>&1 \
-	    | sed --quiet --expression='s/^\([0-9]\+\) blocks$/\1/p')+8))
-	catimg="dd if=$imgfile bs=$skip skip=1 status=noxfer"
-    else
-	catimg="cat -- $imgfile"
-    fi
-    # Determine the compression method
-    if { $catimg 2>/dev/null | zcat --test >/dev/null 2>&1;
-	    [ ${PIPESTATUS[-1]} -eq 0 ]; }; then
-	decomp="zcat"
-    elif { $catimg 2>/dev/null | bzip2 --test >/dev/null 2>&1;
-	    [ ${PIPESTATUS[-1]} -eq 0 ]; }; then
-	decomp="bzip2 --stdout --decompress"
-    elif { $catimg 2>/dev/null | lzop --test >/dev/null 2>&1;
-	    [ ${PIPESTATUS[-1]} -eq 0 ]; }; then
-	decomp="lzop --stdout --decompress"
-    else
-	echo "Error: Could not determine type of $imgfile" >&2
-	continue
-    fi
-    $catimg 2>/dev/null | $decomp | ( cd -- "$imgdir" && $cpio --quiet )
-    if [ ${PIPESTATUS[-1]} -eq 0 ]; then
-	echo "$imgfile unpacked into $imgdir"
-    fi
-done

=== modified file 'intro.xml'
--- intro.xml	2019-02-10 04:20:26 +0000
+++ intro.xml	2011-10-05 16:00:56 +0000
@@ -1,7 +1,7 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -31,14 +31,6 @@
     </authorgroup>
     <copyright>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -68,10 +60,10 @@
       The computers run a small client program in the initial RAM disk
       environment which will communicate with a server over a network.
       All network communication is encrypted using TLS.  The clients
-      are identified by the server using a TLS public key; each client
+      are identified by the server using an OpenPGP key; each client
       has one unique to it.  The server sends the clients an encrypted
       password.  The encrypted password is decrypted by the clients
-      using a separate OpenPGP key, and the password is then used to
+      using the same OpenPGP key, and the password is then used to
       unlock the root file system, whereupon the computers can
       continue booting normally.
     </para>
@@ -80,8 +72,6 @@
   <refsect1 id="introduction">
     <title>INTRODUCTION</title>
     <para>
-      <!-- This paragraph is a combination and paraphrase of two
-           quotes from the 1995 movie “The Usual Suspects”. -->
       You know how it is.  You’ve heard of it happening.  The Man
       comes and takes away your servers, your friends’ servers, the
       servers of everybody in the same hosting facility. The servers
@@ -201,18 +191,8 @@
       <para>
 	No.  The server only gives out the passwords to clients which
 	have <emphasis>in the TLS handshake</emphasis> proven that
-	they do indeed hold the private key corresponding to that
-	client.
-      </para>
-    </refsect2>
-    
-    <refsect2 id="sniff">
-      <title>How about sniffing the network traffic and decrypting it
-      later by physically grabbing the Mandos client and using its
-      key?</title>
-      <para>
-	We only use <acronym>PFS</acronym> (Perfect Forward Security)
-	key exchange algorithms in TLS, which protects against this.
+	they do indeed hold the OpenPGP private key corresponding to
+	that client.
       </para>
     </refsect2>
     
@@ -234,22 +214,17 @@
       </para>
     </refsect2>
     
-    <refsect2 id="fakecheck">
-      <title>Faking checker results?</title>
+    <refsect2 id="fakeping">
+      <title>Faking ping replies?</title>
       <para>
-	If the Mandos client does not have an SSH server, the default
-	is for the Mandos server to use
+	The default for the server is to use
 	<quote><literal>fping</literal></quote>, the replies to which
 	could be faked to eliminate the timeout.  But this could
 	easily be changed to any shell command, with any security
-	measures you like.  If the Mandos client
-	<emphasis>has</emphasis> an SSH server, the default
-	configuration (as generated by
-	<command>mandos-keygen</command> with the
-	<option>--password</option> option) is for the Mandos server
-	to use an <command>ssh-keyscan</command> command with strict
-	keychecking, which can not be faked.  Alternatively, IPsec
-	could be used for the ping packets, making them secure.
+	measures you like.  It could, for instance, be changed to an
+	SSH command with strict keychecking, which could not be faked.
+	Or IPsec could be used for the ping packets, making them
+	secure.
       </para>
     </refsect2>
   </refsect1>
@@ -384,11 +359,6 @@
     </para>
   </refsect1>
   
-  <refsect1 id="bugs">
-    <title>BUGS</title>
-    <xi:include href="bugs.xml"/>
-  </refsect1>
-  
   <refsect1 id="see_also">
     <title>SEE ALSO</title>
     <para>
@@ -422,7 +392,7 @@
     <variablelist>
       <varlistentry>
 	<term>
-	  <ulink url="https://www.recompile.se/mandos">Mandos</ulink>
+	  <ulink url="http://www.recompile.se/mandos">Mandos</ulink>
 	</term>
 	<listitem>
 	  <para>

=== modified file 'legalnotice.xml'
--- legalnotice.xml	2017-08-20 16:20:54 +0000
+++ legalnotice.xml	2008-09-06 17:24:58 +0000
@@ -3,26 +3,25 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd">
 <legalnotice>
   <para>
-    This manual page is part of Mandos.
-  </para>
-  
-  <para>
-    Mandos is free software: you can redistribute it and/or modify it
-    under the terms of the <acronym>GNU</acronym> General Public
-    License as published by the Free Software Foundation, either
-    version 3 of the License, or (at your option) any later version.
-  </para>
-  
-  <para>
-    Mandos is distributed in the hope that it will be useful, but
-    WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    <acronym>GNU</acronym> General Public License for more details.
+    This manual page is free software: you can redistribute it and/or
+    modify it under the terms of the <acronym>GNU</acronym> General
+    Public License as published by the Free Software Foundation,
+    either version 3 of the License, or (at your option) any later
+    version.
+  </para>
+  
+  <para>
+    This manual page is distributed in the hope that it will be
+    useful, but WITHOUT ANY WARRANTY; without even the implied
+    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See the <acronym>GNU</acronym> General Public License for more
+    details.
   </para>
   
   <para>
     You should have received a copy of the <acronym>GNU</acronym>
-    General Public License along with Mandos.  If not, see <ulink
+    General Public License along with this program.  If not, see
+    <ulink
     url="http://www.gnu.org/licenses/">http://www.gnu.org/licenses/</ulink>.
   </para>
 </legalnotice>

=== modified file 'mandos'
--- mandos	2019-02-11 06:31:42 +0000
+++ mandos	2011-10-08 17:15:48 +0000
@@ -1,57 +1,51 @@
 #!/usr/bin/python
 # -*- mode: python; coding: utf-8 -*-
-#
+# 
 # Mandos server - give out binary blobs to connecting clients.
-#
+# 
 # This program is partly derived from an example program for an Avahi
 # service publisher, downloaded from
 # <http://avahi.org/wiki/PythonPublishExample>.  This includes the
 # methods "add", "remove", "server_state_changed",
 # "entry_group_state_changed", "cleanup", and "activate" in the
 # "AvahiService" class, and some lines in "main".
-#
+# 
 # Everything else is
-# Copyright © 2008-2019 Teddy Hogeborn
-# Copyright © 2008-2019 Björn Påhlsson
-#
-# This file is part of Mandos.
-#
-# Mandos is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
+# Copyright © 2008-2011 Teddy Hogeborn
+# Copyright © 2008-2011 Björn Påhlsson
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#     Mandos is distributed in the hope that it will be useful, but
-#     WITHOUT ANY WARRANTY; without even the implied warranty of
+#     This program is distributed in the hope that it will be useful,
+#     but WITHOUT ANY WARRANTY; without even the implied warranty of
 #     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #     GNU General Public License for more details.
-#
+# 
 # You should have received a copy of the GNU General Public License
-# along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
-#
+# along with this program.  If not, see
+# <http://www.gnu.org/licenses/>.
+# 
 # Contact the authors at <mandos@recompile.se>.
-#
+# 
 
 from __future__ import (division, absolute_import, print_function,
                         unicode_literals)
 
-try:
-    from future_builtins import *
-except ImportError:
-    pass
-
-try:
-    import SocketServer as socketserver
-except ImportError:
-    import socketserver
+import SocketServer as socketserver
 import socket
 import argparse
 import datetime
 import errno
-try:
-    import ConfigParser as configparser
-except ImportError:
-    import configparser
+import gnutls.crypto
+import gnutls.connection
+import gnutls.errors
+import gnutls.library.functions
+import gnutls.library.constants
+import gnutls.library.types
+import ConfigParser as configparser
 import sys
 import re
 import os
@@ -66,265 +60,70 @@
 import struct
 import fcntl
 import functools
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
+import cPickle as pickle
 import multiprocessing
 import types
-import binascii
-import tempfile
-import itertools
-import collections
-import codecs
 
 import dbus
 import dbus.service
-from gi.repository import GLib
+import gobject
+import avahi
 from dbus.mainloop.glib import DBusGMainLoop
 import ctypes
 import ctypes.util
 import xml.dom.minidom
 import inspect
 
-# Try to find the value of SO_BINDTODEVICE:
 try:
-    # This is where SO_BINDTODEVICE is in Python 3.3 (or 3.4?) and
-    # newer, and it is also the most natural place for it:
     SO_BINDTODEVICE = socket.SO_BINDTODEVICE
 except AttributeError:
     try:
-        # This is where SO_BINDTODEVICE was up to and including Python
-        # 2.6, and also 3.2:
         from IN import SO_BINDTODEVICE
     except ImportError:
-        # In Python 2.7 it seems to have been removed entirely.
-        # Try running the C preprocessor:
-        try:
-            cc = subprocess.Popen(["cc", "--language=c", "-E",
-                                   "/dev/stdin"],
-                                  stdin=subprocess.PIPE,
-                                  stdout=subprocess.PIPE)
-            stdout = cc.communicate(
-                "#include <sys/socket.h>\nSO_BINDTODEVICE\n")[0]
-            SO_BINDTODEVICE = int(stdout.splitlines()[-1])
-        except (OSError, ValueError, IndexError):
-            # No value found
-            SO_BINDTODEVICE = None
-
-if sys.version_info.major == 2:
-    str = unicode
-
-version = "1.8.3"
-stored_state_file = "clients.pickle"
-
-logger = logging.getLogger()
-syslogger = None
-
-try:
-    if_nametoindex = ctypes.cdll.LoadLibrary(
-        ctypes.util.find_library("c")).if_nametoindex
-except (OSError, AttributeError):
-
-    def if_nametoindex(interface):
-        "Get an interface index the hard way, i.e. using fcntl()"
-        SIOCGIFINDEX = 0x8933  # From /usr/include/linux/sockios.h
-        with contextlib.closing(socket.socket()) as s:
-            ifreq = fcntl.ioctl(s, SIOCGIFINDEX,
-                                struct.pack(b"16s16x", interface))
-        interface_index = struct.unpack("I", ifreq[16:20])[0]
-        return interface_index
-
-
-def copy_function(func):
-    """Make a copy of a function"""
-    if sys.version_info.major == 2:
-        return types.FunctionType(func.func_code,
-                                  func.func_globals,
-                                  func.func_name,
-                                  func.func_defaults,
-                                  func.func_closure)
-    else:
-        return types.FunctionType(func.__code__,
-                                  func.__globals__,
-                                  func.__name__,
-                                  func.__defaults__,
-                                  func.__closure__)
-
-
-def initlogger(debug, level=logging.WARNING):
-    """init logger and add loglevel"""
-
-    global syslogger
-    syslogger = (logging.handlers.SysLogHandler(
-        facility=logging.handlers.SysLogHandler.LOG_DAEMON,
-        address="/dev/log"))
-    syslogger.setFormatter(logging.Formatter
-                           ('Mandos [%(process)d]: %(levelname)s:'
-                            ' %(message)s'))
-    logger.addHandler(syslogger)
-
-    if debug:
-        console = logging.StreamHandler()
-        console.setFormatter(logging.Formatter('%(asctime)s %(name)s'
-                                               ' [%(process)d]:'
-                                               ' %(levelname)s:'
-                                               ' %(message)s'))
-        logger.addHandler(console)
-    logger.setLevel(level)
-
-
-class PGPError(Exception):
-    """Exception if encryption/decryption fails"""
-    pass
-
-
-class PGPEngine(object):
-    """A simple class for OpenPGP symmetric encryption & decryption"""
-
-    def __init__(self):
-        self.tempdir = tempfile.mkdtemp(prefix="mandos-")
-        self.gpg = "gpg"
-        try:
-            output = subprocess.check_output(["gpgconf"])
-            for line in output.splitlines():
-                name, text, path = line.split(b":")
-                if name == "gpg":
-                    self.gpg = path
-                    break
-        except OSError as e:
-            if e.errno != errno.ENOENT:
-                raise
-        self.gnupgargs = ['--batch',
-                          '--homedir', self.tempdir,
-                          '--force-mdc',
-                          '--quiet']
-        # Only GPG version 1 has the --no-use-agent option.
-        if self.gpg == "gpg" or self.gpg.endswith("/gpg"):
-            self.gnupgargs.append("--no-use-agent")
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, exc_type, exc_value, traceback):
-        self._cleanup()
-        return False
-
-    def __del__(self):
-        self._cleanup()
-
-    def _cleanup(self):
-        if self.tempdir is not None:
-            # Delete contents of tempdir
-            for root, dirs, files in os.walk(self.tempdir,
-                                             topdown=False):
-                for filename in files:
-                    os.remove(os.path.join(root, filename))
-                for dirname in dirs:
-                    os.rmdir(os.path.join(root, dirname))
-            # Remove tempdir
-            os.rmdir(self.tempdir)
-            self.tempdir = None
-
-    def password_encode(self, password):
-        # Passphrase can not be empty and can not contain newlines or
-        # NUL bytes.  So we prefix it and hex encode it.
-        encoded = b"mandos" + binascii.hexlify(password)
-        if len(encoded) > 2048:
-            # GnuPG can't handle long passwords, so encode differently
-            encoded = (b"mandos" + password.replace(b"\\", b"\\\\")
-                       .replace(b"\n", b"\\n")
-                       .replace(b"\0", b"\\x00"))
-        return encoded
-
-    def encrypt(self, data, password):
-        passphrase = self.password_encode(password)
-        with tempfile.NamedTemporaryFile(
-                dir=self.tempdir) as passfile:
-            passfile.write(passphrase)
-            passfile.flush()
-            proc = subprocess.Popen([self.gpg, '--symmetric',
-                                     '--passphrase-file',
-                                     passfile.name]
-                                    + self.gnupgargs,
-                                    stdin=subprocess.PIPE,
-                                    stdout=subprocess.PIPE,
-                                    stderr=subprocess.PIPE)
-            ciphertext, err = proc.communicate(input=data)
-        if proc.returncode != 0:
-            raise PGPError(err)
-        return ciphertext
-
-    def decrypt(self, data, password):
-        passphrase = self.password_encode(password)
-        with tempfile.NamedTemporaryFile(
-                dir=self.tempdir) as passfile:
-            passfile.write(passphrase)
-            passfile.flush()
-            proc = subprocess.Popen([self.gpg, '--decrypt',
-                                     '--passphrase-file',
-                                     passfile.name]
-                                    + self.gnupgargs,
-                                    stdin=subprocess.PIPE,
-                                    stdout=subprocess.PIPE,
-                                    stderr=subprocess.PIPE)
-            decrypted_plaintext, err = proc.communicate(input=data)
-        if proc.returncode != 0:
-            raise PGPError(err)
-        return decrypted_plaintext
-
-
-# Pretend that we have an Avahi module
-class Avahi(object):
-    """This isn't so much a class as it is a module-like namespace.
-    It is instantiated once, and simulates having an Avahi module."""
-    IF_UNSPEC = -1               # avahi-common/address.h
-    PROTO_UNSPEC = -1            # avahi-common/address.h
-    PROTO_INET = 0               # avahi-common/address.h
-    PROTO_INET6 = 1              # avahi-common/address.h
-    DBUS_NAME = "org.freedesktop.Avahi"
-    DBUS_INTERFACE_ENTRY_GROUP = DBUS_NAME + ".EntryGroup"
-    DBUS_INTERFACE_SERVER = DBUS_NAME + ".Server"
-    DBUS_PATH_SERVER = "/"
-
-    def string_array_to_txt_array(self, t):
-        return dbus.Array((dbus.ByteArray(s.encode("utf-8"))
-                           for s in t), signature="ay")
-    ENTRY_GROUP_ESTABLISHED = 2  # avahi-common/defs.h
-    ENTRY_GROUP_COLLISION = 3    # avahi-common/defs.h
-    ENTRY_GROUP_FAILURE = 4      # avahi-common/defs.h
-    SERVER_INVALID = 0           # avahi-common/defs.h
-    SERVER_REGISTERING = 1       # avahi-common/defs.h
-    SERVER_RUNNING = 2           # avahi-common/defs.h
-    SERVER_COLLISION = 3         # avahi-common/defs.h
-    SERVER_FAILURE = 4           # avahi-common/defs.h
-avahi = Avahi()
-
+        SO_BINDTODEVICE = None
+
+
+version = "1.3.1"
+
+#logger = logging.getLogger('mandos')
+logger = logging.Logger('mandos')
+syslogger = (logging.handlers.SysLogHandler
+             (facility = logging.handlers.SysLogHandler.LOG_DAEMON,
+              address = str("/dev/log")))
+syslogger.setFormatter(logging.Formatter
+                       ('Mandos [%(process)d]: %(levelname)s:'
+                        ' %(message)s'))
+logger.addHandler(syslogger)
+
+console = logging.StreamHandler()
+console.setFormatter(logging.Formatter('%(name)s [%(process)d]:'
+                                       ' %(levelname)s:'
+                                       ' %(message)s'))
+logger.addHandler(console)
 
 class AvahiError(Exception):
     def __init__(self, value, *args, **kwargs):
         self.value = value
-        return super(AvahiError, self).__init__(value, *args,
-                                                **kwargs)
-
+        super(AvahiError, self).__init__(value, *args, **kwargs)
+    def __unicode__(self):
+        return unicode(repr(self.value))
 
 class AvahiServiceError(AvahiError):
     pass
 
-
 class AvahiGroupError(AvahiError):
     pass
 
 
 class AvahiService(object):
     """An Avahi (Zeroconf) service.
-
+    
     Attributes:
     interface: integer; avahi.IF_UNSPEC or an interface index.
                Used to optionally bind to the specified interface.
     name: string; Example: 'Mandos'
     type: string; Example: '_mandos._tcp'.
-     See <https://www.iana.org/assignments/service-names-port-numbers>
+                  See <http://www.dns-sd.org/ServiceTypes.html>
     port: integer; what port to announce
     TXT: list of strings; TXT record for the service
     domain: string; Domain to publish on, default to .local if empty.
@@ -336,18 +135,10 @@
     server: D-Bus Server
     bus: dbus.SystemBus()
     """
-
-    def __init__(self,
-                 interface=avahi.IF_UNSPEC,
-                 name=None,
-                 servicetype=None,
-                 port=None,
-                 TXT=None,
-                 domain="",
-                 host="",
-                 max_renames=32768,
-                 protocol=avahi.PROTO_UNSPEC,
-                 bus=None):
+    def __init__(self, interface = avahi.IF_UNSPEC, name = None,
+                 servicetype = None, port = None, TXT = None,
+                 domain = "", host = "", max_renames = 32768,
+                 protocol = avahi.PROTO_UNSPEC, bus = None):
         self.interface = interface
         self.name = name
         self.type = servicetype
@@ -362,33 +153,29 @@
         self.server = None
         self.bus = bus
         self.entry_group_state_changed_match = None
-
-    def rename(self, remove=True):
+    def rename(self):
         """Derived from the Avahi example code"""
         if self.rename_count >= self.max_renames:
             logger.critical("No suitable Zeroconf service name found"
                             " after %i retries, exiting.",
                             self.rename_count)
             raise AvahiServiceError("Too many renames")
-        self.name = str(
-            self.server.GetAlternativeServiceName(self.name))
-        self.rename_count += 1
+        self.name = unicode(self.server
+                            .GetAlternativeServiceName(self.name))
         logger.info("Changing Zeroconf service name to %r ...",
                     self.name)
-        if remove:
-            self.remove()
+        syslogger.setFormatter(logging.Formatter
+                               ('Mandos (%s) [%%(process)d]:'
+                                ' %%(levelname)s: %%(message)s'
+                                % self.name))
+        self.remove()
         try:
             self.add()
         except dbus.exceptions.DBusException as error:
-            if (error.get_dbus_name()
-                == "org.freedesktop.Avahi.CollisionError"):
-                logger.info("Local Zeroconf service name collision.")
-                return self.rename(remove=False)
-            else:
-                logger.critical("D-Bus Exception", exc_info=error)
-                self.cleanup()
-                os._exit(1)
-
+            logger.critical("DBusException: %s", error)
+            self.cleanup()
+            os._exit(1)
+        self.rename_count += 1
     def remove(self):
         """Derived from the Avahi example code"""
         if self.entry_group_state_changed_match is not None:
@@ -396,7 +183,6 @@
             self.entry_group_state_changed_match = None
         if self.group is not None:
             self.group.Reset()
-
     def add(self):
         """Derived from the Avahi example code"""
         self.remove()
@@ -407,7 +193,7 @@
                 avahi.DBUS_INTERFACE_ENTRY_GROUP)
         self.entry_group_state_changed_match = (
             self.group.connect_to_signal(
-                'StateChanged', self.entry_group_state_changed))
+                'StateChanged', self .entry_group_state_changed))
         logger.debug("Adding Zeroconf service '%s' of type '%s' ...",
                      self.name, self.type)
         self.group.AddService(
@@ -419,11 +205,10 @@
             dbus.UInt16(self.port),
             avahi.string_array_to_txt_array(self.TXT))
         self.group.Commit()
-
     def entry_group_state_changed(self, state, error):
         """Derived from the Avahi example code"""
         logger.debug("Avahi entry group state change: %i", state)
-
+        
         if state == avahi.ENTRY_GROUP_ESTABLISHED:
             logger.debug("Zeroconf service established.")
         elif state == avahi.ENTRY_GROUP_COLLISION:
@@ -431,29 +216,29 @@
             self.rename()
         elif state == avahi.ENTRY_GROUP_FAILURE:
             logger.critical("Avahi: Error in group state changed %s",
-                            str(error))
-            raise AvahiGroupError("State changed: {!s}".format(error))
-
+                            unicode(error))
+            raise AvahiGroupError("State changed: %s"
+                                  % unicode(error))
     def cleanup(self):
         """Derived from the Avahi example code"""
         if self.group is not None:
             try:
                 self.group.Free()
             except (dbus.exceptions.UnknownMethodException,
-                    dbus.exceptions.DBusException):
+                    dbus.exceptions.DBusException) as e:
                 pass
             self.group = None
         self.remove()
-
     def server_state_changed(self, state, error=None):
         """Derived from the Avahi example code"""
         logger.debug("Avahi server state change: %i", state)
-        bad_states = {
-            avahi.SERVER_INVALID: "Zeroconf server invalid",
-            avahi.SERVER_REGISTERING: None,
-            avahi.SERVER_COLLISION: "Zeroconf server name collision",
-            avahi.SERVER_FAILURE: "Zeroconf server failure",
-        }
+        bad_states = { avahi.SERVER_INVALID:
+                           "Zeroconf server invalid",
+                       avahi.SERVER_REGISTERING: None,
+                       avahi.SERVER_COLLISION:
+                           "Zeroconf server name collision",
+                       avahi.SERVER_FAILURE:
+                           "Zeroconf server failure" }
         if state in bad_states:
             if bad_states[state] is not None:
                 if error is None:
@@ -462,24 +247,12 @@
                     logger.error(bad_states[state] + ": %r", error)
             self.cleanup()
         elif state == avahi.SERVER_RUNNING:
-            try:
-                self.add()
-            except dbus.exceptions.DBusException as error:
-                if (error.get_dbus_name()
-                    == "org.freedesktop.Avahi.CollisionError"):
-                    logger.info("Local Zeroconf service name"
-                                " collision.")
-                    return self.rename(remove=False)
-                else:
-                    logger.critical("D-Bus Exception", exc_info=error)
-                    self.cleanup()
-                    os._exit(1)
+            self.add()
         else:
             if error is None:
                 logger.debug("Unknown state: %r", state)
             else:
                 logger.debug("Unknown state: %r: %r", state, error)
-
     def activate(self):
         """Derived from the Avahi example code"""
         if self.server is None:
@@ -489,690 +262,319 @@
                                     follow_name_owner_changes=True),
                 avahi.DBUS_INTERFACE_SERVER)
         self.server.connect_to_signal("StateChanged",
-                                      self.server_state_changed)
+                                 self.server_state_changed)
         self.server_state_changed(self.server.GetState())
 
 
-class AvahiServiceToSyslog(AvahiService):
-    def rename(self, *args, **kwargs):
-        """Add the new name to the syslog messages"""
-        ret = super(AvahiServiceToSyslog, self).rename(*args, **kwargs)
-        syslogger.setFormatter(logging.Formatter(
-            'Mandos ({}) [%(process)d]: %(levelname)s: %(message)s'
-            .format(self.name)))
-        return ret
-
-
-# Pretend that we have a GnuTLS module
-class GnuTLS(object):
-    """This isn't so much a class as it is a module-like namespace.
-    It is instantiated once, and simulates having a GnuTLS module."""
-
-    library = ctypes.util.find_library("gnutls")
-    if library is None:
-        library = ctypes.util.find_library("gnutls-deb0")
-    _library = ctypes.cdll.LoadLibrary(library)
-    del library
-    _need_version = b"3.3.0"
-    _tls_rawpk_version = b"3.6.6"
-
-    def __init__(self):
-        # Need to use "self" here, since this method is called before
-        # the assignment to the "gnutls" global variable happens.
-        if self.check_version(self._need_version) is None:
-            raise self.Error("Needs GnuTLS {} or later"
-                             .format(self._need_version))
-
-    # Unless otherwise indicated, the constants and types below are
-    # all from the gnutls/gnutls.h C header file.
-
-    # Constants
-    E_SUCCESS = 0
-    E_INTERRUPTED = -52
-    E_AGAIN = -28
-    CRT_OPENPGP = 2
-    CRT_RAWPK = 3
-    CLIENT = 2
-    SHUT_RDWR = 0
-    CRD_CERTIFICATE = 1
-    E_NO_CERTIFICATE_FOUND = -49
-    X509_FMT_DER = 0
-    NO_TICKETS = 1<<10
-    ENABLE_RAWPK = 1<<18
-    CTYPE_PEERS = 3
-    KEYID_USE_SHA256 = 1        # gnutls/x509.h
-    OPENPGP_FMT_RAW = 0         # gnutls/openpgp.h
-
-    # Types
-    class session_int(ctypes.Structure):
-        _fields_ = []
-    session_t = ctypes.POINTER(session_int)
-
-    class certificate_credentials_st(ctypes.Structure):
-        _fields_ = []
-    certificate_credentials_t = ctypes.POINTER(
-        certificate_credentials_st)
-    certificate_type_t = ctypes.c_int
-
-    class datum_t(ctypes.Structure):
-        _fields_ = [('data', ctypes.POINTER(ctypes.c_ubyte)),
-                    ('size', ctypes.c_uint)]
-
-    class openpgp_crt_int(ctypes.Structure):
-        _fields_ = []
-    openpgp_crt_t = ctypes.POINTER(openpgp_crt_int)
-    openpgp_crt_fmt_t = ctypes.c_int  # gnutls/openpgp.h
-    log_func = ctypes.CFUNCTYPE(None, ctypes.c_int, ctypes.c_char_p)
-    credentials_type_t = ctypes.c_int
-    transport_ptr_t = ctypes.c_void_p
-    close_request_t = ctypes.c_int
-
-    # Exceptions
-    class Error(Exception):
-        # We need to use the class name "GnuTLS" here, since this
-        # exception might be raised from within GnuTLS.__init__,
-        # which is called before the assignment to the "gnutls"
-        # global variable has happened.
-        def __init__(self, message=None, code=None, args=()):
-            # Default usage is by a message string, but if a return
-            # code is passed, convert it to a string with
-            # gnutls.strerror()
-            self.code = code
-            if message is None and code is not None:
-                message = GnuTLS.strerror(code)
-            return super(GnuTLS.Error, self).__init__(
-                message, *args)
-
-    class CertificateSecurityError(Error):
-        pass
-
-    # Classes
-    class Credentials(object):
-        def __init__(self):
-            self._c_object = gnutls.certificate_credentials_t()
-            gnutls.certificate_allocate_credentials(
-                ctypes.byref(self._c_object))
-            self.type = gnutls.CRD_CERTIFICATE
-
-        def __del__(self):
-            gnutls.certificate_free_credentials(self._c_object)
-
-    class ClientSession(object):
-        def __init__(self, socket, credentials=None):
-            self._c_object = gnutls.session_t()
-            gnutls_flags = gnutls.CLIENT
-            if gnutls.check_version("3.5.6"):
-                gnutls_flags |= gnutls.NO_TICKETS
-            if gnutls.has_rawpk:
-                gnutls_flags |= gnutls.ENABLE_RAWPK
-            gnutls.init(ctypes.byref(self._c_object), gnutls_flags)
-            del gnutls_flags
-            gnutls.set_default_priority(self._c_object)
-            gnutls.transport_set_ptr(self._c_object, socket.fileno())
-            gnutls.handshake_set_private_extensions(self._c_object,
-                                                    True)
-            self.socket = socket
-            if credentials is None:
-                credentials = gnutls.Credentials()
-            gnutls.credentials_set(self._c_object, credentials.type,
-                                   ctypes.cast(credentials._c_object,
-                                               ctypes.c_void_p))
-            self.credentials = credentials
-
-        def __del__(self):
-            gnutls.deinit(self._c_object)
-
-        def handshake(self):
-            return gnutls.handshake(self._c_object)
-
-        def send(self, data):
-            data = bytes(data)
-            data_len = len(data)
-            while data_len > 0:
-                data_len -= gnutls.record_send(self._c_object,
-                                               data[-data_len:],
-                                               data_len)
-
-        def bye(self):
-            return gnutls.bye(self._c_object, gnutls.SHUT_RDWR)
-
-    # Error handling functions
-    def _error_code(result):
-        """A function to raise exceptions on errors, suitable
-        for the 'restype' attribute on ctypes functions"""
-        if result >= 0:
-            return result
-        if result == gnutls.E_NO_CERTIFICATE_FOUND:
-            raise gnutls.CertificateSecurityError(code=result)
-        raise gnutls.Error(code=result)
-
-    def _retry_on_error(result, func, arguments):
-        """A function to retry on some errors, suitable
-        for the 'errcheck' attribute on ctypes functions"""
-        while result < 0:
-            if result not in (gnutls.E_INTERRUPTED, gnutls.E_AGAIN):
-                return _error_code(result)
-            result = func(*arguments)
-        return result
-
-    # Unless otherwise indicated, the function declarations below are
-    # all from the gnutls/gnutls.h C header file.
-
-    # Functions
-    priority_set_direct = _library.gnutls_priority_set_direct
-    priority_set_direct.argtypes = [session_t, ctypes.c_char_p,
-                                    ctypes.POINTER(ctypes.c_char_p)]
-    priority_set_direct.restype = _error_code
-
-    init = _library.gnutls_init
-    init.argtypes = [ctypes.POINTER(session_t), ctypes.c_int]
-    init.restype = _error_code
-
-    set_default_priority = _library.gnutls_set_default_priority
-    set_default_priority.argtypes = [session_t]
-    set_default_priority.restype = _error_code
-
-    record_send = _library.gnutls_record_send
-    record_send.argtypes = [session_t, ctypes.c_void_p,
-                            ctypes.c_size_t]
-    record_send.restype = ctypes.c_ssize_t
-    record_send.errcheck = _retry_on_error
-
-    certificate_allocate_credentials = (
-        _library.gnutls_certificate_allocate_credentials)
-    certificate_allocate_credentials.argtypes = [
-        ctypes.POINTER(certificate_credentials_t)]
-    certificate_allocate_credentials.restype = _error_code
-
-    certificate_free_credentials = (
-        _library.gnutls_certificate_free_credentials)
-    certificate_free_credentials.argtypes = [
-        certificate_credentials_t]
-    certificate_free_credentials.restype = None
-
-    handshake_set_private_extensions = (
-        _library.gnutls_handshake_set_private_extensions)
-    handshake_set_private_extensions.argtypes = [session_t,
-                                                 ctypes.c_int]
-    handshake_set_private_extensions.restype = None
-
-    credentials_set = _library.gnutls_credentials_set
-    credentials_set.argtypes = [session_t, credentials_type_t,
-                                ctypes.c_void_p]
-    credentials_set.restype = _error_code
-
-    strerror = _library.gnutls_strerror
-    strerror.argtypes = [ctypes.c_int]
-    strerror.restype = ctypes.c_char_p
-
-    certificate_type_get = _library.gnutls_certificate_type_get
-    certificate_type_get.argtypes = [session_t]
-    certificate_type_get.restype = _error_code
-
-    certificate_get_peers = _library.gnutls_certificate_get_peers
-    certificate_get_peers.argtypes = [session_t,
-                                      ctypes.POINTER(ctypes.c_uint)]
-    certificate_get_peers.restype = ctypes.POINTER(datum_t)
-
-    global_set_log_level = _library.gnutls_global_set_log_level
-    global_set_log_level.argtypes = [ctypes.c_int]
-    global_set_log_level.restype = None
-
-    global_set_log_function = _library.gnutls_global_set_log_function
-    global_set_log_function.argtypes = [log_func]
-    global_set_log_function.restype = None
-
-    deinit = _library.gnutls_deinit
-    deinit.argtypes = [session_t]
-    deinit.restype = None
-
-    handshake = _library.gnutls_handshake
-    handshake.argtypes = [session_t]
-    handshake.restype = _error_code
-    handshake.errcheck = _retry_on_error
-
-    transport_set_ptr = _library.gnutls_transport_set_ptr
-    transport_set_ptr.argtypes = [session_t, transport_ptr_t]
-    transport_set_ptr.restype = None
-
-    bye = _library.gnutls_bye
-    bye.argtypes = [session_t, close_request_t]
-    bye.restype = _error_code
-    bye.errcheck = _retry_on_error
-
-    check_version = _library.gnutls_check_version
-    check_version.argtypes = [ctypes.c_char_p]
-    check_version.restype = ctypes.c_char_p
-
-    has_rawpk = bool(check_version(_tls_rawpk_version))
-
-    if has_rawpk:
-        # Types
-        class pubkey_st(ctypes.Structure):
-            _fields = []
-        pubkey_t = ctypes.POINTER(pubkey_st)
-
-        x509_crt_fmt_t = ctypes.c_int
-
-        # All the function declarations below are from gnutls/abstract.h
-        pubkey_init = _library.gnutls_pubkey_init
-        pubkey_init.argtypes = [ctypes.POINTER(pubkey_t)]
-        pubkey_init.restype = _error_code
-
-        pubkey_import = _library.gnutls_pubkey_import
-        pubkey_import.argtypes = [pubkey_t, ctypes.POINTER(datum_t),
-                                  x509_crt_fmt_t]
-        pubkey_import.restype = _error_code
-
-        pubkey_get_key_id = _library.gnutls_pubkey_get_key_id
-        pubkey_get_key_id.argtypes = [pubkey_t, ctypes.c_int,
-                                      ctypes.POINTER(ctypes.c_ubyte),
-                                      ctypes.POINTER(ctypes.c_size_t)]
-        pubkey_get_key_id.restype = _error_code
-
-        pubkey_deinit = _library.gnutls_pubkey_deinit
-        pubkey_deinit.argtypes = [pubkey_t]
-        pubkey_deinit.restype = None
-    else:
-        # All the function declarations below are from gnutls/openpgp.h
-
-        openpgp_crt_init = _library.gnutls_openpgp_crt_init
-        openpgp_crt_init.argtypes = [ctypes.POINTER(openpgp_crt_t)]
-        openpgp_crt_init.restype = _error_code
-
-        openpgp_crt_import = _library.gnutls_openpgp_crt_import
-        openpgp_crt_import.argtypes = [openpgp_crt_t,
-                                       ctypes.POINTER(datum_t),
-                                       openpgp_crt_fmt_t]
-        openpgp_crt_import.restype = _error_code
-
-        openpgp_crt_verify_self = _library.gnutls_openpgp_crt_verify_self
-        openpgp_crt_verify_self.argtypes = [openpgp_crt_t, ctypes.c_uint,
-                                            ctypes.POINTER(ctypes.c_uint)]
-        openpgp_crt_verify_self.restype = _error_code
-
-        openpgp_crt_deinit = _library.gnutls_openpgp_crt_deinit
-        openpgp_crt_deinit.argtypes = [openpgp_crt_t]
-        openpgp_crt_deinit.restype = None
-
-        openpgp_crt_get_fingerprint = (
-            _library.gnutls_openpgp_crt_get_fingerprint)
-        openpgp_crt_get_fingerprint.argtypes = [openpgp_crt_t,
-                                                ctypes.c_void_p,
-                                                ctypes.POINTER(
-                                                    ctypes.c_size_t)]
-        openpgp_crt_get_fingerprint.restype = _error_code
-
-    if check_version("3.6.4"):
-        certificate_type_get2 = _library.gnutls_certificate_type_get2
-        certificate_type_get2.argtypes = [session_t, ctypes.c_int]
-        certificate_type_get2.restype = _error_code
-
-    # Remove non-public functions
-    del _error_code, _retry_on_error
-# Create the global "gnutls" object, simulating a module
-gnutls = GnuTLS()
-
-
-def call_pipe(connection,       # : multiprocessing.Connection
-              func, *args, **kwargs):
-    """This function is meant to be called by multiprocessing.Process
-
-    This function runs func(*args, **kwargs), and writes the resulting
-    return value on the provided multiprocessing.Connection.
-    """
-    connection.send(func(*args, **kwargs))
-    connection.close()
-
-
+def _timedelta_to_milliseconds(td):
+    "Convert a datetime.timedelta() to milliseconds"
+    return ((td.days * 24 * 60 * 60 * 1000)
+            + (td.seconds * 1000)
+            + (td.microseconds // 1000))
+        
 class Client(object):
     """A representation of a client host served by this server.
-
+    
     Attributes:
-    approved:   bool(); 'None' if not yet approved/disapproved
+    _approved:   bool(); 'None' if not yet approved/disapproved
     approval_delay: datetime.timedelta(); Time to wait for approval
     approval_duration: datetime.timedelta(); Duration of one approval
     checker:    subprocess.Popen(); a running checker process used
                                     to see if the client lives.
                                     'None' if no process is running.
-    checker_callback_tag: a GLib event source tag, or None
+    checker_callback_tag: a gobject event source tag, or None
     checker_command: string; External command which is run to check
                      if client lives.  %() expansions are done at
                      runtime with vars(self) as dict, so that for
                      instance %(name)s can be used in the command.
-    checker_initiator_tag: a GLib event source tag, or None
+    checker_initiator_tag: a gobject event source tag, or None
     created:    datetime.datetime(); (UTC) object creation
-    client_structure: Object describing what attributes a client has
-                      and is used for storing the client at exit
     current_checker_command: string; current running checker_command
-    disable_initiator_tag: a GLib event source tag, or None
+    disable_hook:  If set, called by disable() as disable_hook(self)
+    disable_initiator_tag: a gobject event source tag, or None
     enabled:    bool()
     fingerprint: string (40 or 32 hexadecimal digits); used to
-                 uniquely identify an OpenPGP client
-    key_id: string (64 hexadecimal digits); used to uniquely identify
-            a client using raw public keys
+                 uniquely identify the client
     host:       string; available for use by the checker command
     interval:   datetime.timedelta(); How often to start a new checker
     last_approval_request: datetime.datetime(); (UTC) or None
     last_checked_ok: datetime.datetime(); (UTC) or None
-    last_checker_status: integer between 0 and 255 reflecting exit
-                         status of last checker. -1 reflects crashed
-                         checker, -2 means no checker completed yet.
-    last_checker_signal: The signal which killed the last checker, if
-                         last_checker_status is -1
-    last_enabled: datetime.datetime(); (UTC) or None
+    last_enabled: datetime.datetime(); (UTC)
     name:       string; from the config file, used in log messages and
                         D-Bus identifiers
     secret:     bytestring; sent verbatim (over TLS) to client
     timeout:    datetime.timedelta(); How long from last_checked_ok
                                       until this client is disabled
-    extended_timeout:   extra long timeout when secret has been sent
+    extended_timeout:   extra long timeout when password has been sent
     runtime_expansions: Allowed attributes for runtime expansion.
     expires:    datetime.datetime(); time (UTC) when a client will be
                 disabled, or None
-    server_settings: The server_settings dict from main()
     """
-
+    
     runtime_expansions = ("approval_delay", "approval_duration",
-                          "created", "enabled", "expires", "key_id",
-                          "fingerprint", "host", "interval",
-                          "last_approval_request", "last_checked_ok",
+                          "created", "enabled", "fingerprint",
+                          "host", "interval", "last_checked_ok",
                           "last_enabled", "name", "timeout")
-    client_defaults = {
-        "timeout": "PT5M",
-        "extended_timeout": "PT15M",
-        "interval": "PT2M",
-        "checker": "fping -q -- %%(host)s",
-        "host": "",
-        "approval_delay": "PT0S",
-        "approval_duration": "PT1S",
-        "approved_by_default": "True",
-        "enabled": "True",
-    }
-
-    @staticmethod
-    def config_parser(config):
-        """Construct a new dict of client settings of this form:
-        { client_name: {setting_name: value, ...}, ...}
-        with exceptions for any special settings as defined above.
-        NOTE: Must be a pure function. Must return the same result
-        value given the same arguments.
-        """
-        settings = {}
-        for client_name in config.sections():
-            section = dict(config.items(client_name))
-            client = settings[client_name] = {}
-
-            client["host"] = section["host"]
-            # Reformat values from string types to Python types
-            client["approved_by_default"] = config.getboolean(
-                client_name, "approved_by_default")
-            client["enabled"] = config.getboolean(client_name,
-                                                  "enabled")
-
-            # Uppercase and remove spaces from key_id and fingerprint
-            # for later comparison purposes with return value from the
-            # key_id() and fingerprint() functions
-            client["key_id"] = (section.get("key_id", "").upper()
-                                .replace(" ", ""))
-            client["fingerprint"] = (section["fingerprint"].upper()
-                                     .replace(" ", ""))
-            if "secret" in section:
-                client["secret"] = codecs.decode(section["secret"]
-                                                 .encode("utf-8"),
-                                                 "base64")
-            elif "secfile" in section:
-                with open(os.path.expanduser(os.path.expandvars
-                                             (section["secfile"])),
-                          "rb") as secfile:
-                    client["secret"] = secfile.read()
-            else:
-                raise TypeError("No secret or secfile for section {}"
-                                .format(section))
-            client["timeout"] = string_to_delta(section["timeout"])
-            client["extended_timeout"] = string_to_delta(
-                section["extended_timeout"])
-            client["interval"] = string_to_delta(section["interval"])
-            client["approval_delay"] = string_to_delta(
-                section["approval_delay"])
-            client["approval_duration"] = string_to_delta(
-                section["approval_duration"])
-            client["checker_command"] = section["checker"]
-            client["last_approval_request"] = None
-            client["last_checked_ok"] = None
-            client["last_checker_status"] = -2
-
-        return settings
-
-    def __init__(self, settings, name=None, server_settings=None):
+    
+    def timeout_milliseconds(self):
+        "Return the 'timeout' attribute in milliseconds"
+        return _timedelta_to_milliseconds(self.timeout)
+    
+    def extended_timeout_milliseconds(self):
+        "Return the 'extended_timeout' attribute in milliseconds"
+        return _timedelta_to_milliseconds(self.extended_timeout)
+    
+    def interval_milliseconds(self):
+        "Return the 'interval' attribute in milliseconds"
+        return _timedelta_to_milliseconds(self.interval)
+    
+    def approval_delay_milliseconds(self):
+        return _timedelta_to_milliseconds(self.approval_delay)
+    
+    def __init__(self, name = None, disable_hook=None, config=None):
+        """Note: the 'checker' key in 'config' sets the
+        'checker_command' attribute and *not* the 'checker'
+        attribute."""
         self.name = name
-        if server_settings is None:
-            server_settings = {}
-        self.server_settings = server_settings
-        # adding all client settings
-        for setting, value in settings.items():
-            setattr(self, setting, value)
-
-        if self.enabled:
-            if not hasattr(self, "last_enabled"):
-                self.last_enabled = datetime.datetime.utcnow()
-            if not hasattr(self, "expires"):
-                self.expires = (datetime.datetime.utcnow()
-                                + self.timeout)
-        else:
-            self.last_enabled = None
-            self.expires = None
-
+        if config is None:
+            config = {}
         logger.debug("Creating client %r", self.name)
-        logger.debug("  Key ID: %s", self.key_id)
+        # Uppercase and remove spaces from fingerprint for later
+        # comparison purposes with return value from the fingerprint()
+        # function
+        self.fingerprint = (config["fingerprint"].upper()
+                            .replace(" ", ""))
         logger.debug("  Fingerprint: %s", self.fingerprint)
-        self.created = settings.get("created",
-                                    datetime.datetime.utcnow())
-
-        # attributes specific for this server instance
+        if "secret" in config:
+            self.secret = config["secret"].decode("base64")
+        elif "secfile" in config:
+            with open(os.path.expanduser(os.path.expandvars
+                                         (config["secfile"])),
+                      "rb") as secfile:
+                self.secret = secfile.read()
+        else:
+            raise TypeError("No secret or secfile for client %s"
+                            % self.name)
+        self.host = config.get("host", "")
+        self.created = datetime.datetime.utcnow()
+        self.enabled = False
+        self.last_approval_request = None
+        self.last_enabled = None
+        self.last_checked_ok = None
+        self.timeout = string_to_delta(config["timeout"])
+        self.extended_timeout = string_to_delta(config
+                                                ["extended_timeout"])
+        self.interval = string_to_delta(config["interval"])
+        self.disable_hook = disable_hook
         self.checker = None
         self.checker_initiator_tag = None
         self.disable_initiator_tag = None
+        self.expires = None
         self.checker_callback_tag = None
+        self.checker_command = config["checker"]
         self.current_checker_command = None
-        self.approved = None
+        self.last_connect = None
+        self._approved = None
+        self.approved_by_default = config.get("approved_by_default",
+                                              True)
         self.approvals_pending = 0
-        self.changedstate = multiprocessing_manager.Condition(
-            multiprocessing_manager.Lock())
-        self.client_structure = [attr
-                                 for attr in self.__dict__.keys()
-                                 if not attr.startswith("_")]
-        self.client_structure.append("client_structure")
-
-        for name, t in inspect.getmembers(
-                type(self), lambda obj: isinstance(obj, property)):
-            if not name.startswith("_"):
-                self.client_structure.append(name)
-
-    # Send notice to process children that client state has changed
+        self.approval_delay = string_to_delta(
+            config["approval_delay"])
+        self.approval_duration = string_to_delta(
+            config["approval_duration"])
+        self.changedstate = (multiprocessing_manager
+                             .Condition(multiprocessing_manager
+                                        .Lock()))
+    
     def send_changedstate(self):
-        with self.changedstate:
-            self.changedstate.notify_all()
-
+        self.changedstate.acquire()
+        self.changedstate.notify_all()
+        self.changedstate.release()
+    
     def enable(self):
         """Start this client's checker and timeout hooks"""
         if getattr(self, "enabled", False):
             # Already enabled
             return
+        self.send_changedstate()
+        # Schedule a new checker to be started an 'interval' from now,
+        # and every interval from then on.
+        self.checker_initiator_tag = (gobject.timeout_add
+                                      (self.interval_milliseconds(),
+                                       self.start_checker))
+        # Schedule a disable() when 'timeout' has passed
         self.expires = datetime.datetime.utcnow() + self.timeout
+        self.disable_initiator_tag = (gobject.timeout_add
+                                   (self.timeout_milliseconds(),
+                                    self.disable))
         self.enabled = True
         self.last_enabled = datetime.datetime.utcnow()
-        self.init_checker()
-        self.send_changedstate()
-
+        # Also start a new checker *right now*.
+        self.start_checker()
+    
     def disable(self, quiet=True):
         """Disable this client."""
         if not getattr(self, "enabled", False):
             return False
         if not quiet:
+            self.send_changedstate()
+        if not quiet:
             logger.info("Disabling client %s", self.name)
-        if getattr(self, "disable_initiator_tag", None) is not None:
-            GLib.source_remove(self.disable_initiator_tag)
+        if getattr(self, "disable_initiator_tag", False):
+            gobject.source_remove(self.disable_initiator_tag)
             self.disable_initiator_tag = None
         self.expires = None
-        if getattr(self, "checker_initiator_tag", None) is not None:
-            GLib.source_remove(self.checker_initiator_tag)
+        if getattr(self, "checker_initiator_tag", False):
+            gobject.source_remove(self.checker_initiator_tag)
             self.checker_initiator_tag = None
         self.stop_checker()
+        if self.disable_hook:
+            self.disable_hook(self)
         self.enabled = False
-        if not quiet:
-            self.send_changedstate()
-        # Do not run this again if called by a GLib.timeout_add
+        # Do not run this again if called by a gobject.timeout_add
         return False
-
+    
     def __del__(self):
+        self.disable_hook = None
         self.disable()
-
-    def init_checker(self):
-        # Schedule a new checker to be started an 'interval' from now,
-        # and every interval from then on.
-        if self.checker_initiator_tag is not None:
-            GLib.source_remove(self.checker_initiator_tag)
-        self.checker_initiator_tag = GLib.timeout_add(
-            int(self.interval.total_seconds() * 1000),
-            self.start_checker)
-        # Schedule a disable() when 'timeout' has passed
-        if self.disable_initiator_tag is not None:
-            GLib.source_remove(self.disable_initiator_tag)
-        self.disable_initiator_tag = GLib.timeout_add(
-            int(self.timeout.total_seconds() * 1000), self.disable)
-        # Also start a new checker *right now*.
-        self.start_checker()
-
-    def checker_callback(self, source, condition, connection,
-                         command):
+    
+    def checker_callback(self, pid, condition, command):
         """The checker has completed, so take appropriate actions."""
         self.checker_callback_tag = None
         self.checker = None
-        # Read return code from connection (see call_pipe)
-        returncode = connection.recv()
-        connection.close()
-
-        if returncode >= 0:
-            self.last_checker_status = returncode
-            self.last_checker_signal = None
-            if self.last_checker_status == 0:
+        if os.WIFEXITED(condition):
+            exitstatus = os.WEXITSTATUS(condition)
+            if exitstatus == 0:
                 logger.info("Checker for %(name)s succeeded",
                             vars(self))
                 self.checked_ok()
             else:
-                logger.info("Checker for %(name)s failed", vars(self))
+                logger.info("Checker for %(name)s failed",
+                            vars(self))
         else:
-            self.last_checker_status = -1
-            self.last_checker_signal = -returncode
             logger.warning("Checker for %(name)s crashed?",
                            vars(self))
-        return False
-
-    def checked_ok(self):
-        """Assert that the client has been seen, alive and well."""
-        self.last_checked_ok = datetime.datetime.utcnow()
-        self.last_checker_status = 0
-        self.last_checker_signal = None
-        self.bump_timeout()
-
-    def bump_timeout(self, timeout=None):
-        """Bump up the timeout for this client."""
+    
+    def checked_ok(self, timeout=None):
+        """Bump up the timeout for this client.
+        
+        This should only be called when the client has been seen,
+        alive and well.
+        """
         if timeout is None:
             timeout = self.timeout
-        if self.disable_initiator_tag is not None:
-            GLib.source_remove(self.disable_initiator_tag)
-            self.disable_initiator_tag = None
-        if getattr(self, "enabled", False):
-            self.disable_initiator_tag = GLib.timeout_add(
-                int(timeout.total_seconds() * 1000), self.disable)
-            self.expires = datetime.datetime.utcnow() + timeout
-
+        self.last_checked_ok = datetime.datetime.utcnow()
+        gobject.source_remove(self.disable_initiator_tag)
+        self.expires = datetime.datetime.utcnow() + timeout
+        self.disable_initiator_tag = (gobject.timeout_add
+                                      (_timedelta_to_milliseconds
+                                       (timeout), self.disable))
+    
     def need_approval(self):
         self.last_approval_request = datetime.datetime.utcnow()
-
+    
     def start_checker(self):
         """Start a new checker subprocess if one is not running.
-
+        
         If a checker already exists, leave it running and do
         nothing."""
         # The reason for not killing a running checker is that if we
-        # did that, and if a checker (for some reason) started running
-        # slowly and taking more than 'interval' time, then the client
-        # would inevitably timeout, since no checker would get a
-        # chance to run to completion.  If we instead leave running
+        # did that, then if a checker (for some reason) started
+        # running slowly and taking more than 'interval' time, the
+        # client would inevitably timeout, since no checker would get
+        # a chance to run to completion.  If we instead leave running
         # checkers alone, the checker would have to take more time
         # than 'timeout' for the client to be disabled, which is as it
         # should be.
-
-        if self.checker is not None and not self.checker.is_alive():
-            logger.warning("Checker was not alive; joining")
-            self.checker.join()
-            self.checker = None
+        
+        # If a checker exists, make sure it is not a zombie
+        try:
+            pid, status = os.waitpid(self.checker.pid, os.WNOHANG)
+        except (AttributeError, OSError) as error:
+            if (isinstance(error, OSError)
+                and error.errno != errno.ECHILD):
+                raise error
+        else:
+            if pid:
+                logger.warning("Checker was a zombie")
+                gobject.source_remove(self.checker_callback_tag)
+                self.checker_callback(pid, status,
+                                      self.current_checker_command)
         # Start a new checker if needed
         if self.checker is None:
-            # Escape attributes for the shell
-            escaped_attrs = {
-                attr: re.escape(str(getattr(self, attr)))
-                for attr in self.runtime_expansions}
             try:
-                command = self.checker_command % escaped_attrs
-            except TypeError as error:
-                logger.error('Could not format string "%s"',
-                             self.checker_command,
-                             exc_info=error)
-                return True     # Try again later
+                # In case checker_command has exactly one % operator
+                command = self.checker_command % self.host
+            except TypeError:
+                # Escape attributes for the shell
+                escaped_attrs = dict(
+                    (attr,
+                     re.escape(unicode(str(getattr(self, attr, "")),
+                                       errors=
+                                       'replace')))
+                    for attr in
+                    self.runtime_expansions)
+                
+                try:
+                    command = self.checker_command % escaped_attrs
+                except TypeError as error:
+                    logger.error('Could not format string "%s":'
+                                 ' %s', self.checker_command, error)
+                    return True # Try again later
             self.current_checker_command = command
-            logger.info("Starting checker %r for %s", command,
-                        self.name)
-            # We don't need to redirect stdout and stderr, since
-            # in normal mode, that is already done by daemon(),
-            # and in debug mode we don't want to.  (Stdin is
-            # always replaced by /dev/null.)
-            # The exception is when not debugging but nevertheless
-            # running in the foreground; use the previously
-            # created wnull.
-            popen_args = {"close_fds": True,
-                          "shell": True,
-                          "cwd": "/"}
-            if (not self.server_settings["debug"]
-                and self.server_settings["foreground"]):
-                popen_args.update({"stdout": wnull,
-                                   "stderr": wnull})
-            pipe = multiprocessing.Pipe(duplex=False)
-            self.checker = multiprocessing.Process(
-                target=call_pipe,
-                args=(pipe[1], subprocess.call, command),
-                kwargs=popen_args)
-            self.checker.start()
-            self.checker_callback_tag = GLib.io_add_watch(
-                pipe[0].fileno(), GLib.IO_IN,
-                self.checker_callback, pipe[0], command)
-        # Re-run this periodically if run by GLib.timeout_add
+            try:
+                logger.info("Starting checker %r for %s",
+                            command, self.name)
+                # We don't need to redirect stdout and stderr, since
+                # in normal mode, that is already done by daemon(),
+                # and in debug mode we don't want to.  (Stdin is
+                # always replaced by /dev/null.)
+                self.checker = subprocess.Popen(command,
+                                                close_fds=True,
+                                                shell=True, cwd="/")
+                self.checker_callback_tag = (gobject.child_watch_add
+                                             (self.checker.pid,
+                                              self.checker_callback,
+                                              data=command))
+                # The checker may have completed before the gobject
+                # watch was added.  Check for this.
+                pid, status = os.waitpid(self.checker.pid, os.WNOHANG)
+                if pid:
+                    gobject.source_remove(self.checker_callback_tag)
+                    self.checker_callback(pid, status, command)
+            except OSError as error:
+                logger.error("Failed to start subprocess: %s",
+                             error)
+        # Re-run this periodically if run by gobject.timeout_add
         return True
-
+    
     def stop_checker(self):
         """Force the checker process, if any, to stop."""
         if self.checker_callback_tag:
-            GLib.source_remove(self.checker_callback_tag)
+            gobject.source_remove(self.checker_callback_tag)
             self.checker_callback_tag = None
         if getattr(self, "checker", None) is None:
             return
         logger.debug("Stopping checker for %(name)s", vars(self))
-        self.checker.terminate()
+        try:
+            os.kill(self.checker.pid, signal.SIGTERM)
+            #time.sleep(0.5)
+            #if self.checker.poll() is None:
+            #    os.kill(self.checker.pid, signal.SIGKILL)
+        except OSError as error:
+            if error.errno != errno.ESRCH: # No such process
+                raise
         self.checker = None
 
 
-def dbus_service_property(dbus_interface,
-                          signature="v",
-                          access="readwrite",
-                          byte_arrays=False):
+def dbus_service_property(dbus_interface, signature="v",
+                          access="readwrite", byte_arrays=False):
     """Decorators for marking methods of a DBusObjectWithProperties to
     become properties on the D-Bus.
-
+    
     The decorated method will be called with no arguments by "Get"
     and with one argument by "Set".
-
+    
     The parameters, where they are supported, are the same as
     dbus.service.method, except there is only "signature", since the
     type from Get() and the type sent to Set() is the same.
@@ -1181,8 +583,7 @@
     # "Set" method, so we fail early here:
     if byte_arrays and signature != "ay":
         raise ValueError("Byte arrays not supported for non-'ay'"
-                         " signature {!r}".format(signature))
-
+                         " signature %r" % signature)
     def decorator(func):
         func._dbus_is_property = True
         func._dbus_interface = dbus_interface
@@ -1191,59 +592,16 @@
         func._dbus_name = func.__name__
         if func._dbus_name.endswith("_dbus_property"):
             func._dbus_name = func._dbus_name[:-14]
-        func._dbus_get_args_options = {'byte_arrays': byte_arrays}
-        return func
-
-    return decorator
-
-
-def dbus_interface_annotations(dbus_interface):
-    """Decorator for marking functions returning interface annotations
-
-    Usage:
-
-    @dbus_interface_annotations("org.example.Interface")
-    def _foo(self):  # Function name does not matter
-        return {"org.freedesktop.DBus.Deprecated": "true",
-                "org.freedesktop.DBus.Property.EmitsChangedSignal":
-                    "false"}
-    """
-
-    def decorator(func):
-        func._dbus_is_interface = True
-        func._dbus_interface = dbus_interface
-        func._dbus_name = dbus_interface
-        return func
-
-    return decorator
-
-
-def dbus_annotations(annotations):
-    """Decorator to annotate D-Bus methods, signals or properties
-    Usage:
-
-    @dbus_annotations({"org.freedesktop.DBus.Deprecated": "true",
-                       "org.freedesktop.DBus.Property."
-                       "EmitsChangedSignal": "false"})
-    @dbus_service_property("org.example.Interface", signature="b",
-                           access="r")
-    def Property_dbus_property(self):
-        return dbus.Boolean(False)
-
-    See also the DBusObjectWithAnnotations class.
-    """
-
-    def decorator(func):
-        func._dbus_annotations = annotations
-        return func
-
+        func._dbus_get_args_options = {'byte_arrays': byte_arrays }
+        return func
     return decorator
 
 
 class DBusPropertyException(dbus.exceptions.DBusException):
     """A base class for D-Bus property-related exceptions
     """
-    pass
+    def __unicode__(self):
+        return unicode(str(self))
 
 
 class DBusPropertyAccessException(DBusPropertyException):
@@ -1258,129 +616,43 @@
     pass
 
 
-class DBusObjectWithAnnotations(dbus.service.Object):
-    """A D-Bus object with annotations.
-
-    Classes inheriting from this can use the dbus_annotations
-    decorator to add annotations to methods or signals.
-    """
-
-    @staticmethod
-    def _is_dbus_thing(thing):
-        """Returns a function testing if an attribute is a D-Bus thing
-
-        If called like _is_dbus_thing("method") it returns a function
-        suitable for use as predicate to inspect.getmembers().
-        """
-        return lambda obj: getattr(obj, "_dbus_is_{}".format(thing),
-                                   False)
-
-    def _get_all_dbus_things(self, thing):
-        """Returns a generator of (name, attribute) pairs
-        """
-        return ((getattr(athing.__get__(self), "_dbus_name", name),
-                 athing.__get__(self))
-                for cls in self.__class__.__mro__
-                for name, athing in
-                inspect.getmembers(cls, self._is_dbus_thing(thing)))
-
-    @dbus.service.method(dbus.INTROSPECTABLE_IFACE,
-                         out_signature="s",
-                         path_keyword='object_path',
-                         connection_keyword='connection')
-    def Introspect(self, object_path, connection):
-        """Overloading of standard D-Bus method.
-
-        Inserts annotation tags on methods and signals.
-        """
-        xmlstring = dbus.service.Object.Introspect(self, object_path,
-                                                   connection)
-        try:
-            document = xml.dom.minidom.parseString(xmlstring)
-
-            for if_tag in document.getElementsByTagName("interface"):
-                # Add annotation tags
-                for typ in ("method", "signal"):
-                    for tag in if_tag.getElementsByTagName(typ):
-                        annots = dict()
-                        for name, prop in (self.
-                                           _get_all_dbus_things(typ)):
-                            if (name == tag.getAttribute("name")
-                                and prop._dbus_interface
-                                == if_tag.getAttribute("name")):
-                                annots.update(getattr(
-                                    prop, "_dbus_annotations", {}))
-                        for name, value in annots.items():
-                            ann_tag = document.createElement(
-                                "annotation")
-                            ann_tag.setAttribute("name", name)
-                            ann_tag.setAttribute("value", value)
-                            tag.appendChild(ann_tag)
-                # Add interface annotation tags
-                for annotation, value in dict(
-                    itertools.chain.from_iterable(
-                        annotations().items()
-                        for name, annotations
-                        in self._get_all_dbus_things("interface")
-                        if name == if_tag.getAttribute("name")
-                        )).items():
-                    ann_tag = document.createElement("annotation")
-                    ann_tag.setAttribute("name", annotation)
-                    ann_tag.setAttribute("value", value)
-                    if_tag.appendChild(ann_tag)
-                # Fix argument name for the Introspect method itself
-                if (if_tag.getAttribute("name")
-                    == dbus.INTROSPECTABLE_IFACE):
-                    for cn in if_tag.getElementsByTagName("method"):
-                        if cn.getAttribute("name") == "Introspect":
-                            for arg in cn.getElementsByTagName("arg"):
-                                if (arg.getAttribute("direction")
-                                    == "out"):
-                                    arg.setAttribute("name",
-                                                     "xml_data")
-            xmlstring = document.toxml("utf-8")
-            document.unlink()
-        except (AttributeError, xml.dom.DOMException,
-                xml.parsers.expat.ExpatError) as error:
-            logger.error("Failed to override Introspection method",
-                         exc_info=error)
-        return xmlstring
-
-
-class DBusObjectWithProperties(DBusObjectWithAnnotations):
+class DBusObjectWithProperties(dbus.service.Object):
     """A D-Bus object with properties.
-
+    
     Classes inheriting from this can use the dbus_service_property
     decorator to expose methods as D-Bus properties.  It exposes the
     standard Get(), Set(), and GetAll() methods on the D-Bus.
     """
-
+    
+    @staticmethod
+    def _is_dbus_property(obj):
+        return getattr(obj, "_dbus_is_property", False)
+    
+    def _get_all_dbus_properties(self):
+        """Returns a generator of (name, attribute) pairs
+        """
+        return ((prop.__get__(self)._dbus_name, prop.__get__(self))
+                for cls in self.__class__.__mro__
+                for name, prop in
+                inspect.getmembers(cls, self._is_dbus_property))
+    
     def _get_dbus_property(self, interface_name, property_name):
         """Returns a bound method if one exists which is a D-Bus
         property with the specified name and interface.
         """
-        for cls in self.__class__.__mro__:
-            for name, value in inspect.getmembers(
-                    cls, self._is_dbus_thing("property")):
+        for cls in  self.__class__.__mro__:
+            for name, value in (inspect.getmembers
+                                (cls, self._is_dbus_property)):
                 if (value._dbus_name == property_name
                     and value._dbus_interface == interface_name):
                     return value.__get__(self)
-
+        
         # No such property
-        raise DBusPropertyNotFound("{}:{}.{}".format(
-            self.dbus_object_path, interface_name, property_name))
-
-    @classmethod
-    def _get_all_interface_names(cls):
-        """Get a sequence of all interfaces supported by an object"""
-        return (name for name in set(getattr(getattr(x, attr),
-                                             "_dbus_interface", None)
-                                     for x in (inspect.getmro(cls))
-                                     for attr in dir(x))
-                if name is not None)
-
-    @dbus.service.method(dbus.PROPERTIES_IFACE,
-                         in_signature="ss",
+        raise DBusPropertyNotFound(self.dbus_object_path + ":"
+                                   + interface_name + "."
+                                   + property_name)
+    
+    @dbus.service.method(dbus.PROPERTIES_IFACE, in_signature="ss",
                          out_signature="v")
     def Get(self, interface_name, property_name):
         """Standard D-Bus property Get() method, see D-Bus standard.
@@ -1392,7 +664,7 @@
         if not hasattr(value, "variant_level"):
             return value
         return type(value)(value, variant_level=value.variant_level+1)
-
+    
     @dbus.service.method(dbus.PROPERTIES_IFACE, in_signature="ssv")
     def Set(self, interface_name, property_name, value):
         """Standard D-Bus property Set() method, see D-Bus standard.
@@ -1404,24 +676,21 @@
             # The byte_arrays option is not supported yet on
             # signatures other than "ay".
             if prop._dbus_signature != "ay":
-                raise ValueError("Byte arrays not supported for non-"
-                                 "'ay' signature {!r}"
-                                 .format(prop._dbus_signature))
-            value = dbus.ByteArray(b''.join(chr(byte)
-                                            for byte in value))
+                raise ValueError
+            value = dbus.ByteArray(''.join(unichr(byte)
+                                           for byte in value))
         prop(value)
-
-    @dbus.service.method(dbus.PROPERTIES_IFACE,
-                         in_signature="s",
+    
+    @dbus.service.method(dbus.PROPERTIES_IFACE, in_signature="s",
                          out_signature="a{sv}")
     def GetAll(self, interface_name):
         """Standard D-Bus property GetAll() method, see D-Bus
         standard.
-
+        
         Note: Will not include properties with access="write".
         """
-        properties = {}
-        for name, prop in self._get_all_dbus_things("property"):
+        all = {}
+        for name, prop in self._get_all_dbus_properties():
             if (interface_name
                 and interface_name != prop._dbus_interface):
                 # Interface non-empty but did not match
@@ -1431,66 +700,36 @@
                 continue
             value = prop()
             if not hasattr(value, "variant_level"):
-                properties[name] = value
+                all[name] = value
                 continue
-            properties[name] = type(value)(
-                value, variant_level=value.variant_level + 1)
-        return dbus.Dictionary(properties, signature="sv")
-
-    @dbus.service.signal(dbus.PROPERTIES_IFACE, signature="sa{sv}as")
-    def PropertiesChanged(self, interface_name, changed_properties,
-                          invalidated_properties):
-        """Standard D-Bus PropertiesChanged() signal, see D-Bus
-        standard.
-        """
-        pass
-
+            all[name] = type(value)(value, variant_level=
+                                    value.variant_level+1)
+        return dbus.Dictionary(all, signature="sv")
+    
     @dbus.service.method(dbus.INTROSPECTABLE_IFACE,
                          out_signature="s",
                          path_keyword='object_path',
                          connection_keyword='connection')
     def Introspect(self, object_path, connection):
-        """Overloading of standard D-Bus method.
-
-        Inserts property tags and interface annotation tags.
+        """Standard D-Bus method, overloaded to insert property tags.
         """
-        xmlstring = DBusObjectWithAnnotations.Introspect(self,
-                                                         object_path,
-                                                         connection)
+        xmlstring = dbus.service.Object.Introspect(self, object_path,
+                                                   connection)
         try:
             document = xml.dom.minidom.parseString(xmlstring)
-
             def make_tag(document, name, prop):
                 e = document.createElement("property")
                 e.setAttribute("name", name)
                 e.setAttribute("type", prop._dbus_signature)
                 e.setAttribute("access", prop._dbus_access)
                 return e
-
             for if_tag in document.getElementsByTagName("interface"):
-                # Add property tags
                 for tag in (make_tag(document, name, prop)
                             for name, prop
-                            in self._get_all_dbus_things("property")
+                            in self._get_all_dbus_properties()
                             if prop._dbus_interface
                             == if_tag.getAttribute("name")):
                     if_tag.appendChild(tag)
-                # Add annotation tags for properties
-                for tag in if_tag.getElementsByTagName("property"):
-                    annots = dict()
-                    for name, prop in self._get_all_dbus_things(
-                            "property"):
-                        if (name == tag.getAttribute("name")
-                            and prop._dbus_interface
-                            == if_tag.getAttribute("name")):
-                            annots.update(getattr(
-                                prop, "_dbus_annotations", {}))
-                    for name, value in annots.items():
-                        ann_tag = document.createElement(
-                            "annotation")
-                        ann_tag.setAttribute("name", name)
-                        ann_tag.setAttribute("value", value)
-                        tag.appendChild(ann_tag)
                 # Add the names to the return values for the
                 # "org.freedesktop.DBus.Properties" methods
                 if (if_tag.getAttribute("name")
@@ -1511,368 +750,207 @@
         except (AttributeError, xml.dom.DOMException,
                 xml.parsers.expat.ExpatError) as error:
             logger.error("Failed to override Introspection method",
-                         exc_info=error)
-        return xmlstring
-
-
-try:
-    dbus.OBJECT_MANAGER_IFACE
-except AttributeError:
-    dbus.OBJECT_MANAGER_IFACE = "org.freedesktop.DBus.ObjectManager"
-
-
-class DBusObjectWithObjectManager(DBusObjectWithAnnotations):
-    """A D-Bus object with an ObjectManager.
-
-    Classes inheriting from this exposes the standard
-    GetManagedObjects call and the InterfacesAdded and
-    InterfacesRemoved signals on the standard
-    "org.freedesktop.DBus.ObjectManager" interface.
-
-    Note: No signals are sent automatically; they must be sent
-    manually.
-    """
-    @dbus.service.method(dbus.OBJECT_MANAGER_IFACE,
-                         out_signature="a{oa{sa{sv}}}")
-    def GetManagedObjects(self):
-        """This function must be overridden"""
-        raise NotImplementedError()
-
-    @dbus.service.signal(dbus.OBJECT_MANAGER_IFACE,
-                         signature="oa{sa{sv}}")
-    def InterfacesAdded(self, object_path, interfaces_and_properties):
-        pass
-
-    @dbus.service.signal(dbus.OBJECT_MANAGER_IFACE, signature="oas")
-    def InterfacesRemoved(self, object_path, interfaces):
-        pass
-
-    @dbus.service.method(dbus.INTROSPECTABLE_IFACE,
-                         out_signature="s",
-                         path_keyword='object_path',
-                         connection_keyword='connection')
-    def Introspect(self, object_path, connection):
-        """Overloading of standard D-Bus method.
-
-        Override return argument name of GetManagedObjects to be
-        "objpath_interfaces_and_properties"
-        """
-        xmlstring = DBusObjectWithAnnotations.Introspect(self,
-                                                         object_path,
-                                                         connection)
-        try:
-            document = xml.dom.minidom.parseString(xmlstring)
-
-            for if_tag in document.getElementsByTagName("interface"):
-                # Fix argument name for the GetManagedObjects method
-                if (if_tag.getAttribute("name")
-                    == dbus.OBJECT_MANAGER_IFACE):
-                    for cn in if_tag.getElementsByTagName("method"):
-                        if (cn.getAttribute("name")
-                            == "GetManagedObjects"):
-                            for arg in cn.getElementsByTagName("arg"):
-                                if (arg.getAttribute("direction")
-                                    == "out"):
-                                    arg.setAttribute(
-                                        "name",
-                                        "objpath_interfaces"
-                                        "_and_properties")
-            xmlstring = document.toxml("utf-8")
-            document.unlink()
-        except (AttributeError, xml.dom.DOMException,
-                xml.parsers.expat.ExpatError) as error:
-            logger.error("Failed to override Introspection method",
-                         exc_info=error)
-        return xmlstring
-
-
-def datetime_to_dbus(dt, variant_level=0):
+                         error)
+        return xmlstring
+
+
+def datetime_to_dbus (dt, variant_level=0):
     """Convert a UTC datetime.datetime() to a D-Bus type."""
     if dt is None:
-        return dbus.String("", variant_level=variant_level)
-    return dbus.String(dt.isoformat(), variant_level=variant_level)
-
-
-def alternate_dbus_interfaces(alt_interface_names, deprecate=True):
-    """A class decorator; applied to a subclass of
-    dbus.service.Object, it will add alternate D-Bus attributes with
-    interface names according to the "alt_interface_names" mapping.
-    Usage:
-
-    @alternate_dbus_interfaces({"org.example.Interface":
-                                    "net.example.AlternateInterface"})
-    class SampleDBusObject(dbus.service.Object):
-        @dbus.service.method("org.example.Interface")
-        def SampleDBusMethod():
-            pass
-
-    The above "SampleDBusMethod" on "SampleDBusObject" will be
-    reachable via two interfaces: "org.example.Interface" and
-    "net.example.AlternateInterface", the latter of which will have
-    its D-Bus annotation "org.freedesktop.DBus.Deprecated" set to
-    "true", unless "deprecate" is passed with a False value.
-
-    This works for methods and signals, and also for D-Bus properties
-    (from DBusObjectWithProperties) and interfaces (from the
-    dbus_interface_annotations decorator).
+        return dbus.String("", variant_level = variant_level)
+    return dbus.String(dt.isoformat(),
+                       variant_level=variant_level)
+
+class AlternateDBusNamesMetaclass(DBusObjectWithProperties
+                                  .__metaclass__):
+    """Applied to an empty subclass of a D-Bus object, this metaclass
+    will add additional D-Bus attributes matching a certain pattern.
     """
-
-    def wrapper(cls):
-        for orig_interface_name, alt_interface_name in (
-                alt_interface_names.items()):
-            attr = {}
-            interface_names = set()
-            # Go though all attributes of the class
-            for attrname, attribute in inspect.getmembers(cls):
+    def __new__(mcs, name, bases, attr):
+        # Go through all the base classes which could have D-Bus
+        # methods, signals, or properties in them
+        for base in (b for b in bases
+                     if issubclass(b, dbus.service.Object)):
+            # Go though all attributes of the base class
+            for attrname, attribute in inspect.getmembers(base):
                 # Ignore non-D-Bus attributes, and D-Bus attributes
                 # with the wrong interface name
                 if (not hasattr(attribute, "_dbus_interface")
-                    or not attribute._dbus_interface.startswith(
-                        orig_interface_name)):
+                    or not attribute._dbus_interface
+                    .startswith("se.recompile.Mandos")):
                     continue
                 # Create an alternate D-Bus interface name based on
                 # the current name
-                alt_interface = attribute._dbus_interface.replace(
-                    orig_interface_name, alt_interface_name)
-                interface_names.add(alt_interface)
+                alt_interface = (attribute._dbus_interface
+                                 .replace("se.recompile.Mandos",
+                                          "se.bsnet.fukt.Mandos"))
                 # Is this a D-Bus signal?
                 if getattr(attribute, "_dbus_is_signal", False):
-                    # Extract the original non-method undecorated
-                    # function by black magic
-                    if sys.version_info.major == 2:
-                        nonmethod_func = (dict(
+                    # Extract the original non-method function by
+                    # black magic
+                    nonmethod_func = (dict(
                             zip(attribute.func_code.co_freevars,
-                                attribute.__closure__))
-                                          ["func"].cell_contents)
-                    else:
-                        nonmethod_func = (dict(
-                            zip(attribute.__code__.co_freevars,
-                                attribute.__closure__))
-                                          ["func"].cell_contents)
+                                attribute.__closure__))["func"]
+                                      .cell_contents)
                     # Create a new, but exactly alike, function
                     # object, and decorate it to be a new D-Bus signal
                     # with the alternate D-Bus interface name
-                    new_function = copy_function(nonmethod_func)
-                    new_function = (dbus.service.signal(
-                        alt_interface,
-                        attribute._dbus_signature)(new_function))
-                    # Copy annotations, if any
-                    try:
-                        new_function._dbus_annotations = dict(
-                            attribute._dbus_annotations)
-                    except AttributeError:
-                        pass
-
+                    new_function = (dbus.service.signal
+                                    (alt_interface,
+                                     attribute._dbus_signature)
+                                    (types.FunctionType(
+                                nonmethod_func.func_code,
+                                nonmethod_func.func_globals,
+                                nonmethod_func.func_name,
+                                nonmethod_func.func_defaults,
+                                nonmethod_func.func_closure)))
                     # Define a creator of a function to call both the
-                    # original and alternate functions, so both the
-                    # original and alternate signals gets sent when
-                    # the function is called
+                    # old and new functions, so both the old and new
+                    # signals gets sent when the function is called
                     def fixscope(func1, func2):
                         """This function is a scope container to pass
                         func1 and func2 to the "call_both" function
                         outside of its arguments"""
-
-                        @functools.wraps(func2)
                         def call_both(*args, **kwargs):
                             """This function will emit two D-Bus
                             signals by calling func1 and func2"""
                             func1(*args, **kwargs)
                             func2(*args, **kwargs)
-                        # Make wrapper function look like a D-Bus
-                        # signal
-                        for name, attr in inspect.getmembers(func2):
-                            if name.startswith("_dbus_"):
-                                setattr(call_both, name, attr)
-
                         return call_both
                     # Create the "call_both" function and add it to
                     # the class
-                    attr[attrname] = fixscope(attribute, new_function)
+                    attr[attrname] = fixscope(attribute,
+                                              new_function)
                 # Is this a D-Bus method?
                 elif getattr(attribute, "_dbus_is_method", False):
                     # Create a new, but exactly alike, function
                     # object.  Decorate it to be a new D-Bus method
                     # with the alternate D-Bus interface name.  Add it
                     # to the class.
-                    attr[attrname] = (
-                        dbus.service.method(
-                            alt_interface,
-                            attribute._dbus_in_signature,
-                            attribute._dbus_out_signature)
-                        (copy_function(attribute)))
-                    # Copy annotations, if any
-                    try:
-                        attr[attrname]._dbus_annotations = dict(
-                            attribute._dbus_annotations)
-                    except AttributeError:
-                        pass
+                    attr[attrname] = (dbus.service.method
+                                      (alt_interface,
+                                       attribute._dbus_in_signature,
+                                       attribute._dbus_out_signature)
+                                      (types.FunctionType
+                                       (attribute.func_code,
+                                        attribute.func_globals,
+                                        attribute.func_name,
+                                        attribute.func_defaults,
+                                        attribute.func_closure)))
                 # Is this a D-Bus property?
                 elif getattr(attribute, "_dbus_is_property", False):
                     # Create a new, but exactly alike, function
                     # object, and decorate it to be a new D-Bus
                     # property with the alternate D-Bus interface
                     # name.  Add it to the class.
-                    attr[attrname] = (dbus_service_property(
-                        alt_interface, attribute._dbus_signature,
-                        attribute._dbus_access,
-                        attribute._dbus_get_args_options
-                        ["byte_arrays"])
-                                      (copy_function(attribute)))
-                    # Copy annotations, if any
-                    try:
-                        attr[attrname]._dbus_annotations = dict(
-                            attribute._dbus_annotations)
-                    except AttributeError:
-                        pass
-                # Is this a D-Bus interface?
-                elif getattr(attribute, "_dbus_is_interface", False):
-                    # Create a new, but exactly alike, function
-                    # object.  Decorate it to be a new D-Bus interface
-                    # with the alternate D-Bus interface name.  Add it
-                    # to the class.
-                    attr[attrname] = (
-                        dbus_interface_annotations(alt_interface)
-                        (copy_function(attribute)))
-            if deprecate:
-                # Deprecate all alternate interfaces
-                iname = "_AlternateDBusNames_interface_annotation{}"
-                for interface_name in interface_names:
-
-                    @dbus_interface_annotations(interface_name)
-                    def func(self):
-                        return {"org.freedesktop.DBus.Deprecated":
-                                "true"}
-                    # Find an unused name
-                    for aname in (iname.format(i)
-                                  for i in itertools.count()):
-                        if aname not in attr:
-                            attr[aname] = func
-                            break
-            if interface_names:
-                # Replace the class with a new subclass of it with
-                # methods, signals, etc. as created above.
-                if sys.version_info.major == 2:
-                    cls = type(b"{}Alternate".format(cls.__name__),
-                               (cls, ), attr)
-                else:
-                    cls = type("{}Alternate".format(cls.__name__),
-                               (cls, ), attr)
-        return cls
-
-    return wrapper
-
-
-@alternate_dbus_interfaces({"se.recompile.Mandos":
-                            "se.bsnet.fukt.Mandos"})
+                    attr[attrname] = (dbus_service_property
+                                      (alt_interface,
+                                       attribute._dbus_signature,
+                                       attribute._dbus_access,
+                                       attribute
+                                       ._dbus_get_args_options
+                                       ["byte_arrays"])
+                                      (types.FunctionType
+                                       (attribute.func_code,
+                                        attribute.func_globals,
+                                        attribute.func_name,
+                                        attribute.func_defaults,
+                                        attribute.func_closure)))
+        return type.__new__(mcs, name, bases, attr)
+
 class ClientDBus(Client, DBusObjectWithProperties):
     """A Client class using D-Bus
-
+    
     Attributes:
     dbus_object_path: dbus.ObjectPath
     bus: dbus.SystemBus()
     """
-
+    
     runtime_expansions = (Client.runtime_expansions
-                          + ("dbus_object_path", ))
-
-    _interface = "se.recompile.Mandos.Client"
-
+                          + ("dbus_object_path",))
+    
     # dbus.service.Object doesn't use super(), so we can't either.
-
-    def __init__(self, bus=None, *args, **kwargs):
+    
+    def __init__(self, bus = None, *args, **kwargs):
+        self._approvals_pending = 0
         self.bus = bus
         Client.__init__(self, *args, **kwargs)
         # Only now, when this client is initialized, can it show up on
         # the D-Bus
-        client_object_name = str(self.name).translate(
+        client_object_name = unicode(self.name).translate(
             {ord("."): ord("_"),
              ord("-"): ord("_")})
-        self.dbus_object_path = dbus.ObjectPath(
-            "/clients/" + client_object_name)
+        self.dbus_object_path = (dbus.ObjectPath
+                                 ("/clients/" + client_object_name))
         DBusObjectWithProperties.__init__(self, self.bus,
                                           self.dbus_object_path)
-
-    def notifychangeproperty(transform_func, dbus_name,
-                             type_func=lambda x: x,
-                             variant_level=1,
-                             invalidate_only=False,
-                             _interface=_interface):
+        
+    def notifychangeproperty(transform_func,
+                             dbus_name, type_func=lambda x: x,
+                             variant_level=1):
         """ Modify a variable so that it's a property which announces
         its changes to DBus.
 
-        transform_fun: Function that takes a value and a variant_level
-                       and transforms it to a D-Bus type.
+        transform_fun: Function that takes a value and transforms it
+                       to a D-Bus type.
         dbus_name: D-Bus name of the variable
         type_func: Function that transform the value before sending it
                    to the D-Bus.  Default: no transform
         variant_level: D-Bus variant level.  Default: 1
         """
-        attrname = "_{}".format(dbus_name)
-
+        attrname = "_{0}".format(dbus_name)
         def setter(self, value):
             if hasattr(self, "dbus_object_path"):
                 if (not hasattr(self, attrname) or
                     type_func(getattr(self, attrname, None))
                     != type_func(value)):
-                    if invalidate_only:
-                        self.PropertiesChanged(
-                            _interface, dbus.Dictionary(),
-                            dbus.Array((dbus_name, )))
-                    else:
-                        dbus_value = transform_func(
-                            type_func(value),
-                            variant_level=variant_level)
-                        self.PropertyChanged(dbus.String(dbus_name),
-                                             dbus_value)
-                        self.PropertiesChanged(
-                            _interface,
-                            dbus.Dictionary({dbus.String(dbus_name):
-                                             dbus_value}),
-                            dbus.Array())
+                    dbus_value = transform_func(type_func(value),
+                                                variant_level)
+                    self.PropertyChanged(dbus.String(dbus_name),
+                                         dbus_value)
             setattr(self, attrname, value)
-
+        
         return property(lambda self: getattr(self, attrname), setter)
-
+    
+    
     expires = notifychangeproperty(datetime_to_dbus, "Expires")
     approvals_pending = notifychangeproperty(dbus.Boolean,
                                              "ApprovalPending",
-                                             type_func=bool)
+                                             type_func = bool)
     enabled = notifychangeproperty(dbus.Boolean, "Enabled")
     last_enabled = notifychangeproperty(datetime_to_dbus,
                                         "LastEnabled")
-    checker = notifychangeproperty(
-        dbus.Boolean, "CheckerRunning",
-        type_func=lambda checker: checker is not None)
+    checker = notifychangeproperty(dbus.Boolean, "CheckerRunning",
+                                   type_func = lambda checker:
+                                       checker is not None)
     last_checked_ok = notifychangeproperty(datetime_to_dbus,
                                            "LastCheckedOK")
-    last_checker_status = notifychangeproperty(dbus.Int16,
-                                               "LastCheckerStatus")
     last_approval_request = notifychangeproperty(
         datetime_to_dbus, "LastApprovalRequest")
     approved_by_default = notifychangeproperty(dbus.Boolean,
                                                "ApprovedByDefault")
-    approval_delay = notifychangeproperty(
-        dbus.UInt64, "ApprovalDelay",
-        type_func=lambda td: td.total_seconds() * 1000)
+    approval_delay = notifychangeproperty(dbus.UInt16,
+                                          "ApprovalDelay",
+                                          type_func =
+                                          _timedelta_to_milliseconds)
     approval_duration = notifychangeproperty(
-        dbus.UInt64, "ApprovalDuration",
-        type_func=lambda td: td.total_seconds() * 1000)
+        dbus.UInt16, "ApprovalDuration",
+        type_func = _timedelta_to_milliseconds)
     host = notifychangeproperty(dbus.String, "Host")
-    timeout = notifychangeproperty(
-        dbus.UInt64, "Timeout",
-        type_func=lambda td: td.total_seconds() * 1000)
+    timeout = notifychangeproperty(dbus.UInt16, "Timeout",
+                                   type_func =
+                                   _timedelta_to_milliseconds)
     extended_timeout = notifychangeproperty(
-        dbus.UInt64, "ExtendedTimeout",
-        type_func=lambda td: td.total_seconds() * 1000)
-    interval = notifychangeproperty(
-        dbus.UInt64, "Interval",
-        type_func=lambda td: td.total_seconds() * 1000)
+        dbus.UInt16, "ExtendedTimeout",
+        type_func = _timedelta_to_milliseconds)
+    interval = notifychangeproperty(dbus.UInt16,
+                                    "Interval",
+                                    type_func =
+                                    _timedelta_to_milliseconds)
     checker_command = notifychangeproperty(dbus.String, "Checker")
-    secret = notifychangeproperty(dbus.ByteArray, "Secret",
-                                  invalidate_only=True)
-
+    
     del notifychangeproperty
-
+    
     def __del__(self, *args, **kwargs):
         try:
             self.remove_from_connection()
@@ -1881,31 +959,32 @@
         if hasattr(DBusObjectWithProperties, "__del__"):
             DBusObjectWithProperties.__del__(self, *args, **kwargs)
         Client.__del__(self, *args, **kwargs)
-
-    def checker_callback(self, source, condition,
-                         connection, command, *args, **kwargs):
-        ret = Client.checker_callback(self, source, condition,
-                                      connection, command, *args,
-                                      **kwargs)
-        exitstatus = self.last_checker_status
-        if exitstatus >= 0:
+    
+    def checker_callback(self, pid, condition, command,
+                         *args, **kwargs):
+        self.checker_callback_tag = None
+        self.checker = None
+        if os.WIFEXITED(condition):
+            exitstatus = os.WEXITSTATUS(condition)
             # Emit D-Bus signal
             self.CheckerCompleted(dbus.Int16(exitstatus),
-                                  # This is specific to GNU libC
-                                  dbus.Int64(exitstatus << 8),
+                                  dbus.Int64(condition),
                                   dbus.String(command))
         else:
             # Emit D-Bus signal
             self.CheckerCompleted(dbus.Int16(-1),
-                                  dbus.Int64(
-                                      # This is specific to GNU libC
-                                      (exitstatus << 8)
-                                      | self.last_checker_signal),
+                                  dbus.Int64(condition),
                                   dbus.String(command))
-        return ret
-
+        
+        return Client.checker_callback(self, pid, condition, command,
+                                       *args, **kwargs)
+    
     def start_checker(self, *args, **kwargs):
-        old_checker_pid = getattr(self.checker, "pid", None)
+        old_checker = self.checker
+        if self.checker is not None:
+            old_checker_pid = self.checker.pid
+        else:
+            old_checker_pid = None
         r = Client.start_checker(self, *args, **kwargs)
         # Only if new checker process was started
         if (self.checker is not None
@@ -1913,42 +992,42 @@
             # Emit D-Bus signal
             self.CheckerStarted(self.current_checker_command)
         return r
-
+    
     def _reset_approved(self):
-        self.approved = None
+        self._approved = None
         return False
-
+    
     def approve(self, value=True):
-        self.approved = value
-        GLib.timeout_add(int(self.approval_duration.total_seconds()
-                             * 1000), self._reset_approved)
         self.send_changedstate()
-
-    #  D-Bus methods, signals & properties
-
-    #  Interfaces
-
-    #  Signals
-
+        self._approved = value
+        gobject.timeout_add(_timedelta_to_milliseconds
+                            (self.approval_duration),
+                            self._reset_approved)
+    
+    
+    ## D-Bus methods, signals & properties
+    _interface = "se.recompile.Mandos.Client"
+    
+    ## Signals
+    
     # CheckerCompleted - signal
     @dbus.service.signal(_interface, signature="nxs")
     def CheckerCompleted(self, exitcode, waitstatus, command):
         "D-Bus signal"
         pass
-
+    
     # CheckerStarted - signal
     @dbus.service.signal(_interface, signature="s")
     def CheckerStarted(self, command):
         "D-Bus signal"
         pass
-
+    
     # PropertyChanged - signal
-    @dbus_annotations({"org.freedesktop.DBus.Deprecated": "true"})
     @dbus.service.signal(_interface, signature="sv")
     def PropertyChanged(self, property, value):
         "D-Bus signal"
         pass
-
+    
     # GotSecret - signal
     @dbus.service.signal(_interface)
     def GotSecret(self):
@@ -1957,139 +1036,116 @@
         server to mandos-client
         """
         pass
-
+    
     # Rejected - signal
     @dbus.service.signal(_interface, signature="s")
     def Rejected(self, reason):
         "D-Bus signal"
         pass
-
+    
     # NeedApproval - signal
     @dbus.service.signal(_interface, signature="tb")
     def NeedApproval(self, timeout, default):
         "D-Bus signal"
         return self.need_approval()
-
-    #  Methods
-
+    
+    ## Methods
+    
     # Approve - method
     @dbus.service.method(_interface, in_signature="b")
     def Approve(self, value):
         self.approve(value)
-
+    
     # CheckedOK - method
     @dbus.service.method(_interface)
     def CheckedOK(self):
         self.checked_ok()
-
+    
     # Enable - method
-    @dbus_annotations({"org.freedesktop.DBus.Deprecated": "true"})
     @dbus.service.method(_interface)
     def Enable(self):
         "D-Bus method"
         self.enable()
-
+    
     # StartChecker - method
-    @dbus_annotations({"org.freedesktop.DBus.Deprecated": "true"})
     @dbus.service.method(_interface)
     def StartChecker(self):
         "D-Bus method"
         self.start_checker()
-
+    
     # Disable - method
-    @dbus_annotations({"org.freedesktop.DBus.Deprecated": "true"})
     @dbus.service.method(_interface)
     def Disable(self):
         "D-Bus method"
         self.disable()
-
+    
     # StopChecker - method
-    @dbus_annotations({"org.freedesktop.DBus.Deprecated": "true"})
     @dbus.service.method(_interface)
     def StopChecker(self):
         self.stop_checker()
-
-    #  Properties
-
+    
+    ## Properties
+    
     # ApprovalPending - property
     @dbus_service_property(_interface, signature="b", access="read")
     def ApprovalPending_dbus_property(self):
         return dbus.Boolean(bool(self.approvals_pending))
-
+    
     # ApprovedByDefault - property
-    @dbus_service_property(_interface,
-                           signature="b",
+    @dbus_service_property(_interface, signature="b",
                            access="readwrite")
     def ApprovedByDefault_dbus_property(self, value=None):
         if value is None:       # get
             return dbus.Boolean(self.approved_by_default)
         self.approved_by_default = bool(value)
-
+    
     # ApprovalDelay - property
-    @dbus_service_property(_interface,
-                           signature="t",
+    @dbus_service_property(_interface, signature="t",
                            access="readwrite")
     def ApprovalDelay_dbus_property(self, value=None):
         if value is None:       # get
-            return dbus.UInt64(self.approval_delay.total_seconds()
-                               * 1000)
+            return dbus.UInt64(self.approval_delay_milliseconds())
         self.approval_delay = datetime.timedelta(0, 0, 0, value)
-
+    
     # ApprovalDuration - property
-    @dbus_service_property(_interface,
-                           signature="t",
+    @dbus_service_property(_interface, signature="t",
                            access="readwrite")
     def ApprovalDuration_dbus_property(self, value=None):
         if value is None:       # get
-            return dbus.UInt64(self.approval_duration.total_seconds()
-                               * 1000)
+            return dbus.UInt64(_timedelta_to_milliseconds(
+                    self.approval_duration))
         self.approval_duration = datetime.timedelta(0, 0, 0, value)
-
+    
     # Name - property
-    @dbus_annotations(
-        {"org.freedesktop.DBus.Property.EmitsChangedSignal": "const"})
     @dbus_service_property(_interface, signature="s", access="read")
     def Name_dbus_property(self):
         return dbus.String(self.name)
-
-    # KeyID - property
-    @dbus_annotations(
-        {"org.freedesktop.DBus.Property.EmitsChangedSignal": "const"})
-    @dbus_service_property(_interface, signature="s", access="read")
-    def KeyID_dbus_property(self):
-        return dbus.String(self.key_id)
-
+    
     # Fingerprint - property
-    @dbus_annotations(
-        {"org.freedesktop.DBus.Property.EmitsChangedSignal": "const"})
     @dbus_service_property(_interface, signature="s", access="read")
     def Fingerprint_dbus_property(self):
         return dbus.String(self.fingerprint)
-
+    
     # Host - property
-    @dbus_service_property(_interface,
-                           signature="s",
+    @dbus_service_property(_interface, signature="s",
                            access="readwrite")
     def Host_dbus_property(self, value=None):
         if value is None:       # get
             return dbus.String(self.host)
-        self.host = str(value)
-
+        self.host = value
+    
     # Created - property
-    @dbus_annotations(
-        {"org.freedesktop.DBus.Property.EmitsChangedSignal": "const"})
     @dbus_service_property(_interface, signature="s", access="read")
     def Created_dbus_property(self):
-        return datetime_to_dbus(self.created)
-
+        return dbus.String(datetime_to_dbus(self.created))
+    
     # LastEnabled - property
     @dbus_service_property(_interface, signature="s", access="read")
     def LastEnabled_dbus_property(self):
         return datetime_to_dbus(self.last_enabled)
-
+    
     # Enabled - property
-    @dbus_service_property(_interface,
-                           signature="b",
+    @dbus_service_property(_interface, signature="b",
                            access="readwrite")
     def Enabled_dbus_property(self, value=None):
         if value is None:       # get
@@ -2098,96 +1154,88 @@
             self.enable()
         else:
             self.disable()
-
+    
     # LastCheckedOK - property
-    @dbus_service_property(_interface,
-                           signature="s",
+    @dbus_service_property(_interface, signature="s",
                            access="readwrite")
     def LastCheckedOK_dbus_property(self, value=None):
         if value is not None:
             self.checked_ok()
             return
         return datetime_to_dbus(self.last_checked_ok)
-
-    # LastCheckerStatus - property
-    @dbus_service_property(_interface, signature="n", access="read")
-    def LastCheckerStatus_dbus_property(self):
-        return dbus.Int16(self.last_checker_status)
-
+    
     # Expires - property
     @dbus_service_property(_interface, signature="s", access="read")
     def Expires_dbus_property(self):
         return datetime_to_dbus(self.expires)
-
+    
     # LastApprovalRequest - property
     @dbus_service_property(_interface, signature="s", access="read")
     def LastApprovalRequest_dbus_property(self):
         return datetime_to_dbus(self.last_approval_request)
-
+    
     # Timeout - property
-    @dbus_service_property(_interface,
-                           signature="t",
+    @dbus_service_property(_interface, signature="t",
                            access="readwrite")
     def Timeout_dbus_property(self, value=None):
         if value is None:       # get
-            return dbus.UInt64(self.timeout.total_seconds() * 1000)
-        old_timeout = self.timeout
+            return dbus.UInt64(self.timeout_milliseconds())
         self.timeout = datetime.timedelta(0, 0, 0, value)
-        # Reschedule disabling
-        if self.enabled:
-            now = datetime.datetime.utcnow()
-            self.expires += self.timeout - old_timeout
-            if self.expires <= now:
-                # The timeout has passed
-                self.disable()
-            else:
-                if (getattr(self, "disable_initiator_tag", None)
-                    is None):
-                    return
-                GLib.source_remove(self.disable_initiator_tag)
-                self.disable_initiator_tag = GLib.timeout_add(
-                    int((self.expires - now).total_seconds() * 1000),
-                    self.disable)
-
+        if getattr(self, "disable_initiator_tag", None) is None:
+            return
+        # Reschedule timeout
+        gobject.source_remove(self.disable_initiator_tag)
+        self.disable_initiator_tag = None
+        self.expires = None
+        time_to_die = (self.
+                       _timedelta_to_milliseconds((self
+                                                   .last_checked_ok
+                                                   + self.timeout)
+                                                  - datetime.datetime
+                                                  .utcnow()))
+        if time_to_die <= 0:
+            # The timeout has passed
+            self.disable()
+        else:
+            self.expires = (datetime.datetime.utcnow()
+                            + datetime.timedelta(milliseconds =
+                                                 time_to_die))
+            self.disable_initiator_tag = (gobject.timeout_add
+                                          (time_to_die, self.disable))
+    
     # ExtendedTimeout - property
-    @dbus_service_property(_interface,
-                           signature="t",
+    @dbus_service_property(_interface, signature="t",
                            access="readwrite")
     def ExtendedTimeout_dbus_property(self, value=None):
         if value is None:       # get
-            return dbus.UInt64(self.extended_timeout.total_seconds()
-                               * 1000)
+            return dbus.UInt64(self.extended_timeout_milliseconds())
         self.extended_timeout = datetime.timedelta(0, 0, 0, value)
-
+    
     # Interval - property
-    @dbus_service_property(_interface,
-                           signature="t",
+    @dbus_service_property(_interface, signature="t",
                            access="readwrite")
     def Interval_dbus_property(self, value=None):
         if value is None:       # get
-            return dbus.UInt64(self.interval.total_seconds() * 1000)
+            return dbus.UInt64(self.interval_milliseconds())
         self.interval = datetime.timedelta(0, 0, 0, value)
         if getattr(self, "checker_initiator_tag", None) is None:
             return
-        if self.enabled:
-            # Reschedule checker run
-            GLib.source_remove(self.checker_initiator_tag)
-            self.checker_initiator_tag = GLib.timeout_add(
-                value, self.start_checker)
-            self.start_checker()  # Start one now, too
-
+        # Reschedule checker run
+        gobject.source_remove(self.checker_initiator_tag)
+        self.checker_initiator_tag = (gobject.timeout_add
+                                      (value, self.start_checker))
+        self.start_checker()    # Start one now, too
+    
     # Checker - property
-    @dbus_service_property(_interface,
-                           signature="s",
+    @dbus_service_property(_interface, signature="s",
                            access="readwrite")
     def Checker_dbus_property(self, value=None):
         if value is None:       # get
             return dbus.String(self.checker_command)
-        self.checker_command = str(value)
-
+        self.checker_command = value
+    
     # CheckerRunning - property
-    @dbus_service_property(_interface,
-                           signature="b",
+    @dbus_service_property(_interface, signature="b",
                            access="readwrite")
     def CheckerRunning_dbus_property(self, value=None):
         if value is None:       # get
@@ -2196,158 +1244,147 @@
             self.start_checker()
         else:
             self.stop_checker()
-
+    
     # ObjectPath - property
-    @dbus_annotations(
-        {"org.freedesktop.DBus.Property.EmitsChangedSignal": "const",
-         "org.freedesktop.DBus.Deprecated": "true"})
     @dbus_service_property(_interface, signature="o", access="read")
     def ObjectPath_dbus_property(self):
-        return self.dbus_object_path  # is already a dbus.ObjectPath
-
+        return self.dbus_object_path # is already a dbus.ObjectPath
+    
     # Secret = property
-    @dbus_annotations(
-        {"org.freedesktop.DBus.Property.EmitsChangedSignal":
-         "invalidates"})
-    @dbus_service_property(_interface,
-                           signature="ay",
-                           access="write",
-                           byte_arrays=True)
+    @dbus_service_property(_interface, signature="ay",
+                           access="write", byte_arrays=True)
     def Secret_dbus_property(self, value):
-        self.secret = bytes(value)
-
+        self.secret = str(value)
+    
     del _interface
 
 
 class ProxyClient(object):
-    def __init__(self, child_pipe, key_id, fpr, address):
+    def __init__(self, child_pipe, fpr, address):
         self._pipe = child_pipe
-        self._pipe.send(('init', key_id, fpr, address))
+        self._pipe.send(('init', fpr, address))
         if not self._pipe.recv():
-            raise KeyError(key_id or fpr)
-
+            raise KeyError()
+    
     def __getattribute__(self, name):
-        if name == '_pipe':
+        if(name == '_pipe'):
             return super(ProxyClient, self).__getattribute__(name)
         self._pipe.send(('getattr', name))
         data = self._pipe.recv()
         if data[0] == 'data':
             return data[1]
         if data[0] == 'function':
-
             def func(*args, **kwargs):
                 self._pipe.send(('funcall', name, args, kwargs))
                 return self._pipe.recv()[1]
-
             return func
-
+    
     def __setattr__(self, name, value):
-        if name == '_pipe':
+        if(name == '_pipe'):
             return super(ProxyClient, self).__setattr__(name, value)
         self._pipe.send(('setattr', name, value))
 
+class ClientDBusTransitional(ClientDBus):
+    __metaclass__ = AlternateDBusNamesMetaclass
 
 class ClientHandler(socketserver.BaseRequestHandler, object):
     """A class to handle client connections.
-
+    
     Instantiated once for each connection to handle it.
     Note: This will run in its own forked process."""
-
+    
     def handle(self):
         with contextlib.closing(self.server.child_pipe) as child_pipe:
             logger.info("TCP connection from: %s",
-                        str(self.client_address))
+                        unicode(self.client_address))
             logger.debug("Pipe FD: %d",
                          self.server.child_pipe.fileno())
-
-            session = gnutls.ClientSession(self.request)
-
-            # priority = ':'.join(("NONE", "+VERS-TLS1.1",
-            #                       "+AES-256-CBC", "+SHA1",
-            #                       "+COMP-NULL", "+CTYPE-OPENPGP",
-            #                       "+DHE-DSS"))
+            
+            session = (gnutls.connection
+                       .ClientSession(self.request,
+                                      gnutls.connection
+                                      .X509Credentials()))
+            
+            # Note: gnutls.connection.X509Credentials is really a
+            # generic GnuTLS certificate credentials object so long as
+            # no X.509 keys are added to it.  Therefore, we can use it
+            # here despite using OpenPGP certificates.
+            
+            #priority = ':'.join(("NONE", "+VERS-TLS1.1",
+            #                      "+AES-256-CBC", "+SHA1",
+            #                      "+COMP-NULL", "+CTYPE-OPENPGP",
+            #                      "+DHE-DSS"))
             # Use a fallback default, since this MUST be set.
             priority = self.server.gnutls_priority
             if priority is None:
                 priority = "NORMAL"
-            gnutls.priority_set_direct(session._c_object,
-                                       priority.encode("utf-8"),
-                                       None)
-
+            (gnutls.library.functions
+             .gnutls_priority_set_direct(session._c_object,
+                                         priority, None))
+            
             # Start communication using the Mandos protocol
             # Get protocol number
             line = self.request.makefile().readline()
             logger.debug("Protocol version: %r", line)
             try:
                 if int(line.strip().split()[0]) > 1:
-                    raise RuntimeError(line)
+                    raise RuntimeError
             except (ValueError, IndexError, RuntimeError) as error:
                 logger.error("Unknown protocol version: %s", error)
                 return
-
+            
             # Start GnuTLS connection
             try:
                 session.handshake()
-            except gnutls.Error as error:
+            except gnutls.errors.GNUTLSError as error:
                 logger.warning("Handshake failed: %s", error)
                 # Do not run session.bye() here: the session is not
                 # established.  Just abandon the request.
                 return
             logger.debug("Handshake succeeded")
-
+            
             approval_required = False
             try:
-                if gnutls.has_rawpk:
-                    fpr = ""
-                    try:
-                        key_id = self.key_id(
-                            self.peer_certificate(session))
-                    except (TypeError, gnutls.Error) as error:
-                        logger.warning("Bad certificate: %s", error)
-                        return
-                    logger.debug("Key ID: %s", key_id)
-
-                else:
-                    key_id = ""
-                    try:
-                        fpr = self.fingerprint(
-                            self.peer_certificate(session))
-                    except (TypeError, gnutls.Error) as error:
-                        logger.warning("Bad certificate: %s", error)
-                        return
-                    logger.debug("Fingerprint: %s", fpr)
-
-                try:
-                    client = ProxyClient(child_pipe, key_id, fpr,
+                try:
+                    fpr = self.fingerprint(self.peer_certificate
+                                           (session))
+                except (TypeError,
+                        gnutls.errors.GNUTLSError) as error:
+                    logger.warning("Bad certificate: %s", error)
+                    return
+                logger.debug("Fingerprint: %s", fpr)
+                
+                try:
+                    client = ProxyClient(child_pipe, fpr,
                                          self.client_address)
                 except KeyError:
                     return
-
+                
                 if client.approval_delay:
                     delay = client.approval_delay
                     client.approvals_pending += 1
                     approval_required = True
-
+                
                 while True:
                     if not client.enabled:
                         logger.info("Client %s is disabled",
-                                    client.name)
+                                       client.name)
                         if self.server.use_dbus:
                             # Emit D-Bus signal
                             client.Rejected("Disabled")
                         return
-
-                    if client.approved or not client.approval_delay:
-                        # We are approved or approval is disabled
+                    
+                    if client._approved or not client.approval_delay:
+                        #We are approved or approval is disabled
                         break
-                    elif client.approved is None:
+                    elif client._approved is None:
                         logger.info("Client %s needs approval",
                                     client.name)
                         if self.server.use_dbus:
                             # Emit D-Bus signal
                             client.NeedApproval(
-                                client.approval_delay.total_seconds()
-                                * 1000, client.approved_by_default)
+                                client.approval_delay_milliseconds(),
+                                client.approved_by_default)
                     else:
                         logger.warning("Client %s was not approved",
                                        client.name)
@@ -2355,11 +1392,15 @@
                             # Emit D-Bus signal
                             client.Rejected("Denied")
                         return
-
-                    # wait until timeout or approved
+                    
+                    #wait until timeout or approved
+                    #x = float(client
+                    #          ._timedelta_to_milliseconds(delay))
                     time = datetime.datetime.now()
                     client.changedstate.acquire()
-                    client.changedstate.wait(delay.total_seconds())
+                    (client.changedstate.wait
+                     (float(client._timedelta_to_milliseconds(delay)
+                            / 1000)))
                     client.changedstate.release()
                     time2 = datetime.datetime.now()
                     if (time2 - time) >= delay:
@@ -2375,256 +1416,186 @@
                             break
                     else:
                         delay -= time2 - time
-
-                try:
-                    session.send(client.secret)
-                except gnutls.Error as error:
-                    logger.warning("gnutls send failed",
-                                   exc_info=error)
-                    return
-
+                
+                sent_size = 0
+                while sent_size < len(client.secret):
+                    try:
+                        sent = session.send(client.secret[sent_size:])
+                    except gnutls.errors.GNUTLSError as error:
+                        logger.warning("gnutls send failed")
+                        return
+                    logger.debug("Sent: %d, remaining: %d",
+                                 sent, len(client.secret)
+                                 - (sent_size + sent))
+                    sent_size += sent
+                
                 logger.info("Sending secret to %s", client.name)
-                # bump the timeout using extended_timeout
-                client.bump_timeout(client.extended_timeout)
+                # bump the timeout as if seen
+                client.checked_ok(client.extended_timeout)
                 if self.server.use_dbus:
                     # Emit D-Bus signal
                     client.GotSecret()
-
+            
             finally:
                 if approval_required:
                     client.approvals_pending -= 1
                 try:
                     session.bye()
-                except gnutls.Error as error:
-                    logger.warning("GnuTLS bye failed",
-                                   exc_info=error)
-
+                except gnutls.errors.GNUTLSError as error:
+                    logger.warning("GnuTLS bye failed")
+    
     @staticmethod
     def peer_certificate(session):
-        "Return the peer's certificate as a bytestring"
-        try:
-            cert_type = gnutls.certificate_type_get2(session._c_object,
-                                                     gnutls.CTYPE_PEERS)
-        except AttributeError:
-            cert_type = gnutls.certificate_type_get(session._c_object)
-        if gnutls.has_rawpk:
-            valid_cert_types = frozenset((gnutls.CRT_RAWPK,))
-        else:
-            valid_cert_types = frozenset((gnutls.CRT_OPENPGP,))
-        # If not a valid certificate type...
-        if cert_type not in valid_cert_types:
-            logger.info("Cert type %r not in %r", cert_type,
-                        valid_cert_types)
-            # ...return invalid data
-            return b""
+        "Return the peer's OpenPGP certificate as a bytestring"
+        # If not an OpenPGP certificate...
+        if (gnutls.library.functions
+            .gnutls_certificate_type_get(session._c_object)
+            != gnutls.library.constants.GNUTLS_CRT_OPENPGP):
+            # ...do the normal thing
+            return session.peer_certificate
         list_size = ctypes.c_uint(1)
-        cert_list = (gnutls.certificate_get_peers
+        cert_list = (gnutls.library.functions
+                     .gnutls_certificate_get_peers
                      (session._c_object, ctypes.byref(list_size)))
         if not bool(cert_list) and list_size.value != 0:
-            raise gnutls.Error("error getting peer certificate")
+            raise gnutls.errors.GNUTLSError("error getting peer"
+                                            " certificate")
         if list_size.value == 0:
             return None
         cert = cert_list[0]
         return ctypes.string_at(cert.data, cert.size)
-
-    @staticmethod
-    def key_id(certificate):
-        "Convert a certificate bytestring to a hexdigit key ID"
-        # New GnuTLS "datum" with the public key
-        datum = gnutls.datum_t(
-            ctypes.cast(ctypes.c_char_p(certificate),
-                        ctypes.POINTER(ctypes.c_ubyte)),
-            ctypes.c_uint(len(certificate)))
-        # XXX all these need to be created in the gnutls "module"
-        # New empty GnuTLS certificate
-        pubkey = gnutls.pubkey_t()
-        gnutls.pubkey_init(ctypes.byref(pubkey))
-        # Import the raw public key into the certificate
-        gnutls.pubkey_import(pubkey,
-                             ctypes.byref(datum),
-                             gnutls.X509_FMT_DER)
-        # New buffer for the key ID
-        buf = ctypes.create_string_buffer(32)
-        buf_len = ctypes.c_size_t(len(buf))
-        # Get the key ID from the raw public key into the buffer
-        gnutls.pubkey_get_key_id(pubkey,
-                                 gnutls.KEYID_USE_SHA256,
-                                 ctypes.cast(ctypes.byref(buf),
-                                             ctypes.POINTER(ctypes.c_ubyte)),
-                                 ctypes.byref(buf_len))
-        # Deinit the certificate
-        gnutls.pubkey_deinit(pubkey)
-
-        # Convert the buffer to a Python bytestring
-        key_id = ctypes.string_at(buf, buf_len.value)
-        # Convert the bytestring to hexadecimal notation
-        hex_key_id = binascii.hexlify(key_id).upper()
-        return hex_key_id
-
+    
     @staticmethod
     def fingerprint(openpgp):
         "Convert an OpenPGP bytestring to a hexdigit fingerprint"
         # New GnuTLS "datum" with the OpenPGP public key
-        datum = gnutls.datum_t(
-            ctypes.cast(ctypes.c_char_p(openpgp),
-                        ctypes.POINTER(ctypes.c_ubyte)),
-            ctypes.c_uint(len(openpgp)))
+        datum = (gnutls.library.types
+                 .gnutls_datum_t(ctypes.cast(ctypes.c_char_p(openpgp),
+                                             ctypes.POINTER
+                                             (ctypes.c_ubyte)),
+                                 ctypes.c_uint(len(openpgp))))
         # New empty GnuTLS certificate
-        crt = gnutls.openpgp_crt_t()
-        gnutls.openpgp_crt_init(ctypes.byref(crt))
+        crt = gnutls.library.types.gnutls_openpgp_crt_t()
+        (gnutls.library.functions
+         .gnutls_openpgp_crt_init(ctypes.byref(crt)))
         # Import the OpenPGP public key into the certificate
-        gnutls.openpgp_crt_import(crt, ctypes.byref(datum),
-                                  gnutls.OPENPGP_FMT_RAW)
+        (gnutls.library.functions
+         .gnutls_openpgp_crt_import(crt, ctypes.byref(datum),
+                                    gnutls.library.constants
+                                    .GNUTLS_OPENPGP_FMT_RAW))
         # Verify the self signature in the key
         crtverify = ctypes.c_uint()
-        gnutls.openpgp_crt_verify_self(crt, 0,
-                                       ctypes.byref(crtverify))
+        (gnutls.library.functions
+         .gnutls_openpgp_crt_verify_self(crt, 0,
+                                         ctypes.byref(crtverify)))
         if crtverify.value != 0:
-            gnutls.openpgp_crt_deinit(crt)
-            raise gnutls.CertificateSecurityError(code
-                                                  =crtverify.value)
+            gnutls.library.functions.gnutls_openpgp_crt_deinit(crt)
+            raise (gnutls.errors.CertificateSecurityError
+                   ("Verify failed"))
         # New buffer for the fingerprint
         buf = ctypes.create_string_buffer(20)
         buf_len = ctypes.c_size_t()
         # Get the fingerprint from the certificate into the buffer
-        gnutls.openpgp_crt_get_fingerprint(crt, ctypes.byref(buf),
-                                           ctypes.byref(buf_len))
+        (gnutls.library.functions
+         .gnutls_openpgp_crt_get_fingerprint(crt, ctypes.byref(buf),
+                                             ctypes.byref(buf_len)))
         # Deinit the certificate
-        gnutls.openpgp_crt_deinit(crt)
+        gnutls.library.functions.gnutls_openpgp_crt_deinit(crt)
         # Convert the buffer to a Python bytestring
         fpr = ctypes.string_at(buf, buf_len.value)
         # Convert the bytestring to hexadecimal notation
-        hex_fpr = binascii.hexlify(fpr).upper()
+        hex_fpr = ''.join("%02X" % ord(char) for char in fpr)
         return hex_fpr
 
 
 class MultiprocessingMixIn(object):
     """Like socketserver.ThreadingMixIn, but with multiprocessing"""
-
     def sub_process_main(self, request, address):
         try:
             self.finish_request(request, address)
-        except Exception:
+        except:
             self.handle_error(request, address)
         self.close_request(request)
-
+            
     def process_request(self, request, address):
         """Start a new process to process the request."""
-        proc = multiprocessing.Process(target=self.sub_process_main,
-                                       args=(request, address))
-        proc.start()
-        return proc
+        multiprocessing.Process(target = self.sub_process_main,
+                                args = (request, address)).start()
 
 
 class MultiprocessingMixInWithPipe(MultiprocessingMixIn, object):
     """ adds a pipe to the MixIn """
-
     def process_request(self, request, client_address):
         """Overrides and wraps the original process_request().
-
+        
         This function creates a new pipe in self.pipe
         """
         parent_pipe, self.child_pipe = multiprocessing.Pipe()
-
-        proc = MultiprocessingMixIn.process_request(self, request,
-                                                    client_address)
+        
+        super(MultiprocessingMixInWithPipe,
+              self).process_request(request, client_address)
         self.child_pipe.close()
-        self.add_pipe(parent_pipe, proc)
-
-    def add_pipe(self, parent_pipe, proc):
+        self.add_pipe(parent_pipe)
+    
+    def add_pipe(self, parent_pipe):
         """Dummy function; override as necessary"""
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
 class IPv6_TCPServer(MultiprocessingMixInWithPipe,
                      socketserver.TCPServer, object):
     """IPv6-capable TCP server.  Accepts 'None' as address and/or port
-
+    
     Attributes:
         enabled:        Boolean; whether this server is activated yet
         interface:      None or a network interface name (string)
         use_ipv6:       Boolean; to use IPv6 or not
     """
-
     def __init__(self, server_address, RequestHandlerClass,
-                 interface=None,
-                 use_ipv6=True,
-                 socketfd=None):
-        """If socketfd is set, use that file descriptor instead of
-        creating a new one with socket.socket().
-        """
+                 interface=None, use_ipv6=True):
         self.interface = interface
         if use_ipv6:
             self.address_family = socket.AF_INET6
-        if socketfd is not None:
-            # Save the file descriptor
-            self.socketfd = socketfd
-            # Save the original socket.socket() function
-            self.socket_socket = socket.socket
-
-            # To implement --socket, we monkey patch socket.socket.
-            #
-            # (When socketserver.TCPServer is a new-style class, we
-            # could make self.socket into a property instead of monkey
-            # patching socket.socket.)
-            #
-            # Create a one-time-only replacement for socket.socket()
-            @functools.wraps(socket.socket)
-            def socket_wrapper(*args, **kwargs):
-                # Restore original function so subsequent calls are
-                # not affected.
-                socket.socket = self.socket_socket
-                del self.socket_socket
-                # This time only, return a new socket object from the
-                # saved file descriptor.
-                return socket.fromfd(self.socketfd, *args, **kwargs)
-            # Replace socket.socket() function with wrapper
-            socket.socket = socket_wrapper
-        # The socketserver.TCPServer.__init__ will call
-        # socket.socket(), which might be our replacement,
-        # socket_wrapper(), if socketfd was set.
         socketserver.TCPServer.__init__(self, server_address,
                                         RequestHandlerClass)
-
     def server_bind(self):
         """This overrides the normal server_bind() function
         to bind to an interface if one was specified, and also NOT to
         bind to an address or port if they were not specified."""
-        global SO_BINDTODEVICE
         if self.interface is not None:
             if SO_BINDTODEVICE is None:
-                # Fall back to a hard-coded value which seems to be
-                # common enough.
-                logger.warning("SO_BINDTODEVICE not found, trying 25")
-                SO_BINDTODEVICE = 25
-            try:
-                self.socket.setsockopt(
-                    socket.SOL_SOCKET, SO_BINDTODEVICE,
-                    (self.interface + "\0").encode("utf-8"))
-            except socket.error as error:
-                if error.errno == errno.EPERM:
-                    logger.error("No permission to bind to"
-                                 " interface %s", self.interface)
-                elif error.errno == errno.ENOPROTOOPT:
-                    logger.error("SO_BINDTODEVICE not available;"
-                                 " cannot bind to interface %s",
-                                 self.interface)
-                elif error.errno == errno.ENODEV:
-                    logger.error("Interface %s does not exist,"
-                                 " cannot bind", self.interface)
-                else:
-                    raise
+                logger.error("SO_BINDTODEVICE does not exist;"
+                             " cannot bind to interface %s",
+                             self.interface)
+            else:
+                try:
+                    self.socket.setsockopt(socket.SOL_SOCKET,
+                                           SO_BINDTODEVICE,
+                                           str(self.interface
+                                               + '\0'))
+                except socket.error as error:
+                    if error[0] == errno.EPERM:
+                        logger.error("No permission to"
+                                     " bind to interface %s",
+                                     self.interface)
+                    elif error[0] == errno.ENOPROTOOPT:
+                        logger.error("SO_BINDTODEVICE not available;"
+                                     " cannot bind to interface %s",
+                                     self.interface)
+                    else:
+                        raise
         # Only bind(2) the socket if we really need to.
         if self.server_address[0] or self.server_address[1]:
             if not self.server_address[0]:
                 if self.address_family == socket.AF_INET6:
-                    any_address = "::"  # in6addr_any
+                    any_address = "::" # in6addr_any
                 else:
-                    any_address = "0.0.0.0"  # INADDR_ANY
+                    any_address = socket.INADDR_ANY
                 self.server_address = (any_address,
                                        self.server_address[1])
             elif not self.server_address[1]:
-                self.server_address = (self.server_address[0], 0)
+                self.server_address = (self.server_address[0],
+                                       0)
 #                 if self.interface:
 #                     self.server_address = (self.server_address[0],
 #                                            0, # port
@@ -2636,95 +1607,89 @@
 
 class MandosServer(IPv6_TCPServer):
     """Mandos server.
-
+    
     Attributes:
         clients:        set of Client objects
         gnutls_priority GnuTLS priority string
         use_dbus:       Boolean; to emit D-Bus signals or not
-
-    Assumes a GLib.MainLoop event loop.
+    
+    Assumes a gobject.MainLoop event loop.
     """
-
     def __init__(self, server_address, RequestHandlerClass,
-                 interface=None,
-                 use_ipv6=True,
-                 clients=None,
-                 gnutls_priority=None,
-                 use_dbus=True,
-                 socketfd=None):
+                 interface=None, use_ipv6=True, clients=None,
+                 gnutls_priority=None, use_dbus=True):
         self.enabled = False
         self.clients = clients
         if self.clients is None:
-            self.clients = {}
+            self.clients = set()
         self.use_dbus = use_dbus
         self.gnutls_priority = gnutls_priority
         IPv6_TCPServer.__init__(self, server_address,
                                 RequestHandlerClass,
-                                interface=interface,
-                                use_ipv6=use_ipv6,
-                                socketfd=socketfd)
-
+                                interface = interface,
+                                use_ipv6 = use_ipv6)
     def server_activate(self):
         if self.enabled:
             return socketserver.TCPServer.server_activate(self)
-
     def enable(self):
         self.enabled = True
-
-    def add_pipe(self, parent_pipe, proc):
+    def add_pipe(self, parent_pipe):
         # Call "handle_ipc" for both data and EOF events
-        GLib.io_add_watch(
-            parent_pipe.fileno(),
-            GLib.IO_IN | GLib.IO_HUP,
-            functools.partial(self.handle_ipc,
-                              parent_pipe=parent_pipe,
-                              proc=proc))
-
-    def handle_ipc(self, source, condition,
-                   parent_pipe=None,
-                   proc=None,
+        gobject.io_add_watch(parent_pipe.fileno(),
+                             gobject.IO_IN | gobject.IO_HUP,
+                             functools.partial(self.handle_ipc,
+                                               parent_pipe =
+                                               parent_pipe))
+        
+    def handle_ipc(self, source, condition, parent_pipe=None,
                    client_object=None):
-        # error, or the other end of multiprocessing.Pipe has closed
-        if condition & (GLib.IO_ERR | GLib.IO_HUP):
-            # Wait for other process to exit
-            proc.join()
+        condition_names = {
+            gobject.IO_IN: "IN",   # There is data to read.
+            gobject.IO_OUT: "OUT", # Data can be written (without
+                                    # blocking).
+            gobject.IO_PRI: "PRI", # There is urgent data to read.
+            gobject.IO_ERR: "ERR", # Error condition.
+            gobject.IO_HUP: "HUP"  # Hung up (the connection has been
+                                    # broken, usually for pipes and
+                                    # sockets).
+            }
+        conditions_string = ' | '.join(name
+                                       for cond, name in
+                                       condition_names.iteritems()
+                                       if cond & condition)
+        # error or the other end of multiprocessing.Pipe has closed
+        if condition & (gobject.IO_ERR | condition & gobject.IO_HUP):
             return False
-
+        
         # Read a request from the child
         request = parent_pipe.recv()
         command = request[0]
-
+        
         if command == 'init':
-            key_id = request[1].decode("ascii")
-            fpr = request[2].decode("ascii")
-            address = request[3]
-
-            for c in self.clients.values():
-                if key_id == "E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855":
-                    continue
-                if key_id and c.key_id == key_id:
-                    client = c
-                    break
-                if fpr and c.fingerprint == fpr:
+            fpr = request[1]
+            address = request[2]
+            
+            for c in self.clients:
+                if c.fingerprint == fpr:
                     client = c
                     break
             else:
-                logger.info("Client not found for key ID: %s, address"
-                            ": %s", key_id or fpr, address)
+                logger.info("Client not found for fingerprint: %s, ad"
+                            "dress: %s", fpr, address)
                 if self.use_dbus:
                     # Emit D-Bus signal
-                    mandos_dbus_service.ClientNotFound(key_id or fpr,
+                    mandos_dbus_service.ClientNotFound(fpr,
                                                        address[0])
                 parent_pipe.send(False)
                 return False
-
-            GLib.io_add_watch(
-                parent_pipe.fileno(),
-                GLib.IO_IN | GLib.IO_HUP,
-                functools.partial(self.handle_ipc,
-                                  parent_pipe=parent_pipe,
-                                  proc=proc,
-                                  client_object=client))
+            
+            gobject.io_add_watch(parent_pipe.fileno(),
+                                 gobject.IO_IN | gobject.IO_HUP,
+                                 functools.partial(self.handle_ipc,
+                                                   parent_pipe =
+                                                   parent_pipe,
+                                                   client_object =
+                                                   client))
             parent_pipe.send(True)
             # remove the old hook in favor of the new above hook on
             # same fileno
@@ -2733,130 +1698,30 @@
             funcname = request[1]
             args = request[2]
             kwargs = request[3]
-
+            
             parent_pipe.send(('data', getattr(client_object,
                                               funcname)(*args,
-                                                        **kwargs)))
-
+                                                         **kwargs)))
+        
         if command == 'getattr':
             attrname = request[1]
-            if isinstance(client_object.__getattribute__(attrname),
-                          collections.Callable):
-                parent_pipe.send(('function', ))
+            if callable(client_object.__getattribute__(attrname)):
+                parent_pipe.send(('function',))
             else:
-                parent_pipe.send((
-                    'data', client_object.__getattribute__(attrname)))
-
+                parent_pipe.send(('data', client_object
+                                  .__getattribute__(attrname)))
+        
         if command == 'setattr':
             attrname = request[1]
             value = request[2]
             setattr(client_object, attrname, value)
-
+        
         return True
 
 
-def rfc3339_duration_to_delta(duration):
-    """Parse an RFC 3339 "duration" and return a datetime.timedelta
-
-    >>> rfc3339_duration_to_delta("P7D")
-    datetime.timedelta(7)
-    >>> rfc3339_duration_to_delta("PT60S")
-    datetime.timedelta(0, 60)
-    >>> rfc3339_duration_to_delta("PT60M")
-    datetime.timedelta(0, 3600)
-    >>> rfc3339_duration_to_delta("PT24H")
-    datetime.timedelta(1)
-    >>> rfc3339_duration_to_delta("P1W")
-    datetime.timedelta(7)
-    >>> rfc3339_duration_to_delta("PT5M30S")
-    datetime.timedelta(0, 330)
-    >>> rfc3339_duration_to_delta("P1DT3M20S")
-    datetime.timedelta(1, 200)
-    """
-
-    # Parsing an RFC 3339 duration with regular expressions is not
-    # possible - there would have to be multiple places for the same
-    # values, like seconds.  The current code, while more esoteric, is
-    # cleaner without depending on a parsing library.  If Python had a
-    # built-in library for parsing we would use it, but we'd like to
-    # avoid excessive use of external libraries.
-
-    # New type for defining tokens, syntax, and semantics all-in-one
-    Token = collections.namedtuple("Token", (
-        "regexp",  # To match token; if "value" is not None, must have
-                   # a "group" containing digits
-        "value",   # datetime.timedelta or None
-        "followers"))           # Tokens valid after this token
-    # RFC 3339 "duration" tokens, syntax, and semantics; taken from
-    # the "duration" ABNF definition in RFC 3339, Appendix A.
-    token_end = Token(re.compile(r"$"), None, frozenset())
-    token_second = Token(re.compile(r"(\d+)S"),
-                         datetime.timedelta(seconds=1),
-                         frozenset((token_end, )))
-    token_minute = Token(re.compile(r"(\d+)M"),
-                         datetime.timedelta(minutes=1),
-                         frozenset((token_second, token_end)))
-    token_hour = Token(re.compile(r"(\d+)H"),
-                       datetime.timedelta(hours=1),
-                       frozenset((token_minute, token_end)))
-    token_time = Token(re.compile(r"T"),
-                       None,
-                       frozenset((token_hour, token_minute,
-                                  token_second)))
-    token_day = Token(re.compile(r"(\d+)D"),
-                      datetime.timedelta(days=1),
-                      frozenset((token_time, token_end)))
-    token_month = Token(re.compile(r"(\d+)M"),
-                        datetime.timedelta(weeks=4),
-                        frozenset((token_day, token_end)))
-    token_year = Token(re.compile(r"(\d+)Y"),
-                       datetime.timedelta(weeks=52),
-                       frozenset((token_month, token_end)))
-    token_week = Token(re.compile(r"(\d+)W"),
-                       datetime.timedelta(weeks=1),
-                       frozenset((token_end, )))
-    token_duration = Token(re.compile(r"P"), None,
-                           frozenset((token_year, token_month,
-                                      token_day, token_time,
-                                      token_week)))
-    # Define starting values:
-    # Value so far
-    value = datetime.timedelta()
-    found_token = None
-    # Following valid tokens
-    followers = frozenset((token_duration, ))
-    # String left to parse
-    s = duration
-    # Loop until end token is found
-    while found_token is not token_end:
-        # Search for any currently valid tokens
-        for token in followers:
-            match = token.regexp.match(s)
-            if match is not None:
-                # Token found
-                if token.value is not None:
-                    # Value found, parse digits
-                    factor = int(match.group(1), 10)
-                    # Add to value so far
-                    value += factor * token.value
-                # Strip token from string
-                s = token.regexp.sub("", s, 1)
-                # Go to found token
-                found_token = token
-                # Set valid next tokens
-                followers = found_token.followers
-                break
-        else:
-            # No currently valid tokens were found
-            raise ValueError("Invalid RFC 3339 duration: {!r}"
-                             .format(duration))
-    # End token found
-    return value
-
-
 def string_to_delta(interval):
     """Parse a string and return a datetime.timedelta
-
+    
     >>> string_to_delta('7d')
     datetime.timedelta(7)
     >>> string_to_delta('60s')
@@ -2870,16 +1735,10 @@
     >>> string_to_delta('5m 30s')
     datetime.timedelta(0, 330)
     """
-
-    try:
-        return rfc3339_duration_to_delta(interval)
-    except ValueError:
-        pass
-
     timevalue = datetime.timedelta(0)
     for s in interval.split():
         try:
-            suffix = s[-1]
+            suffix = unicode(s[-1])
             value = int(s[:-1])
             if suffix == "d":
                 delta = datetime.timedelta(value)
@@ -2892,16 +1751,40 @@
             elif suffix == "w":
                 delta = datetime.timedelta(0, 0, 0, 0, 0, 0, value)
             else:
-                raise ValueError("Unknown suffix {!r}".format(suffix))
-        except IndexError as e:
+                raise ValueError("Unknown suffix %r" % suffix)
+        except (ValueError, IndexError) as e:
             raise ValueError(*(e.args))
         timevalue += delta
     return timevalue
 
 
-def daemon(nochdir=False, noclose=False):
+def if_nametoindex(interface):
+    """Call the C function if_nametoindex(), or equivalent
+    
+    Note: This function cannot accept a unicode string."""
+    global if_nametoindex
+    try:
+        if_nametoindex = (ctypes.cdll.LoadLibrary
+                          (ctypes.util.find_library("c"))
+                          .if_nametoindex)
+    except (OSError, AttributeError):
+        logger.warning("Doing if_nametoindex the hard way")
+        def if_nametoindex(interface):
+            "Get an interface index the hard way, i.e. using fcntl()"
+            SIOCGIFINDEX = 0x8933  # From /usr/include/linux/sockios.h
+            with contextlib.closing(socket.socket()) as s:
+                ifreq = fcntl.ioctl(s, SIOCGIFINDEX,
+                                    struct.pack(str("16s16x"),
+                                                interface))
+            interface_index = struct.unpack(str("I"),
+                                            ifreq[16:20])[0]
+            return interface_index
+    return if_nametoindex(interface)
+
+
+def daemon(nochdir = False, noclose = False):
     """See daemon(3).  Standard BSD Unix function.
-
+    
     This should really exist as os.daemon, but it doesn't (yet)."""
     if os.fork():
         sys.exit()
@@ -2912,11 +1795,11 @@
         sys.exit()
     if not noclose:
         # Close all standard open file descriptors
-        null = os.open(os.devnull, os.O_NOCTTY | os.O_RDWR)
+        null = os.open(os.path.devnull, os.O_NOCTTY | os.O_RDWR)
         if not stat.S_ISCHR(os.fstat(null).st_mode):
             raise OSError(errno.ENODEV,
-                          "{} not a character device"
-                          .format(os.devnull))
+                          "%s not a character device"
+                          % os.path.devnull)
         os.dup2(null, sys.stdin.fileno())
         os.dup2(null, sys.stdout.fileno())
         os.dup2(null, sys.stderr.fileno())
@@ -2925,13 +1808,13 @@
 
 
 def main():
-
+    
     ##################################################################
     # Parsing of options, both command line and config file
-
+    
     parser = argparse.ArgumentParser()
     parser.add_argument("-v", "--version", action="version",
-                        version="%(prog)s {}".format(version),
+                        version = "%%(prog)s %s" % version,
                         help="show version number and exit")
     parser.add_argument("-i", "--interface", metavar="IF",
                         help="Bind to interface IF")
@@ -2943,7 +1826,7 @@
                         help="Run self-test")
     parser.add_argument("--debug", action="store_true",
                         help="Debug mode; run in foreground and log"
-                        " to terminal", default=None)
+                        " to terminal")
     parser.add_argument("--debuglevel", metavar="LEVEL",
                         help="Debug level for stdout output")
     parser.add_argument("--priority", help="GnuTLS"
@@ -2956,660 +1839,351 @@
                         " files")
     parser.add_argument("--no-dbus", action="store_false",
                         dest="use_dbus", help="Do not provide D-Bus"
-                        " system bus interface", default=None)
+                        " system bus interface")
     parser.add_argument("--no-ipv6", action="store_false",
-                        dest="use_ipv6", help="Do not use IPv6",
-                        default=None)
-    parser.add_argument("--no-restore", action="store_false",
-                        dest="restore", help="Do not restore stored"
-                        " state", default=None)
-    parser.add_argument("--socket", type=int,
-                        help="Specify a file descriptor to a network"
-                        " socket to use instead of creating one")
-    parser.add_argument("--statedir", metavar="DIR",
-                        help="Directory to save/restore state in")
-    parser.add_argument("--foreground", action="store_true",
-                        help="Run in foreground", default=None)
-    parser.add_argument("--no-zeroconf", action="store_false",
-                        dest="zeroconf", help="Do not use Zeroconf",
-                        default=None)
-
+                        dest="use_ipv6", help="Do not use IPv6")
     options = parser.parse_args()
-
+    
     if options.check:
         import doctest
-        fail_count, test_count = doctest.testmod()
-        sys.exit(os.EX_OK if fail_count == 0 else 1)
-
+        doctest.testmod()
+        sys.exit()
+    
     # Default values for config file for server-global settings
-    if gnutls.has_rawpk:
-        priority = ("SECURE128:!CTYPE-X.509:+CTYPE-RAWPK:!RSA"
-                    ":!VERS-ALL:+VERS-TLS1.3:%PROFILE_ULTRA")
-    else:
-        priority = ("SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:!RSA"
-                    ":+SIGN-DSA-SHA256")
-    server_defaults = {"interface": "",
-                       "address": "",
-                       "port": "",
-                       "debug": "False",
-                       "priority": priority,
-                       "servicename": "Mandos",
-                       "use_dbus": "True",
-                       "use_ipv6": "True",
-                       "debuglevel": "",
-                       "restore": "True",
-                       "socket": "",
-                       "statedir": "/var/lib/mandos",
-                       "foreground": "False",
-                       "zeroconf": "True",
-                       }
-    del priority
-
+    server_defaults = { "interface": "",
+                        "address": "",
+                        "port": "",
+                        "debug": "False",
+                        "priority":
+                        "SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP",
+                        "servicename": "Mandos",
+                        "use_dbus": "True",
+                        "use_ipv6": "True",
+                        "debuglevel": "",
+                        }
+    
     # Parse config file for server-global settings
     server_config = configparser.SafeConfigParser(server_defaults)
     del server_defaults
-    server_config.read(os.path.join(options.configdir, "mandos.conf"))
+    server_config.read(os.path.join(options.configdir,
+                                    "mandos.conf"))
     # Convert the SafeConfigParser object to a dict
     server_settings = server_config.defaults()
     # Use the appropriate methods on the non-string config options
-    for option in ("debug", "use_dbus", "use_ipv6", "restore",
-                   "foreground", "zeroconf"):
+    for option in ("debug", "use_dbus", "use_ipv6"):
         server_settings[option] = server_config.getboolean("DEFAULT",
                                                            option)
     if server_settings["port"]:
         server_settings["port"] = server_config.getint("DEFAULT",
                                                        "port")
-    if server_settings["socket"]:
-        server_settings["socket"] = server_config.getint("DEFAULT",
-                                                         "socket")
-        # Later, stdin will, and stdout and stderr might, be dup'ed
-        # over with an opened os.devnull.  But we don't want this to
-        # happen with a supplied network socket.
-        if 0 <= server_settings["socket"] <= 2:
-            server_settings["socket"] = os.dup(server_settings
-                                               ["socket"])
     del server_config
-
+    
     # Override the settings from the config file with command line
     # options, if set.
     for option in ("interface", "address", "port", "debug",
-                   "priority", "servicename", "configdir", "use_dbus",
-                   "use_ipv6", "debuglevel", "restore", "statedir",
-                   "socket", "foreground", "zeroconf"):
+                   "priority", "servicename", "configdir",
+                   "use_dbus", "use_ipv6", "debuglevel"):
         value = getattr(options, option)
         if value is not None:
             server_settings[option] = value
     del options
     # Force all strings to be unicode
     for option in server_settings.keys():
-        if isinstance(server_settings[option], bytes):
-            server_settings[option] = (server_settings[option]
-                                       .decode("utf-8"))
-    # Force all boolean options to be boolean
-    for option in ("debug", "use_dbus", "use_ipv6", "restore",
-                   "foreground", "zeroconf"):
-        server_settings[option] = bool(server_settings[option])
-    # Debug implies foreground
-    if server_settings["debug"]:
-        server_settings["foreground"] = True
+        if type(server_settings[option]) is str:
+            server_settings[option] = unicode(server_settings[option])
     # Now we have our good server settings in "server_settings"
-
+    
     ##################################################################
-
-    if (not server_settings["zeroconf"]
-        and not (server_settings["port"]
-                 or server_settings["socket"] != "")):
-        parser.error("Needs port or socket to work without Zeroconf")
-
+    
     # For convenience
     debug = server_settings["debug"]
     debuglevel = server_settings["debuglevel"]
     use_dbus = server_settings["use_dbus"]
     use_ipv6 = server_settings["use_ipv6"]
-    stored_state_path = os.path.join(server_settings["statedir"],
-                                     stored_state_file)
-    foreground = server_settings["foreground"]
-    zeroconf = server_settings["zeroconf"]
-
-    if debug:
-        initlogger(debug, logging.DEBUG)
-    else:
-        if not debuglevel:
-            initlogger(debug)
-        else:
-            level = getattr(logging, debuglevel.upper())
-            initlogger(debug, level)
-
+    
     if server_settings["servicename"] != "Mandos":
-        syslogger.setFormatter(
-            logging.Formatter('Mandos ({}) [%(process)d]:'
-                              ' %(levelname)s: %(message)s'.format(
-                                  server_settings["servicename"])))
-
+        syslogger.setFormatter(logging.Formatter
+                               ('Mandos (%s) [%%(process)d]:'
+                                ' %%(levelname)s: %%(message)s'
+                                % server_settings["servicename"]))
+    
     # Parse config file with clients
-    client_config = configparser.SafeConfigParser(Client
-                                                  .client_defaults)
+    client_defaults = { "timeout": "5m",
+                        "extended_timeout": "15m",
+                        "interval": "2m",
+                        "checker": "fping -q -- %%(host)s",
+                        "host": "",
+                        "approval_delay": "0s",
+                        "approval_duration": "1s",
+                        }
+    client_config = configparser.SafeConfigParser(client_defaults)
     client_config.read(os.path.join(server_settings["configdir"],
                                     "clients.conf"))
-
+    
     global mandos_dbus_service
     mandos_dbus_service = None
-
-    socketfd = None
-    if server_settings["socket"] != "":
-        socketfd = server_settings["socket"]
-    tcp_server = MandosServer(
-        (server_settings["address"], server_settings["port"]),
-        ClientHandler,
-        interface=(server_settings["interface"] or None),
-        use_ipv6=use_ipv6,
-        gnutls_priority=server_settings["priority"],
-        use_dbus=use_dbus,
-        socketfd=socketfd)
-    if not foreground:
-        pidfilename = "/run/mandos.pid"
-        if not os.path.isdir("/run/."):
-            pidfilename = "/var/run/mandos.pid"
-        pidfile = None
-        try:
-            pidfile = codecs.open(pidfilename, "w", encoding="utf-8")
-        except IOError as e:
-            logger.error("Could not open file %r", pidfilename,
-                         exc_info=e)
-
-    for name, group in (("_mandos", "_mandos"),
-                        ("mandos", "mandos"),
-                        ("nobody", "nogroup")):
-        try:
-            uid = pwd.getpwnam(name).pw_uid
-            gid = pwd.getpwnam(group).pw_gid
-            break
+    
+    tcp_server = MandosServer((server_settings["address"],
+                               server_settings["port"]),
+                              ClientHandler,
+                              interface=(server_settings["interface"]
+                                         or None),
+                              use_ipv6=use_ipv6,
+                              gnutls_priority=
+                              server_settings["priority"],
+                              use_dbus=use_dbus)
+    if not debug:
+        pidfilename = "/var/run/mandos.pid"
+        try:
+            pidfile = open(pidfilename, "w")
+        except IOError:
+            logger.error("Could not open file %r", pidfilename)
+    
+    try:
+        uid = pwd.getpwnam("_mandos").pw_uid
+        gid = pwd.getpwnam("_mandos").pw_gid
+    except KeyError:
+        try:
+            uid = pwd.getpwnam("mandos").pw_uid
+            gid = pwd.getpwnam("mandos").pw_gid
         except KeyError:
-            continue
-    else:
-        uid = 65534
-        gid = 65534
+            try:
+                uid = pwd.getpwnam("nobody").pw_uid
+                gid = pwd.getpwnam("nobody").pw_gid
+            except KeyError:
+                uid = 65534
+                gid = 65534
     try:
         os.setgid(gid)
         os.setuid(uid)
-        if debug:
-            logger.debug("Did setuid/setgid to {}:{}".format(uid,
-                                                             gid))
     except OSError as error:
-        logger.warning("Failed to setuid/setgid to {}:{}: {}"
-                       .format(uid, gid, os.strerror(error.errno)))
-        if error.errno != errno.EPERM:
-            raise
-
+        if error[0] != errno.EPERM:
+            raise error
+    
+    if not debug and not debuglevel:
+        syslogger.setLevel(logging.WARNING)
+        console.setLevel(logging.WARNING)
+    if debuglevel:
+        level = getattr(logging, debuglevel.upper())
+        syslogger.setLevel(level)
+        console.setLevel(level)
+    
     if debug:
         # Enable all possible GnuTLS debugging
-
+        
         # "Use a log level over 10 to enable all debugging options."
         # - GnuTLS manual
-        gnutls.global_set_log_level(11)
-
-        @gnutls.log_func
+        gnutls.library.functions.gnutls_global_set_log_level(11)
+        
+        @gnutls.library.types.gnutls_log_func
         def debug_gnutls(level, string):
             logger.debug("GnuTLS: %s", string[:-1])
-
-        gnutls.global_set_log_function(debug_gnutls)
-
+        
+        (gnutls.library.functions
+         .gnutls_global_set_log_function(debug_gnutls))
+        
         # Redirect stdin so all checkers get /dev/null
-        null = os.open(os.devnull, os.O_NOCTTY | os.O_RDWR)
+        null = os.open(os.path.devnull, os.O_NOCTTY | os.O_RDWR)
         os.dup2(null, sys.stdin.fileno())
         if null > 2:
             os.close(null)
-
+    else:
+        # No console logging
+        logger.removeHandler(console)
+    
     # Need to fork before connecting to D-Bus
-    if not foreground:
+    if not debug:
         # Close all input and output, do double fork, etc.
         daemon()
-
-    # multiprocessing will use threads, so before we use GLib we need
-    # to inform GLib that threads will be used.
-    GLib.threads_init()
-
+    
     global main_loop
     # From the Avahi example code
-    DBusGMainLoop(set_as_default=True)
-    main_loop = GLib.MainLoop()
+    DBusGMainLoop(set_as_default=True )
+    main_loop = gobject.MainLoop()
     bus = dbus.SystemBus()
     # End of Avahi example code
     if use_dbus:
         try:
             bus_name = dbus.service.BusName("se.recompile.Mandos",
-                                            bus,
-                                            do_not_queue=True)
-            old_bus_name = dbus.service.BusName(
-                "se.bsnet.fukt.Mandos", bus,
-                do_not_queue=True)
-        except dbus.exceptions.DBusException as e:
-            logger.error("Disabling D-Bus:", exc_info=e)
+                                            bus, do_not_queue=True)
+            old_bus_name = (dbus.service.BusName
+                            ("se.bsnet.fukt.Mandos", bus,
+                             do_not_queue=True))
+        except dbus.exceptions.NameExistsException as e:
+            logger.error(unicode(e) + ", disabling D-Bus")
             use_dbus = False
             server_settings["use_dbus"] = False
             tcp_server.use_dbus = False
-    if zeroconf:
-        protocol = avahi.PROTO_INET6 if use_ipv6 else avahi.PROTO_INET
-        service = AvahiServiceToSyslog(
-            name=server_settings["servicename"],
-            servicetype="_mandos._tcp",
-            protocol=protocol,
-            bus=bus)
-        if server_settings["interface"]:
-            service.interface = if_nametoindex(
-                server_settings["interface"].encode("utf-8"))
-
+    protocol = avahi.PROTO_INET6 if use_ipv6 else avahi.PROTO_INET
+    service = AvahiService(name = server_settings["servicename"],
+                           servicetype = "_mandos._tcp",
+                           protocol = protocol, bus = bus)
+    if server_settings["interface"]:
+        service.interface = (if_nametoindex
+                             (str(server_settings["interface"])))
+    
     global multiprocessing_manager
     multiprocessing_manager = multiprocessing.Manager()
-
+    
     client_class = Client
     if use_dbus:
-        client_class = functools.partial(ClientDBus, bus=bus)
-
-    client_settings = Client.config_parser(client_config)
-    old_client_settings = {}
-    clients_data = {}
-
-    # This is used to redirect stdout and stderr for checker processes
-    global wnull
-    wnull = open(os.devnull, "w")  # A writable /dev/null
-    # Only used if server is running in foreground but not in debug
-    # mode
-    if debug or not foreground:
-        wnull.close()
-
-    # Get client data and settings from last running state.
-    if server_settings["restore"]:
-        try:
-            with open(stored_state_path, "rb") as stored_state:
-                if sys.version_info.major == 2:
-                    clients_data, old_client_settings = pickle.load(
-                        stored_state)
-                else:
-                    bytes_clients_data, bytes_old_client_settings = (
-                        pickle.load(stored_state, encoding="bytes"))
-                    #   Fix bytes to strings
-                    #  clients_data
-                    # .keys()
-                    clients_data = {(key.decode("utf-8")
-                                     if isinstance(key, bytes)
-                                     else key): value
-                                    for key, value in
-                                    bytes_clients_data.items()}
-                    del bytes_clients_data
-                    for key in clients_data:
-                        value = {(k.decode("utf-8")
-                                  if isinstance(k, bytes) else k): v
-                                 for k, v in
-                                 clients_data[key].items()}
-                        clients_data[key] = value
-                        # .client_structure
-                        value["client_structure"] = [
-                            (s.decode("utf-8")
-                             if isinstance(s, bytes)
-                             else s) for s in
-                            value["client_structure"]]
-                        # .name & .host
-                        for k in ("name", "host"):
-                            if isinstance(value[k], bytes):
-                                value[k] = value[k].decode("utf-8")
-                        if not value.has_key("key_id"):
-                            value["key_id"] = ""
-                        elif not value.has_key("fingerprint"):
-                            value["fingerprint"] = ""
-                    #  old_client_settings
-                    # .keys()
-                    old_client_settings = {
-                        (key.decode("utf-8")
-                         if isinstance(key, bytes)
-                         else key): value
-                        for key, value in
-                        bytes_old_client_settings.items()}
-                    del bytes_old_client_settings
-                    # .host
-                    for value in old_client_settings.values():
-                        if isinstance(value["host"], bytes):
-                            value["host"] = (value["host"]
-                                             .decode("utf-8"))
-            os.remove(stored_state_path)
-        except IOError as e:
-            if e.errno == errno.ENOENT:
-                logger.warning("Could not load persistent state:"
-                               " {}".format(os.strerror(e.errno)))
-            else:
-                logger.critical("Could not load persistent state:",
-                                exc_info=e)
-                raise
-        except EOFError as e:
-            logger.warning("Could not load persistent state: "
-                           "EOFError:",
-                           exc_info=e)
-
-    with PGPEngine() as pgp:
-        for client_name, client in clients_data.items():
-            # Skip removed clients
-            if client_name not in client_settings:
-                continue
-
-            # Decide which value to use after restoring saved state.
-            # We have three different values: Old config file,
-            # new config file, and saved state.
-            # New config value takes precedence if it differs from old
-            # config value, otherwise use saved state.
-            for name, value in client_settings[client_name].items():
-                try:
-                    # For each value in new config, check if it
-                    # differs from the old config value (Except for
-                    # the "secret" attribute)
-                    if (name != "secret"
-                        and (value !=
-                             old_client_settings[client_name][name])):
-                        client[name] = value
-                except KeyError:
-                    pass
-
-            # Clients who has passed its expire date can still be
-            # enabled if its last checker was successful.  A Client
-            # whose checker succeeded before we stored its state is
-            # assumed to have successfully run all checkers during
-            # downtime.
-            if client["enabled"]:
-                if datetime.datetime.utcnow() >= client["expires"]:
-                    if not client["last_checked_ok"]:
-                        logger.warning(
-                            "disabling client {} - Client never "
-                            "performed a successful checker".format(
-                                client_name))
-                        client["enabled"] = False
-                    elif client["last_checker_status"] != 0:
-                        logger.warning(
-                            "disabling client {} - Client last"
-                            " checker failed with error code"
-                            " {}".format(
-                                client_name,
-                                client["last_checker_status"]))
-                        client["enabled"] = False
-                    else:
-                        client["expires"] = (
-                            datetime.datetime.utcnow()
-                            + client["timeout"])
-                        logger.debug("Last checker succeeded,"
-                                     " keeping {} enabled".format(
-                                         client_name))
+        client_class = functools.partial(ClientDBusTransitional,
+                                         bus = bus)
+    def client_config_items(config, section):
+        special_settings = {
+            "approved_by_default":
+                lambda: config.getboolean(section,
+                                          "approved_by_default"),
+            }
+        for name, value in config.items(section):
             try:
-                client["secret"] = pgp.decrypt(
-                    client["encrypted_secret"],
-                    client_settings[client_name]["secret"])
-            except PGPError:
-                # If decryption fails, we use secret from new settings
-                logger.debug("Failed to decrypt {} old secret".format(
-                    client_name))
-                client["secret"] = (client_settings[client_name]
-                                    ["secret"])
-
-    # Add/remove clients based on new changes made to config
-    for client_name in (set(old_client_settings)
-                        - set(client_settings)):
-        del clients_data[client_name]
-    for client_name in (set(client_settings)
-                        - set(old_client_settings)):
-        clients_data[client_name] = client_settings[client_name]
-
-    # Create all client objects
-    for client_name, client in clients_data.items():
-        tcp_server.clients[client_name] = client_class(
-            name=client_name,
-            settings=client,
-            server_settings=server_settings)
-
+                yield (name, special_settings[name]())
+            except KeyError:
+                yield (name, value)
+    
+    tcp_server.clients.update(set(
+            client_class(name = section,
+                         config= dict(client_config_items(
+                        client_config, section)))
+            for section in client_config.sections()))
     if not tcp_server.clients:
         logger.warning("No clients defined")
-
-    if not foreground:
-        if pidfile is not None:
-            pid = os.getpid()
-            try:
-                with pidfile:
-                    print(pid, file=pidfile)
-            except IOError:
-                logger.error("Could not write to file %r with PID %d",
-                             pidfilename, pid)
-        del pidfile
+        
+    if not debug:
+        try:
+            with pidfile:
+                pid = os.getpid()
+                pidfile.write(str(pid) + "\n".encode("utf-8"))
+            del pidfile
+        except IOError:
+            logger.error("Could not write to file %r with PID %d",
+                         pidfilename, pid)
+        except NameError:
+            # "pidfile" was never created
+            pass
         del pidfilename
-
-    for termsig in (signal.SIGHUP, signal.SIGTERM):
-        GLib.unix_signal_add(GLib.PRIORITY_HIGH, termsig,
-                             lambda: main_loop.quit() and False)
-
+        
+        signal.signal(signal.SIGINT, signal.SIG_IGN)
+    
+    signal.signal(signal.SIGHUP, lambda signum, frame: sys.exit())
+    signal.signal(signal.SIGTERM, lambda signum, frame: sys.exit())
+    
     if use_dbus:
-
-        @alternate_dbus_interfaces(
-            {"se.recompile.Mandos": "se.bsnet.fukt.Mandos"})
-        class MandosDBusService(DBusObjectWithObjectManager):
+        class MandosDBusService(dbus.service.Object):
             """A D-Bus proxy object"""
-
             def __init__(self):
                 dbus.service.Object.__init__(self, bus, "/")
-
             _interface = "se.recompile.Mandos"
-
+            
             @dbus.service.signal(_interface, signature="o")
             def ClientAdded(self, objpath):
                 "D-Bus signal"
                 pass
-
+            
             @dbus.service.signal(_interface, signature="ss")
-            def ClientNotFound(self, key_id, address):
+            def ClientNotFound(self, fingerprint, address):
                 "D-Bus signal"
                 pass
-
-            @dbus_annotations({"org.freedesktop.DBus.Deprecated":
-                               "true"})
+            
             @dbus.service.signal(_interface, signature="os")
             def ClientRemoved(self, objpath, name):
                 "D-Bus signal"
                 pass
-
-            @dbus_annotations({"org.freedesktop.DBus.Deprecated":
-                               "true"})
+            
             @dbus.service.method(_interface, out_signature="ao")
             def GetAllClients(self):
                 "D-Bus method"
-                return dbus.Array(c.dbus_object_path for c in
-                                  tcp_server.clients.values())
-
-            @dbus_annotations({"org.freedesktop.DBus.Deprecated":
-                               "true"})
+                return dbus.Array(c.dbus_object_path
+                                  for c in tcp_server.clients)
+            
             @dbus.service.method(_interface,
                                  out_signature="a{oa{sv}}")
             def GetAllClientsWithProperties(self):
                 "D-Bus method"
                 return dbus.Dictionary(
-                    {c.dbus_object_path: c.GetAll(
-                        "se.recompile.Mandos.Client")
-                     for c in tcp_server.clients.values()},
+                    ((c.dbus_object_path, c.GetAll(""))
+                     for c in tcp_server.clients),
                     signature="oa{sv}")
-
+            
             @dbus.service.method(_interface, in_signature="o")
             def RemoveClient(self, object_path):
                 "D-Bus method"
-                for c in tcp_server.clients.values():
+                for c in tcp_server.clients:
                     if c.dbus_object_path == object_path:
-                        del tcp_server.clients[c.name]
+                        tcp_server.clients.remove(c)
                         c.remove_from_connection()
-                        # Don't signal the disabling
+                        # Don't signal anything except ClientRemoved
                         c.disable(quiet=True)
-                        # Emit D-Bus signal for removal
-                        self.client_removed_signal(c)
+                        # Emit D-Bus signal
+                        self.ClientRemoved(object_path, c.name)
                         return
                 raise KeyError(object_path)
-
+            
             del _interface
-
-            @dbus.service.method(dbus.OBJECT_MANAGER_IFACE,
-                                 out_signature="a{oa{sa{sv}}}")
-            def GetManagedObjects(self):
-                """D-Bus method"""
-                return dbus.Dictionary(
-                    {client.dbus_object_path:
-                     dbus.Dictionary(
-                         {interface: client.GetAll(interface)
-                          for interface in
-                          client._get_all_interface_names()})
-                     for client in tcp_server.clients.values()})
-
-            def client_added_signal(self, client):
-                """Send the new standard signal and the old signal"""
-                if use_dbus:
-                    # New standard signal
-                    self.InterfacesAdded(
-                        client.dbus_object_path,
-                        dbus.Dictionary(
-                            {interface: client.GetAll(interface)
-                             for interface in
-                             client._get_all_interface_names()}))
-                    # Old signal
-                    self.ClientAdded(client.dbus_object_path)
-
-            def client_removed_signal(self, client):
-                """Send the new standard signal and the old signal"""
-                if use_dbus:
-                    # New standard signal
-                    self.InterfacesRemoved(
-                        client.dbus_object_path,
-                        client._get_all_interface_names())
-                    # Old signal
-                    self.ClientRemoved(client.dbus_object_path,
-                                       client.name)
-
-        mandos_dbus_service = MandosDBusService()
-
-    # Save modules to variables to exempt the modules from being
-    # unloaded before the function registered with atexit() is run.
-    mp = multiprocessing
-    wn = wnull
-
+        
+        class MandosDBusServiceTransitional(MandosDBusService):
+            __metaclass__ = AlternateDBusNamesMetaclass
+        mandos_dbus_service = MandosDBusServiceTransitional()
+    
     def cleanup():
         "Cleanup function; run on exit"
-        if zeroconf:
-            service.cleanup()
-
-        mp.active_children()
-        wn.close()
-        if not (tcp_server.clients or client_settings):
-            return
-
-        # Store client before exiting. Secrets are encrypted with key
-        # based on what config file has. If config file is
-        # removed/edited, old secret will thus be unrecovable.
-        clients = {}
-        with PGPEngine() as pgp:
-            for client in tcp_server.clients.values():
-                key = client_settings[client.name]["secret"]
-                client.encrypted_secret = pgp.encrypt(client.secret,
-                                                      key)
-                client_dict = {}
-
-                # A list of attributes that can not be pickled
-                # + secret.
-                exclude = {"bus", "changedstate", "secret",
-                           "checker", "server_settings"}
-                for name, typ in inspect.getmembers(dbus.service
-                                                    .Object):
-                    exclude.add(name)
-
-                client_dict["encrypted_secret"] = (client
-                                                   .encrypted_secret)
-                for attr in client.client_structure:
-                    if attr not in exclude:
-                        client_dict[attr] = getattr(client, attr)
-
-                clients[client.name] = client_dict
-                del client_settings[client.name]["secret"]
-
-        try:
-            with tempfile.NamedTemporaryFile(
-                    mode='wb',
-                    suffix=".pickle",
-                    prefix='clients-',
-                    dir=os.path.dirname(stored_state_path),
-                    delete=False) as stored_state:
-                pickle.dump((clients, client_settings), stored_state,
-                            protocol=2)
-                tempname = stored_state.name
-            os.rename(tempname, stored_state_path)
-        except (IOError, OSError) as e:
-            if not debug:
-                try:
-                    os.remove(tempname)
-                except NameError:
-                    pass
-            if e.errno in (errno.ENOENT, errno.EACCES, errno.EEXIST):
-                logger.warning("Could not save persistent state: {}"
-                               .format(os.strerror(e.errno)))
-            else:
-                logger.warning("Could not save persistent state:",
-                               exc_info=e)
-                raise
-
-        # Delete all clients, and settings from config
+        service.cleanup()
+        
         while tcp_server.clients:
-            name, client = tcp_server.clients.popitem()
+            client = tcp_server.clients.pop()
             if use_dbus:
                 client.remove_from_connection()
-            # Don't signal the disabling
+            client.disable_hook = None
+            # Don't signal anything except ClientRemoved
             client.disable(quiet=True)
-            # Emit D-Bus signal for removal
             if use_dbus:
-                mandos_dbus_service.client_removed_signal(client)
-        client_settings.clear()
-
+                # Emit D-Bus signal
+                mandos_dbus_service.ClientRemoved(client
+                                                  .dbus_object_path,
+                                                  client.name)
+    
     atexit.register(cleanup)
-
-    for client in tcp_server.clients.values():
+    
+    for client in tcp_server.clients:
         if use_dbus:
-            # Emit D-Bus signal for adding
-            mandos_dbus_service.client_added_signal(client)
-        # Need to initiate checking of clients
-        if client.enabled:
-            client.init_checker()
-
+            # Emit D-Bus signal
+            mandos_dbus_service.ClientAdded(client.dbus_object_path)
+        client.enable()
+    
     tcp_server.enable()
     tcp_server.server_activate()
-
+    
     # Find out what port we got
-    if zeroconf:
-        service.port = tcp_server.socket.getsockname()[1]
+    service.port = tcp_server.socket.getsockname()[1]
     if use_ipv6:
         logger.info("Now listening on address %r, port %d,"
-                    " flowinfo %d, scope_id %d",
-                    *tcp_server.socket.getsockname())
+                    " flowinfo %d, scope_id %d"
+                    % tcp_server.socket.getsockname())
     else:                       # IPv4
-        logger.info("Now listening on address %r, port %d",
-                    *tcp_server.socket.getsockname())
-
-    # service.interface = tcp_server.socket.getsockname()[3]
-
+        logger.info("Now listening on address %r, port %d"
+                    % tcp_server.socket.getsockname())
+    
+    #service.interface = tcp_server.socket.getsockname()[3]
+    
     try:
-        if zeroconf:
-            # From the Avahi example code
-            try:
-                service.activate()
-            except dbus.exceptions.DBusException as error:
-                logger.critical("D-Bus Exception", exc_info=error)
-                cleanup()
-                sys.exit(1)
-            # End of Avahi example code
-
-        GLib.io_add_watch(tcp_server.fileno(), GLib.IO_IN,
-                          lambda *args, **kwargs:
-                          (tcp_server.handle_request
-                           (*args[2:], **kwargs) or True))
-
+        # From the Avahi example code
+        try:
+            service.activate()
+        except dbus.exceptions.DBusException as error:
+            logger.critical("DBusException: %s", error)
+            cleanup()
+            sys.exit(1)
+        # End of Avahi example code
+        
+        gobject.io_add_watch(tcp_server.fileno(), gobject.IO_IN,
+                             lambda *args, **kwargs:
+                             (tcp_server.handle_request
+                              (*args[2:], **kwargs) or True))
+        
         logger.debug("Starting main loop")
         main_loop.run()
     except AvahiError as error:
-        logger.critical("Avahi Error", exc_info=error)
+        logger.critical("AvahiError: %s", error)
         cleanup()
         sys.exit(1)
     except KeyboardInterrupt:

=== modified file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	2019-02-10 04:20:26 +0000
+++ mandos-clients.conf.xml	2011-10-05 16:00:56 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos-clients.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -36,14 +36,6 @@
       <year>2009</year>
       <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -73,13 +65,8 @@
       ><refentrytitle>mandos</refentrytitle>
       <manvolnum>8</manvolnum></citerefentry>, read by it at startup.
       The file needs to list all clients that should be able to use
-      the service.  The settings in this file can be overridden by
-      runtime changes to the server, which it saves across restarts.
-      (See the section called <quote>PERSISTENT STATE</quote> in
-      <citerefentry><refentrytitle>mandos</refentrytitle><manvolnum
-      >8</manvolnum></citerefentry>.)  However, any <emphasis
-      >changes</emphasis> to this file (including adding and removing
-      clients) will, at startup, override changes done during runtime.
+      the service.  All clients listed will be regarded as enabled,
+      even if a client was disabled in a previous run of the server.
     </para>
     <para>
       The format starts with a <literal>[<replaceable>section
@@ -124,7 +111,7 @@
 	  <para>
 	    How long to wait for external approval before resorting to
 	    use the <option>approved_by_default</option> value.  The
-	    default is <quote>PT0S</quote>, i.e. not to wait.
+	    default is <quote>0s</quote>, i.e. not to wait.
 	  </para>
 	  <para>
 	    The format of <replaceable>TIME</replaceable> is the same
@@ -174,23 +161,17 @@
 	    This option is <emphasis>optional</emphasis>.
 	  </para>
 	  <para>
-	    This option overrides the default shell command that the
-	    server will use to check if the client is still up.  Any
-	    output of the command will be ignored, only the exit code
-	    is checked:  If the exit code of the command is zero, the
-	    client is considered up.  The command will be run using
-	    <quote><command><filename>/bin/sh</filename>
+	    This option allows you to override the default shell
+	    command that the server will use to check if the client is
+	    still up.  Any output of the command will be ignored, only
+	    the exit code is checked:  If the exit code of the command
+	    is zero, the client is considered up.  The command will be
+	    run using <quote><command><filename>/bin/sh</filename>
 	    <option>-c</option></command></quote>, so
 	    <varname>PATH</varname> will be searched.  The default
 	    value for the checker command is <quote><literal
 	    ><command>fping</command> <option>-q</option> <option
-	    >--</option> %%(host)s</literal></quote>.  Note that
-	    <command>mandos-keygen</command>, when generating output
-	    to be inserted into this file, normally looks for an SSH
-	    server on the Mandos client, and, if it finds one, outputs
-	    a <option>checker</option> option to check for the
-	    client’s SSH key fingerprint – this is more secure against
-	    spoofing.
+	    >--</option> %%(host)s</literal></quote>.
 	  </para>
 	  <para>
 	    In addition to normal start time expansion, this option
@@ -209,7 +190,7 @@
 	  </para>
 	  <para>
 	    Extended timeout is an added timeout that is given once
-	    after a password has been sent successfully to a client.
+	    after a password has been sent sucessfully to a client.
 	    The timeout is by default longer than the normal timeout,
 	    and is used for handling the extra long downtime while a
 	    machine is booting up.  Time to take into consideration
@@ -233,23 +214,7 @@
 	  <para>
 	    This option sets the OpenPGP fingerprint that identifies
 	    the public key that clients authenticate themselves with
-	    through TLS.  The string needs to be in hexadecimal form,
-	    but spaces or upper/lower case are not significant.
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>key_id<literal> = </literal
-	><replaceable>HEXSTRING</replaceable></option></term>
-	<listitem>
-	  <para>
-	    This option is <emphasis>optional</emphasis>.
-	  </para>
-	  <para>
-	    This option sets the certificate key ID that identifies
-	    the public key that clients authenticate themselves with
-	    through TLS.  The string needs to be in hexadecimal form,
+	    through TLS.  The string needs to be in hexidecimal form,
 	    but spaces or upper/lower case are not significant.
 	  </para>
 	</listitem>
@@ -331,9 +296,9 @@
 	  <para>
 	    If present, this option must be set to a string of
 	    base64-encoded binary data.  It will be decoded and sent
-	    to the client matching the above <option>key_id</option>
-	    or <option>fingerprint</option>.  This should, of course,
-	    be OpenPGP encrypted data, decryptable only by the client.
+	    to the client matching the above
+	    <option>fingerprint</option>.  This should, of course, be
+	    OpenPGP encrypted data, decryptable only by the client.
 	    The program <citerefentry><refentrytitle><command
 	    >mandos-keygen</command></refentrytitle><manvolnum
 	    >8</manvolnum></citerefentry> can, using its
@@ -364,26 +329,17 @@
 	    <option>extended_timeout</option> option.
 	  </para>
 	  <para>
-	    The <replaceable>TIME</replaceable> is specified as an RFC
-	    3339 duration; for example
-	    <quote><literal>P1Y2M3DT4H5M6S</literal></quote> meaning
-	    one year, two months, three days, four hours, five
-	    minutes, and six seconds.  Some values can be omitted, see
-	    RFC 3339 Appendix A for details.
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>enabled<literal> = </literal>{ <literal
-	>1</literal> | <literal>yes</literal> |	<literal>true</literal
-	> | <literal >on</literal> | <literal>0</literal> | <literal
-	>no</literal> | <literal>false</literal> | <literal
-	>off</literal> }</option></term>
-	<listitem>
-	  <para>
-	    Whether this client should be enabled by default.  The
-	    default is <quote>true</quote>.
+	    The <replaceable>TIME</replaceable> is specified as a
+	    space-separated number of values, each of which is a
+	    number and a one-character suffix.  The suffix must be one
+	    of <quote>d</quote>, <quote>s</quote>, <quote>m</quote>,
+	    <quote>h</quote>, and <quote>w</quote> for days, seconds,
+	    minutes, hours, and weeks, respectively.  The values are
+	    added together to give the total time value, so all of
+	    <quote><literal>330s</literal></quote>,
+	    <quote><literal>110s 110s 110s</literal></quote>, and
+	    <quote><literal>5m 30s</literal></quote> will give a value
+	    of five minutes and thirty seconds.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -433,8 +389,6 @@
 	<quote><literal>approval_duration</literal></quote>,
 	<quote><literal>created</literal></quote>,
 	<quote><literal>enabled</literal></quote>,
-	<quote><literal>expires</literal></quote>,
-	<quote><literal>key_id</literal></quote>,
 	<quote><literal>fingerprint</literal></quote>,
 	<quote><literal>host</literal></quote>,
 	<quote><literal>interval</literal></quote>,
@@ -483,7 +437,6 @@
       <literal>%(<replaceable>foo</replaceable>)s</literal> is
       obscure.
     </para>
-    <xi:include href="bugs.xml"/>
   </refsect1>
   
   <refsect1 id="example">
@@ -491,13 +444,12 @@
     <informalexample>
       <programlisting>
 [DEFAULT]
-timeout = PT5M
-interval = PT2M
+timeout = 5m
+interval = 2m
 checker = fping -q -- %%(host)s
 
 # Client "foo"
 [foo]
-key_id = 788cd77115cd0bb7b2d5e0ae8496f6b48149d5e712c652076b1fd2d957ef7c1f
 fingerprint =  7788 2722 5BA7 DE53 9C5A  7CFA 59CF F7CD BD9A 5920
 secret =
         hQIOA6QdEjBs2L/HEAf/TCyrDe5Xnm9esa+Pb/vWF9CUqfn4srzVgSu234
@@ -516,16 +468,15 @@
         4T2zw4dxS5NswXWU0sVEXxjs6PYxuIiCTL7vdpx8QjBkrPWDrAbcMyBr2O
         QlnHIvPzEArRQLo=
 host = foo.example.org
-interval = PT1M
+interval = 1m
 
 # Client "bar"
 [bar]
-key_id = F90C7A81D72D1EA69A51031A91FF8885F36C8B46D155C8C58709A4C99AE9E361
 fingerprint = 3e393aeaefb84c7e89e2f547b3a107558fca3a27
 secfile = /etc/mandos/bar-secret
-timeout = PT15M
+timeout = 15m
 approved_by_default = False
-approval_delay = PT30S
+approval_delay = 30s
       </programlisting>
     </informalexample>
   </refsect1>
@@ -540,24 +491,8 @@
       <citerefentry><refentrytitle>mandos.conf</refentrytitle>
       <manvolnum>5</manvolnum></citerefentry>,
       <citerefentry><refentrytitle>mandos</refentrytitle>
-      <manvolnum>8</manvolnum></citerefentry>,
-      <citerefentry><refentrytitle>fping</refentrytitle>
       <manvolnum>8</manvolnum></citerefentry>
     </para>
-    <variablelist>
-      <varlistentry>
-	<term>
-	  RFC 3339: <citetitle>Date and Time on the Internet:
-	  Timestamps</citetitle>
-	</term>
-      <listitem>
-	<para>
-	  The time intervals are in the "duration" format, as
-	  specified in ABNF in Appendix A of RFC 3339.
-	</para>
-      </listitem>
-      </varlistentry>
-    </variablelist>
   </refsect1>
 </refentry>
 <!-- Local Variables: -->

=== modified file 'mandos-ctl'
--- mandos-ctl	2019-03-09 15:47:52 +0000
+++ mandos-ctl	2011-10-05 16:00:56 +0000
@@ -1,1540 +1,356 @@
 #!/usr/bin/python
-# -*- mode: python; coding: utf-8; after-save-hook: (lambda () (let ((command (if (and (boundp 'tramp-file-name-structure) (string-match (car tramp-file-name-structure) (buffer-file-name))) (tramp-file-name-localname (tramp-dissect-file-name (buffer-file-name))) (buffer-file-name)))) (if (= (shell-command (format "%s --check" (shell-quote-argument command)) "*Test*") 0) (let ((w (get-buffer-window "*Test*"))) (if w (delete-window w)) (kill-buffer "*Test*")) (display-buffer "*Test*")))); -*-
-#
+# -*- mode: python; coding: utf-8 -*-
+# 
 # Mandos Monitor - Control and monitor the Mandos server
-#
-# Copyright © 2008-2019 Teddy Hogeborn
-# Copyright © 2008-2019 Björn Påhlsson
-#
-# This file is part of Mandos.
-#
-# Mandos is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
+# 
+# Copyright © 2008-2010 Teddy Hogeborn
+# Copyright © 2008-2010 Björn Påhlsson
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#     Mandos is distributed in the hope that it will be useful, but
-#     WITHOUT ANY WARRANTY; without even the implied warranty of
+#     This program is distributed in the hope that it will be useful,
+#     but WITHOUT ANY WARRANTY; without even the implied warranty of
 #     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #     GNU General Public License for more details.
-#
+# 
 # You should have received a copy of the GNU General Public License
-# along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
-#
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# 
 # Contact the authors at <mandos@recompile.se>.
-#
+# 
 
 from __future__ import (division, absolute_import, print_function,
                         unicode_literals)
 
-try:
-    from future_builtins import *
-except ImportError:
-    pass
-
 import sys
+import dbus
 import argparse
 import locale
 import datetime
 import re
 import os
-import collections
-import json
-import unittest
-import logging
-import io
-import tempfile
-import contextlib
-
-import dbus
-
-# Show warnings by default
-if not sys.warnoptions:
-    import warnings
-    warnings.simplefilter("default")
-
-log = logging.getLogger(sys.argv[0])
-logging.basicConfig(level="INFO", # Show info level messages
-                    format="%(message)s") # Show basic log messages
-
-logging.captureWarnings(True)   # Show warnings via the logging system
-
-if sys.version_info.major == 2:
-    str = unicode
 
 locale.setlocale(locale.LC_ALL, "")
 
+tablewords = {
+    "Name": "Name",
+    "Enabled": "Enabled",
+    "Timeout": "Timeout",
+    "LastCheckedOK": "Last Successful Check",
+    "LastApprovalRequest": "Last Approval Request",
+    "Created": "Created",
+    "Interval": "Interval",
+    "Host": "Host",
+    "Fingerprint": "Fingerprint",
+    "CheckerRunning": "Check Is Running",
+    "LastEnabled": "Last Enabled",
+    "ApprovalPending": "Approval Is Pending",
+    "ApprovedByDefault": "Approved By Default",
+    "ApprovalDelay": "Approval Delay",
+    "ApprovalDuration": "Approval Duration",
+    "Checker": "Checker",
+    "ExtendedTimeout" : "Extended Timeout"
+    }
+defaultkeywords = ("Name", "Enabled", "Timeout", "LastCheckedOK")
 domain = "se.recompile"
 busname = domain + ".Mandos"
 server_path = "/"
 server_interface = domain + ".Mandos"
 client_interface = domain + ".Mandos.Client"
-version = "1.8.3"
-
-
-try:
-    dbus.OBJECT_MANAGER_IFACE
-except AttributeError:
-    dbus.OBJECT_MANAGER_IFACE = "org.freedesktop.DBus.ObjectManager"
-
+version = "1.3.1"
+
+def timedelta_to_milliseconds(td):
+    """Convert a datetime.timedelta object to milliseconds"""
+    return ((td.days * 24 * 60 * 60 * 1000)
+            + (td.seconds * 1000)
+            + (td.microseconds // 1000))
 
 def milliseconds_to_string(ms):
     td = datetime.timedelta(0, 0, 0, ms)
-    return ("{days}{hours:02}:{minutes:02}:{seconds:02}"
-            .format(days="{}T".format(td.days) if td.days else "",
-                    hours=td.seconds // 3600,
-                    minutes=(td.seconds % 3600) // 60,
-                    seconds=td.seconds % 60))
-
-
-def rfc3339_duration_to_delta(duration):
-    """Parse an RFC 3339 "duration" and return a datetime.timedelta
-
-    >>> rfc3339_duration_to_delta("P7D")
-    datetime.timedelta(7)
-    >>> rfc3339_duration_to_delta("PT60S")
-    datetime.timedelta(0, 60)
-    >>> rfc3339_duration_to_delta("PT60M")
-    datetime.timedelta(0, 3600)
-    >>> rfc3339_duration_to_delta("P60M")
-    datetime.timedelta(1680)
-    >>> rfc3339_duration_to_delta("PT24H")
-    datetime.timedelta(1)
-    >>> rfc3339_duration_to_delta("P1W")
-    datetime.timedelta(7)
-    >>> rfc3339_duration_to_delta("PT5M30S")
-    datetime.timedelta(0, 330)
-    >>> rfc3339_duration_to_delta("P1DT3M20S")
-    datetime.timedelta(1, 200)
-    >>> # Can not be empty:
-    >>> rfc3339_duration_to_delta("")
-    Traceback (most recent call last):
-    ...
-    ValueError: Invalid RFC 3339 duration: u''
-    >>> # Must start with "P":
-    >>> rfc3339_duration_to_delta("1D")
-    Traceback (most recent call last):
-    ...
-    ValueError: Invalid RFC 3339 duration: u'1D'
-    >>> # Must use correct order
-    >>> rfc3339_duration_to_delta("PT1S2M")
-    Traceback (most recent call last):
-    ...
-    ValueError: Invalid RFC 3339 duration: u'PT1S2M'
-    >>> # Time needs time marker
-    >>> rfc3339_duration_to_delta("P1H2S")
-    Traceback (most recent call last):
-    ...
-    ValueError: Invalid RFC 3339 duration: u'P1H2S'
-    >>> # Weeks can not be combined with anything else
-    >>> rfc3339_duration_to_delta("P1D2W")
-    Traceback (most recent call last):
-    ...
-    ValueError: Invalid RFC 3339 duration: u'P1D2W'
-    >>> rfc3339_duration_to_delta("P2W2H")
-    Traceback (most recent call last):
-    ...
-    ValueError: Invalid RFC 3339 duration: u'P2W2H'
-    """
-
-    # Parsing an RFC 3339 duration with regular expressions is not
-    # possible - there would have to be multiple places for the same
-    # values, like seconds.  The current code, while more esoteric, is
-    # cleaner without depending on a parsing library.  If Python had a
-    # built-in library for parsing we would use it, but we'd like to
-    # avoid excessive use of external libraries.
-
-    # New type for defining tokens, syntax, and semantics all-in-one
-    Token = collections.namedtuple("Token", (
-        "regexp",  # To match token; if "value" is not None, must have
-                   # a "group" containing digits
-        "value",   # datetime.timedelta or None
-        "followers"))           # Tokens valid after this token
-    # RFC 3339 "duration" tokens, syntax, and semantics; taken from
-    # the "duration" ABNF definition in RFC 3339, Appendix A.
-    token_end = Token(re.compile(r"$"), None, frozenset())
-    token_second = Token(re.compile(r"(\d+)S"),
-                         datetime.timedelta(seconds=1),
-                         frozenset((token_end, )))
-    token_minute = Token(re.compile(r"(\d+)M"),
-                         datetime.timedelta(minutes=1),
-                         frozenset((token_second, token_end)))
-    token_hour = Token(re.compile(r"(\d+)H"),
-                       datetime.timedelta(hours=1),
-                       frozenset((token_minute, token_end)))
-    token_time = Token(re.compile(r"T"),
-                       None,
-                       frozenset((token_hour, token_minute,
-                                  token_second)))
-    token_day = Token(re.compile(r"(\d+)D"),
-                      datetime.timedelta(days=1),
-                      frozenset((token_time, token_end)))
-    token_month = Token(re.compile(r"(\d+)M"),
-                        datetime.timedelta(weeks=4),
-                        frozenset((token_day, token_end)))
-    token_year = Token(re.compile(r"(\d+)Y"),
-                       datetime.timedelta(weeks=52),
-                       frozenset((token_month, token_end)))
-    token_week = Token(re.compile(r"(\d+)W"),
-                       datetime.timedelta(weeks=1),
-                       frozenset((token_end, )))
-    token_duration = Token(re.compile(r"P"), None,
-                           frozenset((token_year, token_month,
-                                      token_day, token_time,
-                                      token_week)))
-    # Define starting values:
-    # Value so far
-    value = datetime.timedelta()
-    found_token = None
-    # Following valid tokens
-    followers = frozenset((token_duration, ))
-    # String left to parse
-    s = duration
-    # Loop until end token is found
-    while found_token is not token_end:
-        # Search for any currently valid tokens
-        for token in followers:
-            match = token.regexp.match(s)
-            if match is not None:
-                # Token found
-                if token.value is not None:
-                    # Value found, parse digits
-                    factor = int(match.group(1), 10)
-                    # Add to value so far
-                    value += factor * token.value
-                # Strip token from string
-                s = token.regexp.sub("", s, 1)
-                # Go to found token
-                found_token = token
-                # Set valid next tokens
-                followers = found_token.followers
-                break
-        else:
-            # No currently valid tokens were found
-            raise ValueError("Invalid RFC 3339 duration: {!r}"
-                             .format(duration))
-    # End token found
-    return value
-
+    return ("%(days)s%(hours)02d:%(minutes)02d:%(seconds)02d"
+            % { "days": "%dT" % td.days if td.days else "",
+                "hours": td.seconds // 3600,
+                "minutes": (td.seconds % 3600) // 60,
+                "seconds": td.seconds % 60,
+                })
 
 def string_to_delta(interval):
-    """Parse a string and return a datetime.timedelta"""
-
-    try:
-        return rfc3339_duration_to_delta(interval)
-    except ValueError as e:
-        log.warning("%s - Parsing as pre-1.6.1 interval instead",
-                    ' '.join(e.args))
-    return parse_pre_1_6_1_interval(interval)
-
-
-def parse_pre_1_6_1_interval(interval):
-    """Parse an interval string as documented by Mandos before 1.6.1,
-    and return a datetime.timedelta
-
-    >>> parse_pre_1_6_1_interval('7d')
+    """Parse a string and return a datetime.timedelta
+    
+    >>> string_to_delta("7d")
     datetime.timedelta(7)
-    >>> parse_pre_1_6_1_interval('60s')
+    >>> string_to_delta("60s")
     datetime.timedelta(0, 60)
-    >>> parse_pre_1_6_1_interval('60m')
+    >>> string_to_delta("60m")
     datetime.timedelta(0, 3600)
-    >>> parse_pre_1_6_1_interval('24h')
+    >>> string_to_delta("24h")
     datetime.timedelta(1)
-    >>> parse_pre_1_6_1_interval('1w')
+    >>> string_to_delta("1w")
     datetime.timedelta(7)
-    >>> parse_pre_1_6_1_interval('5m 30s')
+    >>> string_to_delta("5m 30s")
     datetime.timedelta(0, 330)
-    >>> parse_pre_1_6_1_interval('')
-    datetime.timedelta(0)
-    >>> # Ignore unknown characters, allow any order and repetitions
-    >>> parse_pre_1_6_1_interval('2dxy7zz11y3m5m')
-    datetime.timedelta(2, 480, 18000)
-
     """
-
-    value = datetime.timedelta(0)
-    regexp = re.compile(r"(\d+)([dsmhw]?)")
-
-    for num, suffix in regexp.findall(interval):
-        if suffix == "d":
-            value += datetime.timedelta(int(num))
-        elif suffix == "s":
-            value += datetime.timedelta(0, int(num))
-        elif suffix == "m":
-            value += datetime.timedelta(0, 0, 0, 0, int(num))
-        elif suffix == "h":
-            value += datetime.timedelta(0, 0, 0, 0, 0, int(num))
-        elif suffix == "w":
-            value += datetime.timedelta(0, 0, 0, 0, 0, 0, int(num))
-        elif suffix == "":
-            value += datetime.timedelta(0, 0, 0, int(num))
-    return value
-
-
-## Classes for commands.
-
-# Abstract classes first
-class Command(object):
-    """Abstract class for commands"""
-    def run(self, mandos, mandos_objmgr, clients):
-        """Normal commands should implement run_on_one_client(), but
-        commands which want to operate on all clients at the same time
-        can override this run() method instead."""
-        self.mandos = mandos
-        for client, properties in clients.items():
-            self.run_on_one_client(client, properties)
-
-class PrintCmd(Command):
-    """Abstract class for commands printing client details"""
-    all_keywords = ("Name", "Enabled", "Timeout", "LastCheckedOK",
-                    "Created", "Interval", "Host", "KeyID",
-                    "Fingerprint", "CheckerRunning", "LastEnabled",
-                    "ApprovalPending", "ApprovedByDefault",
-                    "LastApprovalRequest", "ApprovalDelay",
-                    "ApprovalDuration", "Checker", "ExtendedTimeout",
-                    "Expires", "LastCheckerStatus")
-    def run(self, mandos, mandos_objmgr, clients):
-        print(self.output(mandos_objmgr, clients.values()))
-
-class PropertyCmd(Command):
-    """Abstract class for Actions for setting one client property"""
-    def run_on_one_client(self, client, properties):
-        """Set the Client's D-Bus property"""
-        log.debug("D-Bus: %s:%s:%s.Set(%r, %r, %r)", busname,
-                  client.__dbus_object_path__,
-                  dbus.PROPERTIES_IFACE, client_interface,
-                  self.property, self.value_to_set
-                  if not isinstance(self.value_to_set, dbus.Boolean)
-                  else bool(self.value_to_set))
-        client.Set(client_interface, self.property, self.value_to_set,
-                   dbus_interface=dbus.PROPERTIES_IFACE)
-
-class ValueArgumentMixIn(object):
-    """Mixin class for commands taking a value as argument"""
-    def __init__(self, value):
-        self.value_to_set = value
-
-class MillisecondsValueArgumentMixIn(ValueArgumentMixIn):
-    """Mixin class for commands taking a value argument as
-    milliseconds."""
-    @property
-    def value_to_set(self):
-        return self._vts
-    @value_to_set.setter
-    def value_to_set(self, value):
-        """When setting, convert value to a datetime.timedelta"""
-        self._vts = int(round(value.total_seconds() * 1000))
-
-# Actual (non-abstract) command classes
-
-class PrintTableCmd(PrintCmd):
-    def __init__(self, verbose=False):
-        self.verbose = verbose
-
-    def output(self, mandos_objmgr, clients):
-        default_keywords = ("Name", "Enabled", "Timeout", "LastCheckedOK")
-        keywords = default_keywords
-        if self.verbose:
-            keywords = self.all_keywords
-        return str(self.TableOfClients(clients, keywords))
-
-    class TableOfClients(object):
-        tableheaders = {
-            "Name": "Name",
-            "Enabled": "Enabled",
-            "Timeout": "Timeout",
-            "LastCheckedOK": "Last Successful Check",
-            "LastApprovalRequest": "Last Approval Request",
-            "Created": "Created",
-            "Interval": "Interval",
-            "Host": "Host",
-            "Fingerprint": "Fingerprint",
-            "KeyID": "Key ID",
-            "CheckerRunning": "Check Is Running",
-            "LastEnabled": "Last Enabled",
-            "ApprovalPending": "Approval Is Pending",
-            "ApprovedByDefault": "Approved By Default",
-            "ApprovalDelay": "Approval Delay",
-            "ApprovalDuration": "Approval Duration",
-            "Checker": "Checker",
-            "ExtendedTimeout": "Extended Timeout",
-            "Expires": "Expires",
-            "LastCheckerStatus": "Last Checker Status",
-        }
-
-        def __init__(self, clients, keywords, tableheaders=None):
-            self.clients = clients
-            self.keywords = keywords
-            if tableheaders is not None:
-                self.tableheaders = tableheaders
-
-        def __str__(self):
-            return "\n".join(self.rows())
-
-        if sys.version_info.major == 2:
-            __unicode__ = __str__
-            def __str__(self):
-                return str(self).encode(locale.getpreferredencoding())
-
-        def rows(self):
-            format_string = self.row_formatting_string()
-            rows = [self.header_line(format_string)]
-            rows.extend(self.client_line(client, format_string)
-                        for client in self.clients)
-            return rows
-
-        def row_formatting_string(self):
-            "Format string used to format table rows"
-            return " ".join("{{{key}:{width}}}".format(
-                width=max(len(self.tableheaders[key]),
-                          *(len(self.string_from_client(client, key))
-                            for client in self.clients)),
-                key=key)
-                            for key in self.keywords)
-
-        def string_from_client(self, client, key):
-            return self.valuetostring(client[key], key)
-
-        @staticmethod
-        def valuetostring(value, keyword):
-            if isinstance(value, dbus.Boolean):
-                return "Yes" if value else "No"
-            if keyword in ("Timeout", "Interval", "ApprovalDelay",
-                           "ApprovalDuration", "ExtendedTimeout"):
-                return milliseconds_to_string(value)
-            return str(value)
-
-        def header_line(self, format_string):
-            return format_string.format(**self.tableheaders)
-
-        def client_line(self, client, format_string):
-            return format_string.format(
-                **{key: self.string_from_client(client, key)
-                   for key in self.keywords})
-
-
-
-class DumpJSONCmd(PrintCmd):
-    def output(self, mandos_objmgr, clients):
-        data = {client["Name"]:
-                {key: self.dbus_boolean_to_bool(client[key])
-                 for key in self.all_keywords}
-                for client in clients.values()}
-        return json.dumps(data, indent=4, separators=(',', ': '))
-    @staticmethod
-    def dbus_boolean_to_bool(value):
-        if isinstance(value, dbus.Boolean):
-            value = bool(value)
-        return value
-
-class IsEnabledCmd(Command):
-    def run_on_one_client(self, client, properties):
-        if self.is_enabled(client, properties):
-            sys.exit(0)
-        sys.exit(1)
-    def is_enabled(self, client, properties):
-        log.debug("D-Bus: %s:%s:%s.Get(%r, %r)", busname,
-                  client.__dbus_object_path__,
-                  dbus.PROPERTIES_IFACE, client_interface,
-                  "Enabled")
-        return bool(client.Get(client_interface, "Enabled",
-                               dbus_interface=dbus.PROPERTIES_IFACE))
-
-class RemoveCmd(Command):
-    def run_on_one_client(self, client, properties):
-        log.debug("D-Bus: %s:%s:%s.RemoveClient(%r)", busname,
-                  server_path, server_interface,
-                  str(client.__dbus_object_path__))
-        self.mandos.RemoveClient(client.__dbus_object_path__)
-
-class ApproveCmd(Command):
-    def run_on_one_client(self, client, properties):
-        log.debug("D-Bus: %s:%s.Approve(True)",
-                  client.__dbus_object_path__, client_interface)
-        client.Approve(dbus.Boolean(True),
-                       dbus_interface=client_interface)
-
-class DenyCmd(Command):
-    def run_on_one_client(self, client, properties):
-        log.debug("D-Bus: %s:%s.Approve(False)",
-                  client.__dbus_object_path__, client_interface)
-        client.Approve(dbus.Boolean(False),
-                       dbus_interface=client_interface)
-
-class EnableCmd(PropertyCmd):
-    property = "Enabled"
-    value_to_set = dbus.Boolean(True)
-
-class DisableCmd(PropertyCmd):
-    property = "Enabled"
-    value_to_set = dbus.Boolean(False)
-
-class BumpTimeoutCmd(PropertyCmd):
-    property = "LastCheckedOK"
-    value_to_set = ""
-
-class StartCheckerCmd(PropertyCmd):
-    property = "CheckerRunning"
-    value_to_set = dbus.Boolean(True)
-
-class StopCheckerCmd(PropertyCmd):
-    property = "CheckerRunning"
-    value_to_set = dbus.Boolean(False)
-
-class ApproveByDefaultCmd(PropertyCmd):
-    property = "ApprovedByDefault"
-    value_to_set = dbus.Boolean(True)
-
-class DenyByDefaultCmd(PropertyCmd):
-    property = "ApprovedByDefault"
-    value_to_set = dbus.Boolean(False)
-
-class SetCheckerCmd(PropertyCmd, ValueArgumentMixIn):
-    property = "Checker"
-
-class SetHostCmd(PropertyCmd, ValueArgumentMixIn):
-    property = "Host"
-
-class SetSecretCmd(PropertyCmd, ValueArgumentMixIn):
-    @property
-    def value_to_set(self):
-        return self._vts
-    @value_to_set.setter
-    def value_to_set(self, value):
-        """When setting, read data from supplied file object"""
-        self._vts = value.read()
-        value.close()
-    property = "Secret"
-
-class SetTimeoutCmd(PropertyCmd, MillisecondsValueArgumentMixIn):
-    property = "Timeout"
-
-class SetExtendedTimeoutCmd(PropertyCmd,
-                            MillisecondsValueArgumentMixIn):
-    property = "ExtendedTimeout"
-
-class SetIntervalCmd(PropertyCmd, MillisecondsValueArgumentMixIn):
-    property = "Interval"
-
-class SetApprovalDelayCmd(PropertyCmd,
-                          MillisecondsValueArgumentMixIn):
-    property = "ApprovalDelay"
-
-class SetApprovalDurationCmd(PropertyCmd,
-                             MillisecondsValueArgumentMixIn):
-    property = "ApprovalDuration"
-
-def add_command_line_options(parser):
+    timevalue = datetime.timedelta(0)
+    regexp = re.compile("\d+[dsmhw]")
+    
+    for s in regexp.findall(interval):
+        try:
+            suffix = unicode(s[-1])
+            value = int(s[:-1])
+            if suffix == "d":
+                delta = datetime.timedelta(value)
+            elif suffix == "s":
+                delta = datetime.timedelta(0, value)
+            elif suffix == "m":
+                delta = datetime.timedelta(0, 0, 0, 0, value)
+            elif suffix == "h":
+                delta = datetime.timedelta(0, 0, 0, 0, 0, value)
+            elif suffix == "w":
+                delta = datetime.timedelta(0, 0, 0, 0, 0, 0, value)
+            else:
+                raise ValueError
+        except (ValueError, IndexError):
+            raise ValueError
+        timevalue += delta
+    return timevalue
+
+def print_clients(clients, keywords):
+    def valuetostring(value, keyword):
+        if type(value) is dbus.Boolean:
+            return "Yes" if value else "No"
+        if keyword in ("Timeout", "Interval", "ApprovalDelay",
+                       "ApprovalDuration"):
+            return milliseconds_to_string(value)
+        return unicode(value)
+    
+    # Create format string to print table rows
+    format_string = " ".join("%%-%ds" %
+                             max(len(tablewords[key]),
+                                 max(len(valuetostring(client[key],
+                                                       key))
+                                     for client in
+                                     clients))
+                             for key in keywords)
+    # Print header line
+    print(format_string % tuple(tablewords[key] for key in keywords))
+    for client in clients:
+        print(format_string % tuple(valuetostring(client[key], key)
+                                    for key in keywords))
+
+def has_actions(options):
+    return any((options.enable,
+                options.disable,
+                options.bump_timeout,
+                options.start_checker,
+                options.stop_checker,
+                options.is_enabled,
+                options.remove,
+                options.checker is not None,
+                options.timeout is not None,
+                options.extended_timeout is not None,
+                options.interval is not None,
+                options.approved_by_default is not None,
+                options.approval_delay is not None,
+                options.approval_duration is not None,
+                options.host is not None,
+                options.secret is not None,
+                options.approve,
+                options.deny))
+
+def main():
+    parser = argparse.ArgumentParser()
     parser.add_argument("--version", action="version",
-                        version="%(prog)s {}".format(version),
+                        version = "%%prog %s" % version,
                         help="show version number and exit")
     parser.add_argument("-a", "--all", action="store_true",
                         help="Select all clients")
     parser.add_argument("-v", "--verbose", action="store_true",
                         help="Print all fields")
-    parser.add_argument("-j", "--dump-json", action="store_true",
-                        help="Dump client data in JSON format")
-    enable_disable = parser.add_mutually_exclusive_group()
-    enable_disable.add_argument("-e", "--enable", action="store_true",
-                                help="Enable client")
-    enable_disable.add_argument("-d", "--disable",
-                                action="store_true",
-                                help="disable client")
+    parser.add_argument("-e", "--enable", action="store_true",
+                        help="Enable client")
+    parser.add_argument("-d", "--disable", action="store_true",
+                        help="disable client")
     parser.add_argument("-b", "--bump-timeout", action="store_true",
                         help="Bump timeout for client")
-    start_stop_checker = parser.add_mutually_exclusive_group()
-    start_stop_checker.add_argument("--start-checker",
-                                    action="store_true",
-                                    help="Start checker for client")
-    start_stop_checker.add_argument("--stop-checker",
-                                    action="store_true",
-                                    help="Stop checker for client")
+    parser.add_argument("--start-checker", action="store_true",
+                        help="Start checker for client")
+    parser.add_argument("--stop-checker", action="store_true",
+                        help="Stop checker for client")
     parser.add_argument("-V", "--is-enabled", action="store_true",
                         help="Check if client is enabled")
     parser.add_argument("-r", "--remove", action="store_true",
                         help="Remove client")
     parser.add_argument("-c", "--checker",
                         help="Set checker command for client")
-    parser.add_argument("-t", "--timeout", type=string_to_delta,
+    parser.add_argument("-t", "--timeout",
                         help="Set timeout for client")
-    parser.add_argument("--extended-timeout", type=string_to_delta,
+    parser.add_argument("--extended-timeout",
                         help="Set extended timeout for client")
-    parser.add_argument("-i", "--interval", type=string_to_delta,
+    parser.add_argument("-i", "--interval",
                         help="Set checker interval for client")
-    approve_deny_default = parser.add_mutually_exclusive_group()
-    approve_deny_default.add_argument(
-        "--approve-by-default", action="store_true",
-        default=None, dest="approved_by_default",
-        help="Set client to be approved by default")
-    approve_deny_default.add_argument(
-        "--deny-by-default", action="store_false",
-        dest="approved_by_default",
-        help="Set client to be denied by default")
-    parser.add_argument("--approval-delay", type=string_to_delta,
+    parser.add_argument("--approve-by-default", action="store_true",
+                        default=None, dest="approved_by_default",
+                        help="Set client to be approved by default")
+    parser.add_argument("--deny-by-default", action="store_false",
+                        dest="approved_by_default",
+                        help="Set client to be denied by default")
+    parser.add_argument("--approval-delay",
                         help="Set delay before client approve/deny")
-    parser.add_argument("--approval-duration", type=string_to_delta,
+    parser.add_argument("--approval-duration",
                         help="Set duration of one client approval")
     parser.add_argument("-H", "--host", help="Set host for client")
-    parser.add_argument("-s", "--secret",
-                        type=argparse.FileType(mode="rb"),
+    parser.add_argument("-s", "--secret", type=file,
                         help="Set password blob (file) for client")
-    approve_deny = parser.add_mutually_exclusive_group()
-    approve_deny.add_argument(
-        "-A", "--approve", action="store_true",
-        help="Approve any current client request")
-    approve_deny.add_argument("-D", "--deny", action="store_true",
-                              help="Deny any current client request")
-    parser.add_argument("--debug", action="store_true",
-                        help="Debug mode (show D-Bus commands)")
-    parser.add_argument("--check", action="store_true",
-                        help="Run self-test")
+    parser.add_argument("-A", "--approve", action="store_true",
+                        help="Approve any current client request")
+    parser.add_argument("-D", "--deny", action="store_true",
+                        help="Deny any current client request")
     parser.add_argument("client", nargs="*", help="Client name")
-
-
-def commands_from_options(options):
-
-    commands = []
-
-    if options.dump_json:
-        commands.append(DumpJSONCmd())
-
-    if options.enable:
-        commands.append(EnableCmd())
-
-    if options.disable:
-        commands.append(DisableCmd())
-
-    if options.bump_timeout:
-        commands.append(BumpTimeoutCmd())
-
-    if options.start_checker:
-        commands.append(StartCheckerCmd())
-
-    if options.stop_checker:
-        commands.append(StopCheckerCmd())
-
-    if options.is_enabled:
-        commands.append(IsEnabledCmd())
-
-    if options.checker is not None:
-        commands.append(SetCheckerCmd(options.checker))
-
-    if options.timeout is not None:
-        commands.append(SetTimeoutCmd(options.timeout))
-
-    if options.extended_timeout:
-        commands.append(
-            SetExtendedTimeoutCmd(options.extended_timeout))
-
-    if options.interval is not None:
-        commands.append(SetIntervalCmd(options.interval))
-
-    if options.approved_by_default is not None:
-        if options.approved_by_default:
-            commands.append(ApproveByDefaultCmd())
-        else:
-            commands.append(DenyByDefaultCmd())
-
-    if options.approval_delay is not None:
-        commands.append(SetApprovalDelayCmd(options.approval_delay))
-
-    if options.approval_duration is not None:
-        commands.append(
-            SetApprovalDurationCmd(options.approval_duration))
-
-    if options.host is not None:
-        commands.append(SetHostCmd(options.host))
-
-    if options.secret is not None:
-        commands.append(SetSecretCmd(options.secret))
-
-    if options.approve:
-        commands.append(ApproveCmd())
-
-    if options.deny:
-        commands.append(DenyCmd())
-
-    if options.remove:
-        commands.append(RemoveCmd())
-
-    # If no command option has been given, show table of clients,
-    # optionally verbosely
-    if not commands:
-        commands.append(PrintTableCmd(verbose=options.verbose))
-
-    return commands
-
-
-def check_option_syntax(parser, options):
-    """Apply additional restrictions on options, not expressible in
-argparse"""
-
-    def has_actions(options):
-        return any((options.enable,
-                    options.disable,
-                    options.bump_timeout,
-                    options.start_checker,
-                    options.stop_checker,
-                    options.is_enabled,
-                    options.remove,
-                    options.checker is not None,
-                    options.timeout is not None,
-                    options.extended_timeout is not None,
-                    options.interval is not None,
-                    options.approved_by_default is not None,
-                    options.approval_delay is not None,
-                    options.approval_duration is not None,
-                    options.host is not None,
-                    options.secret is not None,
-                    options.approve,
-                    options.deny))
-
-    if has_actions(options) and not (options.client or options.all):
+    options = parser.parse_args()
+    
+    if has_actions(options) and not options.client and not options.all:
         parser.error("Options require clients names or --all.")
     if options.verbose and has_actions(options):
-        parser.error("--verbose can only be used alone.")
-    if options.dump_json and (options.verbose
-                              or has_actions(options)):
-        parser.error("--dump-json can only be used alone.")
+        parser.error("--verbose can only be used alone or with"
+                     " --all.")
     if options.all and not has_actions(options):
         parser.error("--all requires an action.")
-    if options.is_enabled and len(options.client) > 1:
-        parser.error("--is-enabled requires exactly one client")
-    if options.remove:
-        options.remove = False
-        if has_actions(options) and not options.deny:
-            parser.error("--remove can only be combined with --deny")
-        options.remove = True
-
-
-def main():
-    parser = argparse.ArgumentParser()
-
-    add_command_line_options(parser)
-
-    options = parser.parse_args()
-
-    check_option_syntax(parser, options)
-
-    clientnames = options.client
-
-    if options.debug:
-        log.setLevel(logging.DEBUG)
-
+    
     try:
         bus = dbus.SystemBus()
-        log.debug("D-Bus: Connect to: (name=%r, path=%r)", busname,
-                  server_path)
         mandos_dbus_objc = bus.get_object(busname, server_path)
     except dbus.exceptions.DBusException:
-        log.critical("Could not connect to Mandos server")
+        print("Could not connect to Mandos server",
+              file=sys.stderr)
         sys.exit(1)
-
+    
     mandos_serv = dbus.Interface(mandos_dbus_objc,
-                                 dbus_interface=server_interface)
-    mandos_serv_object_manager = dbus.Interface(
-        mandos_dbus_objc, dbus_interface=dbus.OBJECT_MANAGER_IFACE)
-
-    # Filter out log message from dbus module
-    dbus_logger = logging.getLogger("dbus.proxies")
-    class NullFilter(logging.Filter):
-        def filter(self, record):
-            return False
-    dbus_filter = NullFilter()
+                                 dbus_interface = server_interface)
+    
+    #block stderr since dbus library prints to stderr
+    null = os.open(os.path.devnull, os.O_RDWR)
+    stderrcopy = os.dup(sys.stderr.fileno())
+    os.dup2(null, sys.stderr.fileno())
+    os.close(null)
     try:
-        dbus_logger.addFilter(dbus_filter)
-        log.debug("D-Bus: %s:%s:%s.GetManagedObjects()", busname,
-                  server_path, dbus.OBJECT_MANAGER_IFACE)
-        mandos_clients = {path: ifs_and_props[client_interface]
-                          for path, ifs_and_props in
-                          mandos_serv_object_manager
-                          .GetManagedObjects().items()
-                          if client_interface in ifs_and_props}
-    except dbus.exceptions.DBusException as e:
-        log.critical("Failed to access Mandos server through D-Bus:"
-                     "\n%s", e)
+        try:
+            mandos_clients = mandos_serv.GetAllClientsWithProperties()
+        finally:
+            #restore stderr
+            os.dup2(stderrcopy, sys.stderr.fileno())
+            os.close(stderrcopy)
+    except dbus.exceptions.DBusException, e:
+        print("Access denied: Accessing mandos server through dbus.",
+              file=sys.stderr)
         sys.exit(1)
-    finally:
-        # restore dbus logger
-        dbus_logger.removeFilter(dbus_filter)
-
+    
     # Compile dict of (clients: properties) to process
-    clients = {}
-
-    if not clientnames:
-        clients = {bus.get_object(busname, path): properties
-                   for path, properties in mandos_clients.items()}
+    clients={}
+    
+    if options.all or not options.client:
+        clients = dict((bus.get_object(busname, path), properties)
+                       for path, properties in
+                       mandos_clients.iteritems())
     else:
-        for name in clientnames:
-            for path, client in mandos_clients.items():
+        for name in options.client:
+            for path, client in mandos_clients.iteritems():
                 if client["Name"] == name:
                     client_objc = bus.get_object(busname, path)
                     clients[client_objc] = client
                     break
             else:
-                log.critical("Client not found on server: %r", name)
+                print("Client not found on server: %r" % name,
+                      file=sys.stderr)
                 sys.exit(1)
-
-    # Run all commands on clients
-    commands = commands_from_options(options)
-    for command in commands:
-        command.run(mandos_serv, mandos_serv_object_manager, clients)
-
-
-class Test_milliseconds_to_string(unittest.TestCase):
-    def test_all(self):
-        self.assertEqual(milliseconds_to_string(93785000),
-                         "1T02:03:05")
-    def test_no_days(self):
-        self.assertEqual(milliseconds_to_string(7385000), "02:03:05")
-    def test_all_zero(self):
-        self.assertEqual(milliseconds_to_string(0), "00:00:00")
-    def test_no_fractional_seconds(self):
-        self.assertEqual(milliseconds_to_string(400), "00:00:00")
-        self.assertEqual(milliseconds_to_string(900), "00:00:00")
-        self.assertEqual(milliseconds_to_string(1900), "00:00:01")
-
-class Test_string_to_delta(unittest.TestCase):
-    def test_handles_basic_rfc3339(self):
-        self.assertEqual(string_to_delta("PT0S"),
-                         datetime.timedelta())
-        self.assertEqual(string_to_delta("P0D"),
-                         datetime.timedelta())
-        self.assertEqual(string_to_delta("PT1S"),
-                         datetime.timedelta(0, 1))
-        self.assertEqual(string_to_delta("PT2H"),
-                         datetime.timedelta(0, 7200))
-    def test_falls_back_to_pre_1_6_1_with_warning(self):
-        # assertLogs only exists in Python 3.4
-        if hasattr(self, "assertLogs"):
-            with self.assertLogs(log, logging.WARNING):
-                value = string_to_delta("2h")
-        else:
-            class WarningFilter(logging.Filter):
-                """Don't show, but record the presence of, warnings"""
-                def filter(self, record):
-                    is_warning = record.levelno >= logging.WARNING
-                    self.found = is_warning or getattr(self, "found",
-                                                       False)
-                    return not is_warning
-            warning_filter = WarningFilter()
-            log.addFilter(warning_filter)
-            try:
-                value = string_to_delta("2h")
-            finally:
-                log.removeFilter(warning_filter)
-            self.assertTrue(getattr(warning_filter, "found", False))
-        self.assertEqual(value, datetime.timedelta(0, 7200))
-
-
-class TestCmd(unittest.TestCase):
-    """Abstract class for tests of command classes"""
-    def setUp(self):
-        testcase = self
-        class MockClient(object):
-            def __init__(self, name, **attributes):
-                self.__dbus_object_path__ = "objpath_{}".format(name)
-                self.attributes = attributes
-                self.attributes["Name"] = name
-                self.calls = []
-            def Set(self, interface, property, value, dbus_interface):
-                testcase.assertEqual(interface, client_interface)
-                testcase.assertEqual(dbus_interface,
-                                     dbus.PROPERTIES_IFACE)
-                self.attributes[property] = value
-            def Get(self, interface, property, dbus_interface):
-                testcase.assertEqual(interface, client_interface)
-                testcase.assertEqual(dbus_interface,
-                                     dbus.PROPERTIES_IFACE)
-                return self.attributes[property]
-            def Approve(self, approve, dbus_interface):
-                testcase.assertEqual(dbus_interface, client_interface)
-                self.calls.append(("Approve", (approve,
-                                               dbus_interface)))
-        self.client = MockClient(
-            "foo",
-            KeyID=("92ed150794387c03ce684574b1139a65"
-                   "94a34f895daaaf09fd8ea90a27cddb12"),
-            Secret=b"secret",
-            Host="foo.example.org",
-            Enabled=dbus.Boolean(True),
-            Timeout=300000,
-            LastCheckedOK="2019-02-03T00:00:00",
-            Created="2019-01-02T00:00:00",
-            Interval=120000,
-            Fingerprint=("778827225BA7DE539C5A"
-                         "7CFA59CFF7CDBD9A5920"),
-            CheckerRunning=dbus.Boolean(False),
-            LastEnabled="2019-01-03T00:00:00",
-            ApprovalPending=dbus.Boolean(False),
-            ApprovedByDefault=dbus.Boolean(True),
-            LastApprovalRequest="",
-            ApprovalDelay=0,
-            ApprovalDuration=1000,
-            Checker="fping -q -- %(host)s",
-            ExtendedTimeout=900000,
-            Expires="2019-02-04T00:00:00",
-            LastCheckerStatus=0)
-        self.other_client = MockClient(
-            "barbar",
-            KeyID=("0558568eedd67d622f5c83b35a115f79"
-                   "6ab612cff5ad227247e46c2b020f441c"),
-            Secret=b"secretbar",
-            Host="192.0.2.3",
-            Enabled=dbus.Boolean(True),
-            Timeout=300000,
-            LastCheckedOK="2019-02-04T00:00:00",
-            Created="2019-01-03T00:00:00",
-            Interval=120000,
-            Fingerprint=("3E393AEAEFB84C7E89E2"
-                         "F547B3A107558FCA3A27"),
-            CheckerRunning=dbus.Boolean(True),
-            LastEnabled="2019-01-04T00:00:00",
-            ApprovalPending=dbus.Boolean(False),
-            ApprovedByDefault=dbus.Boolean(False),
-            LastApprovalRequest="2019-01-03T00:00:00",
-            ApprovalDelay=30000,
-            ApprovalDuration=1000,
-            Checker=":",
-            ExtendedTimeout=900000,
-            Expires="2019-02-05T00:00:00",
-            LastCheckerStatus=-2)
-        self.clients =  collections.OrderedDict(
-            [
-                (self.client, self.client.attributes),
-                (self.other_client, self.other_client.attributes),
-            ])
-        self.one_client = {self.client: self.client.attributes}
-
-class TestPrintTableCmd(TestCmd):
-    def test_normal(self):
-        output = PrintTableCmd().output(None, self.clients.values())
-        expected_output = """
-Name   Enabled Timeout  Last Successful Check
-foo    Yes     00:05:00 2019-02-03T00:00:00  
-barbar Yes     00:05:00 2019-02-04T00:00:00  
-"""[1:-1]
-        self.assertEqual(output, expected_output)
-    def test_verbose(self):
-        output = PrintTableCmd(verbose=True).output(
-            None, self.clients.values())
-        expected_output = """
-Name   Enabled Timeout  Last Successful Check Created             Interval Host            Key ID                                                           Fingerprint                              Check Is Running Last Enabled        Approval Is Pending Approved By Default Last Approval Request Approval Delay Approval Duration Checker              Extended Timeout Expires             Last Checker Status
-foo    Yes     00:05:00 2019-02-03T00:00:00   2019-01-02T00:00:00 00:02:00 foo.example.org 92ed150794387c03ce684574b1139a6594a34f895daaaf09fd8ea90a27cddb12 778827225BA7DE539C5A7CFA59CFF7CDBD9A5920 No               2019-01-03T00:00:00 No                  Yes                                       00:00:00       00:00:01          fping -q -- %(host)s 00:15:00         2019-02-04T00:00:00 0                  
-barbar Yes     00:05:00 2019-02-04T00:00:00   2019-01-03T00:00:00 00:02:00 192.0.2.3       0558568eedd67d622f5c83b35a115f796ab612cff5ad227247e46c2b020f441c 3E393AEAEFB84C7E89E2F547B3A107558FCA3A27 Yes              2019-01-04T00:00:00 No                  No                  2019-01-03T00:00:00   00:00:30       00:00:01          :                    00:15:00         2019-02-05T00:00:00 -2                 
-"""[1:-1]
-        self.assertEqual(output, expected_output)
-    def test_one_client(self):
-        output = PrintTableCmd().output(None, self.one_client.values())
-        expected_output = """
-Name Enabled Timeout  Last Successful Check
-foo  Yes     00:05:00 2019-02-03T00:00:00  
-"""[1:-1]
-        self.assertEqual(output, expected_output)
-
-class TestDumpJSONCmd(TestCmd):
-    def setUp(self):
-        self.expected_json = {
-            "foo": {
-                "Name": "foo",
-                "KeyID": ("92ed150794387c03ce684574b1139a65"
-                          "94a34f895daaaf09fd8ea90a27cddb12"),
-                "Host": "foo.example.org",
-                "Enabled": True,
-                "Timeout": 300000,
-                "LastCheckedOK": "2019-02-03T00:00:00",
-                "Created": "2019-01-02T00:00:00",
-                "Interval": 120000,
-                "Fingerprint": ("778827225BA7DE539C5A"
-                                "7CFA59CFF7CDBD9A5920"),
-                "CheckerRunning": False,
-                "LastEnabled": "2019-01-03T00:00:00",
-                "ApprovalPending": False,
-                "ApprovedByDefault": True,
-                "LastApprovalRequest": "",
-                "ApprovalDelay": 0,
-                "ApprovalDuration": 1000,
-                "Checker": "fping -q -- %(host)s",
-                "ExtendedTimeout": 900000,
-                "Expires": "2019-02-04T00:00:00",
-                "LastCheckerStatus": 0,
-            },
-            "barbar": {
-                "Name": "barbar",
-                "KeyID": ("0558568eedd67d622f5c83b35a115f79"
-                          "6ab612cff5ad227247e46c2b020f441c"),
-                "Host": "192.0.2.3",
-                "Enabled": True,
-                "Timeout": 300000,
-                "LastCheckedOK": "2019-02-04T00:00:00",
-                "Created": "2019-01-03T00:00:00",
-                "Interval": 120000,
-                "Fingerprint": ("3E393AEAEFB84C7E89E2"
-                                "F547B3A107558FCA3A27"),
-                "CheckerRunning": True,
-                "LastEnabled": "2019-01-04T00:00:00",
-                "ApprovalPending": False,
-                "ApprovedByDefault": False,
-                "LastApprovalRequest": "2019-01-03T00:00:00",
-                "ApprovalDelay": 30000,
-                "ApprovalDuration": 1000,
-                "Checker": ":",
-                "ExtendedTimeout": 900000,
-                "Expires": "2019-02-05T00:00:00",
-                "LastCheckerStatus": -2,
-            },
-        }
-        return super(TestDumpJSONCmd, self).setUp()
-    def test_normal(self):
-        json_data = json.loads(DumpJSONCmd().output(None,
-                                                    self.clients))
-        self.assertDictEqual(json_data, self.expected_json)
-    def test_one_client(self):
-        clients = self.one_client
-        json_data = json.loads(DumpJSONCmd().output(None, clients))
-        expected_json = {"foo": self.expected_json["foo"]}
-        self.assertDictEqual(json_data, expected_json)
-
-class TestIsEnabledCmd(TestCmd):
-    def test_is_enabled(self):
-        self.assertTrue(all(IsEnabledCmd().is_enabled(client, properties)
-                            for client, properties in self.clients.items()))
-    def test_is_enabled_run_exits_successfully(self):
-        with self.assertRaises(SystemExit) as e:
-            IsEnabledCmd().run(None, None, self.one_client)
-        if e.exception.code is not None:
-            self.assertEqual(e.exception.code, 0)
-        else:
-            self.assertIsNone(e.exception.code)
-    def test_is_enabled_run_exits_with_failure(self):
-        self.client.attributes["Enabled"] = dbus.Boolean(False)
-        with self.assertRaises(SystemExit) as e:
-            IsEnabledCmd().run(None, None, self.one_client)
-        if isinstance(e.exception.code, int):
-            self.assertNotEqual(e.exception.code, 0)
-        else:
-            self.assertIsNotNone(e.exception.code)
-
-class TestRemoveCmd(TestCmd):
-    def test_remove(self):
-        class MockMandos(object):
-            def __init__(self):
-                self.calls = []
-            def RemoveClient(self, dbus_path):
-                self.calls.append(("RemoveClient", (dbus_path,)))
-        mandos = MockMandos()
-        super(TestRemoveCmd, self).setUp()
-        RemoveCmd().run(mandos, None, self.clients)
-        self.assertEqual(len(mandos.calls), 2)
-        for client in self.clients:
-            self.assertIn(("RemoveClient",
-                           (client.__dbus_object_path__,)),
-                          mandos.calls)
-
-class TestApproveCmd(TestCmd):
-    def test_approve(self):
-        ApproveCmd().run(None, None, self.clients)
-        for client in self.clients:
-            self.assertIn(("Approve", (True, client_interface)),
-                          client.calls)
-
-class TestDenyCmd(TestCmd):
-    def test_deny(self):
-        DenyCmd().run(None, None, self.clients)
-        for client in self.clients:
-            self.assertIn(("Approve", (False, client_interface)),
-                          client.calls)
-
-class TestEnableCmd(TestCmd):
-    def test_enable(self):
-        for client in self.clients:
-            client.attributes["Enabled"] = False
-
-        EnableCmd().run(None, None, self.clients)
-
-        for client in self.clients:
-            self.assertTrue(client.attributes["Enabled"])
-
-class TestDisableCmd(TestCmd):
-    def test_disable(self):
-        DisableCmd().run(None, None, self.clients)
-
-        for client in self.clients:
-            self.assertFalse(client.attributes["Enabled"])
-
-class Unique(object):
-    """Class for objects which exist only to be unique objects, since
-unittest.mock.sentinel only exists in Python 3.3"""
-
-class TestPropertyCmd(TestCmd):
-    """Abstract class for tests of PropertyCmd classes"""
-    def runTest(self):
-        if not hasattr(self, "command"):
-            return
-        values_to_get = getattr(self, "values_to_get",
-                                self.values_to_set)
-        for value_to_set, value_to_get in zip(self.values_to_set,
-                                              values_to_get):
-            for client in self.clients:
-                old_value = client.attributes[self.property]
-                self.assertNotIsInstance(old_value, Unique)
-                client.attributes[self.property] = Unique()
-            self.run_command(value_to_set, self.clients)
-            for client in self.clients:
-                value = client.attributes[self.property]
-                self.assertNotIsInstance(value, Unique)
-                self.assertEqual(value, value_to_get)
-    def run_command(self, value, clients):
-        self.command().run(None, None, clients)
-
-class TestBumpTimeoutCmd(TestPropertyCmd):
-    command = BumpTimeoutCmd
-    property = "LastCheckedOK"
-    values_to_set = [""]
-
-class TestStartCheckerCmd(TestPropertyCmd):
-    command = StartCheckerCmd
-    property = "CheckerRunning"
-    values_to_set = [dbus.Boolean(True)]
-
-class TestStopCheckerCmd(TestPropertyCmd):
-    command = StopCheckerCmd
-    property = "CheckerRunning"
-    values_to_set = [dbus.Boolean(False)]
-
-class TestApproveByDefaultCmd(TestPropertyCmd):
-    command = ApproveByDefaultCmd
-    property = "ApprovedByDefault"
-    values_to_set = [dbus.Boolean(True)]
-
-class TestDenyByDefaultCmd(TestPropertyCmd):
-    command = DenyByDefaultCmd
-    property = "ApprovedByDefault"
-    values_to_set = [dbus.Boolean(False)]
-
-class TestValueArgumentPropertyCmd(TestPropertyCmd):
-    """Abstract class for tests of PropertyCmd classes using the
-ValueArgumentMixIn"""
-    def runTest(self):
-        if type(self) is TestValueArgumentPropertyCmd:
-            return
-        return super(TestValueArgumentPropertyCmd, self).runTest()
-    def run_command(self, value, clients):
-        self.command(value).run(None, None, clients)
-
-class TestSetCheckerCmd(TestValueArgumentPropertyCmd):
-    command = SetCheckerCmd
-    property = "Checker"
-    values_to_set = ["", ":", "fping -q -- %s"]
-
-class TestSetHostCmd(TestValueArgumentPropertyCmd):
-    command = SetHostCmd
-    property = "Host"
-    values_to_set = ["192.0.2.3", "foo.example.org"]
-
-class TestSetSecretCmd(TestValueArgumentPropertyCmd):
-    command = SetSecretCmd
-    property = "Secret"
-    values_to_set = [io.BytesIO(b""),
-                     io.BytesIO(b"secret\0xyzzy\nbar")]
-    values_to_get = [b"", b"secret\0xyzzy\nbar"]
-
-class TestSetTimeoutCmd(TestValueArgumentPropertyCmd):
-    command = SetTimeoutCmd
-    property = "Timeout"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class TestSetExtendedTimeoutCmd(TestValueArgumentPropertyCmd):
-    command = SetExtendedTimeoutCmd
-    property = "ExtendedTimeout"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class TestSetIntervalCmd(TestValueArgumentPropertyCmd):
-    command = SetIntervalCmd
-    property = "Interval"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class TestSetApprovalDelayCmd(TestValueArgumentPropertyCmd):
-    command = SetApprovalDelayCmd
-    property = "ApprovalDelay"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class TestSetApprovalDurationCmd(TestValueArgumentPropertyCmd):
-    command = SetApprovalDurationCmd
-    property = "ApprovalDuration"
-    values_to_set = [datetime.timedelta(),
-                     datetime.timedelta(minutes=5),
-                     datetime.timedelta(seconds=1),
-                     datetime.timedelta(weeks=1),
-                     datetime.timedelta(weeks=52)]
-    values_to_get = [0, 300000, 1000, 604800000, 31449600000]
-
-class Test_command_from_options(unittest.TestCase):
-    def setUp(self):
-        self.parser = argparse.ArgumentParser()
-        add_command_line_options(self.parser)
-    def assert_command_from_args(self, args, command_cls, **cmd_attrs):
-        """Assert that parsing ARGS should result in an instance of
-COMMAND_CLS with (optionally) all supplied attributes (CMD_ATTRS)."""
-        options = self.parser.parse_args(args)
-        check_option_syntax(self.parser, options)
-        commands = commands_from_options(options)
-        self.assertEqual(len(commands), 1)
-        command = commands[0]
-        self.assertIsInstance(command, command_cls)
-        for key, value in cmd_attrs.items():
-            self.assertEqual(getattr(command, key), value)
-    def test_print_table(self):
-        self.assert_command_from_args([], PrintTableCmd,
-                                      verbose=False)
-
-    def test_print_table_verbose(self):
-        self.assert_command_from_args(["--verbose"], PrintTableCmd,
-                                      verbose=True)
-
-    def test_print_table_verbose_short(self):
-        self.assert_command_from_args(["-v"], PrintTableCmd,
-                                      verbose=True)
-
-    def test_enable(self):
-        self.assert_command_from_args(["--enable", "foo"], EnableCmd)
-
-    def test_enable_short(self):
-        self.assert_command_from_args(["-e", "foo"], EnableCmd)
-
-    def test_disable(self):
-        self.assert_command_from_args(["--disable", "foo"],
-                                      DisableCmd)
-
-    def test_disable_short(self):
-        self.assert_command_from_args(["-d", "foo"], DisableCmd)
-
-    def test_bump_timeout(self):
-        self.assert_command_from_args(["--bump-timeout", "foo"],
-                                      BumpTimeoutCmd)
-
-    def test_bump_timeout_short(self):
-        self.assert_command_from_args(["-b", "foo"], BumpTimeoutCmd)
-
-    def test_start_checker(self):
-        self.assert_command_from_args(["--start-checker", "foo"],
-                                      StartCheckerCmd)
-
-    def test_stop_checker(self):
-        self.assert_command_from_args(["--stop-checker", "foo"],
-                                      StopCheckerCmd)
-
-    def test_remove(self):
-        self.assert_command_from_args(["--remove", "foo"],
-                                      RemoveCmd)
-
-    def test_remove_short(self):
-        self.assert_command_from_args(["-r", "foo"], RemoveCmd)
-
-    def test_checker(self):
-        self.assert_command_from_args(["--checker", ":", "foo"],
-                                      SetCheckerCmd, value_to_set=":")
-
-    def test_checker_empty(self):
-        self.assert_command_from_args(["--checker", "", "foo"],
-                                      SetCheckerCmd, value_to_set="")
-
-    def test_checker_short(self):
-        self.assert_command_from_args(["-c", ":", "foo"],
-                                      SetCheckerCmd, value_to_set=":")
-
-    def test_timeout(self):
-        self.assert_command_from_args(["--timeout", "PT5M", "foo"],
-                                      SetTimeoutCmd,
-                                      value_to_set=300000)
-
-    def test_timeout_short(self):
-        self.assert_command_from_args(["-t", "PT5M", "foo"],
-                                      SetTimeoutCmd,
-                                      value_to_set=300000)
-
-    def test_extended_timeout(self):
-        self.assert_command_from_args(["--extended-timeout", "PT15M",
-                                       "foo"],
-                                      SetExtendedTimeoutCmd,
-                                      value_to_set=900000)
-
-    def test_interval(self):
-        self.assert_command_from_args(["--interval", "PT2M", "foo"],
-                                      SetIntervalCmd,
-                                      value_to_set=120000)
-
-    def test_interval_short(self):
-        self.assert_command_from_args(["-i", "PT2M", "foo"],
-                                      SetIntervalCmd,
-                                      value_to_set=120000)
-
-    def test_approve_by_default(self):
-        self.assert_command_from_args(["--approve-by-default", "foo"],
-                                      ApproveByDefaultCmd)
-
-    def test_deny_by_default(self):
-        self.assert_command_from_args(["--deny-by-default", "foo"],
-                                      DenyByDefaultCmd)
-
-    def test_approval_delay(self):
-        self.assert_command_from_args(["--approval-delay", "PT30S",
-                                       "foo"], SetApprovalDelayCmd,
-                                      value_to_set=30000)
-
-    def test_approval_duration(self):
-        self.assert_command_from_args(["--approval-duration", "PT1S",
-                                       "foo"], SetApprovalDurationCmd,
-                                      value_to_set=1000)
-
-    def test_host(self):
-        self.assert_command_from_args(["--host", "foo.example.org",
-                                       "foo"], SetHostCmd,
-                                      value_to_set="foo.example.org")
-
-    def test_host_short(self):
-        self.assert_command_from_args(["-H", "foo.example.org",
-                                       "foo"], SetHostCmd,
-                                      value_to_set="foo.example.org")
-
-    def test_secret_devnull(self):
-        self.assert_command_from_args(["--secret", os.path.devnull,
-                                       "foo"], SetSecretCmd,
-                                      value_to_set=b"")
-
-    def test_secret_tempfile(self):
-        with tempfile.NamedTemporaryFile(mode="r+b") as f:
-            value = b"secret\0xyzzy\nbar"
-            f.write(value)
-            f.seek(0)
-            self.assert_command_from_args(["--secret", f.name,
-                                           "foo"], SetSecretCmd,
-                                          value_to_set=value)
-
-    def test_secret_devnull_short(self):
-        self.assert_command_from_args(["-s", os.path.devnull, "foo"],
-                                      SetSecretCmd, value_to_set=b"")
-
-    def test_secret_tempfile_short(self):
-        with tempfile.NamedTemporaryFile(mode="r+b") as f:
-            value = b"secret\0xyzzy\nbar"
-            f.write(value)
-            f.seek(0)
-            self.assert_command_from_args(["-s", f.name, "foo"],
-                                          SetSecretCmd,
-                                          value_to_set=value)
-
-    def test_approve(self):
-        self.assert_command_from_args(["--approve", "foo"],
-                                      ApproveCmd)
-
-    def test_approve_short(self):
-        self.assert_command_from_args(["-A", "foo"], ApproveCmd)
-
-    def test_deny(self):
-        self.assert_command_from_args(["--deny", "foo"], DenyCmd)
-
-    def test_deny_short(self):
-        self.assert_command_from_args(["-D", "foo"], DenyCmd)
-
-    def test_dump_json(self):
-        self.assert_command_from_args(["--dump-json"], DumpJSONCmd)
-
-    def test_is_enabled(self):
-        self.assert_command_from_args(["--is-enabled", "foo"],
-                                      IsEnabledCmd)
-
-    def test_is_enabled_short(self):
-        self.assert_command_from_args(["-V", "foo"], IsEnabledCmd)
-
-    def test_deny_before_remove(self):
-        options = self.parser.parse_args(["--deny", "--remove", "foo"])
-        check_option_syntax(self.parser, options)
-        commands = commands_from_options(options)
-        self.assertEqual(len(commands), 2)
-        self.assertIsInstance(commands[0], DenyCmd)
-        self.assertIsInstance(commands[1], RemoveCmd)
-
-    def test_deny_before_remove_reversed(self):
-        options = self.parser.parse_args(["--remove", "--deny", "--all"])
-        check_option_syntax(self.parser, options)
-        commands = commands_from_options(options)
-        self.assertEqual(len(commands), 2)
-        self.assertIsInstance(commands[0], DenyCmd)
-        self.assertIsInstance(commands[1], RemoveCmd)
-
-
-class Test_check_option_syntax(unittest.TestCase):
-    # This mostly corresponds to the definition from has_actions() in
-    # check_option_syntax()
-    actions = {
-        # The actual values set here are not that important, but we do
-        # at least stick to the correct types, even though they are
-        # never used
-        "enable": True,
-        "disable": True,
-        "bump_timeout": True,
-        "start_checker": True,
-        "stop_checker": True,
-        "is_enabled": True,
-        "remove": True,
-        "checker": "x",
-        "timeout": datetime.timedelta(),
-        "extended_timeout": datetime.timedelta(),
-        "interval": datetime.timedelta(),
-        "approved_by_default": True,
-        "approval_delay": datetime.timedelta(),
-        "approval_duration": datetime.timedelta(),
-        "host": "x",
-        "secret": io.BytesIO(b"x"),
-        "approve": True,
-        "deny": True,
-    }
-
-    def setUp(self):
-        self.parser = argparse.ArgumentParser()
-        add_command_line_options(self.parser)
-
-    @contextlib.contextmanager
-    def assertParseError(self):
-        with self.assertRaises(SystemExit) as e:
-            with self.temporarily_suppress_stderr():
-                yield
-        # Exit code from argparse is guaranteed to be "2".  Reference:
-        # https://docs.python.org/3/library/argparse.html#exiting-methods
-        self.assertEqual(e.exception.code, 2)
-
-    @staticmethod
-    @contextlib.contextmanager
-    def temporarily_suppress_stderr():
-        null = os.open(os.path.devnull, os.O_RDWR)
-        stderrcopy = os.dup(sys.stderr.fileno())
-        os.dup2(null, sys.stderr.fileno())
-        os.close(null)
-        try:
-            yield
-        finally:
-            # restore stderr
-            os.dup2(stderrcopy, sys.stderr.fileno())
-            os.close(stderrcopy)
-
-    def check_option_syntax(self, options):
-        check_option_syntax(self.parser, options)
-
-    def test_actions_requires_client_or_all(self):
-        for action, value in self.actions.items():
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            with self.assertParseError():
-                self.check_option_syntax(options)
-
-    def test_actions_conflicts_with_verbose(self):
-        for action, value in self.actions.items():
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            options.verbose = True
-            with self.assertParseError():
-                self.check_option_syntax(options)
-
-    def test_dump_json_conflicts_with_verbose(self):
-        options = self.parser.parse_args()
-        options.dump_json = True
-        options.verbose = True
-        with self.assertParseError():
-            self.check_option_syntax(options)
-
-    def test_dump_json_conflicts_with_action(self):
-        for action, value in self.actions.items():
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            options.dump_json = True
-            with self.assertParseError():
-                self.check_option_syntax(options)
-
-    def test_all_can_not_be_alone(self):
-        options = self.parser.parse_args()
-        options.all = True
-        with self.assertParseError():
-            self.check_option_syntax(options)
-
-    def test_all_is_ok_with_any_action(self):
-        for action, value in self.actions.items():
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            options.all = True
-            self.check_option_syntax(options)
-
-    def test_is_enabled_fails_without_client(self):
-        options = self.parser.parse_args()
-        options.is_enabled = True
-        with self.assertParseError():
-            self.check_option_syntax(options)
-
-    def test_is_enabled_works_with_one_client(self):
-        options = self.parser.parse_args()
-        options.is_enabled = True
-        options.client = ["foo"]
-        self.check_option_syntax(options)
-
-    def test_is_enabled_fails_with_two_clients(self):
-        options = self.parser.parse_args()
-        options.is_enabled = True
-        options.client = ["foo", "barbar"]
-        with self.assertParseError():
-            self.check_option_syntax(options)
-
-    def test_remove_can_only_be_combined_with_action_deny(self):
-        for action, value in self.actions.items():
-            if action in {"remove", "deny"}:
-                continue
-            options = self.parser.parse_args()
-            setattr(options, action, value)
-            options.all = True
-            options.remove = True
-            with self.assertParseError():
-                self.check_option_syntax(options)
-
-
-
-def should_only_run_tests():
-    parser = argparse.ArgumentParser(add_help=False)
-    parser.add_argument("--check", action='store_true')
-    args, unknown_args = parser.parse_known_args()
-    run_tests = args.check
-    if run_tests:
-        # Remove --check argument from sys.argv
-        sys.argv[1:] = unknown_args
-    return run_tests
-
-# Add all tests from doctest strings
-def load_tests(loader, tests, none):
-    import doctest
-    tests.addTests(doctest.DocTestSuite())
-    return tests
+    
+    if not has_actions(options) and clients:
+        if options.verbose:
+            keywords = ("Name", "Enabled", "Timeout",
+                        "LastCheckedOK", "Created", "Interval",
+                        "Host", "Fingerprint", "CheckerRunning",
+                        "LastEnabled", "ApprovalPending",
+                        "ApprovedByDefault",
+                        "LastApprovalRequest", "ApprovalDelay",
+                        "ApprovalDuration", "Checker",
+                        "ExtendedTimeout")
+        else:
+            keywords = defaultkeywords
+        
+        print_clients(clients.values(), keywords)
+    else:
+        # Process each client in the list by all selected options
+        for client in clients:
+            if options.remove:
+                mandos_serv.RemoveClient(client.__dbus_object_path__)
+            if options.enable:
+                client.Enable(dbus_interface=client_interface)
+            if options.disable:
+                client.Disable(dbus_interface=client_interface)
+            if options.bump_timeout:
+                client.CheckedOK(dbus_interface=client_interface)
+            if options.start_checker:
+                client.StartChecker(dbus_interface=client_interface)
+            if options.stop_checker:
+                client.StopChecker(dbus_interface=client_interface)
+            if options.is_enabled:
+                sys.exit(0 if client.Get(client_interface,
+                                         "Enabled",
+                                         dbus_interface=
+                                         dbus.PROPERTIES_IFACE)
+                         else 1)
+            if options.checker:
+                client.Set(client_interface, "Checker",
+                           options.checker,
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.host:
+                client.Set(client_interface, "Host", options.host,
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.interval:
+                client.Set(client_interface, "Interval",
+                           timedelta_to_milliseconds
+                           (string_to_delta(options.interval)),
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.approval_delay:
+                client.Set(client_interface, "ApprovalDelay",
+                           timedelta_to_milliseconds
+                           (string_to_delta(options.
+                                            approval_delay)),
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.approval_duration:
+                client.Set(client_interface, "ApprovalDuration",
+                           timedelta_to_milliseconds
+                           (string_to_delta(options.
+                                            approval_duration)),
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.timeout:
+                client.Set(client_interface, "Timeout",
+                           timedelta_to_milliseconds
+                           (string_to_delta(options.timeout)),
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.extended_timeout:
+                client.Set(client_interface, "ExtendedTimeout",
+                           timedelta_to_milliseconds
+                           (string_to_delta(options.extended_timeout)),
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.secret:
+                client.Set(client_interface, "Secret",
+                           dbus.ByteArray(open(options.secret,
+                                               "rb").read()),
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.approved_by_default is not None:
+                client.Set(client_interface, "ApprovedByDefault",
+                           dbus.Boolean(options
+                                        .approved_by_default),
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            if options.approve:
+                client.Approve(dbus.Boolean(True),
+                               dbus_interface=client_interface)
+            elif options.deny:
+                client.Approve(dbus.Boolean(False),
+                               dbus_interface=client_interface)
 
 if __name__ == "__main__":
-    if should_only_run_tests():
-        # Call using ./tdd-python-script --check [--verbose]
-        unittest.main()
-    else:
-        main()
+    main()

=== modified file 'mandos-ctl.xml'
--- mandos-ctl.xml	2019-03-08 23:55:34 +0000
+++ mandos-ctl.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-ctl">
-<!ENTITY TIMESTAMP "2019-03-09">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,14 +33,6 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -55,7 +47,7 @@
   <refnamediv>
     <refname><command>&COMMANDNAME;</command></refname>
     <refpurpose>
-      Control or query the operation of the Mandos server
+      Control the operation of the Mandos server
     </refpurpose>
   </refnamediv>
   
@@ -63,127 +55,103 @@
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
       <group>
-	  <arg choice="plain"><option>--verbose</option></arg>
-	  <arg choice="plain"><option>-v</option></arg>
-	  <sbr/>
-	  <arg choice="plain"><option>--dump-json</option></arg>
-	  <arg choice="plain"><option>-j</option></arg>
-      </group>
-      <arg><option>--debug</option></arg>
-      <group>
-	<arg rep='repeat' choice='plain'>
-	  <replaceable>CLIENT</replaceable>
-	</arg>
-      </group>
-    </cmdsynopsis>
-    <cmdsynopsis>
-      <command>&COMMANDNAME;</command>
-      <group choice="req">
-	<group>
-	  <arg choice="plain"><option>--enable</option></arg>
-	  <arg choice="plain"><option>-e</option></arg>
-	  <sbr/>
-	  <arg choice="plain"><option>--disable</option></arg>
-	  <arg choice="plain"><option>-d</option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--bump-timeout</option></arg>
-	  <arg choice="plain"><option>-b</option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--start-checker</option></arg>
-	  <arg choice="plain"><option>--stop-checker</option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--checker
-	  <replaceable>COMMAND</replaceable></option></arg>
-	  <arg choice="plain"><option>-c
-	  <replaceable>COMMAND</replaceable></option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--timeout
-	  <replaceable>TIME</replaceable></option></arg>
-	  <arg choice="plain"><option>-t
-	  <replaceable>TIME</replaceable></option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--extended-timeout
-	  <replaceable>TIME</replaceable></option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--interval
-	  <replaceable>TIME</replaceable></option></arg>
-	  <arg choice="plain"><option>-i
-	  <replaceable>TIME</replaceable></option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--approve-by-default</option
-          ></arg>
-	  <sbr/>
-	  <arg choice="plain"><option>--deny-by-default</option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--approval-delay
-	  <replaceable>TIME</replaceable></option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--approval-duration
-	  <replaceable>TIME</replaceable></option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--host
-	  <replaceable>STRING</replaceable></option></arg>
-	  <arg choice="plain"><option>-H
-	  <replaceable>STRING</replaceable></option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--secret
-	  <replaceable>FILENAME</replaceable></option></arg>
-	  <arg choice="plain"><option>-s
-	  <replaceable>FILENAME</replaceable></option></arg>
-	</group>
-	<sbr/>
-	<group>
-	  <arg choice="plain"><option>--approve</option></arg>
-	  <arg choice="plain"><option>-A</option></arg>
-	  <sbr/>
-	  <arg choice="plain"><option>--deny</option></arg>
-	  <arg choice="plain"><option>-D</option></arg>
-	</group>
-      </group>
-      <sbr/>
-      <arg><option>--debug</option></arg>
-      <group choice="req">
-	<arg choice="plain"><option>--all</option></arg>
-	<arg choice="plain"><option>-a</option></arg>
-	<arg rep='repeat' choice='plain'>
-	  <replaceable>CLIENT</replaceable>
-	</arg>
-      </group>
-    </cmdsynopsis>
-    <cmdsynopsis>
-      <command>&COMMANDNAME;</command>
-      <group>
+	<arg choice="plain"><option>--enable</option></arg>
+	<arg choice="plain"><option>-e</option></arg>
+	<sbr/>
+	<arg choice="plain"><option>--disable</option></arg>
+	<arg choice="plain"><option>-d</option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--bump-timeout</option></arg>
+	<arg choice="plain"><option>-b</option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--start-checker</option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--stop-checker</option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--remove</option></arg>
+	<arg choice="plain"><option>-r</option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--checker
+	<replaceable>COMMAND</replaceable></option></arg>
+	<arg choice="plain"><option>-c
+	<replaceable>COMMAND</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--timeout
+	<replaceable>TIME</replaceable></option></arg>
+	<arg choice="plain"><option>-t
+	<replaceable>TIME</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--extended-timeout
+	<replaceable>TIME</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--interval
+	<replaceable>TIME</replaceable></option></arg>
+	<arg choice="plain"><option>-i
+	<replaceable>TIME</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--approve-by-default</option
+        ></arg>
+	<sbr/>
+	<arg choice="plain"><option>--deny-by-default</option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--approval-delay
+	<replaceable>TIME</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--approval-duration
+	<replaceable>TIME</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--interval
+	<replaceable>TIME</replaceable></option></arg>
+	<arg choice="plain"><option>-i
+	<replaceable>TIME</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--host
+	<replaceable>STRING</replaceable></option></arg>
+	<arg choice="plain"><option>-H
+	<replaceable>STRING</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--secret
+	<replaceable>FILENAME</replaceable></option></arg>
+	<arg choice="plain"><option>-s
+	<replaceable>FILENAME</replaceable></option></arg>
+      </group>
+      <sbr/>
+      <group>
+	<arg choice="plain"><option>--approve</option></arg>
+	<arg choice="plain"><option>-A</option></arg>
+	<sbr/>
 	<arg choice="plain"><option>--deny</option></arg>
 	<arg choice="plain"><option>-D</option></arg>
       </group>
-      <group choice="req">
-	  <arg choice="plain"><option>--remove</option></arg>
-	  <arg choice="plain"><option>-r</option></arg>
-      </group>
       <sbr/>
-      <arg><option>--debug</option></arg>
       <group choice="req">
 	<arg choice="plain"><option>--all</option></arg>
 	<arg choice="plain"><option>-a</option></arg>
@@ -194,11 +162,22 @@
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
+      <group>
+	<arg choice="plain"><option>--verbose</option></arg>
+	<arg choice="plain"><option>-v</option></arg>
+      </group>
+      <group>
+	<arg rep='repeat' choice='plain'>
+	  <replaceable>CLIENT</replaceable>
+	</arg>
+      </group>
+    </cmdsynopsis>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
       <group choice="req">
 	<arg choice="plain"><option>--is-enabled</option></arg>
 	<arg choice="plain"><option>-V</option></arg>
       </group>
-      <arg><option>--debug</option></arg>
       <arg choice='plain'><replaceable>CLIENT</replaceable></arg>
     </cmdsynopsis>
     <cmdsynopsis>
@@ -215,19 +194,14 @@
 	<arg choice="plain"><option>-v</option></arg>
       </group>
     </cmdsynopsis>
-    <cmdsynopsis>
-      <command>&COMMANDNAME;</command>
-      <arg choice="plain"><option>--check</option></arg>
-    </cmdsynopsis>
   </refsynopsisdiv>
   
   <refsect1 id="description">
     <title>DESCRIPTION</title>
     <para>
-      <command>&COMMANDNAME;</command> is a program to control or
-      query the operation of the Mandos server
-      <citerefentry><refentrytitle>mandos</refentrytitle><manvolnum
-      >8</manvolnum></citerefentry>.
+      <command>&COMMANDNAME;</command> is a program to control the
+      operation of the Mandos server <citerefentry><refentrytitle
+      >mandos</refentrytitle><manvolnum>8</manvolnum></citerefentry>.
     </para>
     <para>
       This program can be used to change client settings, approve or
@@ -491,16 +465,6 @@
       </varlistentry>
       
       <varlistentry>
-	<term><option>--dump-json</option></term>
-	<term><option>-j</option></term>
-	<listitem>
-	  <para>
-	    Dump client settings as JSON to standard output.
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
 	<term><option>--is-enabled</option></term>
 	<term><option>-V</option></term>
 	<listitem>
@@ -511,24 +475,6 @@
 	</listitem>
       </varlistentry>
       
-      <varlistentry>
-	<term><option>--debug</option></term>
-	<listitem>
-	  <para>
-	    Show debug output; currently, this means show D-Bus calls.
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>--check</option></term>
-	<listitem>
-	  <para>
-	    Run self-tests.  This includes any unit tests, etc.
-	  </para>
-	</listitem>
-      </varlistentry>
-      
     </variablelist>
   </refsect1>
   
@@ -550,10 +496,11 @@
     </para>
   </refsect1>
   
-  <refsect1 id="bugs">
-    <title>BUGS</title>
-    <xi:include href="bugs.xml"/>
-  </refsect1>
+<!--   <refsect1 id="bugs"> -->
+<!--     <title>BUGS</title> -->
+<!--     <para> -->
+<!--     </para> -->
+<!--   </refsect1> -->
   
   <refsect1 id="example">
     <title>EXAMPLE</title>
@@ -598,7 +545,7 @@
       <para>
 
 <!-- do not wrap this line -->
-<userinput>&COMMANDNAME; --timeout="PT5M" --interval="PT1M" foo1.example.org foo2.example.org</userinput>
+<userinput>&COMMANDNAME; --timeout="5m" --interval="1m" foo1.example.org foo2.example.org</userinput>
 
       </para>
     </informalexample>

=== modified file 'mandos-keygen'
--- mandos-keygen	2019-02-11 06:31:42 +0000
+++ mandos-keygen	2011-10-05 16:00:56 +0000
@@ -1,42 +1,38 @@
 #!/bin/sh -e
 # 
-# Mandos key generator - create new keys for a Mandos client
-# 
-# Copyright © 2008-2019 Teddy Hogeborn
-# Copyright © 2008-2019 Björn Påhlsson
-# 
-# This file is part of Mandos.
-#
-# Mandos is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
+# Mandos key generator - create a new OpenPGP key for a Mandos client
+# 
+# Copyright © 2008-2011 Teddy Hogeborn
+# Copyright © 2008-2011 Björn Påhlsson
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#     Mandos is distributed in the hope that it will be useful, but
-#     WITHOUT ANY WARRANTY; without even the implied warranty of
+#     This program is distributed in the hope that it will be useful,
+#     but WITHOUT ANY WARRANTY; without even the implied warranty of
 #     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #     GNU General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
-# along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # 
 # Contact the authors at <mandos@recompile.se>.
 # 
 
-VERSION="1.8.3"
+VERSION="1.3.1"
 
 KEYDIR="/etc/keys/mandos"
-KEYTYPE=RSA
-KEYLENGTH=4096
-SUBKEYTYPE=RSA
-SUBKEYLENGTH=4096
+KEYTYPE=DSA
+KEYLENGTH=2048
+SUBKEYTYPE=ELG-E
+SUBKEYLENGTH=2048
 KEYNAME="`hostname --fqdn 2>/dev/null || hostname`"
 KEYEMAIL=""
-KEYCOMMENT=""
+KEYCOMMENT="Mandos client key"
 KEYEXPIRE=0
-TLS_KEYTYPE=ed25519
 FORCE=no
-SSH=yes
 KEYCOMMENT_ORIG="$KEYCOMMENT"
 mode=keygen
 
@@ -45,12 +41,12 @@
 fi
 
 # Parse options
-TEMP=`getopt --options vhpF:d:t:l:s:L:n:e:c:x:T:fS \
-    --longoptions version,help,password,passfile:,dir:,type:,length:,subtype:,sublength:,name:,email:,comment:,expire:,tls-keytype:,force,no-ssh \
+TEMP=`getopt --options vhpF:d:t:l:s:L:n:e:c:x:f \
+    --longoptions version,help,password,passfile:,dir:,type:,length:,subtype:,sublength:,name:,email:,comment:,expire:,force \
     --name "$0" -- "$@"`
 
 help(){
-basename="`basename "$0"`"
+basename="`basename $0`"
 cat <<EOF
 Usage: $basename [ -v | --version ]
        $basename [ -h | --help ]
@@ -64,23 +60,22 @@
   -v, --version         Show program's version number and exit
   -h, --help            Show this help message and exit
   -d DIR, --dir DIR     Target directory for key files
-  -t TYPE, --type TYPE  OpenPGP key type.  Default is RSA.
+  -t TYPE, --type TYPE  Key type.  Default is DSA.
   -l BITS, --length BITS
-                        OpenPGP key length in bits.  Default is 4096.
+                        Key length in bits.  Default is 2048.
   -s TYPE, --subtype TYPE
-                        OpenPGP subkey type.  Default is RSA.
+                        Subkey type.  Default is ELG-E.
   -L BITS, --sublength BITS
-                        OpenPGP subkey length in bits.  Default 4096.
+                        Subkey length in bits.  Default is 2048.
   -n NAME, --name NAME  Name of key.  Default is the FQDN.
   -e ADDRESS, --email ADDRESS
-                        Email address of OpenPGP key.  Default empty.
+                        Email address of key.  Default is empty.
   -c TEXT, --comment TEXT
-                        Comment field for OpenPGP key.  Default empty.
+                        Comment field for key.  The default value is
+                        "Mandos client key".
   -x TIME, --expire TIME
-                        OpenPGP key expire time.  Default is none.
+                        Key expire time.  Default is no expiration.
                         See gpg(1) for syntax.
-  -T TYPE, --tls-keytype TYPE
-                        TLS key type.  Default is ed25519.
   -f, --force           Force overwriting old key files.
 
 Password creation options:
@@ -91,7 +86,6 @@
                         Encrypt a password from FILE using the key in
                         the key directory.  All options other than
                         --dir and --name are ignored.
-  -S, --no-ssh          Don't get SSH key or set "checker" option.
 EOF
 }
 
@@ -109,9 +103,7 @@
 	-e|--email) KEYEMAIL="$2"; shift 2;;
 	-c|--comment) KEYCOMMENT="$2"; shift 2;;
 	-x|--expire) KEYEXPIRE="$2"; shift 2;;
-	-T|--tls-keytype) TLS_KEYTYPE="$2"; shift 2;;
 	-f|--force) FORCE=yes; shift;;
-	-S|--no-ssh) SSH=no; shift;;
 	-v|--version) echo "$0 $VERSION"; exit;;
 	-h|--help) help; exit;;
 	--) shift; break;;
@@ -119,14 +111,12 @@
     esac
 done
 if [ "$#" -gt 0 ]; then
-    echo "Unknown arguments: '$*'" >&2
+    echo "Unknown arguments: '$@'" >&2
     exit 1
 fi
 
 SECKEYFILE="$KEYDIR/seckey.txt"
 PUBKEYFILE="$KEYDIR/pubkey.txt"
-TLS_PRIVKEYFILE="$KEYDIR/tls-privkey.pem"
-TLS_PUBKEYFILE="$KEYDIR/tls-pubkey.pem"
 
 # Check for some invalid values
 if [ ! -d "$KEYDIR" ]; then
@@ -169,10 +159,8 @@
 	[Nn][Oo]|[Ff][Aa][Ll][Ss][Ee]|*) FORCE=0;;
     esac
     
-    if { [ -e "$SECKEYFILE" ] || [ -e "$PUBKEYFILE" ] \
-	     || [ -e "$TLS_PRIVKEYFILE" ] \
-	     || [ -e "$TLS_PUBKEYFILE" ]; } \
-	&& [ "$FORCE" -eq 0 ]; then
+    if [ \( -e "$SECKEYFILE" -o -e "$PUBKEYFILE" \) \
+	-a "$FORCE" -eq 0 ]; then
 	echo "Refusing to overwrite old key files; use --force" >&2
 	exit 1
     fi
@@ -187,7 +175,6 @@
     
     # Create temporary gpg batch file
     BATCHFILE="`mktemp -t mandos-keygen-batch.XXXXXXXXXX`"
-    TLS_PRIVKEYTMP="`mktemp -t mandos-keygen-privkey.XXXXXXXXXX`"
 fi
 
 if [ "$mode" = password ]; then
@@ -202,8 +189,7 @@
 trap "
 set +e; \
 test -n \"$SECFILE\" && shred --remove \"$SECFILE\"; \
-test -n \"$TLS_PRIVKEYTMP\" && shred --remove \"$TLS_PRIVKEYTMP\"; \
-shred --remove \"$RINGDIR\"/sec* 2>/dev/null;
+shred --remove \"$RINGDIR\"/sec*;
 test -n \"$BATCHFILE\" && rm --force \"$BATCHFILE\"; \
 rm --recursive --force \"$RINGDIR\";
 tty --quiet && stty echo; \
@@ -218,10 +204,10 @@
     cat >"$BATCHFILE" <<-EOF
 	Key-Type: $KEYTYPE
 	Key-Length: $KEYLENGTH
-	Key-Usage: sign,auth
+	#Key-Usage: encrypt,sign,auth
 	Subkey-Type: $SUBKEYTYPE
 	Subkey-Length: $SUBKEYLENGTH
-	Subkey-Usage: encrypt
+	#Subkey-Usage: encrypt,sign,auth
 	Name-Real: $KEYNAME
 	$KEYCOMMENTLINE
 	$KEYEMAILLINE
@@ -230,7 +216,6 @@
 	#Handle: <no-spaces>
 	#%pubring pubring.gpg
 	#%secring secring.gpg
-	%no-protection
 	%commit
 	EOF
     
@@ -243,45 +228,7 @@
 	echo -n "Started: "
 	date
     fi
-
-    # Generate TLS private key
-    if certtool --generate-privkey --password='' \
-		--outfile "$TLS_PRIVKEYTMP" --sec-param ultra \
-		--key-type="$TLS_KEYTYPE" --pkcs8 --no-text 2>/dev/null; then
-	
-	# Backup any old key files
-	if cp --backup=numbered --force "$TLS_PRIVKEYFILE" "$TLS_PRIVKEYFILE" \
-	      2>/dev/null; then
-	    shred --remove "$TLS_PRIVKEYFILE" 2>/dev/null || :
-	fi
-	if cp --backup=numbered --force "$TLS_PUBKEYFILE" "$TLS_PUBKEYFILE" \
-	      2>/dev/null; then
-	    rm --force "$TLS_PUBKEYFILE"
-	fi
-	cp --archive "$TLS_PRIVKEYTMP" "$TLS_PRIVKEYFILE"
-	shred --remove "$TLS_PRIVKEYTMP" 2>/dev/null || :
-
-	## TLS public key
-
-	# First try certtool from GnuTLS
-	if ! certtool --password='' --load-privkey="$TLS_PRIVKEYFILE" \
-	     --outfile="$TLS_PUBKEYFILE" --pubkey-info --no-text \
-	     2>/dev/null; then
-	    # Otherwise try OpenSSL
-	    if ! openssl pkey -in "$TLS_PRIVKEYFILE" \
-		 -out "$TLS_PUBKEYFILE" -pubout; then
-		rm --force "$TLS_PUBKEYFILE"
-		# None of the commands succeded; give up
-		return 1
-	    fi
-	fi
-    fi
     
-    # Make sure trustdb.gpg exists;
-    # this is a workaround for Debian bug #737128
-    gpg --quiet --batch --no-tty --no-options --enable-dsa2 \
-	--homedir "$RINGDIR" \
-	--import-ownertrust < /dev/null
     # Generate a new key in the key rings
     gpg --quiet --batch --no-tty --no-options --enable-dsa2 \
 	--homedir "$RINGDIR" --trust-model always \
@@ -296,7 +243,7 @@
     # Backup any old key files
     if cp --backup=numbered --force "$SECKEYFILE" "$SECKEYFILE" \
 	2>/dev/null; then
-	shred --remove "$SECKEYFILE" 2>/dev/null || :
+	shred --remove "$SECKEYFILE"
     fi
     if cp --backup=numbered --force "$PUBKEYFILE" "$PUBKEYFILE" \
 	2>/dev/null; then
@@ -323,30 +270,6 @@
 fi
 
 if [ "$mode" = password ]; then
-    
-    # Make SSH be 0 or 1
-    case "$SSH" in
-	[Yy][Ee][Ss]|[Tt][Rr][Uu][Ee]) SSH=1;;
-	[Nn][Oo]|[Ff][Aa][Ll][Ss][Ee]|*) SSH=0;;
-    esac
-    
-    if [ $SSH -eq 1 ]; then
-	for ssh_keytype in ecdsa-sha2-nistp256 ed25519 rsa; do
-	    set +e
-	    ssh_fingerprint="`ssh-keyscan -t $ssh_keytype localhost 2>/dev/null`"
-	    err=$?
-	    set -e
-	    if [ $err -ne 0 ]; then
-		ssh_fingerprint=""
-		continue
-	    fi
-	    if [ -n "$ssh_fingerprint" ]; then
-		ssh_fingerprint="${ssh_fingerprint#localhost }"
-		break
-	    fi
-	done
-    fi
-    
     # Import key into temporary key rings
     gpg --quiet --batch --no-tty --no-options --enable-dsa2 \
 	--homedir "$RINGDIR" --trust-model always --armor \
@@ -357,26 +280,13 @@
     
     # Get fingerprint of key
     FINGERPRINT="`gpg --quiet --batch --no-tty --no-options \
-	--enable-dsa2 --homedir "$RINGDIR" --trust-model always \
+	--enable-dsa2 --homedir \"$RINGDIR\" --trust-model always \
 	--fingerprint --with-colons \
 	| sed --quiet \
 	--expression='/^fpr:/{s/^fpr:.*:\\([0-9A-Z]*\\):\$/\\1/p;q}'`"
     
     test -n "$FINGERPRINT"
     
-    if [ -r "$TLS_PUBKEYFILE" ]; then
-       KEY_ID="$(certtool --key-id --hash=sha256 \
-    		       --infile="$TLS_PUBKEYFILE" 2>/dev/null || :)"
-
-       if [ -z "$KEY_ID" ]; then
-	   KEY_ID=$(openssl pkey -pubin -in "$TLS_PUBKEYFILE" \
-			    -outform der \
-			| openssl sha256 \
-			| sed --expression='s/^.*[^[:xdigit:]]//')
-       fi
-       test -n "$KEY_ID"
-    fi
-    
     FILECOMMENT="Encrypted password for a Mandos client"
     
     while [ ! -s "$SECFILE" ]; do
@@ -384,11 +294,9 @@
 	    cat "$PASSFILE"
 	else
 	    tty --quiet && stty -echo
-	    echo -n "Enter passphrase: " >/dev/tty
-	    read -r first
+	    read -p "Enter passphrase: " first
 	    tty --quiet && echo >&2
-	    echo -n "Repeat passphrase: " >/dev/tty
-	    read -r second
+	    read -p "Repeat passphrase: " second
 	    if tty --quiet; then
 		echo >&2
 		stty echo
@@ -416,11 +324,6 @@
     cat <<-EOF
 	[$KEYNAME]
 	host = $KEYNAME
-	EOF
-    if [ -n "$KEY_ID" ]; then
-	echo "key_id = $KEY_ID"
-    fi
-    cat <<-EOF
 	fingerprint = $FINGERPRINT
 	secret =
 	EOF
@@ -433,10 +336,6 @@
 		/^[^-]/s/^/    /p
 	    }
 	}' < "$SECFILE"
-    if [ -n "$ssh_fingerprint" ]; then
-	echo 'checker = ssh-keyscan -t '"$ssh_keytype"' %%(host)s 2>/dev/null | grep --fixed-strings --line-regexp --quiet --regexp=%%(host)s" %(ssh_fingerprint)s"'
-	echo "ssh_fingerprint = ${ssh_fingerprint}"
-    fi
 fi
 
 trap - EXIT
@@ -444,8 +343,8 @@
 set +e
 # Remove the password file, if any
 if [ -n "$SECFILE" ]; then
-    shred --remove "$SECFILE" 2>/dev/null
+    shred --remove "$SECFILE"
 fi
 # Remove the key rings
-shred --remove "$RINGDIR"/sec* 2>/dev/null
+shred --remove "$RINGDIR"/sec*
 rm --recursive --force "$RINGDIR"

=== modified file 'mandos-keygen.xml'
--- mandos-keygen.xml	2019-02-10 04:20:26 +0000
+++ mandos-keygen.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-keygen">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,16 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -127,17 +118,7 @@
 	<replaceable>TIME</replaceable></option></arg>
       </group>
       <sbr/>
-      <group>
-	<arg choice="plain"><option>--tls-keytype
-	<replaceable>KEYTYPE</replaceable></option></arg>
-	<arg choice="plain"><option>-T
-	<replaceable>KEYTYPE</replaceable></option></arg>
-      </group>
-      <sbr/>
-      <group>
-	<arg choice="plain"><option>--force</option></arg>
-	<arg choice="plain"><option>-f</option></arg>
-      </group>
+      <arg><option>--force</option></arg>
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
@@ -163,10 +144,6 @@
 	<arg choice="plain"><option>-n
 	<replaceable>NAME</replaceable></option></arg>
       </group>
-      <group>
-	<arg choice="plain"><option>--no-ssh</option></arg>
-	<arg choice="plain"><option>-S</option></arg>
-      </group>
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
@@ -188,12 +165,12 @@
     <title>DESCRIPTION</title>
     <para>
       <command>&COMMANDNAME;</command> is a program to generate the
-      TLS and OpenPGP keys used by
+      OpenPGP key used by
       <citerefentry><refentrytitle>mandos-client</refentrytitle>
-      <manvolnum>8mandos</manvolnum></citerefentry>.  The keys are
-      normally written to /etc/keys/mandos for later installation into
-      the initrd image, but this, and most other things, can be
-      changed with command line options.
+      <manvolnum>8mandos</manvolnum></citerefentry>.  The key is
+      normally written to /etc/mandos for later installation into the
+      initrd image, but this, and most other things, can be changed
+      with command line options.
     </para>
     <para>
       This program can also be used with the
@@ -236,8 +213,8 @@
 	<replaceable>DIRECTORY</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Target directory for key files.  Default is <filename
-	    class="directory">/etc/keys/mandos</filename>.
+	    Target directory for key files.  Default is
+	    <filename>/etc/mandos</filename>.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -249,7 +226,7 @@
 	<replaceable>TYPE</replaceable></option></term>
 	<listitem>
 	  <para>
-	    OpenPGP key type.  Default is <quote>RSA</quote>.
+	    Key type.  Default is <quote>DSA</quote>.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -261,7 +238,7 @@
 	<replaceable>BITS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    OpenPGP key length in bits.  Default is 4096.
+	    Key length in bits.  Default is 2048.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -273,7 +250,8 @@
 	<replaceable>KEYTYPE</replaceable></option></term>
 	<listitem>
 	  <para>
-	    OpenPGP subkey type.  Default is <quote>RSA</quote>
+	    Subkey type.  Default is <quote>ELG-E</quote> (Elgamal
+	    encryption-only).
 	  </para>
 	</listitem>
       </varlistentry>
@@ -285,7 +263,7 @@
 	<replaceable>BITS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    OpenPGP subkey length in bits.  Default is 4096.
+	    Subkey length in bits.  Default is 2048.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -309,7 +287,8 @@
 	<replaceable>TEXT</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Comment field for key.  Default is empty.
+	    Comment field for key.  The default value is
+	    <quote><literal>Mandos client key</literal></quote>.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -329,18 +308,6 @@
       </varlistentry>
       
       <varlistentry>
-	<term><option>--tls-keytype
-	<replaceable>KEYTYPE</replaceable></option></term>
-	<term><option>-T
-	<replaceable>KEYTYPE</replaceable></option></term>
-	<listitem>
-	  <para>
-	    TLS key type.  Default is <quote>ed25519</quote>
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
 	<term><option>--force</option></term>
 	<term><option>-f</option></term>
 	<listitem>
@@ -355,8 +322,8 @@
 	<listitem>
 	  <para>
 	    Prompt for a password and encrypt it with the key already
-	    present in either <filename>/etc/keys/mandos</filename> or
-	    the directory specified with the <option>--dir</option>
+	    present in either <filename>/etc/mandos</filename> or the
+	    directory specified with the <option>--dir</option>
 	    option.  Outputs, on standard output, a section suitable
 	    for inclusion in <citerefentry><refentrytitle
 	    >mandos-clients.conf</refentrytitle><manvolnum
@@ -379,22 +346,6 @@
 	  </para>
 	</listitem>
       </varlistentry>
-      <varlistentry>
-	<term><option>--no-ssh</option></term>
-	<term><option>-S</option></term>
-	<listitem>
-	  <para>
-	    When <option>--password</option> or
-	    <option>--passfile</option> is given, this option will
-	    prevent <command>&COMMANDNAME;</command> from calling
-	    <command>ssh-keyscan</command> to get an SSH fingerprint
-	    for this host and, if successful, output suitable config
-	    options to use this fingerprint as a
-	    <option>checker</option> option in the output.  This is
-	    otherwise the default behavior.
-	  </para>
-	</listitem>
-      </varlistentry>
     </variablelist>
   </refsect1>
   
@@ -402,9 +353,9 @@
     <title>OVERVIEW</title>
     <xi:include href="overview.xml"/>
     <para>
-      This program is a small utility to generate new TLS and OpenPGP
-      keys for new Mandos clients, and to generate sections for
-      inclusion in <filename>clients.conf</filename> on the server.
+      This program is a small utility to generate new OpenPGP keys for
+      new Mandos clients, and to generate sections for inclusion in
+      <filename>clients.conf</filename> on the server.
     </para>
   </refsect1>
   
@@ -442,7 +393,7 @@
     </para>
     <variablelist>
       <varlistentry>
-	<term><filename>/etc/keys/mandos/seckey.txt</filename></term>
+	<term><filename>/etc/mandos/seckey.txt</filename></term>
 	<listitem>
 	  <para>
 	    OpenPGP secret key file which will be created or
@@ -451,7 +402,7 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename>/etc/keys/mandos/pubkey.txt</filename></term>
+	<term><filename>/etc/mandos/pubkey.txt</filename></term>
 	<listitem>
 	  <para>
 	    OpenPGP public key file which will be created or
@@ -460,23 +411,7 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename>/etc/keys/mandos/tls-privkey.pem</filename></term>
-	<listitem>
-	  <para>
-	    Private key file which will be created or overwritten.
-	  </para>
-	</listitem>
-      </varlistentry>
-      <varlistentry>
-	<term><filename>/etc/keys/mandos/tls-pubkey.pem</filename></term>
-	<listitem>
-	  <para>
-	    Public key file which will be created or overwritten.
-	  </para>
-	</listitem>
-      </varlistentry>
-      <varlistentry>
-	<term><filename class="directory">/tmp</filename></term>
+	<term><filename>/tmp</filename></term>
 	<listitem>
 	  <para>
 	    Temporary files will be written here if
@@ -487,10 +422,11 @@
     </variablelist>
   </refsect1>
   
-  <refsect1 id="bugs">
-    <title>BUGS</title>
-    <xi:include href="bugs.xml"/>
-  </refsect1>
+<!--   <refsect1 id="bugs"> -->
+<!--     <title>BUGS</title> -->
+<!--     <para> -->
+<!--     </para> -->
+<!--   </refsect1> -->
   
   <refsect1 id="example">
     <title>EXAMPLE</title>
@@ -516,9 +452,9 @@
     </informalexample>
     <informalexample>
       <para>
-	Prompt for a password, encrypt it with the keys in <filename
-	class="directory">/etc/keys/mandos</filename> and output a
-	section suitable for <filename>clients.conf</filename>.
+	Prompt for a password, encrypt it with the key in
+	<filename>/etc/mandos</filename> and output a section suitable
+	for <filename>clients.conf</filename>.
       </para>
       <para>
 	<userinput>&COMMANDNAME; --password</userinput>
@@ -526,7 +462,7 @@
     </informalexample>
     <informalexample>
       <para>
-	Prompt for a password, encrypt it with the keys in the
+	Prompt for a password, encrypt it with the key in the
 	<filename>client-key</filename> directory and output a section
 	suitable for <filename>clients.conf</filename>.
       </para>
@@ -566,9 +502,7 @@
       <citerefentry><refentrytitle>mandos</refentrytitle>
       <manvolnum>8</manvolnum></citerefentry>,
       <citerefentry><refentrytitle>mandos-client</refentrytitle>
-      <manvolnum>8mandos</manvolnum></citerefentry>,
-      <citerefentry><refentrytitle>ssh-keyscan</refentrytitle>
-      <manvolnum>1</manvolnum></citerefentry>
+      <manvolnum>8mandos</manvolnum></citerefentry>
     </para>
   </refsect1>
   

=== modified file 'mandos-monitor'
--- mandos-monitor	2019-02-11 06:31:42 +0000
+++ mandos-monitor	2011-10-05 16:00:56 +0000
@@ -1,38 +1,33 @@
 #!/usr/bin/python
 # -*- mode: python; coding: utf-8 -*-
-#
+# 
 # Mandos Monitor - Control and monitor the Mandos server
-#
-# Copyright © 2009-2019 Teddy Hogeborn
-# Copyright © 2009-2019 Björn Påhlsson
-#
-# This file is part of Mandos.
-#
-# Mandos is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
+# 
+# Copyright © 2009-2011 Teddy Hogeborn
+# Copyright © 2009-2011 Björn Påhlsson
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#     Mandos is distributed in the hope that it will be useful, but
-#     WITHOUT ANY WARRANTY; without even the implied warranty of
+#     This program is distributed in the hope that it will be useful,
+#     but WITHOUT ANY WARRANTY; without even the implied warranty of
 #     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #     GNU General Public License for more details.
-#
+# 
 # You should have received a copy of the GNU General Public License
-# along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
-#
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# 
 # Contact the authors at <mandos@recompile.se>.
-#
+# 
 
 from __future__ import (division, absolute_import, print_function,
                         unicode_literals)
-try:
-    from future_builtins import *
-except ImportError:
-    pass
 
 import sys
 import os
+import signal
 
 import datetime
 
@@ -40,32 +35,32 @@
 import urwid
 
 from dbus.mainloop.glib import DBusGMainLoop
-from gi.repository import GLib
+import gobject
 
 import dbus
 
+import UserList
+
 import locale
 
+locale.setlocale(locale.LC_ALL, '')
+
 import logging
-
-if sys.version_info.major == 2:
-    str = unicode
-
-locale.setlocale(locale.LC_ALL, '')
-
 logging.getLogger('dbus.proxies').setLevel(logging.CRITICAL)
 
 # Some useful constants
 domain = 'se.recompile'
 server_interface = domain + '.Mandos'
 client_interface = domain + '.Mandos.Client'
-version = "1.8.3"
-
-try:
-    dbus.OBJECT_MANAGER_IFACE
-except AttributeError:
-    dbus.OBJECT_MANAGER_IFACE = "org.freedesktop.DBus.ObjectManager"
-
+version = "1.3.1"
+
+# Always run in monochrome mode
+urwid.curses_display.curses.has_colors = lambda : False
+
+# Urwid doesn't support blinking, but we want it.  Since we have no
+# use for underline on its own, we make underline also always blink.
+urwid.curses_display.curses.A_UNDERLINE |= (
+    urwid.curses_display.curses.A_BLINK)
 
 def isoformat_to_datetime(iso):
     "Parse an ISO 8601 date string to a datetime.datetime()"
@@ -80,49 +75,51 @@
                              int(day),
                              int(hour),
                              int(minute),
-                             int(second),            # Whole seconds
-                             int(fraction*1000000))  # Microseconds
-
+                             int(second),           # Whole seconds
+                             int(fraction*1000000)) # Microseconds
 
 class MandosClientPropertyCache(object):
     """This wraps a Mandos Client D-Bus proxy object, caches the
     properties and calls a hook function when any of them are
     changed.
     """
-    def __init__(self, proxy_object=None, properties=None, **kwargs):
-        self.proxy = proxy_object  # Mandos Client proxy object
-        self.properties = dict() if properties is None else properties
+    def __init__(self, proxy_object=None, *args, **kwargs):
+        self.proxy = proxy_object # Mandos Client proxy object
+        
+        self.properties = dict()
         self.property_changed_match = (
-            self.proxy.connect_to_signal("PropertiesChanged",
-                                         self.properties_changed,
-                                         dbus.PROPERTIES_IFACE,
+            self.proxy.connect_to_signal("PropertyChanged",
+                                         self.property_changed,
+                                         client_interface,
                                          byte_arrays=True))
-
-        if properties is None:
-            self.properties.update(self.proxy.GetAll(
-                client_interface,
-                dbus_interface=dbus.PROPERTIES_IFACE))
-
-        super(MandosClientPropertyCache, self).__init__(**kwargs)
-
-    def properties_changed(self, interface, properties, invalidated):
-        """This is called whenever we get a PropertiesChanged signal
-        It updates the changed properties in the "properties" dict.
+        
+        self.properties.update(
+            self.proxy.GetAll(client_interface,
+                              dbus_interface = dbus.PROPERTIES_IFACE))
+
+        #XXX This breaks good super behaviour
+#        super(MandosClientPropertyCache, self).__init__(
+#            *args, **kwargs)
+    
+    def property_changed(self, property=None, value=None):
+        """This is called whenever we get a PropertyChanged signal
+        It updates the changed property in the "properties" dict.
         """
         # Update properties dict with new value
-        if interface == client_interface:
-            self.properties.update(properties)
-
-    def delete(self):
+        self.properties[property] = value
+    
+    def delete(self, *args, **kwargs):
         self.property_changed_match.remove()
+        super(MandosClientPropertyCache, self).__init__(
+            *args, **kwargs)
 
 
 class MandosClientWidget(urwid.FlowWidget, MandosClientPropertyCache):
     """A Mandos Client which is visible on the screen.
     """
-
+    
     def __init__(self, server_proxy_object=None, update_hook=None,
-                 delete_hook=None, logger=None, **kwargs):
+                 delete_hook=None, logger=None, *args, **kwargs):
         # Called on update
         self.update_hook = update_hook
         # Called on delete
@@ -131,17 +128,39 @@
         self.server_proxy_object = server_proxy_object
         # Logger
         self.logger = logger
-
+        
         self._update_timer_callback_tag = None
-
+        self._update_timer_callback_lock = 0
+        self.last_checker_failed = False
+        
         # The widget shown normally
         self._text_widget = urwid.Text("")
         # The widget shown when we have focus
         self._focus_text_widget = urwid.Text("")
-        super(MandosClientWidget, self).__init__(**kwargs)
+        super(MandosClientWidget, self).__init__(
+            update_hook=update_hook, delete_hook=delete_hook,
+            *args, **kwargs)
         self.update()
         self.opened = False
-
+        
+        last_checked_ok = isoformat_to_datetime(self.properties
+                                                ["LastCheckedOK"])
+        if last_checked_ok is None:
+            self.last_checker_failed = True
+        else:
+            self.last_checker_failed = ((datetime.datetime.utcnow()
+                                         - last_checked_ok)
+                                        > datetime.timedelta
+                                        (milliseconds=
+                                         self.properties
+                                         ["Interval"]))
+        
+        if self.last_checker_failed:
+            self.using_timer(True)
+        
+        if self.need_approval:
+            self.using_timer(True)
+        
         self.match_objects = (
             self.proxy.connect_to_signal("CheckerCompleted",
                                          self.checker_completed,
@@ -163,134 +182,154 @@
                                          self.rejected,
                                          client_interface,
                                          byte_arrays=True))
-        self.logger('Created client {}'
-                    .format(self.properties["Name"]), level=0)
-
+        #self.logger('Created client %s' % (self.properties["Name"]))
+    
+    def property_changed(self, property=None, value=None):
+        super(self, MandosClientWidget).property_changed(property,
+                                                         value)
+        if property == "ApprovalPending":
+            using_timer(bool(value))
+        
     def using_timer(self, flag):
         """Call this method with True or False when timer should be
         activated or deactivated.
         """
-        if flag and self._update_timer_callback_tag is None:
+        old = self._update_timer_callback_lock
+        if flag:
+            self._update_timer_callback_lock += 1
+        else:
+            self._update_timer_callback_lock -= 1
+        if old == 0 and self._update_timer_callback_lock:
             # Will update the shown timer value every second
-            self._update_timer_callback_tag = (GLib.timeout_add
+            self._update_timer_callback_tag = (gobject.timeout_add
                                                (1000,
                                                 self.update_timer))
-        elif not (flag or self._update_timer_callback_tag is None):
-            GLib.source_remove(self._update_timer_callback_tag)
+        elif old and self._update_timer_callback_lock == 0:
+            gobject.source_remove(self._update_timer_callback_tag)
             self._update_timer_callback_tag = None
-
+    
     def checker_completed(self, exitstatus, condition, command):
         if exitstatus == 0:
-            self.logger('Checker for client {} (command "{}")'
-                        ' succeeded'.format(self.properties["Name"],
-                                            command), level=0)
+            if self.last_checker_failed:
+                self.last_checker_failed = False
+                self.using_timer(False)
+            #self.logger('Checker for client %s (command "%s")'
+            #            ' was successful'
+            #            % (self.properties["Name"], command))
             self.update()
             return
         # Checker failed
+        if not self.last_checker_failed:
+            self.last_checker_failed = True
+            self.using_timer(True)
         if os.WIFEXITED(condition):
-            self.logger('Checker for client {} (command "{}") failed'
-                        ' with exit code {}'
-                        .format(self.properties["Name"], command,
-                                os.WEXITSTATUS(condition)))
+            self.logger('Checker for client %s (command "%s")'
+                        ' failed with exit code %s'
+                        % (self.properties["Name"], command,
+                           os.WEXITSTATUS(condition)))
         elif os.WIFSIGNALED(condition):
-            self.logger('Checker for client {} (command "{}") was'
-                        ' killed by signal {}'
-                        .format(self.properties["Name"], command,
-                                os.WTERMSIG(condition)))
+            self.logger('Checker for client %s (command "%s")'
+                        ' was killed by signal %s'
+                        % (self.properties["Name"], command,
+                           os.WTERMSIG(condition)))
+        elif os.WCOREDUMP(condition):
+            self.logger('Checker for client %s (command "%s")'
+                        ' dumped core'
+                        % (self.properties["Name"], command))
+        else:
+            self.logger('Checker for client %s completed'
+                        ' mysteriously')
         self.update()
-
+    
     def checker_started(self, command):
-        """Server signals that a checker started."""
-        self.logger('Client {} started checker "{}"'
-                    .format(self.properties["Name"],
-                            command), level=0)
-
+        #self.logger('Client %s started checker "%s"'
+        #            % (self.properties["Name"], unicode(command)))
+        pass
+    
     def got_secret(self):
-        self.logger('Client {} received its secret'
-                    .format(self.properties["Name"]))
-
+        self.last_checker_failed = False
+        self.logger('Client %s received its secret'
+                    % self.properties["Name"])
+    
     def need_approval(self, timeout, default):
         if not default:
-            message = 'Client {} needs approval within {} seconds'
+            message = 'Client %s needs approval within %s seconds'
         else:
-            message = 'Client {} will get its secret in {} seconds'
-        self.logger(message.format(self.properties["Name"],
-                                   timeout/1000))
-
+            message = 'Client %s will get its secret in %s seconds'
+        self.logger(message
+                    % (self.properties["Name"], timeout/1000))
+        self.using_timer(True)
+    
     def rejected(self, reason):
-        self.logger('Client {} was rejected; reason: {}'
-                    .format(self.properties["Name"], reason))
-
+        self.logger('Client %s was rejected; reason: %s'
+                    % (self.properties["Name"], reason))
+    
     def selectable(self):
         """Make this a "selectable" widget.
         This overrides the method from urwid.FlowWidget."""
         return True
-
+    
     def rows(self, maxcolrow, focus=False):
         """How many rows this widget will occupy might depend on
         whether we have focus or not.
         This overrides the method from urwid.FlowWidget"""
         return self.current_widget(focus).rows(maxcolrow, focus=focus)
-
+    
     def current_widget(self, focus=False):
         if focus or self.opened:
             return self._focus_widget
         return self._widget
-
+    
     def update(self):
         "Called when what is visible on the screen should be updated."
         # How to add standout mode to a style
-        with_standout = {"normal": "standout",
-                         "bold": "bold-standout",
-                         "underline-blink":
-                         "underline-blink-standout",
-                         "bold-underline-blink":
-                         "bold-underline-blink-standout",
-                         }
+        with_standout = { "normal": "standout",
+                          "bold": "bold-standout",
+                          "underline-blink":
+                              "underline-blink-standout",
+                          "bold-underline-blink":
+                              "bold-underline-blink-standout",
+                          }
 
         # Rebuild focus and non-focus widgets using current properties
 
         # Base part of a client. Name!
-        base = '{name}: '.format(name=self.properties["Name"])
+        base = ('%(name)s: '
+                      % {"name": self.properties["Name"]})
         if not self.properties["Enabled"]:
             message = "DISABLED"
-            self.using_timer(False)
         elif self.properties["ApprovalPending"]:
-            timeout = datetime.timedelta(
-                milliseconds=self.properties["ApprovalDelay"])
+            timeout = datetime.timedelta(milliseconds
+                                         = self.properties
+                                         ["ApprovalDelay"])
             last_approval_request = isoformat_to_datetime(
                 self.properties["LastApprovalRequest"])
             if last_approval_request is not None:
-                timer = max(timeout - (datetime.datetime.utcnow()
-                                       - last_approval_request),
-                            datetime.timedelta())
+                timer = timeout - (datetime.datetime.utcnow()
+                                   - last_approval_request)
             else:
                 timer = datetime.timedelta()
             if self.properties["ApprovedByDefault"]:
-                message = "Approval in {}. (d)eny?"
+                message = "Approval in %s. (d)eny?"
             else:
-                message = "Denial in {}. (a)pprove?"
-            message = message.format(str(timer).rsplit(".", 1)[0])
-            self.using_timer(True)
-        elif self.properties["LastCheckerStatus"] != 0:
-            # When checker has failed, show timer until client expires
+                message = "Denial in %s. (a)pprove?"
+            message = message % unicode(timer).rsplit(".", 1)[0]
+        elif self.last_checker_failed:
+            # When checker has failed, print a timer until client expires
             expires = self.properties["Expires"]
             if expires == "":
                 timer = datetime.timedelta(0)
             else:
-                expires = (datetime.datetime.strptime
-                           (expires, '%Y-%m-%dT%H:%M:%S.%f'))
-                timer = max(expires - datetime.datetime.utcnow(),
-                            datetime.timedelta())
+                expires = datetime.datetime.strptime(expires,
+                                                     '%Y-%m-%dT%H:%M:%S.%f')
+                timer = expires - datetime.datetime.utcnow()
             message = ('A checker has failed! Time until client'
-                       ' gets disabled: {}'
-                       .format(str(timer).rsplit(".", 1)[0]))
-            self.using_timer(True)
+                       ' gets disabled: %s'
+                           % unicode(timer).rsplit(".", 1)[0])
         else:
             message = "enabled"
-            self.using_timer(False)
-        self._text = "{}{}".format(base, message)
-
+        self._text = "%s%s" % (base, message)
+            
         if not urwid.supports_unicode():
             self._text = self._text.encode("ascii", "replace")
         textlist = [("normal", self._text)]
@@ -306,64 +345,59 @@
         # Run update hook, if any
         if self.update_hook is not None:
             self.update_hook()
-
+    
     def update_timer(self):
-        """called by GLib. Will indefinitely loop until
-        GLib.source_remove() on tag is called
-        """
+        """called by gobject. Will indefinitely loop until
+        gobject.source_remove() on tag is called"""
         self.update()
         return True             # Keep calling this
-
-    def delete(self, **kwargs):
+    
+    def delete(self, *args, **kwargs):
         if self._update_timer_callback_tag is not None:
-            GLib.source_remove(self._update_timer_callback_tag)
+            gobject.source_remove(self._update_timer_callback_tag)
             self._update_timer_callback_tag = None
         for match in self.match_objects:
             match.remove()
         self.match_objects = ()
         if self.delete_hook is not None:
             self.delete_hook(self)
-        return super(MandosClientWidget, self).delete(**kwargs)
-
+        return super(MandosClientWidget, self).delete(*args, **kwargs)
+    
     def render(self, maxcolrow, focus=False):
         """Render differently if we have focus.
         This overrides the method from urwid.FlowWidget"""
         return self.current_widget(focus).render(maxcolrow,
                                                  focus=focus)
-
+    
     def keypress(self, maxcolrow, key):
         """Handle keys.
         This overrides the method from urwid.FlowWidget"""
         if key == "+":
-            self.proxy.Set(client_interface, "Enabled",
-                           dbus.Boolean(True), ignore_reply=True,
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+            self.proxy.Enable(dbus_interface = client_interface,
+                              ignore_reply=True)
         elif key == "-":
-            self.proxy.Set(client_interface, "Enabled", False,
-                           ignore_reply=True,
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+            self.proxy.Disable(dbus_interface = client_interface,
+                               ignore_reply=True)
         elif key == "a":
             self.proxy.Approve(dbus.Boolean(True, variant_level=1),
-                               dbus_interface=client_interface,
+                               dbus_interface = client_interface,
                                ignore_reply=True)
         elif key == "d":
             self.proxy.Approve(dbus.Boolean(False, variant_level=1),
-                               dbus_interface=client_interface,
+                                  dbus_interface = client_interface,
                                ignore_reply=True)
         elif key == "R" or key == "_" or key == "ctrl k":
             self.server_proxy_object.RemoveClient(self.proxy
                                                   .object_path,
                                                   ignore_reply=True)
         elif key == "s":
-            self.proxy.Set(client_interface, "CheckerRunning",
-                           dbus.Boolean(True), ignore_reply=True,
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+            self.proxy.StartChecker(dbus_interface = client_interface,
+                                    ignore_reply=True)
         elif key == "S":
-            self.proxy.Set(client_interface, "CheckerRunning",
-                           dbus.Boolean(False), ignore_reply=True,
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+            self.proxy.StopChecker(dbus_interface = client_interface,
+                                   ignore_reply=True)
         elif key == "C":
-            self.proxy.CheckedOK(dbus_interface=client_interface,
+            self.proxy.CheckedOK(dbus_interface = client_interface,
                                  ignore_reply=True)
         # xxx
 #         elif key == "p" or key == "=":
@@ -374,16 +408,16 @@
 #             self.open()
         else:
             return key
-
-    def properties_changed(self, interface, properties, invalidated):
-        """Call self.update() if any properties changed.
+    
+    def property_changed(self, property=None, value=None,
+                         *args, **kwargs):
+        """Call self.update() if old value is not new value.
         This overrides the method from MandosClientPropertyCache"""
-        old_values = {key: self.properties.get(key)
-                      for key in properties.keys()}
-        super(MandosClientWidget, self).properties_changed(
-            interface, properties, invalidated)
-        if any(old_values[key] != self.properties.get(key)
-               for key in old_values):
+        property_name = unicode(property)
+        old_value = self.properties.get(property_name)
+        super(MandosClientWidget, self).property_changed(
+            property=property, value=value, *args, **kwargs)
+        if self.properties.get(property_name) != old_value:
             self.update()
 
 
@@ -392,9 +426,8 @@
     "down" key presses, thus not allowing any containing widgets to
     use them as an excuse to shift focus away from this widget.
     """
-    def keypress(self, *args, **kwargs):
-        ret = (super(ConstrainedListBox, self)
-               .keypress(*args, **kwargs))
+    def keypress(self, maxcolrow, key):
+        ret = super(ConstrainedListBox, self).keypress(maxcolrow, key)
         if ret in ("up", "down"):
             return
         return ret
@@ -404,116 +437,152 @@
     """This is the entire user interface - the whole screen
     with boxes, lists of client widgets, etc.
     """
-    def __init__(self, max_log_length=1000, log_level=1):
+    def __init__(self, max_log_length=1000):
         DBusGMainLoop(set_as_default=True)
-
+        
         self.screen = urwid.curses_display.Screen()
-
+        
         self.screen.register_palette((
                 ("normal",
                  "default", "default", None),
                 ("bold",
-                 "bold", "default", "bold"),
+                 "default", "default", "bold"),
                 ("underline-blink",
-                 "underline,blink", "default", "underline,blink"),
+                 "default", "default", "underline"),
                 ("standout",
-                 "standout", "default", "standout"),
+                 "default", "default", "standout"),
                 ("bold-underline-blink",
-                 "bold,underline,blink", "default",
-                 "bold,underline,blink"),
+                 "default", "default", ("bold", "underline")),
                 ("bold-standout",
-                 "bold,standout", "default", "bold,standout"),
+                 "default", "default", ("bold", "standout")),
                 ("underline-blink-standout",
-                 "underline,blink,standout", "default",
-                 "underline,blink,standout"),
+                 "default", "default", ("underline", "standout")),
                 ("bold-underline-blink-standout",
-                 "bold,underline,blink,standout", "default",
-                 "bold,underline,blink,standout"),
+                 "default", "default", ("bold", "underline",
+                                          "standout")),
                 ))
-
+        
         if urwid.supports_unicode():
-            self.divider = "─"  # \u2500
+            self.divider = "─" # \u2500
+            #self.divider = "━" # \u2501
         else:
-            self.divider = "_"  # \u005f
-
+            #self.divider = "-" # \u002d
+            self.divider = "_" # \u005f
+        
         self.screen.start()
-
+        
         self.size = self.screen.get_cols_rows()
-
+        
         self.clients = urwid.SimpleListWalker([])
         self.clients_dict = {}
-
+        
         # We will add Text widgets to this list
-        self.log = urwid.SimpleListWalker([])
+        self.log = []
         self.max_log_length = max_log_length
-
-        self.log_level = log_level
-
+        
         # We keep a reference to the log widget so we can remove it
         # from the ListWalker without it getting destroyed
         self.logbox = ConstrainedListBox(self.log)
-
+        
         # This keeps track of whether self.uilist currently has
         # self.logbox in it or not
         self.log_visible = True
         self.log_wrap = "any"
-
+        
         self.rebuild()
         self.log_message_raw(("bold",
                               "Mandos Monitor version " + version))
         self.log_message_raw(("bold",
                               "q: Quit  ?: Help"))
-
+        
         self.busname = domain + '.Mandos'
-        self.main_loop = GLib.MainLoop()
-
+        self.main_loop = gobject.MainLoop()
+        self.bus = dbus.SystemBus()
+        mandos_dbus_objc = self.bus.get_object(
+            self.busname, "/", follow_name_owner_changes=True)
+        self.mandos_serv = dbus.Interface(mandos_dbus_objc,
+                                          dbus_interface
+                                          = server_interface)
+        try:
+            mandos_clients = (self.mandos_serv
+                              .GetAllClientsWithProperties())
+        except dbus.exceptions.DBusException:
+            mandos_clients = dbus.Dictionary()
+        
+        (self.mandos_serv
+         .connect_to_signal("ClientRemoved",
+                            self.find_and_remove_client,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        (self.mandos_serv
+         .connect_to_signal("ClientAdded",
+                            self.add_new_client,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        (self.mandos_serv
+         .connect_to_signal("ClientNotFound",
+                            self.client_not_found,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        for path, client in mandos_clients.iteritems():
+            client_proxy_object = self.bus.get_object(self.busname,
+                                                      path)
+            self.add_client(MandosClientWidget(server_proxy_object
+                                               =self.mandos_serv,
+                                               proxy_object
+                                               =client_proxy_object,
+                                               properties=client,
+                                               update_hook
+                                               =self.refresh,
+                                               delete_hook
+                                               =self.remove_client,
+                                               logger
+                                               =self.log_message),
+                            path=path)
+    
     def client_not_found(self, fingerprint, address):
-        self.log_message("Client with address {} and fingerprint {}"
-                         " could not be found"
-                         .format(address, fingerprint))
-
+        self.log_message(("Client with address %s and fingerprint %s"
+                          " could not be found" % (address,
+                                                    fingerprint)))
+    
     def rebuild(self):
         """This rebuilds the User Interface.
         Call this when the widget layout needs to change"""
         self.uilist = []
-        # self.uilist.append(urwid.ListBox(self.clients))
+        #self.uilist.append(urwid.ListBox(self.clients))
         self.uilist.append(urwid.Frame(ConstrainedListBox(self.
                                                           clients),
-                                       # header=urwid.Divider(),
+                                       #header=urwid.Divider(),
                                        header=None,
-                                       footer=urwid.Divider(
-                                           div_char=self.divider)))
+                                       footer=
+                                       urwid.Divider(div_char=
+                                                     self.divider)))
         if self.log_visible:
             self.uilist.append(self.logbox)
+            pass
         self.topwidget = urwid.Pile(self.uilist)
-
-    def log_message(self, message, level=1):
-        """Log message formatted with timestamp"""
-        if level < self.log_level:
-            return
+    
+    def log_message(self, message):
         timestamp = datetime.datetime.now().isoformat()
-        self.log_message_raw("{}: {}".format(timestamp, message),
-                             level=level)
-
-    def log_message_raw(self, markup, level=1):
+        self.log_message_raw(timestamp + ": " + message)
+    
+    def log_message_raw(self, markup):
         """Add a log message to the log buffer."""
-        if level < self.log_level:
-            return
         self.log.append(urwid.Text(markup, wrap=self.log_wrap))
-        if self.max_log_length:
-            if len(self.log) > self.max_log_length:
-                del self.log[0:len(self.log)-self.max_log_length-1]
-        self.logbox.set_focus(len(self.logbox.body.contents)-1,
+        if (self.max_log_length
+            and len(self.log) > self.max_log_length):
+            del self.log[0:len(self.log)-self.max_log_length-1]
+        self.logbox.set_focus(len(self.logbox.body.contents),
                               coming_from="above")
         self.refresh()
-
+    
     def toggle_log_display(self):
         """Toggle visibility of the log buffer."""
         self.log_visible = not self.log_visible
         self.rebuild()
-        self.log_message("Log visibility changed to: {}"
-                         .format(self.log_visible), level=0)
-
+        #self.log_message("Log visibility changed to: "
+        #                 + unicode(self.log_visible))
+    
     def change_log_display(self):
         """Change type of log display.
         Currently, this toggles wrapping of text lines."""
@@ -523,149 +592,101 @@
             self.log_wrap = "clip"
         for textwidget in self.log:
             textwidget.set_wrap_mode(self.log_wrap)
-        self.log_message("Wrap mode: {}".format(self.log_wrap),
-                         level=0)
-
-    def find_and_remove_client(self, path, interfaces):
+        #self.log_message("Wrap mode: " + self.log_wrap)
+    
+    def find_and_remove_client(self, path, name):
         """Find a client by its object path and remove it.
-
-        This is connected to the InterfacesRemoved signal from the
+        
+        This is connected to the ClientRemoved signal from the
         Mandos server object."""
-        if client_interface not in interfaces:
-            # Not a Mandos client object; ignore
-            return
         try:
             client = self.clients_dict[path]
         except KeyError:
             # not found?
-            self.log_message("Unknown client {!r} removed"
-                             .format(path))
+            self.log_message("Unknown client %r (%r) removed", name,
+                             path)
             return
         client.delete()
-
-    def add_new_client(self, path, ifs_and_props):
-        """Find a client by its object path and remove it.
-
-        This is connected to the InterfacesAdded signal from the
-        Mandos server object.
-        """
-        if client_interface not in ifs_and_props:
-            # Not a Mandos client object; ignore
-            return
+    
+    def add_new_client(self, path):
         client_proxy_object = self.bus.get_object(self.busname, path)
-        self.add_client(MandosClientWidget(
-            server_proxy_object=self.mandos_serv,
-            proxy_object=client_proxy_object,
-            update_hook=self.refresh,
-            delete_hook=self.remove_client,
-            logger=self.log_message,
-            properties=dict(ifs_and_props[client_interface])),
+        self.add_client(MandosClientWidget(server_proxy_object
+                                           =self.mandos_serv,
+                                           proxy_object
+                                           =client_proxy_object,
+                                           update_hook
+                                           =self.refresh,
+                                           delete_hook
+                                           =self.remove_client,
+                                           logger
+                                           =self.log_message),
                         path=path)
-
+    
     def add_client(self, client, path=None):
         self.clients.append(client)
         if path is None:
             path = client.proxy.object_path
         self.clients_dict[path] = client
-        self.clients.sort(key=lambda c: c.properties["Name"])
+        self.clients.sort(None, lambda c: c.properties["Name"])
         self.refresh()
-
+    
     def remove_client(self, client, path=None):
         self.clients.remove(client)
         if path is None:
             path = client.proxy.object_path
         del self.clients_dict[path]
+        if not self.clients_dict:
+            # Work around bug in Urwid 0.9.8.3 - if a SimpleListWalker
+            # is completely emptied, we need to recreate it.
+            self.clients = urwid.SimpleListWalker([])
+            self.rebuild()
         self.refresh()
-
+    
     def refresh(self):
         """Redraw the screen"""
         canvas = self.topwidget.render(self.size, focus=True)
         self.screen.draw_screen(self.size, canvas)
-
+    
     def run(self):
         """Start the main loop and exit when it's done."""
-        self.bus = dbus.SystemBus()
-        mandos_dbus_objc = self.bus.get_object(
-            self.busname, "/", follow_name_owner_changes=True)
-        self.mandos_serv = dbus.Interface(
-            mandos_dbus_objc, dbus_interface=server_interface)
-        try:
-            mandos_clients = (self.mandos_serv
-                              .GetAllClientsWithProperties())
-            if not mandos_clients:
-                self.log_message_raw(("bold",
-                                      "Note: Server has no clients."))
-        except dbus.exceptions.DBusException:
-            self.log_message_raw(("bold",
-                                  "Note: No Mandos server running."))
-            mandos_clients = dbus.Dictionary()
-
-        (self.mandos_serv
-         .connect_to_signal("InterfacesRemoved",
-                            self.find_and_remove_client,
-                            dbus_interface=dbus.OBJECT_MANAGER_IFACE,
-                            byte_arrays=True))
-        (self.mandos_serv
-         .connect_to_signal("InterfacesAdded",
-                            self.add_new_client,
-                            dbus_interface=dbus.OBJECT_MANAGER_IFACE,
-                            byte_arrays=True))
-        (self.mandos_serv
-         .connect_to_signal("ClientNotFound",
-                            self.client_not_found,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        for path, client in mandos_clients.items():
-            client_proxy_object = self.bus.get_object(self.busname,
-                                                      path)
-            self.add_client(MandosClientWidget(
-                server_proxy_object=self.mandos_serv,
-                proxy_object=client_proxy_object,
-                properties=client,
-                update_hook=self.refresh,
-                delete_hook=self.remove_client,
-                logger=self.log_message),
-                            path=path)
-
         self.refresh()
-        self._input_callback_tag = (GLib.io_add_watch
+        self._input_callback_tag = (gobject.io_add_watch
                                     (sys.stdin.fileno(),
-                                     GLib.IO_IN,
+                                     gobject.IO_IN,
                                      self.process_input))
         self.main_loop.run()
         # Main loop has finished, we should close everything now
-        GLib.source_remove(self._input_callback_tag)
+        gobject.source_remove(self._input_callback_tag)
         self.screen.stop()
-
+    
     def stop(self):
         self.main_loop.quit()
-
+    
     def process_input(self, source, condition):
         keys = self.screen.get_input()
-        translations = {"ctrl n": "down",       # Emacs
-                        "ctrl p": "up",         # Emacs
-                        "ctrl v": "page down",  # Emacs
-                        "meta v": "page up",    # Emacs
-                        " ": "page down",       # less
-                        "f": "page down",       # less
-                        "b": "page up",         # less
-                        "j": "down",            # vi
-                        "k": "up",              # vi
-                        }
+        translations = { "ctrl n": "down",      # Emacs
+                         "ctrl p": "up",        # Emacs
+                         "ctrl v": "page down", # Emacs
+                         "meta v": "page up",   # Emacs
+                         " ": "page down",      # less
+                         "f": "page down",      # less
+                         "b": "page up",        # less
+                         "j": "down",           # vi
+                         "k": "up",             # vi
+                         }
         for key in keys:
             try:
                 key = translations[key]
             except KeyError:    # :-)
                 pass
-
+            
             if key == "q" or key == "Q":
                 self.stop()
                 break
             elif key == "window resize":
                 self.size = self.screen.get_cols_rows()
                 self.refresh()
-            elif key == "ctrl l":
-                self.screen.clear()
+            elif key == "\f":  # Ctrl-L
                 self.refresh()
             elif key == "l" or key == "D":
                 self.toggle_log_display()
@@ -683,9 +704,7 @@
                                             "?: Help",
                                             "l: Log window toggle",
                                             "TAB: Switch window",
-                                            "w: Wrap (log lines)",
-                                            "v: Toggle verbose log",
-                                            ))))
+                                            "w: Wrap (log)"))))
                 self.log_message_raw(("bold",
                                       "  "
                                       .join(("Clients:",
@@ -704,43 +723,35 @@
                 else:
                     self.topwidget.set_focus(self.logbox)
                 self.refresh()
-            elif key == "v":
-                if self.log_level == 0:
-                    self.log_level = 1
-                    self.log_message("Verbose mode: Off")
-                else:
-                    self.log_level = 0
-                    self.log_message("Verbose mode: On")
-            # elif (key == "end" or key == "meta >" or key == "G"
-            #       or key == ">"):
-            #     pass            # xxx end-of-buffer
-            # elif (key == "home" or key == "meta <" or key == "g"
-            #       or key == "<"):
-            #     pass            # xxx beginning-of-buffer
-            # elif key == "ctrl e" or key == "$":
-            #     pass            # xxx move-end-of-line
-            # elif key == "ctrl a" or key == "^":
-            #     pass            # xxx move-beginning-of-line
-            # elif key == "ctrl b" or key == "meta (" or key == "h":
-            #     pass            # xxx left
-            # elif key == "ctrl f" or key == "meta )" or key == "l":
-            #     pass            # xxx right
-            # elif key == "a":
-            #     pass            # scroll up log
-            # elif key == "z":
-            #     pass            # scroll down log
+            #elif (key == "end" or key == "meta >" or key == "G"
+            #      or key == ">"):
+            #    pass            # xxx end-of-buffer
+            #elif (key == "home" or key == "meta <" or key == "g"
+            #      or key == "<"):
+            #    pass            # xxx beginning-of-buffer
+            #elif key == "ctrl e" or key == "$":
+            #    pass            # xxx move-end-of-line
+            #elif key == "ctrl a" or key == "^":
+            #    pass            # xxx move-beginning-of-line
+            #elif key == "ctrl b" or key == "meta (" or key == "h":
+            #    pass            # xxx left
+            #elif key == "ctrl f" or key == "meta )" or key == "l":
+            #    pass            # xxx right
+            #elif key == "a":
+            #    pass            # scroll up log
+            #elif key == "z":
+            #    pass            # scroll down log
             elif self.topwidget.selectable():
                 self.topwidget.keypress(self.size, key)
                 self.refresh()
         return True
 
-
 ui = UserInterface()
 try:
     ui.run()
 except KeyboardInterrupt:
     ui.screen.stop()
-except Exception as e:
-    ui.log_message(str(e))
+except Exception, e:
+    ui.log_message(unicode(e))
     ui.screen.stop()
     raise

=== modified file 'mandos-monitor.xml'
--- mandos-monitor.xml	2019-02-10 04:20:26 +0000
+++ mandos-monitor.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-monitor">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,14 +33,6 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -136,10 +128,6 @@
 	<entry>Toggle log window line wrap</entry>
       </row>
       <row>
-	<entry>v</entry>
-	<entry>Toggle verbose logging</entry>
-      </row>
-      <row>
 	<entry>Up, Ctrl-P, k</entry>
 	<entry>Move up a line</entry>
       </row>
@@ -202,10 +190,9 @@
     <title>BUGS</title>
     <para>
       This program can currently only be used to monitor and control a
-      Mandos server with the default D-Bus bus name of
-      <quote>se.recompile.Mandos</quote>.
+      Mandos server with the default D-Bus service name of
+      <quote>Mandos</quote>.
     </para>
-    <xi:include href="bugs.xml"/>
   </refsect1>
   
   <refsect1 id="example">

=== modified file 'mandos-options.xml'
--- mandos-options.xml	2019-02-09 23:23:26 +0000
+++ mandos-options.xml	2009-02-13 05:38:21 +0000
@@ -48,17 +48,13 @@
   
   <para id="priority">
     GnuTLS priority string for the <acronym>TLS</acronym> handshake.
-    The default is
-    <quote><literal>SECURE128&#8203;:!CTYPE-X.509&#8203;:+CTYPE-RAWPK&#8203;:!RSA&#8203;:!VERS-ALL&#8203;:+VERS-TLS1.3&#8203;:%PROFILE_ULTRA</literal></quote>
-    when using raw public keys in TLS, and
-    <quote><literal>SECURE256&#8203;:!CTYPE-X.509&#8203;:+CTYPE-OPENPGP&#8203;:!RSA&#8203;:+SIGN-DSA-SHA256</literal></quote>
-    when using OpenPGP keys in TLS,.  See <citerefentry><refentrytitle
-    >gnutls_priority_init</refentrytitle>
+    The default is <quote><literal
+    >SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP</literal></quote>.  See
+    <citerefentry><refentrytitle>gnutls_priority_init</refentrytitle>
     <manvolnum>3</manvolnum></citerefentry> for the syntax.
     <emphasis>Warning</emphasis>: changing this may make the
     <acronym>TLS</acronym> handshake fail, making server-client
-    communication impossible.  Changing this option may also make the
-    network traffic decryptable by an attacker.
+    communication impossible.
   </para>
   
   <para id="servicename">
@@ -90,35 +86,4 @@
     advanced users should consider changing this option</emphasis>.
   </para>
   
-  <para id="restore">
-    This option controls whether the server will restore its state
-    from the last time it ran.  Default is to restore last state.
-  </para>
-  
-  <para id="statedir">
-    Directory to save (and restore) state in.  Default is
-    <quote><filename
-    class="directory">/var/lib/mandos</filename></quote>.
-  </para>
-  
-  <para id="socket">
-    If this option is used, the server will not create a new network
-    socket, but will instead use the supplied file descriptor.  By
-    default, the server will create a new network socket.
-  </para>
-  
-  <para id="foreground">
-    This option will make the server run in the foreground and not
-    write a PID file.  The default is to <emphasis>not</emphasis> run
-    in the foreground, except in <option>debug</option> mode, which
-    implies this option.
-  </para>
-  
-  <para id="zeroconf">
-    This option controls whether the server will announce its
-    existence using Zeroconf.  Default is to use Zeroconf.  If
-    Zeroconf is not used, a <option>port</option> number or a
-    <option>socket</option> is required.
-  </para>
-  
 </section>

=== removed file 'mandos-to-cryptroot-unlock'
--- mandos-to-cryptroot-unlock	2018-08-19 14:58:40 +0000
+++ mandos-to-cryptroot-unlock	1970-01-01 00:00:00 +0000
@@ -1,82 +0,0 @@
-#!/bin/sh
-# 
-# Script to get password from plugin-runner to cryptroot-unlock
-# 
-# Copyright © 2018 Teddy Hogeborn
-# Copyright © 2018 Björn Påhlsson
-# 
-# This file is part of Mandos.
-# 
-# Mandos is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-# 
-#     Mandos is distributed in the hope that it will be useful, but
-#     WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-# 
-# You should have received a copy of the GNU General Public License
-# along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
-# 
-# Contact the authors at <mandos@recompile.se>.
-
-# This script is made to run in the initramfs, and must not be run in
-# the normal system environment.
-
-# Temporary file for the password
-passfile=$(mktemp -p /run -t mandos.XXXXXX)
-trap "rm -f -- $passfile 2>/dev/null" EXIT
-
-# Disable the plugins which conflict with "askpass" as distributed by
-# cryptsetup.
-cat <<-EOF >>/conf/conf.d/mandos/plugin-runner.conf
-
-	--disable=askpass-fifo
-	--disable=password-prompt
-	--disable=plymouth
-EOF
-
-# In case a password is retrieved by other means than by plugin-runner
-# (such as typing it on the console into the prompt given by the
-# "askpass" program), this dummy plugin will be made to exit
-# successfully, thereby forcing plugin-runner to stop all its plugins
-# and also exit itself.
-cat <<-EOF > /lib/mandos/plugins.d/dummy
-	#!/bin/sh
-	
-	while [ -e /run/mandos-keep-running ]; do
-	    sleep 1
-	done
-	
-	# exit successfully to force plugin-runner to finish
-	exit 0
-EOF
-chmod u=rwx,go=rx /lib/mandos/plugins.d/dummy
-
-# This file is the flag which keeps the dummy plugin running
-touch /run/mandos-keep-running
-
-# Keep running plugin-runner and trying any password, until either a
-# password is accepted by cryptroot-unlock, or plugin-runner fails, or
-# the file /run/mandos-keep-running has been removed.
-while type cryptroot-unlock >/dev/null 2>&1; do
-    /lib/mandos/plugin-runner > "$passfile" &
-    echo $! > /run/mandos-plugin-runner.pid
-    wait %% || break
-
-    # Try this password ten times (or ten seconds)
-    for loop in 1 2 3 4 5 6 7 8 9 10; do
-	if [ -e /run/mandos-keep-running ]; then
-	    cryptroot-unlock < "$passfile" >/dev/null 2>&1 && break 2
-	    sleep 1
-	else
-	    break 2
-	fi
-    done
-done
-
-exec >/dev/null 2>&1
-
-rm -f /run/mandos-plugin-runner.pid /run/mandos-keep-running

=== modified file 'mandos.conf'
--- mandos.conf	2015-07-20 03:03:33 +0000
+++ mandos.conf	2009-02-13 05:38:21 +0000
@@ -4,26 +4,32 @@
 # These are the default values for the server, uncomment and change
 # them if needed.
 
+
 # If "interface" is set, the server will only listen to a specific
 # network interface.
 ;interface = 
 
+
 # If "address" is set, the server will only listen to a specific
 # address.  This must currently be an IPv6 address; an IPv4 address
 # can be specified using the "::FFFF:192.0.2.3" syntax.  Also, if this
 # is a link-local address, an interface should be set above.
 ;address =
 
+
 # If "port" is set, the server to bind to that port. By default, the
 # server will listen to an arbitrary port.
 ;port = 
 
+
 # If "debug" is true, the server will run in the foreground and print
 # a lot of debugging information.
 ;debug = False
 
+
 # GnuTLS priority for the TLS handshake.  See gnutls_priority_init(3).
-;priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:!RSA:+SIGN-DSA-SHA256
+;priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP
+
 
 # Zeroconf service name.  You need to change this if you for some
 # reason want to run more than one server on the same *host*.
@@ -36,18 +42,3 @@
 
 # Whether to use IPv6.  (Changing this is NOT recommended.)
 ;use_ipv6 = True
-
-# Whether to restore saved state on startup
-;restore = True
-
-# The directory where state is saved
-;statedir = /var/lib/mandos
-
-# Whether to run in the foreground
-;foreground = False
-
-# File descriptor number to use for network socket
-;socket = 
-
-# Whether to use ZeroConf; if false, requires port or socket
-;zeroconf = True

=== modified file 'mandos.conf.xml'
--- mandos.conf.xml	2019-02-10 04:20:26 +0000
+++ mandos.conf.xml	2011-10-05 16:00:56 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/mandos.conf</filename>">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -34,16 +34,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -163,33 +154,6 @@
 	</listitem>
       </varlistentry>
       
-      <varlistentry>
-	<term><option>restore<literal> = </literal>{ <literal
-	  >1</literal> | <literal>yes</literal> | <literal
-	  >true</literal> | <literal>on</literal> | <literal
-	  >0</literal> | <literal>no</literal> | <literal
-	  >false</literal> | <literal>off</literal> }</option></term>
-	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="restore"/>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>statedir<literal> = </literal><replaceable
-	>DIRECTORY</replaceable></option></term>
-	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="statedir"/>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>socket<literal> = </literal><replaceable
-	>NUMBER</replaceable></option></term>
-	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="socket"/>
-	</listitem>
-      </varlistentry>
-      
     </variablelist>
   </refsect1>
   
@@ -207,7 +171,6 @@
       built-in module <systemitem class="library">ConfigParser</systemitem>
       requires it.
     </para>
-    <xi:include href="bugs.xml"/>
   </refsect1>
   
   <refsect1 id="example">
@@ -230,13 +193,11 @@
 interface = eth0
 address = fe80::aede:48ff:fe71:f6f2
 port = 1025
-debug = True
-priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:!RSA
+debug = true
+priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP
 servicename = Daena
 use_dbus = False
 use_ipv6 = True
-restore = True
-statedir = /var/lib/mandos
       </programlisting>
     </informalexample>
   </refsect1>

=== modified file 'mandos.lsm'
--- mandos.lsm	2019-02-11 06:31:42 +0000
+++ mandos.lsm	2011-10-05 16:00:56 +0000
@@ -1,23 +1,22 @@
 Begin4
 Title:		Mandos
-Version:	1.8.3
-Entered-date:	2019-02-11
+Version:	1.3.1
+Entered-date:	2011-07-27
 Description:	The Mandos system allows computers to have encrypted
-		root file systems and at the same time be capable of
-		remote and/or unattended reboots.
+root file systems and at the same time be capable of remote and/or
+unattended reboots.
 Keywords:	boot, encryption, luks, cryptsetup, network, openpgp,
-		tls, dm-crypt
+tls, dm-crypt
 Author:		teddy@recompile.se (Teddy Hogeborn),
 		belorn@recompile.se (Björn Påhlsson)
 Maintained-by:	teddy@recompile.se (Teddy Hogeborn),
 		belorn@recompile.se (Björn Påhlsson)
-Primary-site:	https://www.recompile.se/mandos
-		182K mandos_1.8.3.orig.tar.gz
-Alternate-site:	ftp://ftp.recompile.se/pub/mandos
-		182K mandos_1.8.3.orig.tar.gz
-Platforms:	Requires GCC, GNU libC, Avahi, GnuPG, Python 2.7, and
-		various other libraries.  While made for Debian
-		GNU/Linux, it is probably portable to other
-		distributions, but not other Unixes.
-Copying-policy:	GNU General Public License version 3.0 or later
+Primary-site: http://www.recompile.se/mandos
+	      136K mandos_1.3.1.orig.tar.gz
+Alternate-site: ftp://ftp.recompile.se/pub/mandos
+		136K mandos_1.3.1.orig.tar.gz
+Platforms:	Requires GCC, GNU libC, Avahi, GnuPG, Python 2.6, and
+various other libraries.  While made for Debian GNU/Linux, it is
+probably portable to other distributions, but not other Unixes.
+Copying-policy: GNU General Public License version 3.0 or later
 End

=== removed file 'mandos.service'
--- mandos.service	2017-08-20 14:14:14 +0000
+++ mandos.service	1970-01-01 00:00:00 +0000
@@ -1,35 +0,0 @@
-[Unit]
-Description=Server of encrypted passwords to Mandos clients
-Documentation=man:intro(8mandos) man:mandos(8)
-## If the server is configured to listen to a specific IP or network
-## interface, it may be necessary to change "network.target" to
-## "network-online.target".
-After=network.target
-## If the server is configured to not use ZeroConf, these two lines
-## become unnecessary and should be removed or commented out.
-After=avahi-daemon.service
-Requisite=avahi-daemon.service
-
-[Service]
-## If the server's D-Bus interface is disabled, the "BusName" setting
-## should be removed or commented out.
-BusName=se.recompile.Mandos
-ExecStart=/usr/sbin/mandos --foreground
-Restart=always
-KillMode=mixed
-## Using socket activation won't work, because systemd always does
-## bind() on the socket, and also won't announce the ZeroConf service.
-#ExecStart=/usr/sbin/mandos --foreground --socket=0
-#StandardInput=socket
-# Restrict what the Mandos daemon can do.  Note that this also affects
-# "checker" programs!
-PrivateTmp=yes
-PrivateDevices=yes
-ProtectSystem=full
-ProtectHome=yes
-CapabilityBoundingSet=CAP_KILL CAP_SETGID CAP_SETUID CAP_DAC_OVERRIDE CAP_NET_RAW
-ProtectKernelTunables=yes
-ProtectControlGroups=yes
-
-[Install]
-WantedBy=multi-user.target

=== modified file 'mandos.xml'
--- mandos.xml	2019-02-10 04:20:26 +0000
+++ mandos.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -35,14 +35,6 @@
       <year>2009</year>
       <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -102,18 +94,6 @@
       <arg><option>--no-dbus</option></arg>
       <sbr/>
       <arg><option>--no-ipv6</option></arg>
-      <sbr/>
-      <arg><option>--no-restore</option></arg>
-      <sbr/>
-      <arg><option>--statedir
-      <replaceable>DIRECTORY</replaceable></option></arg>
-      <sbr/>
-      <arg><option>--socket
-      <replaceable>FD</replaceable></option></arg>
-      <sbr/>
-      <arg><option>--foreground</option></arg>
-      <sbr/>
-      <arg><option>--no-zeroconf</option></arg>
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
@@ -295,48 +275,6 @@
 	  <xi:include href="mandos-options.xml" xpointer="ipv6"/>
 	</listitem>
       </varlistentry>
-      
-      <varlistentry>
-	<term><option>--no-restore</option></term>
-	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="restore"/>
-	  <para>
-	    See also <xref linkend="persistent_state"/>.
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>--statedir
-	<replaceable>DIRECTORY</replaceable></option></term>
-	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="statedir"/>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>--socket
-	<replaceable>FD</replaceable></option></term>
-	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="socket"/>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>--foreground</option></term>
-	<listitem>
-	  <xi:include href="mandos-options.xml"
-		      xpointer="foreground"/>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>--no-zeroconf</option></term>
-	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="zeroconf"/>
-	</listitem>
-      </varlistentry>
-      
     </variablelist>
   </refsect1>
   
@@ -362,11 +300,11 @@
       start a TLS protocol handshake with a slight quirk: the Mandos
       server program acts as a TLS <quote>client</quote> while the
       connecting Mandos client acts as a TLS <quote>server</quote>.
-      The Mandos client must supply a TLS public key, and the key ID
-      of this public key is used by the Mandos server to look up (in a
-      list read from <filename>clients.conf</filename> at start time)
-      which binary blob to give the client.  No other authentication
-      or authorization is done by the server.
+      The Mandos client must supply an OpenPGP certificate, and the
+      fingerprint of this certificate is used by the Mandos server to
+      look up (in a list read from <filename>clients.conf</filename>
+      at start time) which binary blob to give the client.  No other
+      authentication or authorization is done by the server.
     </para>
     <table>
       <title>Mandos Protocol (Version 1)</title><tgroup cols="3"><thead>
@@ -392,7 +330,7 @@
 	</emphasis></entry>
       </row>
       <row>
-	<entry>Public key (part of TLS handshake)</entry>
+	<entry>OpenPGP public key (part of TLS handshake)</entry>
 	<entry>-><!-- &rarr; --></entry>
       </row>
       <row>
@@ -419,7 +357,9 @@
       extended timeout, checker program, and interval between checks
       can be configured both globally and per client; see
       <citerefentry><refentrytitle>mandos-clients.conf</refentrytitle>
-      <manvolnum>5</manvolnum></citerefentry>.
+      <manvolnum>5</manvolnum></citerefentry>.  A client successfully
+      receiving its password will also be treated as a successful
+      checker run.
     </para>
   </refsect1>
   
@@ -447,24 +387,12 @@
     <title>LOGGING</title>
     <para>
       The server will send log message with various severity levels to
-      <filename class="devicefile">/dev/log</filename>.  With the
+      <filename>/dev/log</filename>.  With the
       <option>--debug</option> option, it will log even more messages,
       and also show them on the console.
     </para>
   </refsect1>
   
-  <refsect1 id="persistent_state">
-    <title>PERSISTENT STATE</title>
-    <para>
-      Client settings, initially read from
-      <filename>clients.conf</filename>, are persistent across
-      restarts, and run-time changes will override settings in
-      <filename>clients.conf</filename>.  However, if a setting is
-      <emphasis>changed</emphasis> (or a client added, or removed) in
-      <filename>clients.conf</filename>, this will take precedence.
-    </para>
-  </refsect1>
-  
   <refsect1 id="dbus_interface">
     <title>D-BUS INTERFACE</title>
     <para>
@@ -532,31 +460,16 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename>/run/mandos.pid</filename></term>
+	<term><filename>/var/run/mandos.pid</filename></term>
 	<listitem>
 	  <para>
 	    The file containing the process id of the
 	    <command>&COMMANDNAME;</command> process started last.
-	    <emphasis >Note:</emphasis> If the <filename
-	    class="directory">/run</filename> directory does not
-	    exist, <filename>/var/run/mandos.pid</filename> will be
-	    used instead.
-	  </para>
-	</listitem>
-      </varlistentry>
-      <varlistentry>
-	<term><filename
-	class="directory">/var/lib/mandos</filename></term>
-	<listitem>
-	  <para>
-	    Directory where persistent state will be saved.  Change
-	    this with the <option>--statedir</option> option.  See
-	    also the <option>--no-restore</option> option.
-	  </para>
-	</listitem>
-      </varlistentry>
-      <varlistentry>
-	<term><filename class="devicefile">/dev/log</filename></term>
+	  </para>
+	</listitem>
+      </varlistentry>
+      <varlistentry>
+	<term><filename>/dev/log</filename></term>
 	<listitem>
 	  <para>
 	    The Unix domain socket to where local syslog messages are
@@ -585,9 +498,23 @@
       backtrace.  This could be considered a feature.
     </para>
     <para>
+      Currently, if a client is disabled due to having timed out, the
+      server does not record this fact onto permanent storage.  This
+      has some security implications, see <xref linkend="clients"/>.
+    </para>
+    <para>
       There is no fine-grained control over logging and debug output.
     </para>
-    <xi:include href="bugs.xml"/>
+    <para>
+      Debug mode is conflated with running in the foreground.
+    </para>
+    <para>
+      The console log messages do not show a time stamp.
+    </para>
+    <para>
+      This server does not check the expire time of clients’ OpenPGP
+      keys.
+    </para>
   </refsect1>
   
   <refsect1 id="example">
@@ -603,9 +530,9 @@
     <informalexample>
       <para>
 	Run the server in debug mode, read configuration files from
-	the <filename class="directory">~/mandos</filename> directory,
-	and use the Zeroconf service name <quote>Test</quote> to not
-	collide with any other official Mandos server on this host:
+	the <filename>~/mandos</filename> directory, and use the
+	Zeroconf service name <quote>Test</quote> to not collide with
+	any other official Mandos server on this host:
       </para>
       <para>
 
@@ -643,12 +570,12 @@
       <title>CLIENTS</title>
       <para>
 	The server only gives out its stored data to clients which
-	does have the correct key ID of the stored key ID.  This is
-	guaranteed by the fact that the client sends its public key in
-	the TLS handshake; this ensures it to be genuine.  The server
-	computes the key ID of the key itself and looks up the key ID
-	in its list of clients. The <filename>clients.conf</filename>
-	file (see
+	does have the OpenPGP key of the stored fingerprint.  This is
+	guaranteed by the fact that the client sends its OpenPGP
+	public key in the TLS handshake; this ensures it to be
+	genuine.  The server computes the fingerprint of the key
+	itself and looks up the fingerprint in its list of
+	clients. The <filename>clients.conf</filename> file (see
 	<citerefentry><refentrytitle>mandos-clients.conf</refentrytitle>
 	<manvolnum>5</manvolnum></citerefentry>)
 	<emphasis>must</emphasis> be made non-readable by anyone
@@ -660,6 +587,21 @@
 	compromised if they are gone for too long.
       </para>
       <para>
+	If a client is compromised, its downtime should be duly noted
+	by the server which would therefore disable the client.  But
+	if the server was ever restarted, it would re-read its client
+	list from its configuration file and again regard all clients
+	therein as enabled, and hence eligible to receive their
+	passwords.  Therefore, be careful when restarting servers if
+	it is suspected that a client has, in fact, been compromised
+	by parties who may now be running a fake Mandos client with
+	the keys from the non-encrypted initial <acronym>RAM</acronym>
+	image of the client host.  What should be done in that case
+	(if restarting the server program really is necessary) is to
+	stop the server program, edit the configuration file to omit
+	any suspect clients, and restart the server program.
+      </para>
+      <para>
 	For more details on client-side security, see
 	<citerefentry><refentrytitle>mandos-client</refentrytitle>
 	<manvolnum>8mandos</manvolnum></citerefentry>.
@@ -706,13 +648,14 @@
       </varlistentry>
       <varlistentry>
 	<term>
-	  <ulink url="https://gnutls.org/">GnuTLS</ulink>
+	  <ulink url="http://www.gnu.org/software/gnutls/"
+	  >GnuTLS</ulink>
 	</term>
       <listitem>
 	<para>
 	  GnuTLS is the library this server uses to implement TLS for
 	  communicating securely with the client, and at the same time
-	  confidently get the client’s public key.
+	  confidently get the client’s public OpenPGP key.
 	</para>
       </listitem>
       </varlistentry>
@@ -750,12 +693,12 @@
       </varlistentry>
       <varlistentry>
 	<term>
-	  RFC 5246: <citetitle>The Transport Layer Security (TLS)
-	  Protocol Version 1.2</citetitle>
+	  RFC 4346: <citetitle>The Transport Layer Security (TLS)
+	  Protocol Version 1.1</citetitle>
 	</term>
       <listitem>
 	<para>
-	  TLS 1.2 is the protocol implemented by GnuTLS.
+	  TLS 1.1 is the protocol implemented by GnuTLS.
 	</para>
       </listitem>
       </varlistentry>
@@ -771,28 +714,13 @@
       </varlistentry>
       <varlistentry>
 	<term>
-	  RFC 7250: <citetitle>Using Raw Public Keys in Transport
-	  Layer Security (TLS) and Datagram Transport Layer Security
-	  (DTLS)</citetitle>
-	</term>
-      <listitem>
-	<para>
-	  This is implemented by GnuTLS version 3.6.6 and is, if
-	  present, used by this server so that raw public keys can be
-	  used.
-	</para>
-      </listitem>
-      </varlistentry>
-      <varlistentry>
-	<term>
-	  RFC 6091: <citetitle>Using OpenPGP Keys for Transport Layer
-	  Security (TLS) Authentication</citetitle>
-	</term>
-      <listitem>
-	<para>
-	  This is implemented by GnuTLS before version 3.6.0 and is,
-	  if present, used by this server so that OpenPGP keys can be
-	  used.
+	  RFC 5081: <citetitle>Using OpenPGP Keys for Transport Layer
+	  Security</citetitle>
+	</term>
+      <listitem>
+	<para>
+	  This is implemented by GnuTLS and used by this server so
+	  that OpenPGP keys can be used.
 	</para>
       </listitem>
       </varlistentry>

=== removed directory 'network-hooks.d'
=== removed file 'network-hooks.d/bridge'
--- network-hooks.d/bridge	2018-02-08 10:23:55 +0000
+++ network-hooks.d/bridge	1970-01-01 00:00:00 +0000
@@ -1,93 +0,0 @@
-#!/bin/sh
-#
-# This is an example of a Mandos client network hook.  This hook
-# brings up a bridge interface as specified in a separate
-# configuration file.  To be used, this file and any needed
-# configuration file(s) should be copied into the
-# /etc/mandos/network-hooks.d directory.
-# 
-# Copyright © 2012-2018 Teddy Hogeborn
-# Copyright © 2012-2018 Björn Påhlsson
-# 
-# Copying and distribution of this file, with or without modification,
-# are permitted in any medium without royalty provided the copyright
-# notice and this notice are preserved.  This file is offered as-is,
-# without any warranty.
-
-set -e
-
-CONFIG="$MANDOSNETHOOKDIR/bridge.conf"
-
-addrtoif(){
-    grep -liFe "$1" /sys/class/net/*/address \
-	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,' -e "/^${BRIDGE}\$/d"
-}
-
-# Read config file, which must set "BRIDGE", "PORT_ADDRESSES", and
-# optionally "IPADDRS" and "ROUTES".
-if [ -e "$CONFIG" ]; then
-    . "$CONFIG"
-fi
-
-if [ -z "$BRIDGE" ] || [ -z "$PORT_ADDRESSES" ]; then
-    exit
-fi
-
-if [ -n "$DEVICE" ]; then
-    case "$DEVICE" in
-	*,"$BRIDGE"|*,"$BRIDGE",*|"$BRIDGE",*|"$BRIDGE") :;;
-	*) exit;;
-    esac
-fi
-
-brctl="/sbin/brctl"
-for b in "$brctl" /usr/sbin/brctl; do
-    if [ -e "$b" ]; then
-	brctl="$b"
-	break
-    fi
-done
-
-do_start(){
-    "$brctl" addbr "$BRIDGE"
-    for address in $PORT_ADDRESSES; do
-	interface=`addrtoif "$address"`
-	"$brctl" addif "$BRIDGE" "$interface"
-	ip link set dev "$interface" up
-    done
-    ip link set dev "$BRIDGE" up
-    sleep "${DELAY%%.*}"
-    if [ -n "$IPADDRS" ]; then
-        for ipaddr in $IPADDRS; do
-	    ip addr add "$ipaddr" dev "$BRIDGE"
-	done
-    fi
-    if [ -n "$ROUTES" ]; then
-        for route in $ROUTES; do
-	    ip route add "$route" dev "$BRIDGE"
-	done
-    fi
-}
-
-do_stop(){
-    ip link set dev "$BRIDGE" down
-    for address in $PORT_ADDRESSES; do
-	interface=`addrtoif "$address"`
-	ip link set dev "$interface" down
-	"$brctl" delif "$BRIDGE" "$interface"
-    done
-    "$brctl" delbr "$BRIDGE"
-}
-
-case "${MODE:-$1}" in
-    start|stop)
-	do_"${MODE:-$1}"
-	;;
-    files)
-	echo /bin/ip
-	echo "$brctl"
-	;;
-    modules)
-	echo bridge
-	;;
-esac

=== removed file 'network-hooks.d/bridge.conf'
--- network-hooks.d/bridge.conf	2011-12-31 13:25:58 +0000
+++ network-hooks.d/bridge.conf	1970-01-01 00:00:00 +0000
@@ -1,11 +0,0 @@
-## Required
-
-#BRIDGE=br0
-
-#PORT_ADDRESSES="00:11:22:33:44:55 11:22:33:44:55:66"
-
-## Optional
-
-#IPADDRS="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
-
-#ROUTES="192.0.2.0/24 2001:DB8::/32"

=== removed file 'network-hooks.d/openvpn'
--- network-hooks.d/openvpn	2018-02-08 10:23:55 +0000
+++ network-hooks.d/openvpn	1970-01-01 00:00:00 +0000
@@ -1,66 +0,0 @@
-#!/bin/sh
-#
-# This is an example of a Mandos client network hook.  This hook
-# brings up an OpenVPN interface as specified in a separate
-# configuration file.  To be used, this file and any needed
-# configuration file(s) should be copied into the
-# /etc/mandos/network-hooks.d directory.
-# 
-# Copyright © 2012-2018 Teddy Hogeborn
-# Copyright © 2012-2018 Björn Påhlsson
-# 
-# Copying and distribution of this file, with or without modification,
-# are permitted in any medium without royalty provided the copyright
-# notice and this notice are preserved.  This file is offered as-is,
-# without any warranty.
-
-set -e
-
-CONFIG="openvpn.conf"
-
-# Extract the "dev" setting from the config file
-VPNDEVICE=`sed -n -e 's/[[:space:]]#.*//' \
-    -e 's/^[[:space:]]*dev[[:space:]]\+//p' \
-    "$MANDOSNETHOOKDIR/$CONFIG"`
-
-PIDFILE=/run/openvpn-mandos.pid
-
-# Exit if no device set in config
-if [ -z "$VPNDEVICE" ]; then
-    exit
-fi
-
-# Exit if DEVICE is set and it doesn't match the VPN interface
-if [ -n "$DEVICE" ]; then
-    case "$DEVICE" in
-	*,"$VPNDEVICE"*|"$VPNDEVICE"*) :;;
-	*) exit;;
-    esac
-fi
-
-openvpn=/usr/sbin/openvpn
-
-do_start(){
-    "$openvpn" --cd "$MANDOSNETHOOKDIR" --daemon 'openvpn(Mandos)' \
-	--writepid "$PIDFILE" --config "$CONFIG"
-     sleep "$DELAY"
-}
-
-do_stop(){
-    PID="`cat \"$PIDFILE\"`"
-    if [ "$PID" -gt 0 ]; then
-	kill "$PID"
-    fi
-}
-
-case "${MODE:-$1}" in
-    start|stop)
-	do_"${MODE:-$1}"
-	;;
-    files)
-	echo "$openvpn"
-	;;
-    modules)
-	echo tun
-	;;
-esac

=== removed file 'network-hooks.d/openvpn.conf'
--- network-hooks.d/openvpn.conf	2011-12-02 16:52:50 +0000
+++ network-hooks.d/openvpn.conf	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-# Sample OpenVPN configuration file
-# Uncomment and change - see openvpn(8)
-
-# Network device.
-#dev tun
-
-# Our remote peer
-#remote 192.0.2.3
-#float 192.0.2.3
-#port 1194
-
-# VPN endpoints
-#ifconfig 10.1.0.1 10.1.0.2
-
-# A pre-shared static key
-#secret openvpn.key
-
-# Cipher
-#cipher AES-128-CBC

=== removed file 'network-hooks.d/wireless'
--- network-hooks.d/wireless	2018-02-08 10:23:55 +0000
+++ network-hooks.d/wireless	1970-01-01 00:00:00 +0000
@@ -1,165 +0,0 @@
-#!/bin/sh
-#
-# This is an example of a Mandos client network hook.  This hook
-# brings up a wireless interface as specified in a separate
-# configuration file.  To be used, this file and any needed
-# configuration file(s) should be copied into the
-# /etc/mandos/network-hooks.d directory.
-# 
-# Copyright © 2012-2018 Teddy Hogeborn
-# Copyright © 2012-2018 Björn Påhlsson
-# 
-# Copying and distribution of this file, with or without modification,
-# are permitted in any medium without royalty provided the copyright
-# notice and this notice are preserved.  This file is offered as-is,
-# without any warranty.
-
-set -e
-
-RUNDIR="/run"
-CTRL="$RUNDIR/wpa_supplicant-global"
-CTRLDIR="$RUNDIR/wpa_supplicant"
-PIDFILE="$RUNDIR/wpa_supplicant-mandos.pid"
-
-CONFIG="$MANDOSNETHOOKDIR/wireless.conf"
-
-addrtoif(){
-    grep -liFe "$1" /sys/class/net/*/address \
-	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
-}
-
-# Read config file
-if [ -e "$CONFIG" ]; then
-    . "$CONFIG"
-else
-    exit
-fi
-
-ifkeys=`sed -n -e 's/^ADDRESS_\([^=]*\)=.*/\1/p' "$CONFIG" | sort -u`
-
-# Exit if DEVICE is set and is not any of the wireless interfaces
-if [ -n "$DEVICE" ]; then
-    while :; do
-	for KEY in $ifkeys; do
-	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
-	    INTERFACE=`addrtoif "$ADDRESS"`
-	    
-	    case "$DEVICE" in
-		*,"$INTERFACE"|*,"$INTERFACE",*|"$INTERFACE",*|"$INTERFACE")
-		    break 2;;
-	    esac
-	done
-	exit
-    done
-fi
-
-wpa_supplicant=/sbin/wpa_supplicant
-wpa_cli=/sbin/wpa_cli
-ip=/bin/ip
-
-# Used by the wpa_interface_* functions in the wireless.conf file
-wpa_cli_set(){
-    case "$1" in
-        ssid|psk) arg="\"$2\"" ;;
-        *) arg="$2" ;;
-    esac
-    "$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" set_network "$NETWORK" \
-	"$1" "$arg" 2>&1 | sed -e '/^OK$/d'
-}
-
-if [ $VERBOSITY -gt 0 ]; then
-    WPAS_OPTIONS="-d $WPAS_OPTIONS"
-fi
-if [ -n "$PIDFILE" ]; then
-    WPAS_OPTIONS="-P$PIDFILE $WPAS_OPTIONS"
-fi
-
-do_start(){
-    mkdir -m u=rwx,go= -p "$CTRLDIR"
-    "$wpa_supplicant" -B -g "$CTRL" -p "$CTRLDIR" $WPAS_OPTIONS
-    for KEY in $ifkeys; do
-	ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
-	INTERFACE=`addrtoif "$ADDRESS"`
-	DRIVER=`eval 'echo "$WPA_DRIVER_'"$KEY"\"`
-	IFDELAY=`eval 'echo "$DELAY_'"$KEY"\"`
-	"$wpa_cli" -g "$CTRL" interface_add "$INTERFACE" "" \
-	    "${DRIVER:-wext}" "$CTRLDIR" > /dev/null \
-	    | sed -e '/^OK$/d'
-        NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" add_network`
-	eval wpa_interface_"$KEY"
-	"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" enable_network \
-	    "$NETWORK" | sed -e '/^OK$/d'
-	sleep "${IFDELAY:-$DELAY}" &
-	sleep=$!
-	while :; do
-	    kill -0 $sleep 2>/dev/null || break
-	    STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" status \
-		| sed -n -e 's/^wpa_state=//p'`
-	    if [ "$STATE" = COMPLETED ]; then
-		while :; do
-		    kill -0 $sleep 2>/dev/null || break 2
-		    UP=`cat /sys/class/net/"$INTERFACE"/operstate`
-		    if [ "$UP" = up ]; then
-			kill $sleep 2>/dev/null
-			break 2
-		    fi
-		    sleep 1
-		done
-	    fi
-	    sleep 1
-	done &
-	wait $sleep || :
-	IPADDRS=`eval 'echo "$IPADDRS_'"$KEY"\"`
-	if [ -n "$IPADDRS" ]; then
-	    if [ "$IPADDRS" = dhcp ]; then
-		ipconfig -c dhcp -d "$INTERFACE" || :
-		#dhclient "$INTERFACE"
-	    else
-		for ipaddr in $IPADDRS; do
-		    "$ip" addr add "$ipaddr" dev "$INTERFACE"
-		done
-	    fi
-	fi
-	ROUTES=`eval 'echo "$ROUTES_'"$KEY"\"`
-	if [ -n "$ROUTES" ]; then
-	    for route in $ROUTES; do
-		"$ip" route add "$route" dev "$INTERFACE"
-	    done
-	fi
-    done
-}
-
-do_stop(){
-    "$wpa_cli" -g "$CTRL" terminate 2>&1 | sed -e '/^OK$/d'
-    for KEY in $ifkeys; do
-	ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
-	INTERFACE=`addrtoif "$ADDRESS"`
-	"$ip" addr show scope global permanent dev "$INTERFACE" \
-	    | while read type addr rest; do
-		case "$type" in
-		    inet|inet6)
-			"$ip" addr del "$addr" dev "$INTERFACE"
-			;;
-		esac
-	    done
-	"$ip" link set dev "$INTERFACE" down
-    done
-}
-
-case "${MODE:-$1}" in
-    start|stop)
-	do_"${MODE:-$1}"
-	;;
-    files)
-	echo "$wpa_supplicant"
-	echo "$wpa_cli"
-	echo "$ip"
-	;;
-    modules)
-	if [ "$IPADDRS" = dhcp ]; then
-	    echo af_packet
-	fi
-	sed -n -e 's/#.*$//' -e 's/[ 	]*$//' \
-	    -e 's/^MODULE_[^=]\+=//p' "$CONFIG"
-	;;
-esac

=== removed file 'network-hooks.d/wireless.conf'
--- network-hooks.d/wireless.conf	2011-12-31 13:25:58 +0000
+++ network-hooks.d/wireless.conf	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-# Extra options for wpa_supplicant, if any
-#WPAS_OPTIONS=""
-
-# wlan0
-ADDRESS_0=00:11:22:33:44:55
-MODULE_0=ath9k
-#WPA_DRIVER_0=wext
-wpa_interface_0(){
-    # Use this format to set simple things:
-    wpa_cli_set ssid home
-    wpa_cli_set psk "secret passphrase"
-    # Use this format to do more complex things with wpa_cli:
-    #"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" bssid "$NETWORK" 00:11:22:33:44:55
-    #"$wpa_cli" -g "$CTRL" ping
-}
-#DELAY_0=10
-IPADDRS_0=dhcp
-#IPADDRS_0="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
-#ROUTES_0="192.0.2.0/24 2001:DB8::/32"
-
-#ADDRESS_1=11:22:33:44:55:66
-#MODULE_1=...
-#...

=== modified file 'overview.xml'
--- overview.xml	2019-02-09 23:23:26 +0000
+++ overview.xml	2008-09-13 15:36:18 +0000
@@ -8,10 +8,10 @@
   program in the initial <acronym>RAM</acronym> disk environment which
   will communicate with a server over a network.  All network
   communication is encrypted using <acronym>TLS</acronym>.  The
-  clients are identified by the server using a TLS key; each client
-  has one unique to it.  The server sends the clients an encrypted
-  password.  The encrypted password is decrypted by the clients using
-  a separate OpenPGP key, and the password is then used to unlock the
-  root file system, whereupon the computers can continue booting
-  normally.
+  clients are identified by the server using an OpenPGP key; each
+  client has one unique to it.  The server sends the clients an
+  encrypted password.  The encrypted password is decrypted by the
+  clients using the same OpenPGP key, and the password is then used to
+  unlock the root file system, whereupon the computers can continue
+  booting normally.
 </para>

=== removed directory 'plugin-helpers'
=== removed file 'plugin-helpers/mandos-client-iprouteadddel.c'
--- plugin-helpers/mandos-client-iprouteadddel.c	2018-02-18 01:29:21 +0000
+++ plugin-helpers/mandos-client-iprouteadddel.c	1970-01-01 00:00:00 +0000
@@ -1,282 +0,0 @@
-/*  -*- coding: utf-8 -*- */
-/* 
- * iprouteadddel - Add or delete direct route to a local IP address
- * 
- * Copyright © 2015-2018 Teddy Hogeborn
- * Copyright © 2015-2018 Björn Påhlsson
- * 
- * This file is part of Mandos.
- * 
- * Mandos is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- * 
- * Mandos is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
- * 
- * Contact the authors at <mandos@recompile.se>.
- */
-
-#define _GNU_SOURCE		/* program_invocation_short_name */
-#include <stdbool.h>		/* bool, false, true */
-#include <stdio.h>		/* fprintf(), stderr, FILE, vfprintf */
-#include <errno.h>		/* program_invocation_short_name,
-				   errno, perror(), EINVAL, ENOMEM */
-#include <stdarg.h>		/* va_list, va_start */
-#include <stdlib.h> 		/* EXIT_SUCCESS */
-#include <argp.h>		/* struct argp_option, error_t, struct
-				   argp_state, ARGP_KEY_ARG,
-				   argp_usage(), ARGP_KEY_END,
-				   ARGP_ERR_UNKNOWN, struct argp,
-				   argp_parse() */
-#include <sysexits.h>		/* EX_USAGE, EX_OSERR */
-#include <netinet/ip.h>		/* sa_family_t, AF_INET6, AF_INET */
-#include <inttypes.h>		/* PRIdMAX, intmax_t */
-
-#include <netlink/netlink.h>	/* struct nl_addr, nl_addr_parse(),
-				   nl_geterror(),
-				   nl_addr_get_family(),
-				   nl_addr_put() */
-#include <netlink/route/route.h> /* struct rtnl_route,
-				    struct rtnl_nexthop,
-				    rtnl_route_alloc(),
-				    rtnl_route_set_family(),
-				    rtnl_route_set_protocol(),
-				    RTPROT_BOOT,
-				    rtnl_route_set_scope(),
-				    RT_SCOPE_LINK,
-				    rtnl_route_set_type(),
-				    RTN_UNICAST,
-				    rtnl_route_set_dst(),
-				    rtnl_route_set_table(),
-				    RT_TABLE_MAIN,
-				    rtnl_route_nh_alloc(),
-				    rtnl_route_nh_set_ifindex(),
-				    rtnl_route_add_nexthop(),
-				    rtnl_route_add(),
-				    rtnl_route_delete(),
-				    rtnl_route_put(),
-				    rtnl_route_nh_free() */
-#include <netlink/socket.h>	/* struct nl_sock, nl_socket_alloc(),
-				   nl_connect(), nl_socket_free() */
-#include <netlink/route/link.h>	/* rtnl_link_get_kernel(),
-				   rtnl_link_get_ifindex(),
-				   rtnl_link_put() */
-
-bool debug = false;
-const char *argp_program_version = "mandos-client-iprouteadddel " VERSION;
-const char *argp_program_bug_address = "<mandos@recompile.se>";
-
-/* Function to use when printing errors */
-void perror_plus(const char *print_text){
-  int e = errno;
-  fprintf(stderr, "Mandos plugin helper %s: ",
-	  program_invocation_short_name);
-  errno = e;
-  perror(print_text);
-}
-
-__attribute__((format (gnu_printf, 2, 3), nonnull))
-int fprintf_plus(FILE *stream, const char *format, ...){
-  va_list ap;
-  va_start (ap, format);
-  
-  fprintf(stream, "Mandos plugin helper %s: ",
-	  program_invocation_short_name);
-  return vfprintf(stream, format, ap);
-}
-
-int main(int argc, char *argv[]){
-  int ret;
-  int exitcode = EXIT_SUCCESS;
-  struct arguments {
-    bool add;			/* true: add, false: delete */
-    char *address;		/* IP address as string */
-    struct nl_addr *nl_addr;	/* Netlink IP address */
-    char *interface;		/* interface name */
-  } arguments = { .add = true, .address = NULL, .interface = NULL };
-  struct argp_option options[] = {
-    { .name = "debug", .key = 128,
-      .doc = "Debug mode" },
-    { .name = NULL }
-  };
-  struct rtnl_route *route = NULL;
-  struct rtnl_nexthop *nexthop = NULL;
-  struct nl_sock *sk = NULL;
-  
-  error_t parse_opt(int key, char *arg, struct argp_state *state){
-    int lret;
-    errno = 0;
-    switch(key){
-    case 128:			/* --debug */
-      debug = true;
-      break;
-    case ARGP_KEY_ARG:
-      switch(state->arg_num){
-      case 0:
-	if(strcasecmp(arg, "add") == 0){
-	  ((struct arguments *)(state->input))->add = true;
-	} else if(strcasecmp(arg, "delete") == 0){
-	  ((struct arguments *)(state->input))->add = false;
-	} else {
-	  fprintf_plus(stderr, "Unrecognized command: %s\n", arg);
-	  argp_usage(state);
-	}
-	break;
-      case 1:
-	((struct arguments *)(state->input))->address = arg;
-	lret = nl_addr_parse(arg, AF_UNSPEC, &(((struct arguments *)
-						(state->input))
-					       ->nl_addr));
-	if(lret != 0){
-	  fprintf_plus(stderr, "Failed to parse address %s: %s\n",
-		       arg, nl_geterror(lret));
-	  argp_usage(state);
-	}
-	break;
-      case 2:
-	((struct arguments *)(state->input))->interface = arg;
-	break;
-      default:
-	argp_usage(state);
-      }
-      break;
-    case ARGP_KEY_END:
-      if(state->arg_num < 3){
-	argp_usage(state);
-      }
-      break;
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-    return errno;
-  }
-  
-  struct argp argp = { .options = options, .parser = parse_opt,
-		       .args_doc = "[ add | delete ] ADDRESS INTERFACE",
-		       .doc = "Mandos client helper -- Add or delete"
-		       " local route to IP address on interface" };
-  
-  ret = argp_parse(&argp, argc, argv, ARGP_IN_ORDER, 0, &arguments);
-  switch(ret){
-  case 0:
-    break;
-  case EINVAL:
-    exit(EX_USAGE);
-  case ENOMEM:
-  default:
-    errno = ret;
-    perror_plus("argp_parse");
-    exitcode = EX_OSERR;
-    goto end;
-  }
-  /* Get netlink socket */
-  sk = nl_socket_alloc();
-  if(sk == NULL){
-    fprintf_plus(stderr, "Failed to allocate netlink socket: %s\n",
-		 nl_geterror(ret));
-    exitcode = EX_OSERR;
-    goto end;
-  }
-  /* Connect socket to netlink */
-  ret = nl_connect(sk, NETLINK_ROUTE);
-  if(ret < 0){
-    fprintf_plus(stderr, "Failed to connect socket to netlink: %s\n",
-		 nl_geterror(ret));
-    exitcode = EX_OSERR;
-    goto end;
-  }
-  /* Get link object of specified interface */
-  struct rtnl_link *link = NULL;
-  ret = rtnl_link_get_kernel(sk, 0, arguments.interface, &link);
-  if(ret < 0){
-    fprintf_plus(stderr, "Failed to use interface %s: %s\n",
-		 arguments.interface, nl_geterror(ret));
-    exitcode = EX_OSERR;
-    goto end;
-  }
-  /* Get netlink route object */
-  route = rtnl_route_alloc();
-  if(route == NULL){
-    fprintf_plus(stderr, "Failed to get netlink route:\n");
-    exitcode = EX_OSERR;
-    goto end;
-  }
-  /* Get address family of specified address */
-  sa_family_t af = (sa_family_t)nl_addr_get_family(arguments.nl_addr);
-  if(debug){
-    fprintf_plus(stderr, "Address family of %s is %s (%" PRIdMAX
-		 ")\n", arguments.address,
-		 af == AF_INET6 ? "AF_INET6" :
-		 ( af == AF_INET ? "AF_INET" : "UNKNOWN"),
-		 (intmax_t)af);
-  }
-  /* Set route parameters: */
-  rtnl_route_set_family(route, (uint8_t)af);   /* Address family */
-  rtnl_route_set_protocol(route, RTPROT_BOOT); /* protocol - see
-						  ip-route(8) */
-  rtnl_route_set_scope(route, RT_SCOPE_LINK); /* link scope */
-  rtnl_route_set_type(route, RTN_UNICAST);    /* normal unicast
-						 address route */
-  rtnl_route_set_dst(route, arguments.nl_addr); /* Destination
-						   address */
-  rtnl_route_set_table(route, RT_TABLE_MAIN); /* "main" routing
-						 table */
-  /* Create nexthop */
-  nexthop = rtnl_route_nh_alloc();
-  if(nexthop == NULL){
-    fprintf_plus(stderr, "Failed to get netlink route nexthop\n");
-    exitcode = EX_OSERR;
-    goto end;
-  }
-  /* Get index number of specified interface */
-  int ifindex = rtnl_link_get_ifindex(link);
-  if(debug){
-    fprintf_plus(stderr, "ifindex of %s is %d\n", arguments.interface,
-		 ifindex);
-  }
-  /* Set interface index number on nexthop object */
-  rtnl_route_nh_set_ifindex(nexthop, ifindex);
-  /* Set route to use nexthop object */
-  rtnl_route_add_nexthop(route, nexthop);
-  /* Add or delete route? */
-  if(arguments.add){
-    ret = rtnl_route_add(sk, route, NLM_F_EXCL);
-  } else {
-    ret = rtnl_route_delete(sk, route, 0);
-  }
-  if(ret < 0){
-     fprintf_plus(stderr, "Failed to %s route: %s\n",
-		  arguments.add ? "add" : "delete",
-		  nl_geterror(ret));
-    exitcode = EX_OSERR;
-    goto end;
-  }
- end:
-  /* Deallocate route */
-  if(route){
-    rtnl_route_put(route);
-  } else if(nexthop) {
-    /* Deallocate route nexthop */
-    rtnl_route_nh_free(nexthop);
-  }
-  /* Deallocate parsed address */
-  if(arguments.nl_addr){
-    nl_addr_put(arguments.nl_addr);
-  }
-  /* Deallocate link struct */
-  if(link){
-    rtnl_link_put(link);
-  }
-  /* Deallocate netlink socket struct */
-   if(sk){
-    nl_socket_free(sk);
-  }
-  return exitcode;
-}

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2019-02-11 05:14:10 +0000
+++ plugin-runner.c	2011-10-05 16:00:56 +0000
@@ -2,58 +2,59 @@
 /*
  * Mandos plugin runner - Run Mandos plugins
  *
- * Copyright © 2008-2018 Teddy Hogeborn
- * Copyright © 2008-2018 Björn Påhlsson
- * 
- * This file is part of Mandos.
- * 
- * Mandos is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- * 
- * Mandos is distributed in the hope that it will be useful, but
+ * Copyright © 2008-2010 Teddy Hogeborn
+ * Copyright © 2008-2010 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  * 
  * Contact the authors at <mandos@recompile.se>.
  */
 
 #define _GNU_SOURCE		/* TEMP_FAILURE_RETRY(), getline(),
-				   O_CLOEXEC, pipe2() */
+				   asprintf(), O_CLOEXEC */
 #include <stddef.h>		/* size_t, NULL */
 #include <stdlib.h>		/* malloc(), exit(), EXIT_SUCCESS,
 				   realloc() */
 #include <stdbool.h>		/* bool, true, false */
 #include <stdio.h>		/* fileno(), fprintf(),
-				   stderr, STDOUT_FILENO, fclose() */
-#include <sys/types.h>	        /* fstat(), struct stat, waitpid(),
-				   WIFEXITED(), WEXITSTATUS(), wait(),
-				   pid_t, uid_t, gid_t, getuid(),
-				   getgid() */
+				   stderr, STDOUT_FILENO */
+#include <sys/types.h>	        /* DIR, fdopendir(), stat(), struct
+				   stat, waitpid(), WIFEXITED(),
+				   WEXITSTATUS(), wait(), pid_t,
+				   uid_t, gid_t, getuid(), getgid(),
+				   dirfd() */
 #include <sys/select.h>		/* fd_set, select(), FD_ZERO(),
 				   FD_SET(), FD_ISSET(), FD_CLR */
 #include <sys/wait.h>		/* wait(), waitpid(), WIFEXITED(),
-				   WEXITSTATUS(), WTERMSIG() */
-#include <sys/stat.h>		/* struct stat, fstat(), S_ISREG() */
+				   WEXITSTATUS(), WTERMSIG(),
+				   WCOREDUMP() */
+#include <sys/stat.h>		/* struct stat, stat(), S_ISREG() */
 #include <iso646.h>		/* and, or, not */
-#include <dirent.h>		/* struct dirent, scandirat() */
-#include <unistd.h>		/* fcntl(), F_GETFD, F_SETFD,
-				   FD_CLOEXEC, write(), STDOUT_FILENO,
-				   struct stat, fstat(), close(),
-				   setgid(), setuid(), S_ISREG(),
-				   faccessat() pipe2(), fork(),
-				   _exit(), dup2(), fexecve(), read()
-				*/
+#include <dirent.h>		/* DIR, struct dirent, fdopendir(),
+				   readdir(), closedir(), dirfd() */
+#include <unistd.h>		/* struct stat, stat(), S_ISREG(),
+				   fcntl(), setuid(), setgid(),
+				   F_GETFD, F_SETFD, FD_CLOEXEC,
+				   access(), pipe(), fork(), close()
+				   dup2(), STDOUT_FILENO, _exit(),
+				   execv(), write(), read(),
+				   close() */
 #include <fcntl.h>		/* fcntl(), F_GETFD, F_SETFD,
-				   FD_CLOEXEC, openat(), scandirat(),
-				   pipe2() */
-#include <string.h>		/* strsep, strlen(), strsignal(),
-				   strcmp(), strncmp() */
+				   FD_CLOEXEC */
+#include <string.h>		/* strsep, strlen(), asprintf(),
+				   strsignal(), strcmp(), strncmp() */
 #include <errno.h>		/* errno */
 #include <argp.h>		/* struct argp_option, struct
 				   argp_state, struct argp,
@@ -71,12 +72,10 @@
 				   EX_CONFIG, EX_UNAVAILABLE, EX_OK */
 #include <errno.h> 		/* errno */
 #include <error.h>		/* error() */
-#include <fnmatch.h>		/* fnmatch() */
 
 #define BUFFER_SIZE 256
 
 #define PDIR "/lib/mandos/plugins.d"
-#define PHDIR "/lib/mandos/plugin-helpers"
 #define AFILE "/conf/conf.d/mandos/plugin-runner.conf"
 
 const char *argp_program_version = "plugin-runner " VERSION;
@@ -106,7 +105,6 @@
 
 /* Gets an existing plugin based on name,
    or if none is found, creates a new one */
-__attribute__((warn_unused_result))
 static plugin *getplugin(char *name){
   /* Check for existing plugin with that name */
   for(plugin *p = plugin_list; p != NULL; p = p->next){
@@ -173,20 +171,17 @@
 }
 
 /* Helper function for add_argument and add_environment */
-__attribute__((nonnull, warn_unused_result))
 static bool add_to_char_array(const char *new, char ***array,
 			      int *len){
   /* Resize the pointed-to array to hold one more pointer */
-  char **new_array = NULL;
   do {
-    new_array = realloc(*array, sizeof(char *)
-			* (size_t) ((*len) + 2));
-  } while(new_array == NULL and errno == EINTR);
+    *array = realloc(*array, sizeof(char *)
+		     * (size_t) ((*len) + 2));
+  } while(*array == NULL and errno == EINTR);
   /* Malloc check */
-  if(new_array == NULL){
+  if(*array == NULL){
     return false;
   }
-  *array = new_array;
   /* Make a copy of the new string */
   char *copy;
   do {
@@ -204,7 +199,6 @@
 }
 
 /* Add to a plugin's argument vector */
-__attribute__((nonnull(2), warn_unused_result))
 static bool add_argument(plugin *p, const char *arg){
   if(p == NULL){
     return false;
@@ -213,7 +207,6 @@
 }
 
 /* Add to a plugin's environment */
-__attribute__((nonnull(2), warn_unused_result))
 static bool add_environment(plugin *p, const char *def, bool replace){
   if(p == NULL){
     return false;
@@ -221,19 +214,19 @@
   /* namelen = length of name of environment variable */
   size_t namelen = (size_t)(strchrnul(def, '=') - def);
   /* Search for this environment variable */
-  for(char **envdef = p->environ; *envdef != NULL; envdef++){
-    if(strncmp(*envdef, def, namelen + 1) == 0){
+  for(char **e = p->environ; *e != NULL; e++){
+    if(strncmp(*e, def, namelen + 1) == 0){
       /* It already exists */
       if(replace){
-	char *new_envdef;
+	char *new;
 	do {
-	  new_envdef = realloc(*envdef, strlen(def) + 1);
-	} while(new_envdef == NULL and errno == EINTR);
-	if(new_envdef == NULL){
+	  new = realloc(*e, strlen(def) + 1);
+	} while(new == NULL and errno == EINTR);
+	if(new == NULL){
 	  return false;
 	}
-	*envdef = new_envdef;
-	strcpy(*envdef, def);
+	*e = new;
+	strcpy(*e, def);
       }
       return true;
     }
@@ -241,13 +234,11 @@
   return add_to_char_array(def, &(p->environ), &(p->envc));
 }
 
-#ifndef O_CLOEXEC
 /*
  * Based on the example in the GNU LibC manual chapter 13.13 "File
  * Descriptor Flags".
  | [[info:libc:Descriptor%20Flags][File Descriptor Flags]] |
  */
-__attribute__((warn_unused_result))
 static int set_cloexec_flag(int fd){
   int ret = (int)TEMP_FAILURE_RETRY(fcntl(fd, F_GETFD, 0));
   /* If reading the flags failed, return error indication now. */
@@ -258,7 +249,6 @@
   return (int)TEMP_FAILURE_RETRY(fcntl(fd, F_SETFD,
 				       ret | FD_CLOEXEC));
 }
-#endif	/* not O_CLOEXEC */
 
 
 /* Mark processes as completed when they exit, and save their exit
@@ -296,7 +286,6 @@
 }
 
 /* Prints out a password to stdout */
-__attribute__((nonnull, warn_unused_result))
 static bool print_out_password(const char *buffer, size_t length){
   ssize_t ret;
   for(size_t written = 0; written < length; written += (size_t)ret){
@@ -310,7 +299,6 @@
 }
 
 /* Removes and free a plugin from the plugin list */
-__attribute__((nonnull))
 static void free_plugin(plugin *plugin_node){
   
   for(char **arg = plugin_node->argv; *arg != NULL; arg++){
@@ -348,10 +336,11 @@
 
 int main(int argc, char *argv[]){
   char *plugindir = NULL;
-  char *pluginhelperdir = NULL;
   char *argfile = NULL;
   FILE *conffp;
-  struct dirent **direntries = NULL;
+  size_t d_name_len;
+  DIR *dir = NULL;
+  struct dirent *dirst;
   struct stat st;
   fd_set rfds_all;
   int ret, maxfd = 0;
@@ -365,7 +354,6 @@
 				      .sa_flags = SA_NOCLDSTOP };
   char **custom_argv = NULL;
   int custom_argc = 0;
-  int dir_fd = -1;
   
   /* Establish a signal handler */
   sigemptyset(&sigchld_action.sa_mask);
@@ -416,10 +404,6 @@
       .doc = "Group ID the plugins will run as", .group = 3 },
     { .name = "debug", .key = 132,
       .doc = "Debug mode", .group = 4 },
-    { .name = "plugin-helper-dir", .key = 133,
-      .arg = "DIRECTORY",
-      .doc = "Specify a different plugin helper directory",
-      .group = 2 },
     /*
      * These reproduce what we would get without ARGP_NO_HELP
      */
@@ -432,12 +416,11 @@
     { .name = NULL }
   };
   
-  __attribute__((nonnull(3)))
   error_t parse_opt(int key, char *arg, struct argp_state *state){
     errno = 0;
     switch(key){
       char *tmp;
-      intmax_t tmp_id;
+      intmax_t tmpmax;
     case 'g': 			/* --global-options */
       {
 	char *plugin_option;
@@ -446,13 +429,10 @@
 	    break;
 	  }
 	}
-	errno = 0;
       }
       break;
     case 'G':			/* --global-env */
-      if(add_environment(getplugin(NULL), arg, true)){
-	errno = 0;
-      }
+      add_environment(getplugin(NULL), arg, true);
       break;
     case 'o':			/* --options-for */
       {
@@ -475,7 +455,6 @@
 	    break;
 	  }
 	}
-	errno = 0;
       }
       break;
     case 'E':			/* --env-for */
@@ -493,9 +472,7 @@
 	  errno = EINVAL;
 	  break;
 	}
-	if(add_environment(getplugin(arg), envdef, true)){
-	  errno = 0;
-	}
+	add_environment(getplugin(arg), envdef, true);
       }
       break;
     case 'd':			/* --disable */
@@ -503,7 +480,6 @@
 	plugin *p = getplugin(arg);
 	if(p != NULL){
 	  p->disabled = true;
-	  errno = 0;
 	}
       }
       break;
@@ -512,64 +488,49 @@
 	plugin *p = getplugin(arg);
 	if(p != NULL){
 	  p->disabled = false;
-	  errno = 0;
 	}
       }
       break;
     case 128:			/* --plugin-dir */
       free(plugindir);
       plugindir = strdup(arg);
-      if(plugindir != NULL){
-	errno = 0;
-      }
       break;
     case 129:			/* --config-file */
       /* This is already done by parse_opt_config_file() */
       break;
     case 130:			/* --userid */
-      tmp_id = strtoimax(arg, &tmp, 10);
+      tmpmax = strtoimax(arg, &tmp, 10);
       if(errno != 0 or tmp == arg or *tmp != '\0'
-	 or tmp_id != (uid_t)tmp_id){
+	 or tmpmax != (uid_t)tmpmax){
 	argp_error(state, "Bad user ID number: \"%s\", using %"
 		   PRIdMAX, arg, (intmax_t)uid);
 	break;
       }
-      uid = (uid_t)tmp_id;
-      errno = 0;
+      uid = (uid_t)tmpmax;
       break;
     case 131:			/* --groupid */
-      tmp_id = strtoimax(arg, &tmp, 10);
+      tmpmax = strtoimax(arg, &tmp, 10);
       if(errno != 0 or tmp == arg or *tmp != '\0'
-	 or tmp_id != (gid_t)tmp_id){
+	 or tmpmax != (gid_t)tmpmax){
 	argp_error(state, "Bad group ID number: \"%s\", using %"
 		   PRIdMAX, arg, (intmax_t)gid);
 	break;
       }
-      gid = (gid_t)tmp_id;
-      errno = 0;
+      gid = (gid_t)tmpmax;
       break;
     case 132:			/* --debug */
       debug = true;
       break;
-    case 133:			/* --plugin-helper-dir */
-      free(pluginhelperdir);
-      pluginhelperdir = strdup(arg);
-      if(pluginhelperdir != NULL){
-	errno = 0;
-      }
-      break;
       /*
        * These reproduce what we would get without ARGP_NO_HELP
        */
     case '?':			/* --help */
       state->flags &= ~(unsigned int)ARGP_NO_EXIT; /* force exit */
       argp_state_help(state, state->out_stream, ARGP_HELP_STD_HELP);
-      __builtin_unreachable();
     case -3:			/* --usage */
       state->flags &= ~(unsigned int)ARGP_NO_EXIT; /* force exit */
       argp_state_help(state, state->out_stream,
 		      ARGP_HELP_USAGE | ARGP_HELP_EXIT_OK);
-      __builtin_unreachable();
     case 'V':			/* --version */
       fprintf(state->out_stream, "%s\n", argp_program_version);
       exit(EXIT_SUCCESS);
@@ -585,7 +546,6 @@
       if(arg[0] == '\0'){
 	break;
       }
-      /* FALLTHROUGH */
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -610,14 +570,10 @@
     case 129:			/* --config-file */
       free(argfile);
       argfile = strdup(arg);
-      if(argfile != NULL){
-	errno = 0;
-      }
       break;
     case 130:			/* --userid */
     case 131:			/* --groupid */
     case 132:			/* --debug */
-    case 133:			/* --plugin-helper-dir */
     case '?':			/* --help */
     case -3:			/* --usage */
     case 'V':			/* --version */
@@ -702,18 +658,13 @@
 	}
 	
 	custom_argc += 1;
-	{
-	  char **new_argv = realloc(custom_argv, sizeof(char *)
-				    * ((size_t)custom_argc + 1));
-	  if(new_argv == NULL){
-	    error(0, errno, "realloc");
-	    exitstatus = EX_OSERR;
-	    free(new_arg);
-	    free(org_line);
-	    goto fallback;
-	  } else {
-	    custom_argv = new_argv;
-	  }
+	custom_argv = realloc(custom_argv, sizeof(char *)
+			      * ((unsigned int) custom_argc + 1));
+	if(custom_argv == NULL){
+	  error(0, errno, "realloc");
+	  exitstatus = EX_OSERR;
+	  free(org_line);
+	  goto fallback;
 	}
 	custom_argv[custom_argc-1] = new_arg;
 	custom_argv[custom_argc] = NULL;
@@ -777,26 +728,8 @@
     goto fallback;
   }
   
-  {
-    char *pluginhelperenv;
-    bool bret = true;
-    ret = asprintf(&pluginhelperenv, "MANDOSPLUGINHELPERDIR=%s",
-		   pluginhelperdir != NULL ? pluginhelperdir : PHDIR);
-    if(ret != -1){
-      bret = add_environment(getplugin(NULL), pluginhelperenv, true);
-    }
-    if(ret == -1 or not bret){
-      error(0, errno, "Failed to set MANDOSPLUGINHELPERDIR"
-	    " environment variable to \"%s\" for all plugins\n",
-	    pluginhelperdir != NULL ? pluginhelperdir : PHDIR);
-    }
-    if(ret != -1){
-      free(pluginhelperenv);
-    }
-  }
-  
   if(debug){
-    for(plugin *p = plugin_list; p != NULL; p = p->next){
+    for(plugin *p = plugin_list; p != NULL; p=p->next){
       fprintf(stderr, "Plugin: %s has %d arguments\n",
 	      p->name ? p->name : "Global", p->argc - 1);
       for(char **a = p->argv; *a != NULL; a++){
@@ -809,14 +742,12 @@
     }
   }
   
-  if(getuid() == 0){
+  {
     /* Work around Debian bug #633582:
-       <https://bugs.debian.org/633582> */
+       <http://bugs.debian.org/633582> */
     int plugindir_fd = open(/* plugindir or */ PDIR, O_RDONLY);
     if(plugindir_fd == -1){
-      if(errno != ENOENT){
-	error(0, errno, "open(\"" PDIR "\")");
-      }
+      error(0, errno, "open");
     } else {
       ret = (int)TEMP_FAILURE_RETRY(fstat(plugindir_fd, &st));
       if(ret == -1){
@@ -829,12 +760,12 @@
 	  }
 	}
       }
-      close(plugindir_fd);
+      TEMP_FAILURE_RETRY(close(plugindir_fd));
     }
   }
   
   /* Lower permissions */
-  ret = setgid(gid);
+  setgid(gid);
   if(ret == -1){
     error(0, errno, "setgid");
   }
@@ -845,13 +776,24 @@
   
   /* Open plugin directory with close_on_exec flag */
   {
-    dir_fd = open(plugindir != NULL ? plugindir : PDIR, O_RDONLY |
-#ifdef O_CLOEXEC
-		  O_CLOEXEC
-#else  /* not O_CLOEXEC */
-		  0
-#endif	/* not O_CLOEXEC */
-		  );
+    int dir_fd = -1;
+    if(plugindir == NULL){
+      dir_fd = open(PDIR, O_RDONLY |
+#ifdef O_CLOEXEC
+		    O_CLOEXEC
+#else  /* not O_CLOEXEC */
+		    0
+#endif	/* not O_CLOEXEC */
+		    );
+    } else {
+      dir_fd = open(plugindir, O_RDONLY |
+#ifdef O_CLOEXEC
+		    O_CLOEXEC
+#else  /* not O_CLOEXEC */
+		    0
+#endif	/* not O_CLOEXEC */
+		    );
+    }
     if(dir_fd == -1){
       error(0, errno, "Could not open plugin dir");
       exitstatus = EX_UNAVAILABLE;
@@ -863,94 +805,129 @@
     ret = set_cloexec_flag(dir_fd);
     if(ret < 0){
       error(0, errno, "set_cloexec_flag");
+      TEMP_FAILURE_RETRY(close(dir_fd));
       exitstatus = EX_OSERR;
       goto fallback;
     }
 #endif	/* O_CLOEXEC */
-  }
-  
-  int good_name(const struct dirent * const dirent){
-    const char * const patterns[] = { ".*", "#*#", "*~", "*.dpkg-new",
-				      "*.dpkg-old", "*.dpkg-bak",
-				      "*.dpkg-divert", NULL };
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-    for(const char **pat = (const char **)patterns;
-	*pat != NULL; pat++){
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
-      if(fnmatch(*pat, dirent->d_name, FNM_FILE_NAME | FNM_PERIOD)
-	 != FNM_NOMATCH){
-	if(debug){
-	    fprintf(stderr, "Ignoring plugin dir entry \"%s\""
-		    " matching pattern %s\n", dirent->d_name, *pat);
-	}
-	return 0;
-      }
+    
+    dir = fdopendir(dir_fd);
+    if(dir == NULL){
+      error(0, errno, "Could not open plugin dir");
+      TEMP_FAILURE_RETRY(close(dir_fd));
+      exitstatus = EX_OSERR;
+      goto fallback;
     }
-    return 1;
-  }
-  
-  int numplugins = scandirat(dir_fd, ".", &direntries, good_name,
-			     alphasort);
-  if(numplugins == -1){
-    error(0, errno, "Could not scan plugin dir");
-    direntries = NULL;
-    exitstatus = EX_OSERR;
-    goto fallback;
   }
   
   FD_ZERO(&rfds_all);
   
   /* Read and execute any executable in the plugin directory*/
-  for(int i = 0; i < numplugins; i++){
-    
-    int plugin_fd = openat(dir_fd, direntries[i]->d_name, O_RDONLY);
-    if(plugin_fd == -1){
-      error(0, errno, "Could not open plugin");
-      free(direntries[i]);
+  while(true){
+    do {
+      dirst = readdir(dir);
+    } while(dirst == NULL and errno == EINTR);
+    
+    /* All directory entries have been processed */
+    if(dirst == NULL){
+      if(errno == EBADF){
+	error(0, errno, "readdir");
+	exitstatus = EX_IOERR;
+	goto fallback;
+      }
+      break;
+    }
+    
+    d_name_len = strlen(dirst->d_name);
+    
+    /* Ignore dotfiles, backup files and other junk */
+    {
+      bool bad_name = false;
+      
+      const char const *bad_prefixes[] = { ".", "#", NULL };
+      
+      const char const *bad_suffixes[] = { "~", "#", ".dpkg-new",
+					   ".dpkg-old",
+					   ".dpkg-bak",
+					   ".dpkg-divert", NULL };
+      for(const char **pre = bad_prefixes; *pre != NULL; pre++){
+	size_t pre_len = strlen(*pre);
+	if((d_name_len >= pre_len)
+	   and strncmp((dirst->d_name), *pre, pre_len) == 0){
+	  if(debug){
+	    fprintf(stderr, "Ignoring plugin dir entry \"%s\""
+		    " with bad prefix %s\n", dirst->d_name, *pre);
+	  }
+	  bad_name = true;
+	  break;
+	}
+      }
+      if(bad_name){
+	continue;
+      }
+      for(const char **suf = bad_suffixes; *suf != NULL; suf++){
+	size_t suf_len = strlen(*suf);
+	if((d_name_len >= suf_len)
+	   and (strcmp((dirst->d_name) + d_name_len-suf_len, *suf)
+		== 0)){
+	  if(debug){
+	    fprintf(stderr, "Ignoring plugin dir entry \"%s\""
+		    " with bad suffix %s\n", dirst->d_name, *suf);
+	  }
+	  bad_name = true;
+	  break;
+	}
+      }
+      
+      if(bad_name){
+	continue;
+      }
+    }
+    
+    char *filename;
+    if(plugindir == NULL){
+      ret = (int)TEMP_FAILURE_RETRY(asprintf(&filename, PDIR "/%s",
+					     dirst->d_name));
+    } else {
+      ret = (int)TEMP_FAILURE_RETRY(asprintf(&filename, "%s/%s",
+					     plugindir,
+					     dirst->d_name));
+    }
+    if(ret < 0){
+      error(0, errno, "asprintf");
       continue;
     }
-    ret = (int)TEMP_FAILURE_RETRY(fstat(plugin_fd, &st));
+    
+    ret = (int)TEMP_FAILURE_RETRY(stat(filename, &st));
     if(ret == -1){
       error(0, errno, "stat");
-      close(plugin_fd);
-      free(direntries[i]);
+      free(filename);
       continue;
     }
     
     /* Ignore non-executable files */
     if(not S_ISREG(st.st_mode)
-       or (TEMP_FAILURE_RETRY(faccessat(dir_fd, direntries[i]->d_name,
-					X_OK, 0)) != 0)){
+       or (TEMP_FAILURE_RETRY(access(filename, X_OK)) != 0)){
       if(debug){
-	fprintf(stderr, "Ignoring plugin dir entry \"%s/%s\""
-		" with bad type or mode\n",
-		plugindir != NULL ? plugindir : PDIR,
-		direntries[i]->d_name);
+	fprintf(stderr, "Ignoring plugin dir entry \"%s\""
+		" with bad type or mode\n", filename);
       }
-      close(plugin_fd);
-      free(direntries[i]);
+      free(filename);
       continue;
     }
     
-    plugin *p = getplugin(direntries[i]->d_name);
+    plugin *p = getplugin(dirst->d_name);
     if(p == NULL){
       error(0, errno, "getplugin");
-      close(plugin_fd);
-      free(direntries[i]);
+      free(filename);
       continue;
     }
     if(p->disabled){
       if(debug){
 	fprintf(stderr, "Ignoring disabled plugin \"%s\"\n",
-		direntries[i]->d_name);
+		dirst->d_name);
       }
-      close(plugin_fd);
-      free(direntries[i]);
+      free(filename);
       continue;
     }
     {
@@ -970,8 +947,9 @@
 	}
       }
     }
-    /* If this plugin has any environment variables, we need to
-       duplicate the environment from this process, too. */
+    /* If this plugin has any environment variables, we will call
+       using execve and need to duplicate the environment from this
+       process, too. */
     if(p->environ[0] != NULL){
       for(char **e = environ; *e != NULL; e++){
 	if(not add_environment(p, *e, false)){
@@ -981,47 +959,25 @@
     }
     
     int pipefd[2];
-#ifndef O_CLOEXEC
     ret = (int)TEMP_FAILURE_RETRY(pipe(pipefd));
-#else  /* O_CLOEXEC */
-    ret = (int)TEMP_FAILURE_RETRY(pipe2(pipefd, O_CLOEXEC));
-#endif	/* O_CLOEXEC */
     if(ret == -1){
       error(0, errno, "pipe");
       exitstatus = EX_OSERR;
-      free(direntries[i]);
-      goto fallback;
-    }
-    if(pipefd[0] >= FD_SETSIZE){
-      fprintf(stderr, "pipe()[0] (%d) >= FD_SETSIZE (%d)", pipefd[0],
-	      FD_SETSIZE);
-      close(pipefd[0]);
-      close(pipefd[1]);
-      exitstatus = EX_OSERR;
-      free(direntries[i]);
-      goto fallback;
-    }
-#ifndef O_CLOEXEC
+      goto fallback;
+    }
     /* Ask OS to automatic close the pipe on exec */
     ret = set_cloexec_flag(pipefd[0]);
     if(ret < 0){
       error(0, errno, "set_cloexec_flag");
-      close(pipefd[0]);
-      close(pipefd[1]);
       exitstatus = EX_OSERR;
-      free(direntries[i]);
       goto fallback;
     }
     ret = set_cloexec_flag(pipefd[1]);
     if(ret < 0){
       error(0, errno, "set_cloexec_flag");
-      close(pipefd[0]);
-      close(pipefd[1]);
       exitstatus = EX_OSERR;
-      free(direntries[i]);
       goto fallback;
     }
-#endif	/* not O_CLOEXEC */
     /* Block SIGCHLD until process is safely in process list */
     ret = (int)TEMP_FAILURE_RETRY(sigprocmask(SIG_BLOCK,
 					      &sigchld_action.sa_mask,
@@ -1029,7 +985,6 @@
     if(ret < 0){
       error(0, errno, "sigprocmask");
       exitstatus = EX_OSERR;
-      free(direntries[i]);
       goto fallback;
     }
     /* Starting a new process to be watched */
@@ -1039,12 +994,7 @@
     } while(pid == -1 and errno == EINTR);
     if(pid == -1){
       error(0, errno, "fork");
-      TEMP_FAILURE_RETRY(sigprocmask(SIG_UNBLOCK,
-				     &sigchld_action.sa_mask, NULL));
-      close(pipefd[0]);
-      close(pipefd[1]);
       exitstatus = EX_OSERR;
-      free(direntries[i]);
       goto fallback;
     }
     if(pid == 0){
@@ -1066,19 +1016,29 @@
 	_exit(EX_OSERR);
       }
       
-      if(fexecve(plugin_fd, p->argv,
-		(p->environ[0] != NULL) ? p->environ : environ) < 0){
-	error(0, errno, "fexecve for %s/%s",
-	      plugindir != NULL ? plugindir : PDIR,
-	      direntries[i]->d_name);
-	_exit(EX_OSERR);
+      if(dirfd(dir) < 0){
+	/* If dir has no file descriptor, we could not set FD_CLOEXEC
+	   above and must now close it manually here. */
+	closedir(dir);
+      }
+      if(p->environ[0] == NULL){
+	if(execv(filename, p->argv) < 0){
+	  error(0, errno, "execv for %s", filename);
+	  _exit(EX_OSERR);
+	}
+      } else {
+	if(execve(filename, p->argv, p->environ) < 0){
+	  error(0, errno, "execve for %s", filename);
+	  _exit(EX_OSERR);
+	}
       }
       /* no return */
     }
     /* Parent process */
-    close(pipefd[1]);		/* Close unused write end of pipe */
-    close(plugin_fd);
-    plugin *new_plugin = getplugin(direntries[i]->d_name);
+    TEMP_FAILURE_RETRY(close(pipefd[1])); /* Close unused write end of
+					     pipe */
+    free(filename);
+    plugin *new_plugin = getplugin(dirst->d_name);
     if(new_plugin == NULL){
       error(0, errno, "getplugin");
       ret = (int)(TEMP_FAILURE_RETRY
@@ -1088,19 +1048,12 @@
         error(0, errno, "sigprocmask");
       }
       exitstatus = EX_OSERR;
-      free(direntries[i]);
       goto fallback;
     }
-    free(direntries[i]);
     
     new_plugin->pid = pid;
     new_plugin->fd = pipefd[0];
-
-    if(debug){
-      fprintf(stderr, "Plugin %s started (PID %" PRIdMAX ")\n",
-	      new_plugin->name, (intmax_t) (new_plugin->pid));
-    }
-
+    
     /* Unblock SIGCHLD so signal handler can be run if this process
        has already completed */
     ret = (int)TEMP_FAILURE_RETRY(sigprocmask(SIG_UNBLOCK,
@@ -1112,17 +1065,16 @@
       goto fallback;
     }
     
-    FD_SET(new_plugin->fd, &rfds_all);
+    FD_SET(new_plugin->fd, &rfds_all); /* Spurious warning from
+					  -Wconversion */
     
     if(maxfd < new_plugin->fd){
       maxfd = new_plugin->fd;
     }
   }
   
-  free(direntries);
-  direntries = NULL;
-  close(dir_fd);
-  dir_fd = -1;
+  TEMP_FAILURE_RETRY(closedir(dir));
+  dir = NULL;
   free_plugin(getplugin(NULL));
   
   for(plugin *p = plugin_list; p != NULL; p = p->next){
@@ -1167,11 +1119,15 @@
 		      (intmax_t) (proc->pid),
 		      WTERMSIG(proc->status),
 		      strsignal(WTERMSIG(proc->status)));
+	    } else if(WCOREDUMP(proc->status)){
+	      fprintf(stderr, "Plugin %s [%" PRIdMAX "] dumped"
+		      " core\n", proc->name, (intmax_t) (proc->pid));
 	    }
 	  }
 	  
 	  /* Remove the plugin */
-	  FD_CLR(proc->fd, &rfds_all);
+	  FD_CLR(proc->fd, &rfds_all); /* Spurious warning from
+					  -Wconversion */
 	  
 	  /* Block signal while modifying process_list */
 	  ret = (int)TEMP_FAILURE_RETRY(sigprocmask
@@ -1217,21 +1173,22 @@
       }
       
       /* This process has not completed.  Does it have any output? */
-      if(proc->eof or not FD_ISSET(proc->fd, &rfds)){
+      if(proc->eof or not FD_ISSET(proc->fd, &rfds)){ /* Spurious
+							 warning from
+							 -Wconversion */
 	/* This process had nothing to say at this time */
 	proc = proc->next;
 	continue;
       }
       /* Before reading, make the process' data buffer large enough */
       if(proc->buffer_length + BUFFER_SIZE > proc->buffer_size){
-	char *new_buffer = realloc(proc->buffer, proc->buffer_size
-				   + (size_t) BUFFER_SIZE);
-	if(new_buffer == NULL){
+	proc->buffer = realloc(proc->buffer, proc->buffer_size
+			       + (size_t) BUFFER_SIZE);
+	if(proc->buffer == NULL){
 	  error(0, errno, "malloc");
 	  exitstatus = EX_OSERR;
 	  goto fallback;
 	}
-	proc->buffer = new_buffer;
 	proc->buffer_size += BUFFER_SIZE;
       }
       /* Read from the process */
@@ -1290,10 +1247,8 @@
     free(custom_argv);
   }
   
-  free(direntries);
-  
-  if(dir_fd != -1){
-    close(dir_fd);
+  if(dir != NULL){
+    closedir(dir);
   }
   
   /* Kill the processes */
@@ -1319,7 +1274,6 @@
   free_plugin_list();
   
   free(plugindir);
-  free(pluginhelperdir);
   free(argfile);
   
   return exitstatus;

=== modified file 'plugin-runner.xml'
--- plugin-runner.xml	2019-02-10 04:20:26 +0000
+++ plugin-runner.xml	2011-10-05 16:56:06 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plugin-runner">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-05">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,16 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -123,9 +114,6 @@
       <arg><option>--plugin-dir=<replaceable
       >DIRECTORY</replaceable></option></arg>
       <sbr/>
-      <arg><option>--plugin-helper-dir=<replaceable
-      >DIRECTORY</replaceable></option></arg>
-      <sbr/>
       <arg><option>--config-file=<replaceable
       >FILE</replaceable></option></arg>
       <sbr/>
@@ -332,21 +320,6 @@
       </varlistentry>
       
       <varlistentry>
-	<term><option>--plugin-helper-dir
-	<replaceable>DIRECTORY</replaceable></option></term>
-	<listitem>
-	  <para>
-	    Specify a different plugin helper directory.  The default
-	    is <filename>/lib/mandos/plugin-helpers</filename>, which
-	    will exist in the initial <acronym>RAM</acronym> disk
-	    environment.  (This will simply be passed to all plugins
-	    via the <envar>MANDOSPLUGINHELPERDIR</envar> environment
-	    variable.  See <xref linkend="writing_plugins"/>)
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
 	<term><option>--config-file
 	<replaceable>FILE</replaceable></option></term>
 	<listitem>
@@ -453,11 +426,7 @@
       <para>
 	The plugin will run in the initial RAM disk environment, so
 	care must be taken not to depend on any files or running
-	services not available there.  Any helper executables required
-	by the plugin (which are not in the <envar>PATH</envar>) can
-	be placed in the plugin helper directory, the name of which
-	will be made available to the plugin via the
-	<envar>MANDOSPLUGINHELPERDIR</envar> environment variable.
+	services not available there.
       </para>
       <para>
 	The plugin must exit cleanly and free all allocated resources
@@ -506,9 +475,7 @@
       only passes on its environment to all the plugins.  The
       environment passed to plugins can be modified using the
       <option>--global-env</option> and <option>--env-for</option>
-      options.  Also, the <option>--plugin-helper-dir</option> option
-      will affect the environment variable
-      <envar>MANDOSPLUGINHELPERDIR</envar> for the plugins.
+      options.
     </para>
   </refsect1>
   
@@ -547,26 +514,6 @@
 	    </para>
 	  </listitem>
 	</varlistentry>
-	<varlistentry>
-	  <term><filename class="directory"
-	  >/lib/mandos/plugins.d</filename></term>
-	  <listitem>
-	    <para>
-	      The default plugin directory; can be changed by the
-	      <option>--plugin-dir</option> option.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><filename class="directory"
-	  >/lib/mandos/plugin-helpers</filename></term>
-	  <listitem>
-	    <para>
-	      The default plugin helper directory; can be changed by
-	      the <option>--plugin-helper-dir</option> option.
-	    </para>
-	  </listitem>
-	</varlistentry>
       </variablelist>
     </para>
   </refsect1>
@@ -577,7 +524,6 @@
       The <option>--config-file</option> option is ignored when
       specified from within a configuration file.
     </para>
-    <xi:include href="bugs.xml"/>
   </refsect1>
   
   <refsect1 id="examples">
@@ -626,16 +572,15 @@
     </informalexample>
     <informalexample>
       <para>
-	Read a different configuration file, run plugins from a
-	different directory, specify an alternate plugin helper
-	directory and add two options to the
+	Run plugins from a different directory, read a different
+	configuration file, and add two options to the
 	<citerefentry><refentrytitle >mandos-client</refentrytitle>
 	<manvolnum>8mandos</manvolnum></citerefentry> plugin:
       </para>
       <para>
 
 <!-- do not wrap this line -->
-<userinput>cd /etc/keys/mandos; &COMMANDNAME;  --config-file=/etc/mandos/plugin-runner.conf --plugin-dir /usr/lib/x86_64-linux-gnu/mandos/plugins.d --plugin-helper-dir /usr/lib/x86_64-linux-gnu/mandos/plugin-helpers --options-for=mandos-client:--pubkey=pubkey.txt,--seckey=seckey.txt,--tls-pubkey=tls-pubkey.pem,--tls-privkey=tls-privkey.pem</userinput>
+<userinput>cd /etc/keys/mandos; &COMMANDNAME;  --config-file=/etc/mandos/plugin-runner.conf --plugin-dir /usr/lib/mandos/plugins.d --options-for=mandos-client:--pubkey=pubkey.txt,--seckey=seckey.txt</userinput>
 
       </para>
     </informalexample>

=== modified file 'plugins.d/askpass-fifo.c'
--- plugins.d/askpass-fifo.c	2019-02-11 07:06:55 +0000
+++ plugins.d/askpass-fifo.c	2011-10-05 16:00:56 +0000
@@ -2,29 +2,28 @@
 /*
  * Askpass-FIFO - Read a password from a FIFO and output it
  * 
- * Copyright © 2008-2019 Teddy Hogeborn
- * Copyright © 2008-2019 Björn Påhlsson
- * 
- * This file is part of Mandos.
- * 
- * Mandos is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- * 
- * Mandos is distributed in the hope that it will be useful, but
+ * Copyright © 2008-2011 Teddy Hogeborn
+ * Copyright © 2008-2011 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  * 
  * Contact the authors at <mandos@recompile.se>.
  */
 
 #define _GNU_SOURCE		/* TEMP_FAILURE_RETRY() */
-#include <sys/types.h>		/* uid_t, gid_t, ssize_t */
+#include <sys/types.h>		/* ssize_t */
 #include <sys/stat.h>		/* mkfifo(), S_IRUSR, S_IWUSR */
 #include <iso646.h>		/* and */
 #include <errno.h>		/* errno, EACCES, ENOTDIR, ELOOP,
@@ -45,11 +44,8 @@
 #include <string.h> 		/* strerror() */
 #include <stdarg.h>		/* va_list, va_start(), ... */
 
-uid_t uid = 65534;
-gid_t gid = 65534;
 
 /* Function to use when printing errors */
-__attribute__((format (gnu_printf, 3, 4)))
 void error_plus(int status, int errnum, const char *formatstring,
 		...){
   va_list ap;
@@ -58,23 +54,17 @@
   
   va_start(ap, formatstring);
   ret = vasprintf(&text, formatstring, ap);
-  if(ret == -1){
+  if (ret == -1){
     fprintf(stderr, "Mandos plugin %s: ",
 	    program_invocation_short_name);
     vfprintf(stderr, formatstring, ap);
     fprintf(stderr, ": ");
     fprintf(stderr, "%s\n", strerror(errnum));
     error(status, errno, "vasprintf while printing error");
-    if(status){
-      __builtin_unreachable();
-    }
     return;
   }
   fprintf(stderr, "Mandos plugin ");
   error(status, errnum, "%s", text);
-  if(status){
-    __builtin_unreachable();
-  }
   free(text);
 }
 
@@ -83,9 +73,6 @@
   int ret = 0;
   ssize_t sret;
   
-  uid = getuid();
-  gid = getgid();
-  
   /* Create FIFO */
   const char passfifo[] = "/lib/cryptsetup/passfifo";
   ret = mkfifo(passfifo, S_IRUSR | S_IWUSR);
@@ -96,17 +83,14 @@
     case ENOTDIR:
     case ELOOP:
       error_plus(EX_OSFILE, errno, "mkfifo");
-      __builtin_unreachable();
     case ENAMETOOLONG:
     case ENOSPC:
     case EROFS:
     default:
       error_plus(EX_OSERR, errno, "mkfifo");
-      __builtin_unreachable();
     case ENOENT:
       /* no "/lib/cryptsetup"? */
       error_plus(EX_UNAVAILABLE, errno, "mkfifo");
-      __builtin_unreachable();
     case EEXIST:
       break;			/* not an error */
     }
@@ -134,16 +118,6 @@
     }
   }
   
-  /* Lower group privileges  */
-  if(setgid(gid) == -1){
-    error_plus(0, errno, "setgid");
-  }
-  
-  /* Lower user privileges */
-  if(setuid(uid) == -1){
-    error_plus(0, errno, "setuid");
-  }
-  
   /* Read from FIFO */
   char *buf = NULL;
   size_t buf_len = 0;

=== modified file 'plugins.d/askpass-fifo.xml'
--- plugins.d/askpass-fifo.xml	2019-02-10 04:20:26 +0000
+++ plugins.d/askpass-fifo.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "askpass-fifo">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,19 +33,11 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
+    <xi:include href="../legalnotice.xml"/>
   </refentryinfo>
   
   <refmeta>
@@ -120,11 +112,6 @@
     </variablelist>
   </refsect1>
   
-  <refsect1 id="bugs">
-    <title>BUGS</title>
-    <xi:include href="../bugs.xml"/>
-  </refsect1>
-  
   <refsect1 id="example">
     <title>EXAMPLE</title>
     <para>

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2019-02-11 05:14:10 +0000
+++ plugins.d/mandos-client.c	2011-10-05 16:00:56 +0000
@@ -9,23 +9,22 @@
  * "browse_callback", and parts of "main".
  * 
  * Everything else is
- * Copyright © 2008-2019 Teddy Hogeborn
- * Copyright © 2008-2019 Björn Påhlsson
- * 
- * This file is part of Mandos.
- * 
- * Mandos is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- * 
- * Mandos is distributed in the hope that it will be useful, but
+ * Copyright © 2008-2011 Teddy Hogeborn
+ * Copyright © 2008-2011 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  * 
  * Contact the authors at <mandos@recompile.se>.
  */
@@ -33,42 +32,37 @@
 /* Needed by GPGME, specifically gpgme_data_seek() */
 #ifndef _LARGEFILE_SOURCE
 #define _LARGEFILE_SOURCE
-#endif	/* not _LARGEFILE_SOURCE */
+#endif
 #ifndef _FILE_OFFSET_BITS
 #define _FILE_OFFSET_BITS 64
-#endif	/* not _FILE_OFFSET_BITS */
+#endif
 
 #define _GNU_SOURCE		/* TEMP_FAILURE_RETRY(), asprintf() */
 
 #include <stdio.h>		/* fprintf(), stderr, fwrite(),
-				   stdout, ferror() */
-#include <stdint.h> 		/* uint16_t, uint32_t, intptr_t */
+				   stdout, ferror(), remove() */
+#include <stdint.h> 		/* uint16_t, uint32_t */
 #include <stddef.h>		/* NULL, size_t, ssize_t */
 #include <stdlib.h> 		/* free(), EXIT_SUCCESS, srand(),
 				   strtof(), abort() */
 #include <stdbool.h>		/* bool, false, true */
-#include <string.h>		/* strcmp(), strlen(), strerror(),
-				   asprintf(), strncpy(), strsignal()
-				*/
+#include <string.h>		/* memset(), strcmp(), strlen(),
+				   strerror(), asprintf(), strcpy() */
 #include <sys/ioctl.h>		/* ioctl */
 #include <sys/types.h>		/* socket(), inet_pton(), sockaddr,
 				   sockaddr_in6, PF_INET6,
 				   SOCK_STREAM, uid_t, gid_t, open(),
 				   opendir(), DIR */
-#include <sys/stat.h>		/* open(), S_ISREG */
+#include <sys/stat.h>		/* open() */
 #include <sys/socket.h>		/* socket(), struct sockaddr_in6,
-				   inet_pton(), connect(),
-				   getnameinfo() */
-#include <fcntl.h>		/* open(), unlinkat(), AT_REMOVEDIR */
+				   inet_pton(), connect() */
+#include <fcntl.h>		/* open() */
 #include <dirent.h>		/* opendir(), struct dirent, readdir()
 				 */
 #include <inttypes.h>		/* PRIu16, PRIdMAX, intmax_t,
 				   strtoimax() */
-#include <errno.h>		/* perror(), errno, EINTR, EINVAL,
-				   EAI_SYSTEM, ENETUNREACH,
-				   EHOSTUNREACH, ECONNREFUSED, EPROTO,
-				   EIO, ENOENT, ENXIO, ENOMEM, EISDIR,
-				   ENOTEMPTY,
+#include <assert.h>		/* assert() */
+#include <errno.h>		/* perror(), errno,
 				   program_invocation_short_name */
 #include <time.h>		/* nanosleep(), time(), sleep() */
 #include <net/if.h>		/* ioctl, ifreq, SIOCGIFFLAGS, IFF_UP,
@@ -79,9 +73,8 @@
 				*/
 #include <unistd.h>		/* close(), SEEK_SET, off_t, write(),
 				   getuid(), getgid(), seteuid(),
-				   setgid(), pause(), _exit(),
-				   unlinkat() */
-#include <arpa/inet.h>		/* inet_pton(), htons() */
+				   setgid(), pause() */
+#include <arpa/inet.h>		/* inet_pton(), htons, inet_ntop() */
 #include <iso646.h>		/* not, or, and */
 #include <argp.h>		/* struct argp_option, error_t, struct
 				   argp_state, struct argp,
@@ -92,15 +85,6 @@
 				   raise() */
 #include <sysexits.h>		/* EX_OSERR, EX_USAGE, EX_UNAVAILABLE,
 				   EX_NOHOST, EX_IOERR, EX_PROTOCOL */
-#include <sys/wait.h>		/* waitpid(), WIFEXITED(),
-				   WEXITSTATUS(), WTERMSIG() */
-#include <grp.h>		/* setgroups() */
-#include <argz.h>		/* argz_add_sep(), argz_next(),
-				   argz_delete(), argz_append(),
-				   argz_stringify(), argz_add(),
-				   argz_count() */
-#include <netdb.h>		/* getnameinfo(), NI_NUMERICHOST,
-				   EAI_SYSTEM, gai_strerror() */
 
 #ifdef __linux__
 #include <sys/klog.h> 		/* klogctl() */
@@ -123,15 +107,9 @@
 				   gnutls_*
 				   init_gnutls_session(),
 				   GNUTLS_* */
-#if GNUTLS_VERSION_NUMBER < 0x030600
 #include <gnutls/openpgp.h>
-			 /* gnutls_certificate_set_openpgp_key_file(),
-			    GNUTLS_OPENPGP_FMT_BASE64 */
-#elif GNUTLS_VERSION_NUMBER >= 0x030606
-#include <gnutls/x509.h>	/* gnutls_pkcs_encrypt_flags_t,
-				 GNUTLS_PKCS_PLAIN,
-				 GNUTLS_PKCS_NULL_PASSWORD */
-#endif
+			  /* gnutls_certificate_set_openpgp_key_file(),
+				   GNUTLS_OPENPGP_FMT_BASE64 */
 
 /* GPGME */
 #include <gpgme.h> 		/* All GPGME types, constants and
@@ -145,9 +123,6 @@
 #define PATHDIR "/conf/conf.d/mandos"
 #define SECKEY "seckey.txt"
 #define PUBKEY "pubkey.txt"
-#define TLS_PRIVKEY "tls-privkey.pem"
-#define TLS_PUBKEY "tls-pubkey.pem"
-#define HOOKDIR "/lib/mandos/network-hooks.d"
 
 bool debug = false;
 static const char mandos_protocol_version[] = "1";
@@ -155,15 +130,11 @@
 const char *argp_program_bug_address = "<mandos@recompile.se>";
 static const char sys_class_net[] = "/sys/class/net";
 char *connect_to = NULL;
-const char *hookdir = HOOKDIR;
-int hookdir_fd = -1;
-uid_t uid = 65534;
-gid_t gid = 65534;
 
 /* Doubly linked list that need to be circularly linked when used */
 typedef struct server{
   const char *ip;
-  in_port_t port;
+  uint16_t port;
   AvahiIfIndex if_index;
   int af;
   struct timespec last_seen;
@@ -173,6 +144,7 @@
 
 /* Used for passing in values through the Avahi callback functions */
 typedef struct {
+  AvahiSimplePoll *simple_poll;
   AvahiServer *server;
   gnutls_certificate_credentials_t cred;
   unsigned int dh_bits;
@@ -180,173 +152,92 @@
   const char *priority;
   gpgme_ctx_t ctx;
   server *current_server;
-  char *interfaces;
-  size_t interfaces_size;
 } mandos_context;
 
-/* global so signal handler can reach it*/
-AvahiSimplePoll *simple_poll;
+/* global context so signal handler can reach it*/
+mandos_context mc = { .simple_poll = NULL, .server = NULL,
+		      .dh_bits = 1024, .priority = "SECURE256"
+		      ":!CTYPE-X.509:+CTYPE-OPENPGP",
+		      .current_server = NULL };
 
 sig_atomic_t quit_now = 0;
 int signal_received = 0;
 
 /* Function to use when printing errors */
 void perror_plus(const char *print_text){
-  int e = errno;
   fprintf(stderr, "Mandos plugin %s: ",
 	  program_invocation_short_name);
-  errno = e;
   perror(print_text);
 }
 
-__attribute__((format (gnu_printf, 2, 3), nonnull))
-int fprintf_plus(FILE *stream, const char *format, ...){
-  va_list ap;
-  va_start (ap, format);
-  
-  TEMP_FAILURE_RETRY(fprintf(stream, "Mandos plugin %s: ",
-			     program_invocation_short_name));
-  return (int)TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
-}
-
 /*
  * Make additional room in "buffer" for at least BUFFER_SIZE more
  * bytes. "buffer_capacity" is how much is currently allocated,
  * "buffer_length" is how much is already used.
  */
-__attribute__((nonnull, warn_unused_result))
 size_t incbuffer(char **buffer, size_t buffer_length,
-		 size_t buffer_capacity){
+		  size_t buffer_capacity){
   if(buffer_length + BUFFER_SIZE > buffer_capacity){
-    char *new_buf = realloc(*buffer, buffer_capacity + BUFFER_SIZE);
-    if(new_buf == NULL){
-      int old_errno = errno;
-      free(*buffer);
-      errno = old_errno;
-      *buffer = NULL;
+    *buffer = realloc(*buffer, buffer_capacity + BUFFER_SIZE);
+    if(buffer == NULL){
       return 0;
     }
-    *buffer = new_buf;
     buffer_capacity += BUFFER_SIZE;
   }
   return buffer_capacity;
 }
 
 /* Add server to set of servers to retry periodically */
-__attribute__((nonnull, warn_unused_result))
-bool add_server(const char *ip, in_port_t port, AvahiIfIndex if_index,
-		int af, server **current_server){
+int add_server(const char *ip, uint16_t port,
+		 AvahiIfIndex if_index,
+		 int af){
   int ret;
   server *new_server = malloc(sizeof(server));
   if(new_server == NULL){
     perror_plus("malloc");
-    return false;
+    return -1;
   }
   *new_server = (server){ .ip = strdup(ip),
-			  .port = port,
-			  .if_index = if_index,
-			  .af = af };
+			 .port = port,
+			 .if_index = if_index,
+			 .af = af };
   if(new_server->ip == NULL){
     perror_plus("strdup");
-    free(new_server);
-    return false;
-  }
-  ret = clock_gettime(CLOCK_MONOTONIC, &(new_server->last_seen));
-  if(ret == -1){
-    perror_plus("clock_gettime");
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-    free((char *)(new_server->ip));
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
-    free(new_server);
-    return false;
+    return -1;
   }
   /* Special case of first server */
-  if(*current_server == NULL){
+  if (mc.current_server == NULL){
     new_server->next = new_server;
     new_server->prev = new_server;
-    *current_server = new_server;
+    mc.current_server = new_server;
+  /* Place the new server last in the list */
   } else {
-    /* Place the new server last in the list */
-    new_server->next = *current_server;
-    new_server->prev = (*current_server)->prev;
+    new_server->next = mc.current_server;
+    new_server->prev = mc.current_server->prev;
     new_server->prev->next = new_server;
-    (*current_server)->prev = new_server;
-  }
-  return true;
-}
-
-/* Set effective uid to 0, return errno */
-__attribute__((warn_unused_result))
-int raise_privileges(void){
-  int old_errno = errno;
-  int ret = 0;
-  if(seteuid(0) == -1){
-    ret = errno;
-  }
-  errno = old_errno;
-  return ret;
-}
-
-/* Set effective and real user ID to 0.  Return errno. */
-__attribute__((warn_unused_result))
-int raise_privileges_permanently(void){
-  int old_errno = errno;
-  int ret = raise_privileges();
-  if(ret != 0){
-    errno = old_errno;
-    return ret;
-  }
-  if(setuid(0) == -1){
-    ret = errno;
-  }
-  errno = old_errno;
-  return ret;
-}
-
-/* Set effective user ID to unprivileged saved user ID */
-__attribute__((warn_unused_result))
-int lower_privileges(void){
-  int old_errno = errno;
-  int ret = 0;
-  if(seteuid(uid) == -1){
-    ret = errno;
-  }
-  errno = old_errno;
-  return ret;
-}
-
-/* Lower privileges permanently */
-__attribute__((warn_unused_result))
-int lower_privileges_permanently(void){
-  int old_errno = errno;
-  int ret = 0;
-  if(setuid(uid) == -1){
-    ret = errno;
-  }
-  errno = old_errno;
-  return ret;
+    mc.current_server->prev = new_server;
+  }
+  ret = clock_gettime(CLOCK_MONOTONIC, &mc.current_server->last_seen);
+  if(ret == -1){
+    perror_plus("clock_gettime");
+    return -1;
+  }
+  return 0;
 }
 
 /* 
  * Initialize GPGME.
  */
-__attribute__((nonnull, warn_unused_result))
-static bool init_gpgme(const char * const seckey,
-		       const char * const pubkey,
-		       const char * const tempdir,
-		       mandos_context *mc){
+static bool init_gpgme(const char *seckey,
+		       const char *pubkey, const char *tempdir){
   gpgme_error_t rc;
   gpgme_engine_info_t engine_info;
   
+  
   /*
    * Helper function to insert pub and seckey to the engine keyring.
    */
-  bool import_key(const char * const filename){
+  bool import_key(const char *filename){
     int ret;
     int fd;
     gpgme_data_t pgp_data;
@@ -357,146 +248,21 @@
       return false;
     }
     
-    /* Workaround for systems without a real-time clock; see also
-       Debian bug #894495: <https://bugs.debian.org/894495> */
-    do {
-      {
-	time_t currtime = time(NULL);
-	if(currtime != (time_t)-1){
-	  struct tm tm;
-	  if(gmtime_r(&currtime, &tm) == NULL) {
-	    perror_plus("gmtime_r");
-	    break;
-	  }
-	  if(tm.tm_year != 70 or tm.tm_mon != 0){
-	    break;
-	  }
-	  if(debug){
-	    fprintf_plus(stderr, "System clock is January 1970");
-	  }
-	} else {
-	  if(debug){
-	    fprintf_plus(stderr, "System clock is invalid");
-	  }
-	}
-      }
-      struct stat keystat;
-      ret = fstat(fd, &keystat);
-      if(ret != 0){
-	perror_plus("fstat");
-	break;
-      }
-      ret = raise_privileges();
-      if(ret != 0){
-	errno = ret;
-	perror_plus("Failed to raise privileges");
-	break;
-      }
-      if(debug){
-	fprintf_plus(stderr,
-		     "Setting system clock to key file mtime");
-      }
-      time_t keytime = keystat.st_mtim.tv_sec;
-      if(stime(&keytime) != 0){
-	perror_plus("stime");
-      }
-      ret = lower_privileges();
-      if(ret != 0){
-	errno = ret;
-	perror_plus("Failed to lower privileges");
-      }
-    } while(false);
-
     rc = gpgme_data_new_from_fd(&pgp_data, fd);
     if(rc != GPG_ERR_NO_ERROR){
-      fprintf_plus(stderr, "bad gpgme_data_new_from_fd: %s: %s\n",
-		   gpgme_strsource(rc), gpgme_strerror(rc));
+      fprintf(stderr, "bad gpgme_data_new_from_fd: %s: %s\n",
+	      gpgme_strsource(rc), gpgme_strerror(rc));
       return false;
     }
     
-    rc = gpgme_op_import(mc->ctx, pgp_data);
+    rc = gpgme_op_import(mc.ctx, pgp_data);
     if(rc != GPG_ERR_NO_ERROR){
-      fprintf_plus(stderr, "bad gpgme_op_import: %s: %s\n",
-		   gpgme_strsource(rc), gpgme_strerror(rc));
+      fprintf(stderr, "bad gpgme_op_import: %s: %s\n",
+	      gpgme_strsource(rc), gpgme_strerror(rc));
       return false;
     }
-    {
-      gpgme_import_result_t import_result
-	= gpgme_op_import_result(mc->ctx);
-      if((import_result->imported < 1
-	  or import_result->not_imported > 0)
-	 and import_result->unchanged == 0){
-	fprintf_plus(stderr, "bad gpgme_op_import_results:\n");
-	fprintf_plus(stderr,
-		     "The total number of considered keys: %d\n",
-		     import_result->considered);
-	fprintf_plus(stderr,
-		     "The number of keys without user ID: %d\n",
-		     import_result->no_user_id);
-	fprintf_plus(stderr,
-		     "The total number of imported keys: %d\n",
-		     import_result->imported);
-	fprintf_plus(stderr, "The number of imported RSA keys: %d\n",
-		     import_result->imported_rsa);
-	fprintf_plus(stderr, "The number of unchanged keys: %d\n",
-		     import_result->unchanged);
-	fprintf_plus(stderr, "The number of new user IDs: %d\n",
-		     import_result->new_user_ids);
-	fprintf_plus(stderr, "The number of new sub keys: %d\n",
-		     import_result->new_sub_keys);
-	fprintf_plus(stderr, "The number of new signatures: %d\n",
-		     import_result->new_signatures);
-	fprintf_plus(stderr, "The number of new revocations: %d\n",
-		     import_result->new_revocations);
-	fprintf_plus(stderr,
-		     "The total number of secret keys read: %d\n",
-		     import_result->secret_read);
-	fprintf_plus(stderr,
-		     "The number of imported secret keys: %d\n",
-		     import_result->secret_imported);
-	fprintf_plus(stderr,
-		     "The number of unchanged secret keys: %d\n",
-		     import_result->secret_unchanged);
-	fprintf_plus(stderr, "The number of keys not imported: %d\n",
-		     import_result->not_imported);
-	for(gpgme_import_status_t import_status
-	      = import_result->imports;
-	    import_status != NULL;
-	    import_status = import_status->next){
-	  fprintf_plus(stderr, "Import status for key: %s\n",
-		       import_status->fpr);
-	  if(import_status->result != GPG_ERR_NO_ERROR){
-	    fprintf_plus(stderr, "Import result: %s: %s\n",
-			 gpgme_strsource(import_status->result),
-			 gpgme_strerror(import_status->result));
-	  }
-	  fprintf_plus(stderr, "Key status:\n");
-	  fprintf_plus(stderr,
-		       import_status->status & GPGME_IMPORT_NEW
-		       ? "The key was new.\n"
-		       : "The key was not new.\n");
-	  fprintf_plus(stderr,
-		       import_status->status & GPGME_IMPORT_UID
-		       ? "The key contained new user IDs.\n"
-		       : "The key did not contain new user IDs.\n");
-	  fprintf_plus(stderr,
-		       import_status->status & GPGME_IMPORT_SIG
-		       ? "The key contained new signatures.\n"
-		       : "The key did not contain new signatures.\n");
-	  fprintf_plus(stderr,
-		       import_status->status & GPGME_IMPORT_SUBKEY
-		       ? "The key contained new sub keys.\n"
-		       : "The key did not contain new sub keys.\n");
-	  fprintf_plus(stderr,
-		       import_status->status & GPGME_IMPORT_SECRET
-		       ? "The key contained a secret key.\n"
-		       : "The key did not contain a secret key.\n");
-	}
-	return false;
-      }
-    }
     
-    ret = close(fd);
+    ret = (int)TEMP_FAILURE_RETRY(close(fd));
     if(ret == -1){
       perror_plus("close");
     }
@@ -505,23 +271,23 @@
   }
   
   if(debug){
-    fprintf_plus(stderr, "Initializing GPGME\n");
+    fprintf(stderr, "Initializing GPGME\n");
   }
   
   /* Init GPGME */
   gpgme_check_version(NULL);
   rc = gpgme_engine_check_version(GPGME_PROTOCOL_OpenPGP);
   if(rc != GPG_ERR_NO_ERROR){
-    fprintf_plus(stderr, "bad gpgme_engine_check_version: %s: %s\n",
-		 gpgme_strsource(rc), gpgme_strerror(rc));
+    fprintf(stderr, "bad gpgme_engine_check_version: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   
   /* Set GPGME home directory for the OpenPGP engine only */
   rc = gpgme_get_engine_info(&engine_info);
   if(rc != GPG_ERR_NO_ERROR){
-    fprintf_plus(stderr, "bad gpgme_get_engine_info: %s: %s\n",
-		 gpgme_strsource(rc), gpgme_strerror(rc));
+    fprintf(stderr, "bad gpgme_get_engine_info: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   while(engine_info != NULL){
@@ -533,16 +299,15 @@
     engine_info = engine_info->next;
   }
   if(engine_info == NULL){
-    fprintf_plus(stderr, "Could not set GPGME home dir to %s\n",
-		 tempdir);
+    fprintf(stderr, "Could not set GPGME home dir to %s\n", tempdir);
     return false;
   }
   
   /* Create new GPGME "context" */
-  rc = gpgme_new(&(mc->ctx));
+  rc = gpgme_new(&(mc.ctx));
   if(rc != GPG_ERR_NO_ERROR){
-    fprintf_plus(stderr, "bad gpgme_new: %s: %s\n",
-		 gpgme_strsource(rc), gpgme_strerror(rc));
+    fprintf(stderr, "bad gpgme_new: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   
@@ -557,11 +322,9 @@
  * Decrypt OpenPGP data.
  * Returns -1 on error
  */
-__attribute__((nonnull, warn_unused_result))
 static ssize_t pgp_packet_decrypt(const char *cryptotext,
 				  size_t crypto_size,
-				  char **plaintext,
-				  mandos_context *mc){
+				  char **plaintext){
   gpgme_data_t dh_crypto, dh_plain;
   gpgme_error_t rc;
   ssize_t ret;
@@ -569,57 +332,57 @@
   ssize_t plaintext_length = 0;
   
   if(debug){
-    fprintf_plus(stderr, "Trying to decrypt OpenPGP data\n");
+    fprintf(stderr, "Trying to decrypt OpenPGP data\n");
   }
   
   /* Create new GPGME data buffer from memory cryptotext */
   rc = gpgme_data_new_from_mem(&dh_crypto, cryptotext, crypto_size,
 			       0);
   if(rc != GPG_ERR_NO_ERROR){
-    fprintf_plus(stderr, "bad gpgme_data_new_from_mem: %s: %s\n",
-		 gpgme_strsource(rc), gpgme_strerror(rc));
+    fprintf(stderr, "bad gpgme_data_new_from_mem: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
     return -1;
   }
   
   /* Create new empty GPGME data buffer for the plaintext */
   rc = gpgme_data_new(&dh_plain);
   if(rc != GPG_ERR_NO_ERROR){
-    fprintf_plus(stderr, "bad gpgme_data_new: %s: %s\n",
-		 gpgme_strsource(rc), gpgme_strerror(rc));
+    fprintf(stderr, "bad gpgme_data_new: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
     gpgme_data_release(dh_crypto);
     return -1;
   }
   
   /* Decrypt data from the cryptotext data buffer to the plaintext
      data buffer */
-  rc = gpgme_op_decrypt(mc->ctx, dh_crypto, dh_plain);
+  rc = gpgme_op_decrypt(mc.ctx, dh_crypto, dh_plain);
   if(rc != GPG_ERR_NO_ERROR){
-    fprintf_plus(stderr, "bad gpgme_op_decrypt: %s: %s\n",
-		 gpgme_strsource(rc), gpgme_strerror(rc));
+    fprintf(stderr, "bad gpgme_op_decrypt: %s: %s\n",
+	    gpgme_strsource(rc), gpgme_strerror(rc));
     plaintext_length = -1;
     if(debug){
       gpgme_decrypt_result_t result;
-      result = gpgme_op_decrypt_result(mc->ctx);
+      result = gpgme_op_decrypt_result(mc.ctx);
       if(result == NULL){
-	fprintf_plus(stderr, "gpgme_op_decrypt_result failed\n");
+	fprintf(stderr, "gpgme_op_decrypt_result failed\n");
       } else {
-	if(result->unsupported_algorithm != NULL) {
-	  fprintf_plus(stderr, "Unsupported algorithm: %s\n",
-		       result->unsupported_algorithm);
-	}
-	fprintf_plus(stderr, "Wrong key usage: %s\n",
-		     result->wrong_key_usage ? "Yes" : "No");
+	fprintf(stderr, "Unsupported algorithm: %s\n",
+		result->unsupported_algorithm);
+	fprintf(stderr, "Wrong key usage: %u\n",
+		result->wrong_key_usage);
 	if(result->file_name != NULL){
-	  fprintf_plus(stderr, "File name: %s\n", result->file_name);
+	  fprintf(stderr, "File name: %s\n", result->file_name);
 	}
-
-	for(gpgme_recipient_t r = result->recipients; r != NULL;
-	    r = r->next){
-	  fprintf_plus(stderr, "Public key algorithm: %s\n",
-		       gpgme_pubkey_algo_name(r->pubkey_algo));
-	  fprintf_plus(stderr, "Key ID: %s\n", r->keyid);
-	  fprintf_plus(stderr, "Secret key available: %s\n",
-		       r->status == GPG_ERR_NO_SECKEY ? "No" : "Yes");
+	gpgme_recipient_t recipient;
+	recipient = result->recipients;
+	while(recipient != NULL){
+	  fprintf(stderr, "Public key algorithm: %s\n",
+		  gpgme_pubkey_algo_name(recipient->pubkey_algo));
+	  fprintf(stderr, "Key ID: %s\n", recipient->keyid);
+	  fprintf(stderr, "Secret key available: %s\n",
+		  recipient->status == GPG_ERR_NO_SECKEY
+		  ? "No" : "Yes");
+	  recipient = recipient->next;
 	}
       }
     }
@@ -627,7 +390,7 @@
   }
   
   if(debug){
-    fprintf_plus(stderr, "Decryption of OpenPGP data succeeded\n");
+    fprintf(stderr, "Decryption of OpenPGP data succeeded\n");
   }
   
   /* Seek back to the beginning of the GPGME plaintext data buffer */
@@ -640,12 +403,12 @@
   *plaintext = NULL;
   while(true){
     plaintext_capacity = incbuffer(plaintext,
-				   (size_t)plaintext_length,
-				   plaintext_capacity);
+				      (size_t)plaintext_length,
+				      plaintext_capacity);
     if(plaintext_capacity == 0){
-      perror_plus("incbuffer");
-      plaintext_length = -1;
-      goto decrypt_end;
+	perror_plus("incbuffer");
+	plaintext_length = -1;
+	goto decrypt_end;
     }
     
     ret = gpgme_data_read(dh_plain, *plaintext + plaintext_length,
@@ -664,7 +427,7 @@
   }
   
   if(debug){
-    fprintf_plus(stderr, "Decrypted password is: ");
+    fprintf(stderr, "Decrypted password is: ");
     for(ssize_t i = 0; i < plaintext_length; i++){
       fprintf(stderr, "%02hhX ", (*plaintext)[i]);
     }
@@ -681,35 +444,33 @@
   return plaintext_length;
 }
 
-__attribute__((warn_unused_result, const))
-static const char *safe_string(const char *str){
-  if(str == NULL)
-    return "(unknown)";
-  return str;
-}
-
-__attribute__((warn_unused_result))
-static const char *safer_gnutls_strerror(int value){
-  const char *ret = gnutls_strerror(value);
-  return safe_string(ret);
+static const char * safer_gnutls_strerror(int value){
+  const char *ret = gnutls_strerror(value); /* Spurious warning from
+					       -Wunreachable-code */
+  if(ret == NULL)
+    ret = "(unknown)";
+  return ret;
 }
 
 /* GnuTLS log function callback */
-__attribute__((nonnull))
 static void debuggnutls(__attribute__((unused)) int level,
 			const char* string){
-  fprintf_plus(stderr, "GnuTLS: %s", string);
+  fprintf(stderr, "GnuTLS: %s", string);
 }
 
-__attribute__((nonnull(1, 2, 4), warn_unused_result))
 static int init_gnutls_global(const char *pubkeyfilename,
-			      const char *seckeyfilename,
-			      const char *dhparamsfilename,
-			      mandos_context *mc){
+			      const char *seckeyfilename){
   int ret;
   
   if(debug){
-    fprintf_plus(stderr, "Initializing GnuTLS\n");
+    fprintf(stderr, "Initializing GnuTLS\n");
+  }
+  
+  ret = gnutls_global_init();
+  if(ret != GNUTLS_E_SUCCESS){
+    fprintf(stderr, "GnuTLS global_init: %s\n",
+	    safer_gnutls_strerror(ret));
+    return -1;
   }
   
   if(debug){
@@ -721,285 +482,85 @@
   }
   
   /* OpenPGP credentials */
-  ret = gnutls_certificate_allocate_credentials(&mc->cred);
+  ret = gnutls_certificate_allocate_credentials(&mc.cred);
   if(ret != GNUTLS_E_SUCCESS){
-    fprintf_plus(stderr, "GnuTLS memory error: %s\n",
-		 safer_gnutls_strerror(ret));
+    fprintf(stderr, "GnuTLS memory error: %s\n",
+	    safer_gnutls_strerror(ret));
+    gnutls_global_deinit();
     return -1;
   }
   
   if(debug){
-    fprintf_plus(stderr, "Attempting to use public key %s and"
-		 " private key %s as GnuTLS credentials\n",
-		 pubkeyfilename,
-		 seckeyfilename);
+    fprintf(stderr, "Attempting to use OpenPGP public key %s and"
+	    " secret key %s as GnuTLS credentials\n", pubkeyfilename,
+	    seckeyfilename);
   }
   
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-  ret = gnutls_certificate_set_rawpk_key_file
-    (mc->cred, pubkeyfilename, seckeyfilename,
-     GNUTLS_X509_FMT_PEM,	/* format */
-     NULL,			/* pass */
-     /* key_usage */
-     GNUTLS_KEY_DIGITAL_SIGNATURE | GNUTLS_KEY_KEY_ENCIPHERMENT,
-     NULL, 			/* names */
-     0,				/* names_length */
-     /* privkey_flags */
-     GNUTLS_PKCS_PLAIN | GNUTLS_PKCS_NULL_PASSWORD,
-     0);			/* pkcs11_flags */
-#elif GNUTLS_VERSION_NUMBER < 0x030600
   ret = gnutls_certificate_set_openpgp_key_file
-    (mc->cred, pubkeyfilename, seckeyfilename,
+    (mc.cred, pubkeyfilename, seckeyfilename,
      GNUTLS_OPENPGP_FMT_BASE64);
-#else
-#error "Needs GnuTLS 3.6.6 or later, or before 3.6.0"
-#endif
   if(ret != GNUTLS_E_SUCCESS){
-    fprintf_plus(stderr,
-		 "Error[%d] while reading the key pair ('%s',"
-		 " '%s')\n", ret, pubkeyfilename, seckeyfilename);
-    fprintf_plus(stderr, "The GnuTLS error is: %s\n",
-		 safer_gnutls_strerror(ret));
+    fprintf(stderr,
+	    "Error[%d] while reading the OpenPGP key pair ('%s',"
+	    " '%s')\n", ret, pubkeyfilename, seckeyfilename);
+    fprintf(stderr, "The GnuTLS error is: %s\n",
+	    safer_gnutls_strerror(ret));
     goto globalfail;
   }
   
   /* GnuTLS server initialization */
-  ret = gnutls_dh_params_init(&mc->dh_params);
-  if(ret != GNUTLS_E_SUCCESS){
-    fprintf_plus(stderr, "Error in GnuTLS DH parameter"
-		 " initialization: %s\n",
-		 safer_gnutls_strerror(ret));
-    goto globalfail;
-  }
-  /* If a Diffie-Hellman parameters file was given, try to use it */
-  if(dhparamsfilename != NULL){
-    gnutls_datum_t params = { .data = NULL, .size = 0 };
-    do {
-      int dhpfile = open(dhparamsfilename, O_RDONLY);
-      if(dhpfile == -1){
-	perror_plus("open");
-	dhparamsfilename = NULL;
-	break;
-      }
-      size_t params_capacity = 0;
-      while(true){
-	params_capacity = incbuffer((char **)&params.data,
-				    (size_t)params.size,
-				    (size_t)params_capacity);
-	if(params_capacity == 0){
-	  perror_plus("incbuffer");
-	  free(params.data);
-	  params.data = NULL;
-	  dhparamsfilename = NULL;
-	  break;
-	}
-	ssize_t bytes_read = read(dhpfile,
-				  params.data + params.size,
-				  BUFFER_SIZE);
-	/* EOF */
-	if(bytes_read == 0){
-	  break;
-	}
-	/* check bytes_read for failure */
-	if(bytes_read < 0){
-	  perror_plus("read");
-	  free(params.data);
-	  params.data = NULL;
-	  dhparamsfilename = NULL;
-	  break;
-	}
-	params.size += (unsigned int)bytes_read;
-      }
-      ret = close(dhpfile);
-      if(ret == -1){
-	perror_plus("close");
-      }
-      if(params.data == NULL){
-	dhparamsfilename = NULL;
-      }
-      if(dhparamsfilename == NULL){
-	break;
-      }
-      ret = gnutls_dh_params_import_pkcs3(mc->dh_params, &params,
-					  GNUTLS_X509_FMT_PEM);
-      if(ret != GNUTLS_E_SUCCESS){
-	fprintf_plus(stderr, "Failed to parse DH parameters in file"
-		     " \"%s\": %s\n", dhparamsfilename,
-		     safer_gnutls_strerror(ret));
-	dhparamsfilename = NULL;
-      }
-      free(params.data);
-    } while(false);
-  }
-  if(dhparamsfilename == NULL){
-    if(mc->dh_bits == 0){
-#if GNUTLS_VERSION_NUMBER < 0x030600
-      /* Find out the optimal number of DH bits */
-      /* Try to read the private key file */
-      gnutls_datum_t buffer = { .data = NULL, .size = 0 };
-      do {
-	int secfile = open(seckeyfilename, O_RDONLY);
-	if(secfile == -1){
-	  perror_plus("open");
-	  break;
-	}
-	size_t buffer_capacity = 0;
-	while(true){
-	  buffer_capacity = incbuffer((char **)&buffer.data,
-				      (size_t)buffer.size,
-				      (size_t)buffer_capacity);
-	  if(buffer_capacity == 0){
-	    perror_plus("incbuffer");
-	    free(buffer.data);
-	    buffer.data = NULL;
-	    break;
-	  }
-	  ssize_t bytes_read = read(secfile,
-				    buffer.data + buffer.size,
-				    BUFFER_SIZE);
-	  /* EOF */
-	  if(bytes_read == 0){
-	    break;
-	  }
-	  /* check bytes_read for failure */
-	  if(bytes_read < 0){
-	    perror_plus("read");
-	    free(buffer.data);
-	    buffer.data = NULL;
-	    break;
-	  }
-	  buffer.size += (unsigned int)bytes_read;
-	}
-	close(secfile);
-      } while(false);
-      /* If successful, use buffer to parse private key */
-      gnutls_sec_param_t sec_param = GNUTLS_SEC_PARAM_ULTRA;
-      if(buffer.data != NULL){
-	{
-	  gnutls_openpgp_privkey_t privkey = NULL;
-	  ret = gnutls_openpgp_privkey_init(&privkey);
-	  if(ret != GNUTLS_E_SUCCESS){
-	    fprintf_plus(stderr, "Error initializing OpenPGP key"
-			 " structure: %s",
-			 safer_gnutls_strerror(ret));
-	    free(buffer.data);
-	    buffer.data = NULL;
-	  } else {
-	    ret = gnutls_openpgp_privkey_import
-	      (privkey, &buffer, GNUTLS_OPENPGP_FMT_BASE64, "", 0);
-	    if(ret != GNUTLS_E_SUCCESS){
-	      fprintf_plus(stderr, "Error importing OpenPGP key : %s",
-			   safer_gnutls_strerror(ret));
-	      privkey = NULL;
-	    }
-	    free(buffer.data);
-	    buffer.data = NULL;
-	    if(privkey != NULL){
-	      /* Use private key to suggest an appropriate
-		 sec_param */
-	      sec_param = gnutls_openpgp_privkey_sec_param(privkey);
-	      gnutls_openpgp_privkey_deinit(privkey);
-	      if(debug){
-		fprintf_plus(stderr, "This OpenPGP key implies using"
-			     " a GnuTLS security parameter \"%s\".\n",
-			     safe_string(gnutls_sec_param_get_name
-					 (sec_param)));
-	      }
-	    }
-	  }
-	}
-	if(sec_param == GNUTLS_SEC_PARAM_UNKNOWN){
-	  /* Err on the side of caution */
-	  sec_param = GNUTLS_SEC_PARAM_ULTRA;
-	  if(debug){
-	    fprintf_plus(stderr, "Falling back to security parameter"
-			 " \"%s\"\n",
-			 safe_string(gnutls_sec_param_get_name
-				     (sec_param)));
-	  }
-	}
-      }
-      unsigned int uret = gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH, sec_param);
-      if(uret != 0){
-	mc->dh_bits = uret;
-	if(debug){
-	  fprintf_plus(stderr, "A \"%s\" GnuTLS security parameter"
-		       " implies %u DH bits; using that.\n",
-		       safe_string(gnutls_sec_param_get_name
-				   (sec_param)),
-		       mc->dh_bits);
-	}
-      } else {
-	fprintf_plus(stderr, "Failed to get implied number of DH"
-		     " bits for security parameter \"%s\"): %s\n",
-		     safe_string(gnutls_sec_param_get_name
-				 (sec_param)),
-		     safer_gnutls_strerror(ret));
-	goto globalfail;
-      }
-#endif
-    } else {			/* dh_bits != 0 */
-      if(debug){
-	fprintf_plus(stderr, "DH bits explicitly set to %u\n",
-		     mc->dh_bits);
-      }
-      ret = gnutls_dh_params_generate2(mc->dh_params, mc->dh_bits);
-      if(ret != GNUTLS_E_SUCCESS){
-	fprintf_plus(stderr, "Error in GnuTLS prime generation (%u"
-		     " bits): %s\n", mc->dh_bits,
-		     safer_gnutls_strerror(ret));
-	goto globalfail;
-      }
-      gnutls_certificate_set_dh_params(mc->cred, mc->dh_params);
-    }
-  }
+  ret = gnutls_dh_params_init(&mc.dh_params);
+  if(ret != GNUTLS_E_SUCCESS){
+    fprintf(stderr, "Error in GnuTLS DH parameter initialization:"
+	    " %s\n", safer_gnutls_strerror(ret));
+    goto globalfail;
+  }
+  ret = gnutls_dh_params_generate2(mc.dh_params, mc.dh_bits);
+  if(ret != GNUTLS_E_SUCCESS){
+    fprintf(stderr, "Error in GnuTLS prime generation: %s\n",
+	    safer_gnutls_strerror(ret));
+    goto globalfail;
+  }
+  
+  gnutls_certificate_set_dh_params(mc.cred, mc.dh_params);
   
   return 0;
   
  globalfail:
   
-  gnutls_certificate_free_credentials(mc->cred);
-  gnutls_dh_params_deinit(mc->dh_params);
+  gnutls_certificate_free_credentials(mc.cred);
+  gnutls_global_deinit();
+  gnutls_dh_params_deinit(mc.dh_params);
   return -1;
 }
 
-__attribute__((nonnull, warn_unused_result))
-static int init_gnutls_session(gnutls_session_t *session,
-			       mandos_context *mc){
+static int init_gnutls_session(gnutls_session_t *session){
   int ret;
   /* GnuTLS session creation */
   do {
-    ret = gnutls_init(session, (GNUTLS_SERVER
-#if GNUTLS_VERSION_NUMBER >= 0x030506
-				| GNUTLS_NO_TICKETS
-#endif
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-				| GNUTLS_ENABLE_RAWPK
-#endif
-				));
+    ret = gnutls_init(session, GNUTLS_SERVER);
     if(quit_now){
       return -1;
     }
   } while(ret == GNUTLS_E_INTERRUPTED or ret == GNUTLS_E_AGAIN);
   if(ret != GNUTLS_E_SUCCESS){
-    fprintf_plus(stderr,
-		 "Error in GnuTLS session initialization: %s\n",
-		 safer_gnutls_strerror(ret));
+    fprintf(stderr, "Error in GnuTLS session initialization: %s\n",
+	    safer_gnutls_strerror(ret));
   }
   
   {
     const char *err;
     do {
-      ret = gnutls_priority_set_direct(*session, mc->priority, &err);
+      ret = gnutls_priority_set_direct(*session, mc.priority, &err);
       if(quit_now){
 	gnutls_deinit(*session);
 	return -1;
       }
     } while(ret == GNUTLS_E_INTERRUPTED or ret == GNUTLS_E_AGAIN);
     if(ret != GNUTLS_E_SUCCESS){
-      fprintf_plus(stderr, "Syntax error at: %s\n", err);
-      fprintf_plus(stderr, "GnuTLS error: %s\n",
-		   safer_gnutls_strerror(ret));
+      fprintf(stderr, "Syntax error at: %s\n", err);
+      fprintf(stderr, "GnuTLS error: %s\n",
+	      safer_gnutls_strerror(ret));
       gnutls_deinit(*session);
       return -1;
     }
@@ -1007,15 +568,15 @@
   
   do {
     ret = gnutls_credentials_set(*session, GNUTLS_CRD_CERTIFICATE,
-				 mc->cred);
+				 mc.cred);
     if(quit_now){
       gnutls_deinit(*session);
       return -1;
     }
   } while(ret == GNUTLS_E_INTERRUPTED or ret == GNUTLS_E_AGAIN);
   if(ret != GNUTLS_E_SUCCESS){
-    fprintf_plus(stderr, "Error setting GnuTLS credentials: %s\n",
-		 safer_gnutls_strerror(ret));
+    fprintf(stderr, "Error setting GnuTLS credentials: %s\n",
+	    safer_gnutls_strerror(ret));
     gnutls_deinit(*session);
     return -1;
   }
@@ -1023,6 +584,8 @@
   /* ignore client certificate if any. */
   gnutls_certificate_server_set_request(*session, GNUTLS_CERT_IGNORE);
   
+  gnutls_dh_set_prime_bits(*session, mc.dh_bits);
+  
   return 0;
 }
 
@@ -1030,180 +593,16 @@
 static void empty_log(__attribute__((unused)) AvahiLogLevel level,
 		      __attribute__((unused)) const char *txt){}
 
-/* Helper function to add_local_route() and delete_local_route() */
-__attribute__((nonnull, warn_unused_result))
-static bool add_delete_local_route(const bool add,
-				   const char *address,
-				   AvahiIfIndex if_index){
-  int ret;
-  char helper[] = "mandos-client-iprouteadddel";
-  char add_arg[] = "add";
-  char delete_arg[] = "delete";
-  char debug_flag[] = "--debug";
-  char *pluginhelperdir = getenv("MANDOSPLUGINHELPERDIR");
-  if(pluginhelperdir == NULL){
-    if(debug){
-      fprintf_plus(stderr, "MANDOSPLUGINHELPERDIR environment"
-		   " variable not set; cannot run helper\n");
-    }
-    return false;
-  }
-  
-  char interface[IF_NAMESIZE];
-  if(if_indextoname((unsigned int)if_index, interface) == NULL){
-    perror_plus("if_indextoname");
-    return false;
-  }
-  
-  int devnull = (int)TEMP_FAILURE_RETRY(open("/dev/null", O_RDONLY));
-  if(devnull == -1){
-    perror_plus("open(\"/dev/null\", O_RDONLY)");
-    return false;
-  }
-  pid_t pid = fork();
-  if(pid == 0){
-    /* Child */
-    /* Raise privileges */
-    errno = raise_privileges_permanently();
-    if(errno != 0){
-      perror_plus("Failed to raise privileges");
-      /* _exit(EX_NOPERM); */
-    } else {
-      /* Set group */
-      errno = 0;
-      ret = setgid(0);
-      if(ret == -1){
-	perror_plus("setgid");
-	_exit(EX_NOPERM);
-      }
-      /* Reset supplementary groups */
-      errno = 0;
-      ret = setgroups(0, NULL);
-      if(ret == -1){
-	perror_plus("setgroups");
-	_exit(EX_NOPERM);
-      }
-    }
-    ret = dup2(devnull, STDIN_FILENO);
-    if(ret == -1){
-      perror_plus("dup2(devnull, STDIN_FILENO)");
-      _exit(EX_OSERR);
-    }
-    ret = close(devnull);
-    if(ret == -1){
-      perror_plus("close");
-      _exit(EX_OSERR);
-    }
-    ret = dup2(STDERR_FILENO, STDOUT_FILENO);
-    if(ret == -1){
-      perror_plus("dup2(STDERR_FILENO, STDOUT_FILENO)");
-      _exit(EX_OSERR);
-    }
-    int helperdir_fd = (int)TEMP_FAILURE_RETRY(open(pluginhelperdir,
-						    O_RDONLY
-						    | O_DIRECTORY
-						    | O_PATH
-						    | O_CLOEXEC));
-    if(helperdir_fd == -1){
-      perror_plus("open");
-      _exit(EX_UNAVAILABLE);
-    }
-    int helper_fd = (int)TEMP_FAILURE_RETRY(openat(helperdir_fd,
-						   helper, O_RDONLY));
-    if(helper_fd == -1){
-      perror_plus("openat");
-      close(helperdir_fd);
-      _exit(EX_UNAVAILABLE);
-    }
-    close(helperdir_fd);
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-    if(fexecve(helper_fd, (char *const [])
-	       { helper, add ? add_arg : delete_arg, (char *)address,
-		   interface, debug ? debug_flag : NULL, NULL },
-	       environ) == -1){
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
-      perror_plus("fexecve");
-      _exit(EXIT_FAILURE);
-    }
-  }
-  if(pid == -1){
-    perror_plus("fork");
-    return false;
-  }
-  int status;
-  pid_t pret = -1;
-  errno = 0;
-  do {
-    pret = waitpid(pid, &status, 0);
-    if(pret == -1 and errno == EINTR and quit_now){
-      int errno_raising = 0;
-      if((errno = raise_privileges()) != 0){
-	errno_raising = errno;
-	perror_plus("Failed to raise privileges in order to"
-		    " kill helper program");
-      }
-      if(kill(pid, SIGTERM) == -1){
-	perror_plus("kill");
-      }
-      if((errno_raising == 0) and (errno = lower_privileges()) != 0){
-	perror_plus("Failed to lower privileges after killing"
-		    " helper program");
-      }
-      return false;
-    }
-  } while(pret == -1 and errno == EINTR);
-  if(pret == -1){
-    perror_plus("waitpid");
-    return false;
-  }
-  if(WIFEXITED(status)){
-    if(WEXITSTATUS(status) != 0){
-      fprintf_plus(stderr, "Error: iprouteadddel exited"
-		   " with status %d\n", WEXITSTATUS(status));
-      return false;
-    }
-    return true;
-  }
-  if(WIFSIGNALED(status)){
-    fprintf_plus(stderr, "Error: iprouteadddel died by"
-		 " signal %d\n", WTERMSIG(status));
-    return false;
-  }
-  fprintf_plus(stderr, "Error: iprouteadddel crashed\n");
-  return false;
-}
-
-__attribute__((nonnull, warn_unused_result))
-static bool add_local_route(const char *address,
-			    AvahiIfIndex if_index){
-  if(debug){
-    fprintf_plus(stderr, "Adding route to %s\n", address);
-  }
-  return add_delete_local_route(true, address, if_index);
-}
-
-__attribute__((nonnull, warn_unused_result))
-static bool delete_local_route(const char *address,
-			       AvahiIfIndex if_index){
-  if(debug){
-    fprintf_plus(stderr, "Removing route to %s\n", address);
-  }
-  return add_delete_local_route(false, address, if_index);
-}
-
 /* Called when a Mandos server is found */
-__attribute__((nonnull, warn_unused_result))
-static int start_mandos_communication(const char *ip, in_port_t port,
+static int start_mandos_communication(const char *ip, uint16_t port,
 				      AvahiIfIndex if_index,
-				      int af, mandos_context *mc){
+				      int af){
   int ret, tcp_sd = -1;
   ssize_t sret;
-  struct sockaddr_storage to;
+  union {
+    struct sockaddr_in in;
+    struct sockaddr_in6 in6;
+  } to;
   char *buffer = NULL;
   char *decrypted_buffer = NULL;
   size_t buffer_length = 0;
@@ -1212,7 +611,6 @@
   int retval = -1;
   gnutls_session_t session;
   int pf;			/* Protocol family */
-  bool route_added = false;
   
   errno = 0;
   
@@ -1229,55 +627,22 @@
     pf = PF_INET;
     break;
   default:
-    fprintf_plus(stderr, "Bad address family: %d\n", af);
+    fprintf(stderr, "Bad address family: %d\n", af);
     errno = EINVAL;
     return -1;
   }
   
-  /* If the interface is specified and we have a list of interfaces */
-  if(if_index != AVAHI_IF_UNSPEC and mc->interfaces != NULL){
-    /* Check if the interface is one of the interfaces we are using */
-    bool match = false;
-    {
-      char *interface = NULL;
-      while((interface = argz_next(mc->interfaces,
-				   mc->interfaces_size,
-				   interface))){
-	if(if_nametoindex(interface) == (unsigned int)if_index){
-	  match = true;
-	  break;
-	}
-      }
-    }
-    if(not match){
-      /* This interface does not match any in the list, so we don't
-	 connect to the server */
-      if(debug){
-	char interface[IF_NAMESIZE];
-	if(if_indextoname((unsigned int)if_index, interface) == NULL){
-	  perror_plus("if_indextoname");
-	} else {
-	  fprintf_plus(stderr, "Skipping server on non-used interface"
-		       " \"%s\"\n",
-		       if_indextoname((unsigned int)if_index,
-				      interface));
-	}
-      }
-      return -1;
-    }
-  }
-  
-  ret = init_gnutls_session(&session, mc);
+  ret = init_gnutls_session(&session);
   if(ret != 0){
     return -1;
   }
   
   if(debug){
-    fprintf_plus(stderr, "Setting up a TCP connection to %s, port %"
-		 PRIuMAX "\n", ip, (uintmax_t)port);
+    fprintf(stderr, "Setting up a TCP connection to %s, port %" PRIu16
+	    "\n", ip, port);
   }
   
-  tcp_sd = socket(pf, SOCK_STREAM | SOCK_CLOEXEC, 0);
+  tcp_sd = socket(pf, SOCK_STREAM, 0);
   if(tcp_sd < 0){
     int e = errno;
     perror_plus("socket");
@@ -1290,14 +655,13 @@
     goto mandos_end;
   }
   
+  memset(&to, 0, sizeof(to));
   if(af == AF_INET6){
-    struct sockaddr_in6 *to6 = (struct sockaddr_in6 *)&to;
-    *to6 = (struct sockaddr_in6){ .sin6_family = (sa_family_t)af };
-    ret = inet_pton(af, ip, &to6->sin6_addr);
+    to.in6.sin6_family = (sa_family_t)af;
+    ret = inet_pton(af, ip, &to.in6.sin6_addr);
   } else {			/* IPv4 */
-    struct sockaddr_in *to4 = (struct sockaddr_in *)&to;
-    *to4 = (struct sockaddr_in){ .sin_family = (sa_family_t)af };
-    ret = inet_pton(af, ip, &to4->sin_addr);
+    to.in.sin_family = (sa_family_t)af;
+    ret = inet_pton(af, ip, &to.in.sin_addr);
   }
   if(ret < 0 ){
     int e = errno;
@@ -1307,25 +671,31 @@
   }
   if(ret == 0){
     int e = errno;
-    fprintf_plus(stderr, "Bad address: %s\n", ip);
+    fprintf(stderr, "Bad address: %s\n", ip);
     errno = e;
     goto mandos_end;
   }
   if(af == AF_INET6){
-    ((struct sockaddr_in6 *)&to)->sin6_port = htons(port);
-    if(IN6_IS_ADDR_LINKLOCAL
-       (&((struct sockaddr_in6 *)&to)->sin6_addr)){
+    to.in6.sin6_port = htons(port); /* Spurious warnings from
+				       -Wconversion and
+				       -Wunreachable-code */
+    
+    if(IN6_IS_ADDR_LINKLOCAL /* Spurious warnings from */
+       (&to.in6.sin6_addr)){ /* -Wstrict-aliasing=2 or lower and
+			      -Wunreachable-code*/
       if(if_index == AVAHI_IF_UNSPEC){
-	fprintf_plus(stderr, "An IPv6 link-local address is"
-		     " incomplete without a network interface\n");
+	fprintf(stderr, "An IPv6 link-local address is incomplete"
+		" without a network interface\n");
 	errno = EINVAL;
 	goto mandos_end;
       }
       /* Set the network interface number as scope */
-      ((struct sockaddr_in6 *)&to)->sin6_scope_id = (uint32_t)if_index;
+      to.in6.sin6_scope_id = (uint32_t)if_index;
     }
   } else {
-    ((struct sockaddr_in *)&to)->sin_port = htons(port);
+    to.in.sin_port = htons(port); /* Spurious warnings from
+				     -Wconversion and
+				     -Wunreachable-code */
   }
   
   if(quit_now){
@@ -1339,95 +709,54 @@
       if(if_indextoname((unsigned int)if_index, interface) == NULL){
 	perror_plus("if_indextoname");
       } else {
-	fprintf_plus(stderr, "Connection to: %s%%%s, port %" PRIuMAX
-		     "\n", ip, interface, (uintmax_t)port);
+	fprintf(stderr, "Connection to: %s%%%s, port %" PRIu16 "\n",
+		ip, interface, port);
       }
     } else {
-      fprintf_plus(stderr, "Connection to: %s, port %" PRIuMAX "\n",
-		   ip, (uintmax_t)port);
+      fprintf(stderr, "Connection to: %s, port %" PRIu16 "\n", ip,
+	      port);
     }
     char addrstr[(INET_ADDRSTRLEN > INET6_ADDRSTRLEN) ?
 		 INET_ADDRSTRLEN : INET6_ADDRSTRLEN] = "";
-    if(af == AF_INET6){
-      ret = getnameinfo((struct sockaddr *)&to,
-			sizeof(struct sockaddr_in6),
-			addrstr, sizeof(addrstr), NULL, 0,
-			NI_NUMERICHOST);
-    } else {
-      ret = getnameinfo((struct sockaddr *)&to,
-			sizeof(struct sockaddr_in),
-			addrstr, sizeof(addrstr), NULL, 0,
-			NI_NUMERICHOST);
-    }
-    if(ret == EAI_SYSTEM){
-      perror_plus("getnameinfo");
-    } else if(ret != 0) {
-      fprintf_plus(stderr, "getnameinfo: %s", gai_strerror(ret));
-    } else if(strcmp(addrstr, ip) != 0){
-      fprintf_plus(stderr, "Canonical address form: %s\n", addrstr);
-    }
-  }
-  
-  if(quit_now){
-    errno = EINTR;
-    goto mandos_end;
-  }
-  
-  while(true){
-    if(af == AF_INET6){
-      ret = connect(tcp_sd, (struct sockaddr *)&to,
-		    sizeof(struct sockaddr_in6));
-    } else {
-      ret = connect(tcp_sd, (struct sockaddr *)&to, /* IPv4 */
-		    sizeof(struct sockaddr_in));
-    }
-    if(ret < 0){
-      if(((errno == ENETUNREACH) or (errno == EHOSTUNREACH))
-	 and if_index != AVAHI_IF_UNSPEC
-	 and connect_to == NULL
-	 and not route_added and
-	 ((af == AF_INET6 and not
-	   IN6_IS_ADDR_LINKLOCAL(&(((struct sockaddr_in6 *)
-				    &to)->sin6_addr)))
-	  or (af == AF_INET and
-	      /* Not a a IPv4LL address */
-	      (ntohl(((struct sockaddr_in *)&to)->sin_addr.s_addr)
-	       & 0xFFFF0000L) != 0xA9FE0000L))){
-	/* Work around Avahi bug - Avahi does not announce link-local
-	   addresses if it has a global address, so local hosts with
-	   *only* a link-local address (e.g. Mandos clients) cannot
-	   connect to a Mandos server announced by Avahi on a server
-	   host with a global address.  Work around this by retrying
-	   with an explicit route added with the server's address.
-	   
-	   Avahi bug reference:
-	   https://lists.freedesktop.org/archives/avahi/2010-February/001833.html
-	   https://bugs.debian.org/587961
-	*/
-	if(debug){
-	  fprintf_plus(stderr, "Mandos server unreachable, trying"
-		       " direct route\n");
-	}
-	int e = errno;
-	route_added = add_local_route(ip, if_index);
-	if(route_added){
-	  continue;
-	}
-	errno = e;
-      }
-      if(errno != ECONNREFUSED or debug){
-	int e = errno;
-	perror_plus("connect");
-	errno = e;
-      }
-      goto mandos_end;
-    }
-    
-    if(quit_now){
-      errno = EINTR;
-      goto mandos_end;
-    }
-    break;
+    const char *pcret;
+    if(af == AF_INET6){
+      pcret = inet_ntop(af, &(to.in6.sin6_addr), addrstr,
+			sizeof(addrstr));
+    } else {
+      pcret = inet_ntop(af, &(to.in.sin_addr), addrstr,
+			sizeof(addrstr));
+    }
+    if(pcret == NULL){
+      perror_plus("inet_ntop");
+    } else {
+      if(strcmp(addrstr, ip) != 0){
+	fprintf(stderr, "Canonical address form: %s\n", addrstr);
+      }
+    }
+  }
+  
+  if(quit_now){
+    errno = EINTR;
+    goto mandos_end;
+  }
+  
+  if(af == AF_INET6){
+    ret = connect(tcp_sd, &to.in6, sizeof(to));
+  } else {
+    ret = connect(tcp_sd, &to.in, sizeof(to)); /* IPv4 */
+  }
+  if(ret < 0){
+    if ((errno != ECONNREFUSED and errno != ENETUNREACH) or debug){
+      int e = errno;
+      perror_plus("connect");
+      errno = e;
+    }
+    goto mandos_end;
+  }
+  
+  if(quit_now){
+    errno = EINTR;
+    goto mandos_end;
   }
   
   const char *out = mandos_protocol_version;
@@ -1435,7 +764,7 @@
   while(true){
     size_t out_size = strlen(out);
     ret = (int)TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
-					out_size - written));
+				   out_size - written));
     if(ret == -1){
       int e = errno;
       perror_plus("write");
@@ -1461,7 +790,7 @@
   }
   
   if(debug){
-    fprintf_plus(stderr, "Establishing TLS session with %s\n", ip);
+    fprintf(stderr, "Establishing TLS session with %s\n", ip);
   }
   
   if(quit_now){
@@ -1469,11 +798,8 @@
     goto mandos_end;
   }
   
-  /* This casting via intptr_t is to eliminate warning about casting
-     an int to a pointer type.  This is exactly how the GnuTLS Guile
-     function "set-session-transport-fd!" does it. */
-  gnutls_transport_set_ptr(session,
-			   (gnutls_transport_ptr_t)(intptr_t)tcp_sd);
+  /* Spurious warning from -Wint-to-pointer-cast */
+  gnutls_transport_set_ptr(session, (gnutls_transport_ptr_t) tcp_sd);
   
   if(quit_now){
     errno = EINTR;
@@ -1490,7 +816,7 @@
   
   if(ret != GNUTLS_E_SUCCESS){
     if(debug){
-      fprintf_plus(stderr, "*** GnuTLS Handshake failed ***\n");
+      fprintf(stderr, "*** GnuTLS Handshake failed ***\n");
       gnutls_perror(ret);
     }
     errno = EPROTO;
@@ -1500,8 +826,8 @@
   /* Read OpenPGP packet that contains the wanted password */
   
   if(debug){
-    fprintf_plus(stderr, "Retrieving OpenPGP encrypted password from"
-		 " %s\n", ip);
+    fprintf(stderr, "Retrieving OpenPGP encrypted password from %s\n",
+	    ip);
   }
   
   while(true){
@@ -1512,7 +838,7 @@
     }
     
     buffer_capacity = incbuffer(&buffer, buffer_length,
-				buffer_capacity);
+				   buffer_capacity);
     if(buffer_capacity == 0){
       int e = errno;
       perror_plus("incbuffer");
@@ -1545,16 +871,15 @@
 	  }
 	} while(ret == GNUTLS_E_AGAIN or ret == GNUTLS_E_INTERRUPTED);
 	if(ret < 0){
-	  fprintf_plus(stderr, "*** GnuTLS Re-handshake failed "
-		       "***\n");
+	  fprintf(stderr, "*** GnuTLS Re-handshake failed ***\n");
 	  gnutls_perror(ret);
 	  errno = EPROTO;
 	  goto mandos_end;
 	}
 	break;
       default:
-	fprintf_plus(stderr, "Unknown error while reading data from"
-		     " encrypted session with Mandos server\n");
+	fprintf(stderr, "Unknown error while reading data from"
+		" encrypted session with Mandos server\n");
 	gnutls_bye(session, GNUTLS_SHUT_RDWR);
 	errno = EIO;
 	goto mandos_end;
@@ -1565,7 +890,7 @@
   }
   
   if(debug){
-    fprintf_plus(stderr, "Closing TLS session\n");
+    fprintf(stderr, "Closing TLS session\n");
   }
   
   if(quit_now){
@@ -1583,11 +908,11 @@
   
   if(buffer_length > 0){
     ssize_t decrypted_buffer_size;
-    decrypted_buffer_size = pgp_packet_decrypt(buffer, buffer_length,
-					       &decrypted_buffer, mc);
+    decrypted_buffer_size = pgp_packet_decrypt(buffer,
+					       buffer_length,
+					       &decrypted_buffer);
     if(decrypted_buffer_size >= 0){
       
-      clearerr(stdout);
       written = 0;
       while(written < (size_t) decrypted_buffer_size){
 	if(quit_now){
@@ -1601,24 +926,14 @@
 	if(ret == 0 and ferror(stdout)){
 	  int e = errno;
 	  if(debug){
-	    fprintf_plus(stderr, "Error writing encrypted data: %s\n",
-			 strerror(errno));
+	    fprintf(stderr, "Error writing encrypted data: %s\n",
+		    strerror(errno));
 	  }
 	  errno = e;
 	  goto mandos_end;
 	}
 	written += (size_t)ret;
       }
-      ret = fflush(stdout);
-      if(ret != 0){
-	int e = errno;
-	if(debug){
-	  fprintf_plus(stderr, "Error writing encrypted data: %s\n",
-		       strerror(errno));
-	}
-	errno = e;
-	goto mandos_end;
-      }
       retval = 0;
     }
   }
@@ -1627,17 +942,11 @@
   
  mandos_end:
   {
-    if(route_added){
-      if(not delete_local_route(ip, if_index)){
-	fprintf_plus(stderr, "Failed to delete local route to %s on"
-		     " interface %d", ip, if_index);
-      }
-    }
     int e = errno;
     free(decrypted_buffer);
     free(buffer);
     if(tcp_sd >= 0){
-      ret = close(tcp_sd);
+      ret = (int)TEMP_FAILURE_RETRY(close(tcp_sd));
     }
     if(ret == -1){
       if(e == 0){
@@ -1668,27 +977,22 @@
 			     AVAHI_GCC_UNUSED AvahiStringList *txt,
 			     AVAHI_GCC_UNUSED AvahiLookupResultFlags
 			     flags,
-			     void *mc){
-  if(r == NULL){
-    return;
-  }
+			     AVAHI_GCC_UNUSED void* userdata){
+  assert(r);
   
   /* Called whenever a service has been resolved successfully or
      timed out */
   
   if(quit_now){
-    avahi_s_service_resolver_free(r);
     return;
   }
   
   switch(event){
   default:
   case AVAHI_RESOLVER_FAILURE:
-    fprintf_plus(stderr, "(Avahi Resolver) Failed to resolve service "
-		 "'%s' of type '%s' in domain '%s': %s\n", name, type,
-		 domain,
-		 avahi_strerror(avahi_server_errno
-				(((mandos_context*)mc)->server)));
+    fprintf(stderr, "(Avahi Resolver) Failed to resolve service '%s'"
+	    " of type '%s' in domain '%s': %s\n", name, type, domain,
+	    avahi_strerror(avahi_server_errno(mc.server)));
     break;
     
   case AVAHI_RESOLVER_FOUND:
@@ -1696,23 +1000,17 @@
       char ip[AVAHI_ADDRESS_STR_MAX];
       avahi_address_snprint(ip, sizeof(ip), address);
       if(debug){
-	fprintf_plus(stderr, "Mandos server \"%s\" found on %s (%s, %"
-		     PRIdMAX ") on port %" PRIu16 "\n", name,
-		     host_name, ip, (intmax_t)interface, port);
+	fprintf(stderr, "Mandos server \"%s\" found on %s (%s, %"
+		PRIdMAX ") on port %" PRIu16 "\n", name, host_name,
+		ip, (intmax_t)interface, port);
       }
-      int ret = start_mandos_communication(ip, (in_port_t)port,
-					   interface,
-					   avahi_proto_to_af(proto),
-					   mc);
+      int ret = start_mandos_communication(ip, port, interface,
+					   avahi_proto_to_af(proto));
       if(ret == 0){
-	avahi_simple_poll_quit(simple_poll);
+	avahi_simple_poll_quit(mc.simple_poll);
       } else {
-	if(not add_server(ip, (in_port_t)port, interface,
-			  avahi_proto_to_af(proto),
-			  &((mandos_context*)mc)->current_server)){
-	  fprintf_plus(stderr, "Failed to add server \"%s\" to server"
-		       " list\n", name);
-	}
+	ret = add_server(ip, port, interface,
+			 avahi_proto_to_af(proto));
       }
     }
   }
@@ -1728,10 +1026,8 @@
 			    const char *domain,
 			    AVAHI_GCC_UNUSED AvahiLookupResultFlags
 			    flags,
-			    void *mc){
-  if(b == NULL){
-    return;
-  }
+			    AVAHI_GCC_UNUSED void* userdata){
+  assert(b);
   
   /* Called whenever a new services becomes available on the LAN or
      is removed from the LAN */
@@ -1744,10 +1040,9 @@
   default:
   case AVAHI_BROWSER_FAILURE:
     
-    fprintf_plus(stderr, "(Avahi browser) %s\n",
-		 avahi_strerror(avahi_server_errno
-				(((mandos_context*)mc)->server)));
-    avahi_simple_poll_quit(simple_poll);
+    fprintf(stderr, "(Avahi browser) %s\n",
+	    avahi_strerror(avahi_server_errno(mc.server)));
+    avahi_simple_poll_quit(mc.simple_poll);
     return;
     
   case AVAHI_BROWSER_NEW:
@@ -1756,14 +1051,11 @@
        the callback function is called the Avahi server will free the
        resolver for us. */
     
-    if(avahi_s_service_resolver_new(((mandos_context*)mc)->server,
-				    interface, protocol, name, type,
-				    domain, protocol, 0,
-				    resolve_callback, mc) == NULL)
-      fprintf_plus(stderr, "Avahi: Failed to resolve service '%s':"
-		   " %s\n", name,
-		   avahi_strerror(avahi_server_errno
-				  (((mandos_context*)mc)->server)));
+    if(avahi_s_service_resolver_new(mc.server, interface, protocol,
+				    name, type, domain, protocol, 0,
+				    resolve_callback, NULL) == NULL)
+      fprintf(stderr, "Avahi: Failed to resolve service '%s': %s\n",
+	      name, avahi_strerror(avahi_server_errno(mc.server)));
     break;
     
   case AVAHI_BROWSER_REMOVE:
@@ -1772,8 +1064,7 @@
   case AVAHI_BROWSER_ALL_FOR_NOW:
   case AVAHI_BROWSER_CACHE_EXHAUSTED:
     if(debug){
-      fprintf_plus(stderr, "No Mandos server found, still"
-		   " searching...\n");
+      fprintf(stderr, "No Mandos server found, still searching...\n");
     }
     break;
   }
@@ -1788,152 +1079,117 @@
   signal_received = sig;
   int old_errno = errno;
   /* set main loop to exit */
-  if(simple_poll != NULL){
-    avahi_simple_poll_quit(simple_poll);
+  if(mc.simple_poll != NULL){
+    avahi_simple_poll_quit(mc.simple_poll);
   }
   errno = old_errno;
 }
 
-__attribute__((nonnull, warn_unused_result))
-bool get_flags(const char *ifname, struct ifreq *ifr){
-  int ret;
-  int old_errno;
-  
-  int s = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
-  if(s < 0){
-    old_errno = errno;
-    perror_plus("socket");
-    errno = old_errno;
-    return false;
-  }
-  strncpy(ifr->ifr_name, ifname, IF_NAMESIZE);
-  ifr->ifr_name[IF_NAMESIZE-1] = '\0'; /* NUL terminate */
-  ret = ioctl(s, SIOCGIFFLAGS, ifr);
-  if(ret == -1){
+/* 
+ * This function determines if a directory entry in /sys/class/net
+ * corresponds to an acceptable network device.
+ * (This function is passed to scandir(3) as a filter function.)
+ */
+int good_interface(const struct dirent *if_entry){
+  ssize_t ssret;
+  char *flagname = NULL;
+  if(if_entry->d_name[0] == '.'){
+    return 0;
+  }
+  int ret = asprintf(&flagname, "%s/%s/flags", sys_class_net,
+		     if_entry->d_name);
+  if(ret < 0){
+    perror_plus("asprintf");
+    return 0;
+  }
+  int flags_fd = (int)TEMP_FAILURE_RETRY(open(flagname, O_RDONLY));
+  if(flags_fd == -1){
+    perror_plus("open");
+    free(flagname);
+    return 0;
+  }
+  free(flagname);
+  typedef short ifreq_flags;	/* ifreq.ifr_flags in netdevice(7) */
+  /* read line from flags_fd */
+  ssize_t to_read = 2+(sizeof(ifreq_flags)*2)+1; /* "0x1003\n" */
+  char *flagstring = malloc((size_t)to_read+1); /* +1 for final \0 */
+  flagstring[(size_t)to_read] = '\0';
+  if(flagstring == NULL){
+    perror_plus("malloc");
+    close(flags_fd);
+    return 0;
+  }
+  while(to_read > 0){
+    ssret = (ssize_t)TEMP_FAILURE_RETRY(read(flags_fd, flagstring,
+					     (size_t)to_read));
+    if(ssret == -1){
+      perror_plus("read");
+      free(flagstring);
+      close(flags_fd);
+      return 0;
+    }
+    to_read -= ssret;
+    if(ssret == 0){
+      break;
+    }
+  }
+  close(flags_fd);
+  intmax_t tmpmax;
+  char *tmp;
+  errno = 0;
+  tmpmax = strtoimax(flagstring, &tmp, 0);
+  if(errno != 0 or tmp == flagstring or (*tmp != '\0'
+					 and not (isspace(*tmp)))
+     or tmpmax != (ifreq_flags)tmpmax){
     if(debug){
-      old_errno = errno;
-      perror_plus("ioctl SIOCGIFFLAGS");
-      errno = old_errno;
-    }
-    if((close(s) == -1) and debug){
-      old_errno = errno;
-      perror_plus("close");
-      errno = old_errno;
-    }
-    return false;
-  }
-  if((close(s) == -1) and debug){
-    old_errno = errno;
-    perror_plus("close");
-    errno = old_errno;
-  }
-  return true;
-}
-
-__attribute__((nonnull, warn_unused_result))
-bool good_flags(const char *ifname, const struct ifreq *ifr){
-  
+      fprintf(stderr, "Invalid flags \"%s\" for interface \"%s\"\n",
+	      flagstring, if_entry->d_name);
+    }
+    free(flagstring);
+    return 0;
+  }
+  free(flagstring);
+  ifreq_flags flags = (ifreq_flags)tmpmax;
   /* Reject the loopback device */
-  if(ifr->ifr_flags & IFF_LOOPBACK){
+  if(flags & IFF_LOOPBACK){
     if(debug){
-      fprintf_plus(stderr, "Rejecting loopback interface \"%s\"\n",
-		   ifname);
+      fprintf(stderr, "Rejecting loopback interface \"%s\"\n",
+	      if_entry->d_name);
     }
-    return false;
+    return 0;
   }
   /* Accept point-to-point devices only if connect_to is specified */
-  if(connect_to != NULL and (ifr->ifr_flags & IFF_POINTOPOINT)){
+  if(connect_to != NULL and (flags & IFF_POINTOPOINT)){
     if(debug){
-      fprintf_plus(stderr, "Accepting point-to-point interface"
-		   " \"%s\"\n", ifname);
+      fprintf(stderr, "Accepting point-to-point interface \"%s\"\n",
+	      if_entry->d_name);
     }
-    return true;
+    return 1;
   }
   /* Otherwise, reject non-broadcast-capable devices */
-  if(not (ifr->ifr_flags & IFF_BROADCAST)){
+  if(not (flags & IFF_BROADCAST)){
     if(debug){
-      fprintf_plus(stderr, "Rejecting non-broadcast interface"
-		   " \"%s\"\n", ifname);
+      fprintf(stderr, "Rejecting non-broadcast interface \"%s\"\n",
+	      if_entry->d_name);
     }
-    return false;
+    return 0;
   }
   /* Reject non-ARP interfaces (including dummy interfaces) */
-  if(ifr->ifr_flags & IFF_NOARP){
+  if(flags & IFF_NOARP){
     if(debug){
-      fprintf_plus(stderr, "Rejecting non-ARP interface \"%s\"\n",
-		   ifname);
+      fprintf(stderr, "Rejecting non-ARP interface \"%s\"\n",
+	      if_entry->d_name);
     }
-    return false;
+    return 0;
   }
-  
   /* Accept this device */
   if(debug){
-    fprintf_plus(stderr, "Interface \"%s\" is good\n", ifname);
-  }
-  return true;
-}
-
-/* 
- * This function determines if a directory entry in /sys/class/net
- * corresponds to an acceptable network device.
- * (This function is passed to scandir(3) as a filter function.)
- */
-__attribute__((nonnull, warn_unused_result))
-int good_interface(const struct dirent *if_entry){
-  if(if_entry->d_name[0] == '.'){
-    return 0;
-  }
-  
-  struct ifreq ifr;
-  if(not get_flags(if_entry->d_name, &ifr)){
-    if(debug){
-      fprintf_plus(stderr, "Failed to get flags for interface "
-		   "\"%s\"\n", if_entry->d_name);
-    }
-    return 0;
-  }
-  
-  if(not good_flags(if_entry->d_name, &ifr)){
-    return 0;
+    fprintf(stderr, "Interface \"%s\" is acceptable\n",
+	    if_entry->d_name);
   }
   return 1;
 }
 
-/* 
- * This function determines if a network interface is up.
- */
-__attribute__((nonnull, warn_unused_result))
-bool interface_is_up(const char *interface){
-  struct ifreq ifr;
-  if(not get_flags(interface, &ifr)){
-    if(debug){
-      fprintf_plus(stderr, "Failed to get flags for interface "
-		   "\"%s\"\n", interface);
-    }
-    return false;
-  }
-  
-  return (bool)(ifr.ifr_flags & IFF_UP);
-}
-
-/* 
- * This function determines if a network interface is running
- */
-__attribute__((nonnull, warn_unused_result))
-bool interface_is_running(const char *interface){
-  struct ifreq ifr;
-  if(not get_flags(interface, &ifr)){
-    if(debug){
-      fprintf_plus(stderr, "Failed to get flags for interface "
-		   "\"%s\"\n", interface);
-    }
-    return false;
-  }
-  
-  return (bool)(ifr.ifr_flags & IFF_RUNNING);
-}
-
-__attribute__((nonnull, pure, warn_unused_result))
 int notdotentries(const struct dirent *direntry){
   /* Skip "." and ".." */
   if(direntry->d_name[0] == '.'
@@ -1945,80 +1201,23 @@
   return 1;
 }
 
-/* Is this directory entry a runnable program? */
-__attribute__((nonnull, warn_unused_result))
-int runnable_hook(const struct dirent *direntry){
-  int ret;
-  size_t sret;
-  struct stat st;
-  
-  if((direntry->d_name)[0] == '\0'){
-    /* Empty name? */
-    return 0;
-  }
-  
-  sret = strspn(direntry->d_name, "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-		"abcdefghijklmnopqrstuvwxyz"
-		"0123456789"
-		"_.-");
-  if((direntry->d_name)[sret] != '\0'){
-    /* Contains non-allowed characters */
-    if(debug){
-      fprintf_plus(stderr, "Ignoring hook \"%s\" with bad name\n",
-		   direntry->d_name);
-    }
-    return 0;
-  }
-  
-  ret = fstatat(hookdir_fd, direntry->d_name, &st, 0);
-  if(ret == -1){
-    if(debug){
-      perror_plus("Could not stat hook");
-    }
-    return 0;
-  }
-  if(not (S_ISREG(st.st_mode))){
-    /* Not a regular file */
-    if(debug){
-      fprintf_plus(stderr, "Ignoring hook \"%s\" - not a file\n",
-		   direntry->d_name);
-    }
-    return 0;
-  }
-  if(not (st.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))){
-    /* Not executable */
-    if(debug){
-      fprintf_plus(stderr, "Ignoring hook \"%s\" - not executable\n",
-		   direntry->d_name);
-    }
-    return 0;
-  }
-  if(debug){
-    fprintf_plus(stderr, "Hook \"%s\" is acceptable\n",
-		 direntry->d_name);
-  }
-  return 1;
-}
-
-__attribute__((nonnull, warn_unused_result))
-int avahi_loop_with_timeout(AvahiSimplePoll *s, int retry_interval,
-			    mandos_context *mc){
+int avahi_loop_with_timeout(AvahiSimplePoll *s, int retry_interval){
   int ret;
   struct timespec now;
   struct timespec waited_time;
   intmax_t block_time;
   
   while(true){
-    if(mc->current_server == NULL){
-      if(debug){
-	fprintf_plus(stderr, "Wait until first server is found."
-		     " No timeout!\n");
+    if(mc.current_server == NULL){
+      if (debug){
+	fprintf(stderr,
+		"Wait until first server is found. No timeout!\n");
       }
       ret = avahi_simple_poll_iterate(s, -1);
     } else {
-      if(debug){
-	fprintf_plus(stderr, "Check current_server if we should run"
-		     " it, or wait\n");
+      if (debug){
+	fprintf(stderr, "Check current_server if we should run it,"
+		" or wait\n");
       }
       /* the current time */
       ret = clock_gettime(CLOCK_MONOTONIC, &now);
@@ -2029,9 +1228,9 @@
       /* Calculating in ms how long time between now and server
 	 who we visted longest time ago. Now - last seen.  */
       waited_time.tv_sec = (now.tv_sec
-			    - mc->current_server->last_seen.tv_sec);
+			    - mc.current_server->last_seen.tv_sec);
       waited_time.tv_nsec = (now.tv_nsec
-			     - mc->current_server->last_seen.tv_nsec);
+			     - mc.current_server->last_seen.tv_nsec);
       /* total time is 10s/10,000ms.
 	 Converting to s from ms by dividing by 1,000,
 	 and ns to ms by dividing by 1,000,000. */
@@ -2039,27 +1238,26 @@
 		     - ((intmax_t)waited_time.tv_sec * 1000))
 		    - ((intmax_t)waited_time.tv_nsec / 1000000));
       
-      if(debug){
-	fprintf_plus(stderr, "Blocking for %" PRIdMAX " ms\n",
-		     block_time);
+      if (debug){
+	fprintf(stderr, "Blocking for %" PRIdMAX " ms\n", block_time);
       }
       
       if(block_time <= 0){
-	ret = start_mandos_communication(mc->current_server->ip,
-					 mc->current_server->port,
-					 mc->current_server->if_index,
-					 mc->current_server->af, mc);
+	ret = start_mandos_communication(mc.current_server->ip,
+					 mc.current_server->port,
+					 mc.current_server->if_index,
+					 mc.current_server->af);
 	if(ret == 0){
-	  avahi_simple_poll_quit(s);
+	  avahi_simple_poll_quit(mc.simple_poll);
 	  return 0;
 	}
 	ret = clock_gettime(CLOCK_MONOTONIC,
-			    &mc->current_server->last_seen);
+			    &mc.current_server->last_seen);
 	if(ret == -1){
 	  perror_plus("clock_gettime");
 	  return -1;
 	}
-	mc->current_server = mc->current_server->next;
+	mc.current_server = mc.current_server->next;
 	block_time = 0; 	/* Call avahi to find new Mandos
 				   servers, but don't block */
       }
@@ -2067,432 +1265,31 @@
       ret = avahi_simple_poll_iterate(s, (int)block_time);
     }
     if(ret != 0){
-      if(ret > 0 or errno != EINTR){
+      if (ret > 0 or errno != EINTR) {
 	return (ret != 1) ? ret : 0;
       }
     }
   }
 }
 
-__attribute__((nonnull))
-void run_network_hooks(const char *mode, const char *interface,
-		       const float delay){
-  struct dirent **direntries = NULL;
-  if(hookdir_fd == -1){
-    hookdir_fd = open(hookdir, O_RDONLY | O_DIRECTORY | O_PATH
-		      | O_CLOEXEC);
-    if(hookdir_fd == -1){
-      if(errno == ENOENT){
-	if(debug){
-	  fprintf_plus(stderr, "Network hook directory \"%s\" not"
-		       " found\n", hookdir);
-	}
-      } else {
-	perror_plus("open");
-      }
-      return;
-    }
-  }
-  int devnull = (int)TEMP_FAILURE_RETRY(open("/dev/null", O_RDONLY));
-  if(devnull == -1){
-    perror_plus("open(\"/dev/null\", O_RDONLY)");
-    return;
-  }
-  int numhooks = scandirat(hookdir_fd, ".", &direntries,
-			   runnable_hook, alphasort);
-  if(numhooks == -1){
-    perror_plus("scandir");
-    close(devnull);
-    return;
-  }
-  struct dirent *direntry;
-  int ret;
-  for(int i = 0; i < numhooks; i++){
-    direntry = direntries[i];
-    if(debug){
-      fprintf_plus(stderr, "Running network hook \"%s\"\n",
-		   direntry->d_name);
-    }
-    pid_t hook_pid = fork();
-    if(hook_pid == 0){
-      /* Child */
-      /* Raise privileges */
-      errno = raise_privileges_permanently();
-      if(errno != 0){
-	perror_plus("Failed to raise privileges");
-	_exit(EX_NOPERM);
-      }
-      /* Set group */
-      errno = 0;
-      ret = setgid(0);
-      if(ret == -1){
-	perror_plus("setgid");
-	_exit(EX_NOPERM);
-      }
-      /* Reset supplementary groups */
-      errno = 0;
-      ret = setgroups(0, NULL);
-      if(ret == -1){
-	perror_plus("setgroups");
-	_exit(EX_NOPERM);
-      }
-      ret = setenv("MANDOSNETHOOKDIR", hookdir, 1);
-      if(ret == -1){
-	perror_plus("setenv");
-	_exit(EX_OSERR);
-      }
-      ret = setenv("DEVICE", interface, 1);
-      if(ret == -1){
-	perror_plus("setenv");
-	_exit(EX_OSERR);
-      }
-      ret = setenv("VERBOSITY", debug ? "1" : "0", 1);
-      if(ret == -1){
-	perror_plus("setenv");
-	_exit(EX_OSERR);
-      }
-      ret = setenv("MODE", mode, 1);
-      if(ret == -1){
-	perror_plus("setenv");
-	_exit(EX_OSERR);
-      }
-      char *delaystring;
-      ret = asprintf(&delaystring, "%f", (double)delay);
-      if(ret == -1){
-	perror_plus("asprintf");
-	_exit(EX_OSERR);
-      }
-      ret = setenv("DELAY", delaystring, 1);
-      if(ret == -1){
-	free(delaystring);
-	perror_plus("setenv");
-	_exit(EX_OSERR);
-      }
-      free(delaystring);
-      if(connect_to != NULL){
-	ret = setenv("CONNECT", connect_to, 1);
-	if(ret == -1){
-	  perror_plus("setenv");
-	  _exit(EX_OSERR);
-	}
-      }
-      int hook_fd = (int)TEMP_FAILURE_RETRY(openat(hookdir_fd,
-						   direntry->d_name,
-						   O_RDONLY));
-      if(hook_fd == -1){
-	perror_plus("openat");
-	_exit(EXIT_FAILURE);
-      }
-      if(close(hookdir_fd) == -1){
-	perror_plus("close");
-	_exit(EXIT_FAILURE);
-      }
-      ret = dup2(devnull, STDIN_FILENO);
-      if(ret == -1){
-	perror_plus("dup2(devnull, STDIN_FILENO)");
-	_exit(EX_OSERR);
-      }
-      ret = close(devnull);
-      if(ret == -1){
-	perror_plus("close");
-	_exit(EX_OSERR);
-      }
-      ret = dup2(STDERR_FILENO, STDOUT_FILENO);
-      if(ret == -1){
-	perror_plus("dup2(STDERR_FILENO, STDOUT_FILENO)");
-	_exit(EX_OSERR);
-      }
-      if(fexecve(hook_fd, (char *const []){ direntry->d_name, NULL },
-		 environ) == -1){
-	perror_plus("fexecve");
-	_exit(EXIT_FAILURE);
-      }
-    } else {
-      if(hook_pid == -1){
-	perror_plus("fork");
-	free(direntry);
-	continue;
-      }
-      int status;
-      if(TEMP_FAILURE_RETRY(waitpid(hook_pid, &status, 0)) == -1){
-	perror_plus("waitpid");
-	free(direntry);
-	continue;
-      }
-      if(WIFEXITED(status)){
-	if(WEXITSTATUS(status) != 0){
-	  fprintf_plus(stderr, "Warning: network hook \"%s\" exited"
-		       " with status %d\n", direntry->d_name,
-		       WEXITSTATUS(status));
-	  free(direntry);
-	  continue;
-	}
-      } else if(WIFSIGNALED(status)){
-	fprintf_plus(stderr, "Warning: network hook \"%s\" died by"
-		     " signal %d\n", direntry->d_name,
-		     WTERMSIG(status));
-	free(direntry);
-	continue;
-      } else {
-	fprintf_plus(stderr, "Warning: network hook \"%s\""
-		     " crashed\n", direntry->d_name);
-	free(direntry);
-	continue;
-      }
-    }
-    if(debug){
-      fprintf_plus(stderr, "Network hook \"%s\" ran successfully\n",
-		   direntry->d_name);
-    }
-    free(direntry);
-  }
-  free(direntries);
-  if(close(hookdir_fd) == -1){
-    perror_plus("close");
-  } else {
-    hookdir_fd = -1;
-  }
-  close(devnull);
-}
-
-__attribute__((nonnull, warn_unused_result))
-int bring_up_interface(const char *const interface,
-		       const float delay){
-  int old_errno = errno;
-  int ret;
-  struct ifreq network;
-  unsigned int if_index = if_nametoindex(interface);
-  if(if_index == 0){
-    fprintf_plus(stderr, "No such interface: \"%s\"\n", interface);
-    errno = old_errno;
-    return ENXIO;
-  }
-  
-  if(quit_now){
-    errno = old_errno;
-    return EINTR;
-  }
-  
-  if(not interface_is_up(interface)){
-    int ret_errno = 0;
-    int ioctl_errno = 0;
-    if(not get_flags(interface, &network)){
-      ret_errno = errno;
-      fprintf_plus(stderr, "Failed to get flags for interface "
-		   "\"%s\"\n", interface);
-      errno = old_errno;
-      return ret_errno;
-    }
-    network.ifr_flags |= IFF_UP; /* set flag */
-    
-    int sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
-    if(sd == -1){
-      ret_errno = errno;
-      perror_plus("socket");
-      errno = old_errno;
-      return ret_errno;
-    }
-    
-    if(quit_now){
-      ret = close(sd);
-      if(ret == -1){
-	perror_plus("close");
-      }
-      errno = old_errno;
-      return EINTR;
-    }
-    
-    if(debug){
-      fprintf_plus(stderr, "Bringing up interface \"%s\"\n",
-		   interface);
-    }
-    
-    /* Raise privileges */
-    ret_errno = raise_privileges();
-    if(ret_errno != 0){
-      errno = ret_errno;
-      perror_plus("Failed to raise privileges");
-    }
-    
-#ifdef __linux__
-    int ret_linux;
-    bool restore_loglevel = false;
-    if(ret_errno == 0){
-      /* Lower kernel loglevel to KERN_NOTICE to avoid KERN_INFO
-	 messages about the network interface to mess up the prompt */
-      ret_linux = klogctl(8, NULL, 5);
-      if(ret_linux == -1){
-	perror_plus("klogctl");
-      } else {
-	restore_loglevel = true;
-      }
-    }
-#endif	/* __linux__ */
-    int ret_setflags = ioctl(sd, SIOCSIFFLAGS, &network);
-    ioctl_errno = errno;
-#ifdef __linux__
-    if(restore_loglevel){
-      ret_linux = klogctl(7, NULL, 0);
-      if(ret_linux == -1){
-	perror_plus("klogctl");
-      }
-    }
-#endif	/* __linux__ */
-    
-    /* If raise_privileges() succeeded above */
-    if(ret_errno == 0){
-      /* Lower privileges */
-      ret_errno = lower_privileges();
-      if(ret_errno != 0){
-	errno = ret_errno;
-	perror_plus("Failed to lower privileges");
-      }
-    }
-    
-    /* Close the socket */
-    ret = close(sd);
-    if(ret == -1){
-      perror_plus("close");
-    }
-    
-    if(ret_setflags == -1){
-      errno = ioctl_errno;
-      perror_plus("ioctl SIOCSIFFLAGS +IFF_UP");
-      errno = old_errno;
-      return ioctl_errno;
-    }
-  } else if(debug){
-    fprintf_plus(stderr, "Interface \"%s\" is already up; good\n",
-		 interface);
-  }
-  
-  /* Sleep checking until interface is running.
-     Check every 0.25s, up to total time of delay */
-  for(int i = 0; i < delay * 4; i++){
-    if(interface_is_running(interface)){
-      break;
-    }
-    struct timespec sleeptime = { .tv_nsec = 250000000 };
-    ret = nanosleep(&sleeptime, NULL);
-    if(ret == -1 and errno != EINTR){
-      perror_plus("nanosleep");
-    }
-  }
-  
-  errno = old_errno;
-  return 0;
-}
-
-__attribute__((nonnull, warn_unused_result))
-int take_down_interface(const char *const interface){
-  int old_errno = errno;
-  struct ifreq network;
-  unsigned int if_index = if_nametoindex(interface);
-  if(if_index == 0){
-    fprintf_plus(stderr, "No such interface: \"%s\"\n", interface);
-    errno = old_errno;
-    return ENXIO;
-  }
-  if(interface_is_up(interface)){
-    int ret_errno = 0;
-    int ioctl_errno = 0;
-    if(not get_flags(interface, &network) and debug){
-      ret_errno = errno;
-      fprintf_plus(stderr, "Failed to get flags for interface "
-		   "\"%s\"\n", interface);
-      errno = old_errno;
-      return ret_errno;
-    }
-    network.ifr_flags &= ~(short)IFF_UP; /* clear flag */
-    
-    int sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
-    if(sd == -1){
-      ret_errno = errno;
-      perror_plus("socket");
-      errno = old_errno;
-      return ret_errno;
-    }
-    
-    if(debug){
-      fprintf_plus(stderr, "Taking down interface \"%s\"\n",
-		   interface);
-    }
-    
-    /* Raise privileges */
-    ret_errno = raise_privileges();
-    if(ret_errno != 0){
-      errno = ret_errno;
-      perror_plus("Failed to raise privileges");
-    }
-    
-    int ret_setflags = ioctl(sd, SIOCSIFFLAGS, &network);
-    ioctl_errno = errno;
-    
-    /* If raise_privileges() succeeded above */
-    if(ret_errno == 0){
-      /* Lower privileges */
-      ret_errno = lower_privileges();
-      if(ret_errno != 0){
-	errno = ret_errno;
-	perror_plus("Failed to lower privileges");
-      }
-    }
-    
-    /* Close the socket */
-    int ret = close(sd);
-    if(ret == -1){
-      perror_plus("close");
-    }
-    
-    if(ret_setflags == -1){
-      errno = ioctl_errno;
-      perror_plus("ioctl SIOCSIFFLAGS -IFF_UP");
-      errno = old_errno;
-      return ioctl_errno;
-    }
-  } else if(debug){
-    fprintf_plus(stderr, "Interface \"%s\" is already down; odd\n",
-		 interface);
-  }
-  
-  errno = old_errno;
-  return 0;
-}
-
 int main(int argc, char *argv[]){
-  mandos_context mc = { .server = NULL, .dh_bits = 0,
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-			.priority = "SECURE128:!CTYPE-X.509"
-			":+CTYPE-RAWPK:!RSA:!VERS-ALL:+VERS-TLS1.3"
-			":%PROFILE_ULTRA",
-#elif GNUTLS_VERSION_NUMBER < 0x030600
-			.priority = "SECURE256:!CTYPE-X.509"
-			":+CTYPE-OPENPGP:!RSA:+SIGN-DSA-SHA256",
-#else
-#error "Needs GnuTLS 3.6.6 or later, or before 3.6.0"
-#endif
-			.current_server = NULL, .interfaces = NULL,
-			.interfaces_size = 0 };
   AvahiSServiceBrowser *sb = NULL;
-  error_t ret_errno;
+  int error;
   int ret;
   intmax_t tmpmax;
   char *tmp;
   int exitcode = EXIT_SUCCESS;
-  char *interfaces_to_take_down = NULL;
-  size_t interfaces_to_take_down_size = 0;
-  char run_tempdir[] = "/run/tmp/mandosXXXXXX";
-  char old_tempdir[] = "/tmp/mandosXXXXXX";
-  char *tempdir = NULL;
+  const char *interface = "";
+  struct ifreq network;
+  int sd = -1;
+  bool take_down_interface = false;
+  uid_t uid;
+  gid_t gid;
+  char tempdir[] = "/tmp/mandosXXXXXX";
+  bool tempdir_created = false;
   AvahiIfIndex if_index = AVAHI_IF_UNSPEC;
   const char *seckey = PATHDIR "/" SECKEY;
   const char *pubkey = PATHDIR "/" PUBKEY;
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-  const char *tls_privkey = PATHDIR "/" TLS_PRIVKEY;
-  const char *tls_pubkey = PATHDIR "/" TLS_PUBKEY;
-#endif
-  const char *dh_params_file = NULL;
-  char *interfaces_hooks = NULL;
   
   bool gnutls_initialized = false;
   bool gpgme_initialized = false;
@@ -2544,33 +1341,12 @@
       { .name = "pubkey", .key = 'p',
 	.arg = "FILE",
 	.doc = "OpenPGP public key file base name",
-	.group = 1 },
-      { .name = "tls-privkey", .key = 't',
-	.arg = "FILE",
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-	.doc = "TLS private key file base name",
-#else
-	.doc = "Dummy; ignored (requires GnuTLS 3.6.6)",
-#endif
-	.group = 1 },
-      { .name = "tls-pubkey", .key = 'T',
-	.arg = "FILE",
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-	.doc = "TLS public key file base name",
-#else
-	.doc = "Dummy; ignored (requires GnuTLS 3.6.6)",
-#endif
-	.group = 1 },
+	.group = 2 },
       { .name = "dh-bits", .key = 129,
 	.arg = "BITS",
 	.doc = "Bit length of the prime number used in the"
 	" Diffie-Hellman key exchange",
 	.group = 2 },
-      { .name = "dh-params", .key = 134,
-	.arg = "FILE",
-	.doc = "PEM-encoded PKCS#3 file with pre-generated parameters"
-	" for the Diffie-Hellman key exchange",
-	.group = 2 },
       { .name = "priority", .key = 130,
 	.arg = "STRING",
 	.doc = "GnuTLS priority string for the TLS handshake",
@@ -2581,11 +1357,7 @@
 	.group = 2 },
       { .name = "retry", .key = 132,
 	.arg = "SECONDS",
-	.doc = "Retry interval used when denied by the Mandos server",
-	.group = 2 },
-      { .name = "network-hook-dir", .key = 133,
-	.arg = "DIR",
-	.doc = "Directory where network hooks are located",
+	.doc = "Retry interval used when denied by the mandos server",
 	.group = 2 },
       /*
        * These reproduce what we would get without ARGP_NO_HELP
@@ -2610,11 +1382,7 @@
 	connect_to = arg;
 	break;
       case 'i':			/* --interface */
-	ret_errno = argz_add_sep(&mc.interfaces, &mc.interfaces_size,
-				 arg, (int)',');
-	if(ret_errno != 0){
-	  argp_error(state, "%s", strerror(ret_errno));
-	}
+	interface = arg;
 	break;
       case 's':			/* --seckey */
 	seckey = arg;
@@ -2622,16 +1390,6 @@
       case 'p':			/* --pubkey */
 	pubkey = arg;
 	break;
-      case 't':			/* --tls-privkey */
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-	tls_privkey = arg;
-#endif
-	break;
-      case 'T':			/* --tls-pubkey */
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-	tls_pubkey = arg;
-#endif
-	break;
       case 129:			/* --dh-bits */
 	errno = 0;
 	tmpmax = strtoimax(arg, &tmp, 10);
@@ -2641,9 +1399,6 @@
 	}
 	mc.dh_bits = (typeof(mc.dh_bits))tmpmax;
 	break;
-      case 134:			/* --dh-params */
-	dh_params_file = arg;
-	break;
       case 130:			/* --priority */
 	mc.priority = arg;
 	break;
@@ -2662,9 +1417,6 @@
 	  argp_error(state, "Bad retry interval");
 	}
 	break;
-      case 133:			/* --network-hook-dir */
-	hookdir = arg;
-	break;
 	/*
 	 * These reproduce what we would get without ARGP_NO_HELP
 	 */
@@ -2672,13 +1424,11 @@
 	argp_state_help(state, state->out_stream,
 			(ARGP_HELP_STD_HELP | ARGP_HELP_EXIT_ERR)
 			& ~(unsigned int)ARGP_HELP_EXIT_OK);
-	__builtin_unreachable();
       case -3:			/* --usage */
 	argp_state_help(state, state->out_stream,
 			ARGP_HELP_USAGE | ARGP_HELP_EXIT_ERR);
-	__builtin_unreachable();
       case 'V':			/* --version */
-	fprintf_plus(state->out_stream, "%s\n", argp_program_version);
+	fprintf(state->out_stream, "%s\n", argp_program_version);
 	exit(argp_err_exit_status);
 	break;
       default:
@@ -2691,14 +1441,14 @@
 			 .args_doc = "",
 			 .doc = "Mandos client -- Get and decrypt"
 			 " passwords from a Mandos server" };
-    ret_errno = argp_parse(&argp, argc, argv,
-			   ARGP_IN_ORDER | ARGP_NO_HELP, 0, NULL);
-    switch(ret_errno){
+    ret = argp_parse(&argp, argc, argv,
+		     ARGP_IN_ORDER | ARGP_NO_HELP, 0, NULL);
+    switch(ret){
     case 0:
       break;
     case ENOMEM:
     default:
-      errno = ret_errno;
+      errno = ret;
       perror_plus("argp_parse");
       exitcode = EX_OSERR;
       goto end;
@@ -2707,136 +1457,103 @@
       goto end;
     }
   }
-  
+    
   {
     /* Work around Debian bug #633582:
-       <https://bugs.debian.org/633582> */
-    
-    /* Re-raise privileges */
-    ret = raise_privileges();
-    if(ret != 0){
-      errno = ret;
-      perror_plus("Failed to raise privileges");
-    } else {
-      struct stat st;
-      
-      if(strcmp(seckey, PATHDIR "/" SECKEY) == 0){
-	int seckey_fd = open(seckey, O_RDONLY);
-	if(seckey_fd == -1){
-	  perror_plus("open");
-	} else {
-	  ret = (int)TEMP_FAILURE_RETRY(fstat(seckey_fd, &st));
-	  if(ret == -1){
-	    perror_plus("fstat");
-	  } else {
-	    if(S_ISREG(st.st_mode)
-	       and st.st_uid == 0 and st.st_gid == 0){
-	      ret = fchown(seckey_fd, uid, gid);
-	      if(ret == -1){
-		perror_plus("fchown");
-	      }
-	    }
-	  }
-	  close(seckey_fd);
-	}
-      }
-      
-      if(strcmp(pubkey, PATHDIR "/" PUBKEY) == 0){
-	int pubkey_fd = open(pubkey, O_RDONLY);
-	if(pubkey_fd == -1){
-	  perror_plus("open");
-	} else {
-	  ret = (int)TEMP_FAILURE_RETRY(fstat(pubkey_fd, &st));
-	  if(ret == -1){
-	    perror_plus("fstat");
-	  } else {
-	    if(S_ISREG(st.st_mode)
-	       and st.st_uid == 0 and st.st_gid == 0){
-	      ret = fchown(pubkey_fd, uid, gid);
-	      if(ret == -1){
-		perror_plus("fchown");
-	      }
-	    }
-	  }
-	  close(pubkey_fd);
-	}
-      }
-      
-      if(dh_params_file != NULL
-	 and strcmp(dh_params_file, PATHDIR "/dhparams.pem" ) == 0){
-	int dhparams_fd = open(dh_params_file, O_RDONLY);
-	if(dhparams_fd == -1){
-	  perror_plus("open");
-	} else {
-	  ret = (int)TEMP_FAILURE_RETRY(fstat(dhparams_fd, &st));
-	  if(ret == -1){
-	    perror_plus("fstat");
-	  } else {
-	    if(S_ISREG(st.st_mode)
-	       and st.st_uid == 0 and st.st_gid == 0){
-	      ret = fchown(dhparams_fd, uid, gid);
-	      if(ret == -1){
-		perror_plus("fchown");
-	      }
-	    }
-	  }
-	  close(dhparams_fd);
-	}
-      }
-      
-      /* Lower privileges */
-      ret = lower_privileges();
-      if(ret != 0){
-	errno = ret;
-	perror_plus("Failed to lower privileges");
-      }
-    }
-  }
-  
-  /* Remove invalid interface names (except "none") */
-  {
-    char *interface = NULL;
-    while((interface = argz_next(mc.interfaces, mc.interfaces_size,
-				 interface))){
-      if(strcmp(interface, "none") != 0
-	 and if_nametoindex(interface) == 0){
-	if(interface[0] != '\0'){
-	  fprintf_plus(stderr, "Not using nonexisting interface"
-		       " \"%s\"\n", interface);
-	}
-	argz_delete(&mc.interfaces, &mc.interfaces_size, interface);
-	interface = NULL;
-      }
-    }
-  }
-  
-  /* Run network hooks */
-  {
-    if(mc.interfaces != NULL){
-      interfaces_hooks = malloc(mc.interfaces_size);
-      if(interfaces_hooks == NULL){
-	perror_plus("malloc");
-	goto end;
-      }
-      memcpy(interfaces_hooks, mc.interfaces, mc.interfaces_size);
-      argz_stringify(interfaces_hooks, mc.interfaces_size, (int)',');
-    }
-    run_network_hooks("start", interfaces_hooks != NULL ?
-		      interfaces_hooks : "", delay);
+       <http://bugs.debian.org/633582> */
+    struct stat st;
+    
+    /* Re-raise priviliges */
+    errno = 0;
+    ret = seteuid(0);
+    if(ret == -1){
+      perror_plus("seteuid");
+    }
+    
+    if(strcmp(seckey, PATHDIR "/" SECKEY) == 0){
+      int seckey_fd = open(seckey, O_RDONLY);
+      if(seckey_fd == -1){
+	perror_plus("open");
+      } else {
+	ret = (int)TEMP_FAILURE_RETRY(fstat(seckey_fd, &st));
+	if(ret == -1){
+	  perror_plus("fstat");
+	} else {
+	  if(S_ISREG(st.st_mode) and st.st_uid == 0 and st.st_gid == 0){
+	    ret = fchown(seckey_fd, uid, gid);
+	    if(ret == -1){
+	      perror_plus("fchown");
+	    }
+	  }
+	}
+	TEMP_FAILURE_RETRY(close(seckey_fd));
+      }
+    }
+    
+    if(strcmp(pubkey, PATHDIR "/" PUBKEY) == 0){
+      int pubkey_fd = open(pubkey, O_RDONLY);
+      if(pubkey_fd == -1){
+	perror_plus("open");
+      } else {
+	ret = (int)TEMP_FAILURE_RETRY(fstat(pubkey_fd, &st));
+	if(ret == -1){
+	  perror_plus("fstat");
+	} else {
+	  if(S_ISREG(st.st_mode) and st.st_uid == 0 and st.st_gid == 0){
+	    ret = fchown(pubkey_fd, uid, gid);
+	    if(ret == -1){
+	      perror_plus("fchown");
+	    }
+	  }
+	}
+	TEMP_FAILURE_RETRY(close(pubkey_fd));
+      }
+    }
+    
+    /* Lower privileges */
+    errno = 0;
+    ret = seteuid(uid);
+    if(ret == -1){
+      perror_plus("seteuid");
+    }
   }
   
   if(not debug){
     avahi_set_log_function(empty_log);
   }
+
+  if(interface[0] == '\0'){
+    struct dirent **direntries;
+    ret = scandir(sys_class_net, &direntries, good_interface,
+		  alphasort);
+    if(ret >= 1){
+      /* Pick the first good interface */
+      interface = strdup(direntries[0]->d_name);
+      if(debug){
+	fprintf(stderr, "Using interface \"%s\"\n", interface);
+      }
+      if(interface == NULL){
+	perror_plus("malloc");
+	free(direntries);
+	exitcode = EXIT_FAILURE;
+	goto end;
+      }
+      free(direntries);
+    } else {
+      free(direntries);
+      fprintf(stderr, "Could not find a network interface\n");
+      exitcode = EXIT_FAILURE;
+      goto end;
+    }
+  }
   
   /* Initialize Avahi early so avahi_simple_poll_quit() can be called
      from the signal handler */
   /* Initialize the pseudo-RNG for Avahi */
   srand((unsigned int) time(NULL));
-  simple_poll = avahi_simple_poll_new();
-  if(simple_poll == NULL){
-    fprintf_plus(stderr,
-		 "Avahi: Failed to create simple poll object.\n");
+  mc.simple_poll = avahi_simple_poll_new();
+  if(mc.simple_poll == NULL){
+    fprintf(stderr, "Avahi: Failed to create simple poll object.\n");
     exitcode = EX_UNAVAILABLE;
     goto end;
   }
@@ -2904,104 +1621,158 @@
     }
   }
   
-  /* If no interfaces were specified, make a list */
-  if(mc.interfaces == NULL){
-    struct dirent **direntries = NULL;
-    /* Look for any good interfaces */
-    ret = scandir(sys_class_net, &direntries, good_interface,
-		  alphasort);
-    if(ret >= 1){
-      /* Add all found interfaces to interfaces list */
-      for(int i = 0; i < ret; ++i){
-	ret_errno = argz_add(&mc.interfaces, &mc.interfaces_size,
-			     direntries[i]->d_name);
-	if(ret_errno != 0){
-	  errno = ret_errno;
-	  perror_plus("argz_add");
-	  free(direntries[i]);
-	  continue;
-	}
-	if(debug){
-	  fprintf_plus(stderr, "Will use interface \"%s\"\n",
-		       direntries[i]->d_name);
-	}
-	free(direntries[i]);
-      }
-      free(direntries);
+  /* If the interface is down, bring it up */
+  if(strcmp(interface, "none") != 0){
+    if_index = (AvahiIfIndex) if_nametoindex(interface);
+    if(if_index == 0){
+      fprintf(stderr, "No such interface: \"%s\"\n", interface);
+      exitcode = EX_UNAVAILABLE;
+      goto end;
+    }
+    
+    if(quit_now){
+      goto end;
+    }
+    
+    /* Re-raise priviliges */
+    errno = 0;
+    ret = seteuid(0);
+    if(ret == -1){
+      perror_plus("seteuid");
+    }
+    
+#ifdef __linux__
+    /* Lower kernel loglevel to KERN_NOTICE to avoid KERN_INFO
+       messages about the network interface to mess up the prompt */
+    ret = klogctl(8, NULL, 5);
+    bool restore_loglevel = true;
+    if(ret == -1){
+      restore_loglevel = false;
+      perror_plus("klogctl");
+    }
+#endif	/* __linux__ */
+    
+    sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
+    if(sd < 0){
+      perror_plus("socket");
+      exitcode = EX_OSERR;
+#ifdef __linux__
+      if(restore_loglevel){
+	ret = klogctl(7, NULL, 0);
+	if(ret == -1){
+	  perror_plus("klogctl");
+	}
+      }
+#endif	/* __linux__ */
+      /* Lower privileges */
+      errno = 0;
+      ret = seteuid(uid);
+      if(ret == -1){
+	perror_plus("seteuid");
+      }
+      goto end;
+    }
+    strcpy(network.ifr_name, interface);
+    ret = ioctl(sd, SIOCGIFFLAGS, &network);
+    if(ret == -1){
+      perror_plus("ioctl SIOCGIFFLAGS");
+#ifdef __linux__
+      if(restore_loglevel){
+	ret = klogctl(7, NULL, 0);
+	if(ret == -1){
+	  perror_plus("klogctl");
+	}
+      }
+#endif	/* __linux__ */
+      exitcode = EX_OSERR;
+      /* Lower privileges */
+      errno = 0;
+      ret = seteuid(uid);
+      if(ret == -1){
+	perror_plus("seteuid");
+      }
+      goto end;
+    }
+    if((network.ifr_flags & IFF_UP) == 0){
+      network.ifr_flags |= IFF_UP;
+      take_down_interface = true;
+      ret = ioctl(sd, SIOCSIFFLAGS, &network);
+      if(ret == -1){
+	take_down_interface = false;
+	perror_plus("ioctl SIOCSIFFLAGS +IFF_UP");
+	exitcode = EX_OSERR;
+#ifdef __linux__
+	if(restore_loglevel){
+	  ret = klogctl(7, NULL, 0);
+	  if(ret == -1){
+	    perror_plus("klogctl");
+	  }
+	}
+#endif	/* __linux__ */
+	/* Lower privileges */
+	errno = 0;
+	ret = seteuid(uid);
+	if(ret == -1){
+	  perror_plus("seteuid");
+	}
+	goto end;
+      }
+    }
+    /* Sleep checking until interface is running.
+       Check every 0.25s, up to total time of delay */
+    for(int i=0; i < delay * 4; i++){
+      ret = ioctl(sd, SIOCGIFFLAGS, &network);
+      if(ret == -1){
+	perror_plus("ioctl SIOCGIFFLAGS");
+      } else if(network.ifr_flags & IFF_RUNNING){
+	break;
+      }
+      struct timespec sleeptime = { .tv_nsec = 250000000 };
+      ret = nanosleep(&sleeptime, NULL);
+      if(ret == -1 and errno != EINTR){
+	perror_plus("nanosleep");
+      }
+    }
+    if(not take_down_interface){
+      /* We won't need the socket anymore */
+      ret = (int)TEMP_FAILURE_RETRY(close(sd));
+      if(ret == -1){
+	perror_plus("close");
+      }
+    }
+#ifdef __linux__
+    if(restore_loglevel){
+      /* Restores kernel loglevel to default */
+      ret = klogctl(7, NULL, 0);
+      if(ret == -1){
+	perror_plus("klogctl");
+      }
+    }
+#endif	/* __linux__ */
+    /* Lower privileges */
+    errno = 0;
+    if(take_down_interface){
+      /* Lower privileges */
+      ret = seteuid(uid);
+      if(ret == -1){
+	perror_plus("seteuid");
+      }
     } else {
-      if(ret == 0){
-	free(direntries);
-      }
-      fprintf_plus(stderr, "Could not find a network interface\n");
-      exitcode = EXIT_FAILURE;
-      goto end;
-    }
-  }
-  
-  /* Bring up interfaces which are down, and remove any "none"s */
-  {
-    char *interface = NULL;
-    while((interface = argz_next(mc.interfaces, mc.interfaces_size,
-				 interface))){
-      /* If interface name is "none", stop bringing up interfaces.
-	 Also remove all instances of "none" from the list */
-      if(strcmp(interface, "none") == 0){
-	argz_delete(&mc.interfaces, &mc.interfaces_size,
-		    interface);
-	interface = NULL;
-	while((interface = argz_next(mc.interfaces,
-				     mc.interfaces_size, interface))){
-	  if(strcmp(interface, "none") == 0){
-	    argz_delete(&mc.interfaces, &mc.interfaces_size,
-			interface);
-	    interface = NULL;
-	  }
-	}
-	break;
-      }
-      bool interface_was_up = interface_is_up(interface);
-      errno = bring_up_interface(interface, delay);
-      if(not interface_was_up){
-	if(errno != 0){
-	  fprintf_plus(stderr, "Failed to bring up interface \"%s\":"
-		       " %s\n", interface, strerror(errno));
-	} else {
-	  errno = argz_add(&interfaces_to_take_down,
-			   &interfaces_to_take_down_size,
-			   interface);
-	  if(errno != 0){
-	    perror_plus("argz_add");
-	  }
-	}
-      }
-    }
-    if(debug and (interfaces_to_take_down == NULL)){
-      fprintf_plus(stderr, "No interfaces were brought up\n");
-    }
-  }
-  
-  /* If we only got one interface, explicitly use only that one */
-  if(argz_count(mc.interfaces, mc.interfaces_size) == 1){
-    if(debug){
-      fprintf_plus(stderr, "Using only interface \"%s\"\n",
-		   mc.interfaces);
-    }
-    if_index = (AvahiIfIndex)if_nametoindex(mc.interfaces);
+      /* Lower privileges permanently */
+      ret = setuid(uid);
+      if(ret == -1){
+	perror_plus("setuid");
+      }
+    }
   }
   
   if(quit_now){
     goto end;
   }
   
-#if GNUTLS_VERSION_NUMBER >= 0x030606
-  ret = init_gnutls_global(tls_pubkey, tls_privkey, dh_params_file, &mc);
-#elif GNUTLS_VERSION_NUMBER < 0x030600
-  ret = init_gnutls_global(pubkey, seckey, dh_params_file, &mc);
-#else
-#error "Needs GnuTLS 3.6.6 or later, or before 3.6.0"
-#endif
+  ret = init_gnutls_global(pubkey, seckey);
   if(ret == -1){
-    fprintf_plus(stderr, "init_gnutls_global failed\n");
+    fprintf(stderr, "init_gnutls_global failed\n");
     exitcode = EX_UNAVAILABLE;
     goto end;
   } else {
@@ -3012,26 +1783,18 @@
     goto end;
   }
   
-  /* Try /run/tmp before /tmp */
-  tempdir = mkdtemp(run_tempdir);
-  if(tempdir == NULL and errno == ENOENT){
-      if(debug){
-	fprintf_plus(stderr, "Tempdir %s did not work, trying %s\n",
-		     run_tempdir, old_tempdir);
-      }
-      tempdir = mkdtemp(old_tempdir);
-  }
-  if(tempdir == NULL){
+  if(mkdtemp(tempdir) == NULL){
     perror_plus("mkdtemp");
     goto end;
   }
+  tempdir_created = true;
   
   if(quit_now){
     goto end;
   }
   
-  if(not init_gpgme(pubkey, seckey, tempdir, &mc)){
-    fprintf_plus(stderr, "init_gpgme failed\n");
+  if(not init_gpgme(pubkey, seckey, tempdir)){
+    fprintf(stderr, "init_gpgme failed\n");
     exitcode = EX_UNAVAILABLE;
     goto end;
   } else {
@@ -3046,9 +1809,8 @@
     /* Connect directly, do not use Zeroconf */
     /* (Mainly meant for debugging) */
     char *address = strrchr(connect_to, ':');
-    
     if(address == NULL){
-      fprintf_plus(stderr, "No colon in address\n");
+      fprintf(stderr, "No colon in address\n");
       exitcode = EX_USAGE;
       goto end;
     }
@@ -3057,21 +1819,21 @@
       goto end;
     }
     
-    in_port_t port;
+    uint16_t port;
     errno = 0;
     tmpmax = strtoimax(address+1, &tmp, 10);
     if(errno != 0 or tmp == address+1 or *tmp != '\0'
-       or tmpmax != (in_port_t)tmpmax){
-      fprintf_plus(stderr, "Bad port number\n");
+       or tmpmax != (uint16_t)tmpmax){
+      fprintf(stderr, "Bad port number\n");
       exitcode = EX_USAGE;
       goto end;
     }
-    
+  
     if(quit_now){
       goto end;
     }
     
-    port = (in_port_t)tmpmax;
+    port = (uint16_t)tmpmax;
     *address = '\0';
     /* Colon in address indicates IPv6 */
     int af;
@@ -3093,22 +1855,21 @@
     }
     
     while(not quit_now){
-      ret = start_mandos_communication(address, port, if_index, af,
-				       &mc);
+      ret = start_mandos_communication(address, port, if_index, af);
       if(quit_now or ret == 0){
 	break;
       }
       if(debug){
-	fprintf_plus(stderr, "Retrying in %d seconds\n",
-		     (int)retry_interval);
+	fprintf(stderr, "Retrying in %d seconds\n",
+		(int)retry_interval);
       }
-      sleep((unsigned int)retry_interval);
+      sleep((int)retry_interval);
     }
     
-    if(not quit_now){
+    if (not quit_now){
       exitcode = EXIT_SUCCESS;
     }
-    
+
     goto end;
   }
   
@@ -3126,8 +1887,9 @@
     config.publish_domain = 0;
     
     /* Allocate a new server */
-    mc.server = avahi_server_new(avahi_simple_poll_get(simple_poll),
-				 &config, NULL, NULL, &ret);
+    mc.server = avahi_server_new(avahi_simple_poll_get
+				 (mc.simple_poll), &config, NULL,
+				 NULL, &error);
     
     /* Free the Avahi configuration data */
     avahi_server_config_free(&config);
@@ -3135,8 +1897,8 @@
   
   /* Check if creating the Avahi server object succeeded */
   if(mc.server == NULL){
-    fprintf_plus(stderr, "Failed to create Avahi server: %s\n",
-		 avahi_strerror(ret));
+    fprintf(stderr, "Failed to create Avahi server: %s\n",
+	    avahi_strerror(error));
     exitcode = EX_UNAVAILABLE;
     goto end;
   }
@@ -3148,11 +1910,10 @@
   /* Create the Avahi service browser */
   sb = avahi_s_service_browser_new(mc.server, if_index,
 				   AVAHI_PROTO_UNSPEC, "_mandos._tcp",
-				   NULL, 0, browse_callback,
-				   (void *)&mc);
+				   NULL, 0, browse_callback, NULL);
   if(sb == NULL){
-    fprintf_plus(stderr, "Failed to create service browser: %s\n",
-		 avahi_strerror(avahi_server_errno(mc.server)));
+    fprintf(stderr, "Failed to create service browser: %s\n",
+	    avahi_strerror(avahi_server_errno(mc.server)));
     exitcode = EX_UNAVAILABLE;
     goto end;
   }
@@ -3164,166 +1925,119 @@
   /* Run the main loop */
   
   if(debug){
-    fprintf_plus(stderr, "Starting Avahi loop search\n");
+    fprintf(stderr, "Starting Avahi loop search\n");
   }
-  
-  ret = avahi_loop_with_timeout(simple_poll,
-				(int)(retry_interval * 1000), &mc);
+
+  ret = avahi_loop_with_timeout(mc.simple_poll,
+				(int)(retry_interval * 1000));
   if(debug){
-    fprintf_plus(stderr, "avahi_loop_with_timeout exited %s\n",
-		 (ret == 0) ? "successfully" : "with error");
+    fprintf(stderr, "avahi_loop_with_timeout exited %s\n",
+	    (ret == 0) ? "successfully" : "with error");
   }
   
  end:
   
   if(debug){
-    if(signal_received){
-      fprintf_plus(stderr, "%s exiting due to signal %d: %s\n",
-		   argv[0], signal_received,
-		   strsignal(signal_received));
-    } else {
-      fprintf_plus(stderr, "%s exiting\n", argv[0]);
-    }
+    fprintf(stderr, "%s exiting\n", argv[0]);
   }
   
   /* Cleanup things */
-  free(mc.interfaces);
-  
   if(sb != NULL)
     avahi_s_service_browser_free(sb);
   
   if(mc.server != NULL)
     avahi_server_free(mc.server);
   
-  if(simple_poll != NULL)
-    avahi_simple_poll_free(simple_poll);
+  if(mc.simple_poll != NULL)
+    avahi_simple_poll_free(mc.simple_poll);
   
   if(gnutls_initialized){
     gnutls_certificate_free_credentials(mc.cred);
+    gnutls_global_deinit();
     gnutls_dh_params_deinit(mc.dh_params);
   }
   
   if(gpgme_initialized){
     gpgme_release(mc.ctx);
   }
-  
+
   /* Cleans up the circular linked list of Mandos servers the client
      has seen */
   if(mc.current_server != NULL){
     mc.current_server->prev->next = NULL;
     while(mc.current_server != NULL){
       server *next = mc.current_server->next;
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-      free((char *)(mc.current_server->ip));
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
       free(mc.current_server);
       mc.current_server = next;
     }
   }
   
-  /* Re-raise privileges */
-  {
-    ret = raise_privileges();
-    if(ret != 0){
-      errno = ret;
-      perror_plus("Failed to raise privileges");
-    } else {
-      
-      /* Run network hooks */
-      run_network_hooks("stop", interfaces_hooks != NULL ?
-			interfaces_hooks : "", delay);
-      
-      /* Take down the network interfaces which were brought up */
-      {
-	char *interface = NULL;
-	while((interface = argz_next(interfaces_to_take_down,
-				     interfaces_to_take_down_size,
-				     interface))){
-	  ret = take_down_interface(interface);
-	  if(ret != 0){
-	    errno = ret;
-	    perror_plus("Failed to take down interface");
-	  }
-	}
-	if(debug and (interfaces_to_take_down == NULL)){
-	  fprintf_plus(stderr, "No interfaces needed to be taken"
-		       " down\n");
-	}
-      }
+  /* Take down the network interface */
+  if(take_down_interface){
+    /* Re-raise priviliges */
+    errno = 0;
+    ret = seteuid(0);
+    if(ret == -1){
+      perror_plus("seteuid");
     }
-    
-    ret = lower_privileges_permanently();
-    if(ret != 0){
-      errno = ret;
-      perror_plus("Failed to lower privileges permanently");
+    if(geteuid() == 0){
+      ret = ioctl(sd, SIOCGIFFLAGS, &network);
+      if(ret == -1){
+	perror_plus("ioctl SIOCGIFFLAGS");
+      } else if(network.ifr_flags & IFF_UP) {
+	network.ifr_flags &= ~(short)IFF_UP; /* clear flag */
+	ret = ioctl(sd, SIOCSIFFLAGS, &network);
+	if(ret == -1){
+	  perror_plus("ioctl SIOCSIFFLAGS -IFF_UP");
+	}
+      }
+      ret = (int)TEMP_FAILURE_RETRY(close(sd));
+      if(ret == -1){
+	perror_plus("close");
+      }
+      /* Lower privileges permanently */
+      errno = 0;
+      ret = setuid(uid);
+      if(ret == -1){
+	perror_plus("setuid");
+      }
     }
   }
   
-  free(interfaces_to_take_down);
-  free(interfaces_hooks);
-  
-  void clean_dir_at(int base, const char * const dirname,
-		    uintmax_t level){
+  /* Removes the GPGME temp directory and all files inside */
+  if(tempdir_created){
     struct dirent **direntries = NULL;
-    int dret;
-    int dir_fd = (int)TEMP_FAILURE_RETRY(openat(base, dirname,
-						O_RDONLY
-						| O_NOFOLLOW
-						| O_DIRECTORY
-						| O_PATH));
-    if(dir_fd == -1){
-      perror_plus("open");
-      return;
-    }
-    int numentries = scandirat(dir_fd, ".", &direntries,
-			       notdotentries, alphasort);
-    if(numentries >= 0){
+    struct dirent *direntry = NULL;
+    int numentries = scandir(tempdir, &direntries, notdotentries,
+			     alphasort);
+    if (numentries > 0){
       for(int i = 0; i < numentries; i++){
-	if(debug){
-	  fprintf_plus(stderr, "Unlinking \"%s/%s\"\n",
-		       dirname, direntries[i]->d_name);
-	}
-	dret = unlinkat(dir_fd, direntries[i]->d_name, 0);
-	if(dret == -1){
-	  if(errno == EISDIR){
-	      dret = unlinkat(dir_fd, direntries[i]->d_name,
-			      AT_REMOVEDIR);
-	  }	    
-	  if((dret == -1) and (errno == ENOTEMPTY)
-	     and (strcmp(direntries[i]->d_name, "private-keys-v1.d")
-		  == 0) and (level == 0)){
-	    /* Recurse only in this special case */
-	    clean_dir_at(dir_fd, direntries[i]->d_name, level+1);
-	    dret = 0;
-	  }
-	  if((dret == -1) and (errno != ENOENT)){
-	    fprintf_plus(stderr, "unlink(\"%s/%s\"): %s\n", dirname,
-			 direntries[i]->d_name, strerror(errno));
-	  }
-	}
-	free(direntries[i]);
-      }
-      
-      /* need to clean even if 0 because man page doesn't specify */
-      free(direntries);
-      dret = unlinkat(base, dirname, AT_REMOVEDIR);
-      if(dret == -1 and errno != ENOENT){
-	perror_plus("rmdir");
-      }
-    } else {
-      perror_plus("scandirat");
-    }
-    close(dir_fd);
-  }
-  
-  /* Removes the GPGME temp directory and all files inside */
-  if(tempdir != NULL){
-    clean_dir_at(-1, tempdir, 0);
+	direntry = direntries[i];
+	char *fullname = NULL;
+	ret = asprintf(&fullname, "%s/%s", tempdir,
+		       direntry->d_name);
+	if(ret < 0){
+	  perror_plus("asprintf");
+	  continue;
+	}
+	ret = remove(fullname);
+	if(ret == -1){
+	  fprintf(stderr, "remove(\"%s\"): %s\n", fullname,
+		  strerror(errno));
+	}
+	free(fullname);
+      }
+    }
+
+    /* need to clean even if 0 because man page doesn't specify */
+    free(direntries);
+    if (numentries == -1){
+      perror_plus("scandir");
+    }
+    ret = rmdir(tempdir);
+    if(ret == -1 and errno != ENOENT){
+      perror_plus("rmdir");
+    }
   }
   
   if(quit_now){

=== modified file 'plugins.d/mandos-client.xml'
--- plugins.d/mandos-client.xml	2019-02-10 04:20:26 +0000
+++ plugins.d/mandos-client.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-client">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,16 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -73,13 +64,11 @@
 	><replaceable>PORT</replaceable></option></arg>
       </group>
       <sbr/>
-      <group rep='repeat'>
+      <group>
 	<arg choice="plain"><option>--interface
-	<replaceable>NAME</replaceable><arg rep='repeat'
-	>,<replaceable>NAME</replaceable></arg></option></arg>
-	<arg choice="plain"><option>-i <replaceable>NAME</replaceable
-	><arg rep='repeat'>,<replaceable>NAME</replaceable></arg
-	></option></arg>
+	<replaceable>NAME</replaceable></option></arg>
+	<arg choice="plain"><option>-i
+	<replaceable>NAME</replaceable></option></arg>
       </group>
       <sbr/>
       <group>
@@ -96,20 +85,6 @@
 	<replaceable>FILE</replaceable></option></arg>
       </group>
       <sbr/>
-      <group>
-	<arg choice="plain"><option>--tls-privkey
-	<replaceable>FILE</replaceable></option></arg>
-	<arg choice="plain"><option>-t
-	<replaceable>FILE</replaceable></option></arg>
-      </group>
-      <sbr/>
-      <group>
-	<arg choice="plain"><option>--tls-pubkey
-	<replaceable>FILE</replaceable></option></arg>
-	<arg choice="plain"><option>-T
-	<replaceable>FILE</replaceable></option></arg>
-      </group>
-      <sbr/>
       <arg>
 	<option>--priority <replaceable>STRING</replaceable></option>
       </arg>
@@ -119,10 +94,6 @@
       </arg>
       <sbr/>
       <arg>
-	<option>--dh-params <replaceable>FILE</replaceable></option>
-      </arg>
-      <sbr/>
-      <arg>
 	<option>--delay <replaceable>SECONDS</replaceable></option>
       </arg>
       <sbr/>
@@ -131,11 +102,6 @@
       </arg>
       <sbr/>
       <arg>
-	<option>--network-hook-dir
-	<replaceable>DIR</replaceable></option>
-      </arg>
-      <sbr/>
-      <arg>
 	<option>--debug</option>
       </arg>
     </cmdsynopsis>
@@ -166,35 +132,17 @@
       communicates with <citerefentry><refentrytitle
       >mandos</refentrytitle><manvolnum>8</manvolnum></citerefentry>
       to get a password.  In slightly more detail, this client program
-      brings up network interfaces, uses the interfaces’ IPv6
-      link-local addresses to get network connectivity, uses Zeroconf
-      to find servers on the local network, and communicates with
-      servers using TLS with a raw public key to ensure authenticity
-      and confidentiality.  This client program keeps running, trying
-      all servers on the network, until it receives a satisfactory
-      reply or a TERM signal.  After all servers have been tried, all
+      brings up a network interface, uses the interface’s IPv6
+      link-local address to get network connectivity, uses Zeroconf to
+      find servers on the local network, and communicates with servers
+      using TLS with an OpenPGP key to ensure authenticity and
+      confidentiality.  This client program keeps running, trying all
+      servers on the network, until it receives a satisfactory reply
+      or a TERM signal.  After all servers have been tried, all
       servers are periodically retried.  If no servers are found it
       will wait indefinitely for new servers to appear.
     </para>
     <para>
-      The network interfaces are selected like this: If any interfaces
-      are specified using the <option>--interface</option> option,
-      those interface are used.  Otherwise,
-      <command>&COMMANDNAME;</command> will use all interfaces that
-      are not loopback interfaces, are not point-to-point interfaces,
-      are capable of broadcasting and do not have the NOARP flag (see
-      <citerefentry><refentrytitle>netdevice</refentrytitle>
-      <manvolnum>7</manvolnum></citerefentry>).  (If the
-      <option>--connect</option> option is used, point-to-point
-      interfaces and non-broadcast interfaces are accepted.)  If any
-      used interfaces are not up and running, they are first taken up
-      (and later taken down again on program exit).
-    </para>
-    <para>
-      Before network interfaces are selected, all <quote>network
-      hooks</quote> are run; see <xref linkend="network-hooks"/>.
-    </para>
-    <para>
       This program is not meant to be run directly; it is really meant
       to run as a plugin of the <application>Mandos</application>
       <citerefentry><refentrytitle>plugin-runner</refentrytitle>
@@ -245,50 +193,44 @@
 	    assumed to separate the address from the port number.
 	  </para>
 	  <para>
-	    Normally, Zeroconf would be used to locate Mandos servers,
-	    in which case this option would only be used when testing
-	    and debugging.
+	    This option is normally only useful for testing and
+	    debugging.
 	  </para>
 	</listitem>
       </varlistentry>
       
       <varlistentry>
 	<term><option>--interface=<replaceable
-	>NAME</replaceable><arg rep='repeat'>,<replaceable
-	>NAME</replaceable></arg></option></term>
+	>NAME</replaceable></option></term>
 	<term><option>-i
-	<replaceable>NAME</replaceable><arg rep='repeat'>,<replaceable
-	>NAME</replaceable></arg></option></term>
+	<replaceable>NAME</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Comma separated list of network interfaces that will be
-	    brought up and scanned for Mandos servers to connect to.
-	    The default is the empty string, which will automatically
-	    use all appropriate interfaces.
+	    Network interface that will be brought up and scanned for
+	    Mandos servers to connect to.  The default is the empty
+	    string, which will automatically choose an appropriate
+	    interface.
 	  </para>
 	  <para>
-	    If the <option>--connect</option> option is used, and
-	    exactly one interface name is specified (except
-	    <quote><literal>none</literal></quote>), this specifies
-	    the interface to use to connect to the address given.
+	    If the <option>--connect</option> option is used, this
+	    specifies the interface to use to connect to the address
+	    given.
 	  </para>
 	  <para>
 	    Note that since this program will normally run in the
 	    initial RAM disk environment, the interface must be an
 	    interface which exists at that stage.  Thus, the interface
-	    can normally not be a pseudo-interface such as
-	    <quote>br0</quote> or <quote>tun0</quote>; such interfaces
-	    will not exist until much later in the boot process, and
-	    can not be used by this program, unless created by a
-	    <quote>network hook</quote> — see <xref
-	    linkend="network-hooks"/>.
+	    can not be a pseudo-interface such as <quote>br0</quote>
+	    or <quote>tun0</quote>; such interfaces will not exist
+	    until much later in the boot process, and can not be used
+	    by this program.
 	  </para>
 	  <para>
 	    <replaceable>NAME</replaceable> can be the string
-	    <quote><literal>none</literal></quote>; this will make
-	    <command>&COMMANDNAME;</command> only bring up interfaces
-	    specified <emphasis>before</emphasis> this string.  This
-	    is not recommended, and only meant for advanced users.
+	    <quote><literal>none</literal></quote>; this will not use
+	    any specific interface, and will not bring up an interface
+	    on startup.  This is not recommended, and only meant for
+	    advanced users.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -322,34 +264,6 @@
       </varlistentry>
       
       <varlistentry>
-	<term><option>--tls-pubkey=<replaceable
-	>FILE</replaceable></option></term>
-	<term><option>-T
-	<replaceable>FILE</replaceable></option></term>
-	<listitem>
-	  <para>
-	    TLS raw public key file name.  The default name is
-	    <quote><filename>/conf/conf.d/mandos/tls-pubkey.pem</filename
-	    ></quote>.
-	  </para>
-	</listitem>
-      </varlistentry>
-
-      <varlistentry>
-	<term><option>--tls-privkey=<replaceable
-	>FILE</replaceable></option></term>
-	<term><option>-t
-	<replaceable>FILE</replaceable></option></term>
-	<listitem>
-	  <para>
-	    TLS secret key file name.  The default name is
-	    <quote><filename>/conf/conf.d/mandos/tls-privkey.pem</filename
-	    ></quote>.
-	  </para>
-	</listitem>
-      </varlistentry>
-
-      <varlistentry>
 	<term><option>--priority=<replaceable
 	>STRING</replaceable></option></term>
 	<listitem>
@@ -364,28 +278,7 @@
 	<listitem>
 	  <para>
 	    Sets the number of bits to use for the prime number in the
-	    TLS Diffie-Hellman key exchange.  The default value is
-	    selected automatically based on the GnuTLS security
-	    profile set in its priority string.  Note that if the
-	    <option>--dh-params</option> option is used, the values
-	    from that file will be used instead.
-	  </para>
-	</listitem>
-      </varlistentry>
-      
-      <varlistentry>
-	<term><option>--dh-params=<replaceable
-	>FILE</replaceable></option></term>
-	<listitem>
-	  <para>
-	    Specifies a PEM-encoded PKCS#3 file to read the parameters
-	    needed by the TLS Diffie-Hellman key exchange from.  If
-	    this option is not given, or if the file for some reason
-	    could not be used, the parameters will be generated on
-	    startup, which will take some time and processing power.
-	    Those using servers running under time, power or processor
-	    constraints may want to generate such a file in advance
-	    and use this option.
+	    TLS Diffie-Hellman key exchange.  Default is 1024.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -395,7 +288,7 @@
 	>SECONDS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    After bringing a network interface up, the program waits
+	    After bringing the network interface up, the program waits
 	    for the interface to arrive in a <quote>running</quote>
 	    state before proceeding.  During this time, the kernel log
 	    level will be lowered to reduce clutter on the system
@@ -418,18 +311,6 @@
 	  </para>
 	</listitem>
       </varlistentry>
-
-      <varlistentry>
-	<term><option>--network-hook-dir=<replaceable
-	>DIR</replaceable></option></term>
-	<listitem>
-	  <para>
-	    Network hook directory.  The default directory is
-	    <quote><filename class="directory"
-	    >/lib/mandos/network-hooks.d</filename></quote>.
-	  </para>
-	</listitem>
-      </varlistentry>
       
       <varlistentry>
 	<term><option>--debug</option></term>
@@ -496,10 +377,8 @@
       <refentrytitle>plugin-runner</refentrytitle>
       <manvolnum>8mandos</manvolnum></citerefentry>) is used to run
       both this program and others in in parallel,
-      <emphasis>one</emphasis> of which (<citerefentry>
-      <refentrytitle>password-prompt</refentrytitle>
-      <manvolnum>8mandos</manvolnum></citerefentry>) will prompt for
-      passwords on the system console.
+      <emphasis>one</emphasis> of which will prompt for passwords on
+      the system console.
     </para>
   </refsect1>
   
@@ -518,196 +397,14 @@
   
   <refsect1 id="environment">
     <title>ENVIRONMENT</title>
-    <variablelist>
-      <varlistentry>
-	<term><envar>MANDOSPLUGINHELPERDIR</envar></term>
-	<listitem>
-	  <para>
-	    This environment variable will be assumed to contain the
-	    directory containing any helper executables.  The use and
-	    nature of these helper executables, if any, is
-	    purposefully not documented.
-	</para>
-	</listitem>
-      </varlistentry>
-    </variablelist>
     <para>
-      This program does not use any other environment variables, not
-      even the ones provided by <citerefentry><refentrytitle
+      This program does not use any environment variables, not even
+      the ones provided by <citerefentry><refentrytitle
       >cryptsetup</refentrytitle><manvolnum>8</manvolnum>
     </citerefentry>.
     </para>
   </refsect1>
   
-  <refsect1 id="network-hooks">
-    <title>NETWORK HOOKS</title>
-    <para>
-      If a network interface like a bridge or tunnel is required to
-      find a Mandos server, this requires the interface to be up and
-      running before <command>&COMMANDNAME;</command> starts looking
-      for Mandos servers.  This can be accomplished by creating a
-      <quote>network hook</quote> program, and placing it in a special
-      directory.
-    </para>
-    <para>
-      Before the network is used (and again before program exit), any
-      runnable programs found in the network hook directory are run
-      with the argument <quote><literal>start</literal></quote> or
-      <quote><literal>stop</literal></quote>.  This should bring up or
-      down, respectively, any network interface which
-      <command>&COMMANDNAME;</command> should use.
-    </para>
-    <refsect2 id="hook-requirements">
-      <title>REQUIREMENTS</title>
-      <para>
-	A network hook must be an executable file, and its name must
-	consist entirely of upper and lower case letters, digits,
-	underscores, periods, and hyphens.
-      </para>
-      <para>
-	A network hook will receive one argument, which can be one of
-	the following:
-      </para>
-      <variablelist>
-	<varlistentry>
-	  <term><literal>start</literal></term>
-	  <listitem>
-	    <para>
-	      This should make the network hook create (if necessary)
-	      and bring up a network interface.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><literal>stop</literal></term>
-	  <listitem>
-	    <para>
-	      This should make the network hook take down a network
-	      interface, and delete it if it did not exist previously.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><literal>files</literal></term>
-	  <listitem>
-	    <para>
-	      This should make the network hook print, <emphasis>one
-	      file per line</emphasis>, all the files needed for it to
-	      run.  (These files will be copied into the initial RAM
-	      filesystem.)  Typical use is for a network hook which is
-	      a shell script to print its needed binaries.
-	    </para>
-	    <para>
-	      It is not necessary to print any non-executable files
-	      already in the network hook directory, these will be
-	      copied implicitly if they otherwise satisfy the name
-	      requirements.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><literal>modules</literal></term>
-	  <listitem>
-	    <para>
-	      This should make the network hook print, <emphasis>on
-	      separate lines</emphasis>, all the kernel modules needed
-	      for it to run.  (These modules will be copied into the
-	      initial RAM filesystem.)  For instance, a tunnel
-	      interface needs the
-	      <quote><literal>tun</literal></quote> module.
-	    </para>
-	  </listitem>
-	</varlistentry>
-      </variablelist>
-      <para>
-	The network hook will be provided with a number of environment
-	variables:
-      </para>
-      <variablelist>
-	<varlistentry>
-	  <term><envar>MANDOSNETHOOKDIR</envar></term>
-	  <listitem>
-	    <para>
-	      The network hook directory, specified to
-	      <command>&COMMANDNAME;</command> by the
-	      <option>--network-hook-dir</option> option.  Note: this
-	      should <emphasis>always</emphasis> be used by the
-	      network hook to refer to itself or any files in the hook
-	      directory it may require.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><envar>DEVICE</envar></term>
-	  <listitem>
-	    <para>
-	      The network interfaces, as specified to
-	      <command>&COMMANDNAME;</command> by the
-	      <option>--interface</option> option, combined to one
-	      string and separated by commas.  If this is set, and
-	      does not contain the interface a hook will bring up,
-	      there is no reason for a hook to continue.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><envar>MODE</envar></term>
-	  <listitem>
-	    <para>
-	      This will be the same as the first argument;
-	      i.e. <quote><literal>start</literal></quote>,
-	      <quote><literal>stop</literal></quote>,
-	      <quote><literal>files</literal></quote>, or
-	      <quote><literal>modules</literal></quote>.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><envar>VERBOSITY</envar></term>
-	  <listitem>
-	    <para>
-	      This will be the <quote><literal>1</literal></quote> if
-	      the <option>--debug</option> option is passed to
-	      <command>&COMMANDNAME;</command>, otherwise
-	      <quote><literal>0</literal></quote>.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><envar>DELAY</envar></term>
-	  <listitem>
-	    <para>
-	      This will be the same as the <option>--delay</option>
-	      option passed to <command>&COMMANDNAME;</command>.  Is
-	      only set if <envar>MODE</envar> is
-	      <quote><literal>start</literal></quote> or
-	      <quote><literal>stop</literal></quote>.
-	    </para>
-	  </listitem>
-	</varlistentry>
-	<varlistentry>
-	  <term><envar>CONNECT</envar></term>
-	  <listitem>
-	    <para>
-	      This will be the same as the <option>--connect</option>
-	      option passed to <command>&COMMANDNAME;</command>.  Is
-	      only set if <option>--connect</option> is passed and
-	      <envar>MODE</envar> is
-	      <quote><literal>start</literal></quote> or
-	      <quote><literal>stop</literal></quote>.
-	    </para>
-	  </listitem>
-	</varlistentry>
-      </variablelist>
-      <para>
-	A hook may not read from standard input, and should be
-	restrictive in printing to standard output or standard error
-	unless <varname>VERBOSITY</varname> is
-	<quote><literal>1</literal></quote>.
-      </para>
-    </refsect2>
-  </refsect1>
-  
   <refsect1 id="files">
     <title>FILES</title>
     <variablelist>
@@ -725,38 +422,14 @@
 	  </para>
 	</listitem>
       </varlistentry>
-      <varlistentry>
-	<term><filename>/conf/conf.d/mandos/tls-pubkey.pem</filename
-	></term>
-	<term><filename>/conf/conf.d/mandos/tls-privkey.pem</filename
-	></term>
-	<listitem>
-	  <para>
-	    Public and private raw key files, in <quote>PEM</quote>
-	    format.  These are the default file names, they can be
-	    changed with the <option>--tls-pubkey</option> and
-	    <option>--tls-privkey</option> options.
-	  </para>
-	</listitem>
-      </varlistentry>
-      <varlistentry>
-	<term><filename
-	class="directory">/lib/mandos/network-hooks.d</filename></term>
-	<listitem>
-	  <para>
-	    Directory where network hooks are located.  Change this
-	    with the <option>--network-hook-dir</option> option.  See
-	    <xref linkend="network-hooks"/>.
-	  </para>
-	</listitem>
-      </varlistentry>
     </variablelist>
   </refsect1>
   
-  <refsect1 id="bugs">
-    <title>BUGS</title>
-    <xi:include href="../bugs.xml"/>
-  </refsect1>
+<!--   <refsect1 id="bugs"> -->
+<!--     <title>BUGS</title> -->
+<!--     <para> -->
+<!--     </para> -->
+<!--   </refsect1> -->
   
   <refsect1 id="example">
     <title>EXAMPLE</title>
@@ -768,8 +441,8 @@
     </para>
     <informalexample>
       <para>
-	Normal invocation needs no options, if the network interfaces
-	can be automatically determined:
+	Normal invocation needs no options, if the network interface
+	is <quote>eth0</quote>:
       </para>
       <para>
 	<userinput>&COMMANDNAME;</userinput>
@@ -777,8 +450,8 @@
     </informalexample>
     <informalexample>
       <para>
-	Search for Mandos servers (and connect to them) using one
-	specific interface:
+	Search for Mandos servers (and connect to them) using another
+	interface:
       </para>
       <para>
 	<!-- do not wrap this line -->
@@ -787,18 +460,18 @@
     </informalexample>
     <informalexample>
       <para>
-	Run in debug mode, and use custom keys:
+	Run in debug mode, and use a custom key:
       </para>
       <para>
 
 <!-- do not wrap this line -->
-<userinput>&COMMANDNAME; --debug --pubkey keydir/pubkey.txt --seckey keydir/seckey.txt --tls-pubkey keydir/tls-pubkey.pem --tls-privkey keydir/tls-privkey.pem</userinput>
+<userinput>&COMMANDNAME; --debug --pubkey keydir/pubkey.txt --seckey keydir/seckey.txt</userinput>
 
       </para>
     </informalexample>
     <informalexample>
       <para>
-	Run in debug mode, with custom keys, and do not use Zeroconf
+	Run in debug mode, with a custom key, and do not use Zeroconf
 	to locate a server; connect directly to the IPv6 link-local
 	address <quote><systemitem class="ipaddress"
 	>fe80::aede:48ff:fe71:f6f2</systemitem></quote>, port 4711,
@@ -807,7 +480,7 @@
       <para>
 
 <!-- do not wrap this line -->
-<userinput>&COMMANDNAME; --debug --pubkey keydir/pubkey.txt --seckey keydir/seckey.txt --tls-pubkey keydir/tls-pubkey.pem --tls-privkey keydir/tls-privkey.pem --connect fe80::aede:48ff:fe71:f6f2:4711 --interface eth2</userinput>
+<userinput>&COMMANDNAME; --debug --pubkey keydir/pubkey.txt --seckey keydir/seckey.txt --connect fe80::aede:48ff:fe71:f6f2:4711 --interface eth2</userinput>
 
       </para>
     </informalexample>
@@ -837,20 +510,19 @@
     <para>
       The only remaining weak point is that someone with physical
       access to the client hard drive might turn off the client
-      computer, read the OpenPGP and TLS keys directly from the hard
-      drive, and communicate with the server.  To safeguard against
-      this, the server is supposed to notice the client disappearing
-      and stop giving out the encrypted data.  Therefore, it is
-      important to set the timeout and checker interval values tightly
-      on the server.  See <citerefentry><refentrytitle
+      computer, read the OpenPGP keys directly from the hard drive,
+      and communicate with the server.  To safeguard against this, the
+      server is supposed to notice the client disappearing and stop
+      giving out the encrypted data.  Therefore, it is important to
+      set the timeout and checker interval values tightly on the
+      server.  See <citerefentry><refentrytitle
       >mandos</refentrytitle><manvolnum>8</manvolnum></citerefentry>.
     </para>
     <para>
       It will also help if the checker program on the server is
       configured to request something from the client which can not be
-      spoofed by someone else on the network, like SSH server key
-      fingerprints, and unlike unencrypted <acronym>ICMP</acronym>
-      echo (<quote>ping</quote>) replies.
+      spoofed by someone else on the network, unlike unencrypted
+      <acronym>ICMP</acronym> echo (<quote>ping</quote>) replies.
     </para>
     <para>
       <emphasis>Note</emphasis>: This makes it completely insecure to
@@ -902,19 +574,20 @@
       </varlistentry>
       <varlistentry>
 	<term>
-	  <ulink url="https://www.gnutls.org/">GnuTLS</ulink>
+	  <ulink url="http://www.gnu.org/software/gnutls/"
+	  >GnuTLS</ulink>
 	</term>
       <listitem>
 	<para>
 	  GnuTLS is the library this client uses to implement TLS for
 	  communicating securely with the server, and at the same time
-	  send the public key to the server.
+	  send the public OpenPGP key to the server.
 	</para>
       </listitem>
       </varlistentry>
       <varlistentry>
 	<term>
-	  <ulink url="https://www.gnupg.org/related_software/gpgme/"
+	  <ulink url="http://www.gnupg.org/related_software/gpgme/"
 		 >GPGME</ulink>
 	</term>
 	<listitem>
@@ -948,7 +621,7 @@
 	      <para>
 		This client uses IPv6 link-local addresses, which are
 		immediately usable since a link-local addresses is
-		automatically assigned to a network interface when it
+		automatically assigned to a network interfaces when it
 		is brought up.
 	      </para>
 	    </listitem>
@@ -958,12 +631,12 @@
       </varlistentry>
       <varlistentry>
 	<term>
-	  RFC 5246: <citetitle>The Transport Layer Security (TLS)
-	  Protocol Version 1.2</citetitle>
+	  RFC 4346: <citetitle>The Transport Layer Security (TLS)
+	  Protocol Version 1.1</citetitle>
 	</term>
       <listitem>
 	<para>
-	  TLS 1.2 is the protocol implemented by GnuTLS.
+	  TLS 1.1 is the protocol implemented by GnuTLS.
 	</para>
       </listitem>
       </varlistentry>
@@ -980,28 +653,13 @@
       </varlistentry>
       <varlistentry>
 	<term>
-	  RFC 7250: <citetitle>Using Raw Public Keys in Transport
-	  Layer Security (TLS) and Datagram Transport Layer Security
-	  (DTLS)</citetitle>
-	</term>
-      <listitem>
-	<para>
-	  This is implemented by GnuTLS in version 3.6.6 and is, if
-	  present, used by this program so that raw public keys can be
-	  used.
-	</para>
-      </listitem>
-      </varlistentry>
-      <varlistentry>
-	<term>
-	  RFC 6091: <citetitle>Using OpenPGP Keys for Transport Layer
+	  RFC 5081: <citetitle>Using OpenPGP Keys for Transport Layer
 	  Security</citetitle>
 	</term>
       <listitem>
 	<para>
-	  This is implemented by GnuTLS before version 3.6.0 and is,
-	  if present, used by this program so that OpenPGP keys can be
-	  used.
+	  This is implemented by GnuTLS and used by this program so
+	  that OpenPGP keys can be used.
 	</para>
       </listitem>
       </varlistentry>

=== modified file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	2019-02-11 07:06:55 +0000
+++ plugins.d/password-prompt.c	2011-10-05 16:00:56 +0000
@@ -2,23 +2,22 @@
 /*
  * Password-prompt - Read a password from the terminal and print it
  * 
- * Copyright © 2008-2019 Teddy Hogeborn
- * Copyright © 2008-2019 Björn Påhlsson
- * 
- * This file is part of Mandos.
- * 
- * Mandos is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- * 
- * Mandos is distributed in the hope that it will be useful, but
+ * Copyright © 2008-2010 Teddy Hogeborn
+ * Copyright © 2008-2010 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  * 
  * Contact the authors at <mandos@recompile.se>.
  */
@@ -74,7 +73,6 @@
 const char plymouth_name[] = "plymouthd";
 
 /* Function to use when printing errors */
-__attribute__((format (gnu_printf, 3, 4)))
 void error_plus(int status, int errnum, const char *formatstring,
 		...){
   va_list ap;
@@ -83,11 +81,12 @@
   
   va_start(ap, formatstring);
   ret = vasprintf(&text, formatstring, ap);
-  if(ret == -1){
+  if (ret == -1){
     fprintf(stderr, "Mandos plugin %s: ",
 	    program_invocation_short_name);
     vfprintf(stderr, formatstring, ap);
-    fprintf(stderr, ": %s\n", strerror(errnum));
+    fprintf(stderr, ": ");
+    fprintf(stderr, "%s\n", strerror(errnum));
     error(status, errno, "vasprintf while printing error");
     return;
   }
@@ -110,18 +109,17 @@
      from the terminal.  Password-prompt will exit if it detects
      plymouth since plymouth performs the same functionality.
    */
-  __attribute__((nonnull))
   int is_plymouth(const struct dirent *proc_entry){
     int ret;
     int cl_fd;
     {
-      uintmax_t proc_id;
+      uintmax_t maxvalue;
       char *tmp;
       errno = 0;
-      proc_id = strtoumax(proc_entry->d_name, &tmp, 10);
+      maxvalue = strtoumax(proc_entry->d_name, &tmp, 10);
       
       if(errno != 0 or *tmp != '\0'
-	 or proc_id != (uintmax_t)((pid_t)proc_id)){
+	 or maxvalue != (uintmax_t)((pid_t)maxvalue)){
 	return 0;
       }
     }
@@ -130,7 +128,7 @@
     ret = asprintf(&cmdline_filename, "/proc/%s/cmdline",
 		   proc_entry->d_name);
     if(ret == -1){
-      error_plus(0, errno, "asprintf");
+      error(0, errno, "asprintf");
       return 0;
     }
     
@@ -139,7 +137,7 @@
     free(cmdline_filename);
     if(cl_fd == -1){
       if(errno != ENOENT){
-	error_plus(0, errno, "open");
+	error(0, errno, "open");
       }
       return 0;
     }
@@ -156,7 +154,7 @@
 	if(cmdline_len + blocksize + 1 > cmdline_allocated){
 	  tmp = realloc(cmdline, cmdline_allocated + blocksize + 1);
 	  if(tmp == NULL){
-	    error_plus(0, errno, "realloc");
+	    error(0, errno, "realloc");
 	    free(cmdline);
 	    close(cl_fd);
 	    return 0;
@@ -169,7 +167,7 @@
 	sret = read(cl_fd, cmdline + cmdline_len,
 		    cmdline_allocated - cmdline_len);
 	if(sret == -1){
-	  error_plus(0, errno, "read");
+	  error(0, errno, "read");
 	  free(cmdline);
 	  close(cl_fd);
 	  return 0;
@@ -178,7 +176,7 @@
       } while(sret != 0);
       ret = close(cl_fd);
       if(ret == -1){
-	error_plus(0, errno, "close");
+	error(0, errno, "close");
 	free(cmdline);
 	return 0;
       }
@@ -213,14 +211,8 @@
   struct dirent **direntries = NULL;
   int ret;
   ret = scandir("/proc", &direntries, is_plymouth, alphasort);
-  if(ret == -1){
-    error_plus(1, errno, "scandir");
-  }
-  {
-    int i = ret;
-    while(i--){
-      free(direntries[i]);
-    }
+  if (ret == -1){
+    error(1, errno, "scandir");
   }
   free(direntries);
   return ret > 0;
@@ -257,7 +249,6 @@
       { .name = NULL }
     };
     
-    __attribute__((nonnull(3)))
     error_t parse_opt (int key, char *arg, struct argp_state *state){
       errno = 0;
       switch (key){
@@ -274,11 +265,9 @@
 	argp_state_help(state, state->out_stream,
 			(ARGP_HELP_STD_HELP | ARGP_HELP_EXIT_ERR)
 			& ~(unsigned int)ARGP_HELP_EXIT_OK);
-	__builtin_unreachable();
       case -3:			/* --usage */
 	argp_state_help(state, state->out_stream,
 			ARGP_HELP_USAGE | ARGP_HELP_EXIT_ERR);
-	__builtin_unreachable();
       case 'V':			/* --version */
 	fprintf(state->out_stream, "%s\n", argp_program_version);
 	exit(argp_err_exit_status);
@@ -301,7 +290,7 @@
     case ENOMEM:
     default:
       errno = ret;
-      error_plus(0, errno, "argp_parse");
+      error(0, errno, "argp_parse");
       return EX_OSERR;
     case EINVAL:
       return EX_USAGE;
@@ -312,7 +301,7 @@
     fprintf(stderr, "Starting %s\n", argv[0]);
   }
 
-  if(conflict_detection()){
+  if (conflict_detection()){
     if(debug){
       fprintf(stderr, "Stopping %s because of conflict\n", argv[0]);
     }
@@ -325,7 +314,7 @@
   
   if(tcgetattr(STDIN_FILENO, &t_old) != 0){
     int e = errno;
-    error_plus(0, errno, "tcgetattr");
+    error(0, errno, "tcgetattr");
     switch(e){
     case EBADF:
     case ENOTTY:
@@ -338,17 +327,17 @@
   sigemptyset(&new_action.sa_mask);
   ret = sigaddset(&new_action.sa_mask, SIGINT);
   if(ret == -1){
-    error_plus(0, errno, "sigaddset");
+    error(0, errno, "sigaddset");
     return EX_OSERR;
   }
   ret = sigaddset(&new_action.sa_mask, SIGHUP);
   if(ret == -1){
-    error_plus(0, errno, "sigaddset");
+    error(0, errno, "sigaddset");
     return EX_OSERR;
   }
   ret = sigaddset(&new_action.sa_mask, SIGTERM);
   if(ret == -1){
-    error_plus(0, errno, "sigaddset");
+    error(0, errno, "sigaddset");
     return EX_OSERR;
   }
   /* Need to check if the handler is SIG_IGN before handling:
@@ -357,37 +346,37 @@
   */
   ret = sigaction(SIGINT, NULL, &old_action);
   if(ret == -1){
-    error_plus(0, errno, "sigaction");
+    error(0, errno, "sigaction");
     return EX_OSERR;
   }
   if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGINT, &new_action, NULL);
     if(ret == -1){
-      error_plus(0, errno, "sigaction");
+      error(0, errno, "sigaction");
       return EX_OSERR;
     }
   }
   ret = sigaction(SIGHUP, NULL, &old_action);
   if(ret == -1){
-    error_plus(0, errno, "sigaction");
+    error(0, errno, "sigaction");
     return EX_OSERR;
   }
   if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGHUP, &new_action, NULL);
     if(ret == -1){
-      error_plus(0, errno, "sigaction");
+      error(0, errno, "sigaction");
       return EX_OSERR;
     }
   }
   ret = sigaction(SIGTERM, NULL, &old_action);
   if(ret == -1){
-    error_plus(0, errno, "sigaction");
+    error(0, errno, "sigaction");
     return EX_OSERR;
   }
   if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGTERM, &new_action, NULL);
     if(ret == -1){
-      error_plus(0, errno, "sigaction");
+      error(0, errno, "sigaction");
       return EX_OSERR;
     }
   }
@@ -401,7 +390,7 @@
   t_new.c_lflag &= ~(tcflag_t)ECHO;
   if(tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
     int e = errno;
-    error_plus(0, errno, "tcsetattr-echo");
+    error(0, errno, "tcsetattr-echo");
     switch(e){
     case EBADF:
     case ENOTTY:
@@ -471,7 +460,7 @@
 	sret = write(STDOUT_FILENO, buffer + written, n - written);
 	if(sret < 0){
 	  int e = errno;
-	  error_plus(0, errno, "write");
+	  error(0, errno, "write");
 	  switch(e){
 	  case EBADF:
 	  case EFAULT:
@@ -493,7 +482,7 @@
       sret = close(STDOUT_FILENO);
       if(sret == -1){
 	int e = errno;
-	error_plus(0, errno, "close");
+	error(0, errno, "close");
 	switch(e){
 	case EBADF:
 	  status = EX_OSFILE;
@@ -509,11 +498,10 @@
     if(sret < 0){
       int e = errno;
       if(errno != EINTR and not feof(stdin)){
-	error_plus(0, errno, "getline");
+	error(0, errno, "getline");
 	switch(e){
 	case EBADF:
 	  status = EX_UNAVAILABLE;
-	  break;
 	case EIO:
 	case EINVAL:
 	default:
@@ -539,7 +527,7 @@
     fprintf(stderr, "Restoring terminal attributes\n");
   }
   if(tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_old) != 0){
-    error_plus(0, errno, "tcsetattr+echo");
+    error(0, errno, "tcsetattr+echo");
   }
   
   if(quit_now){
@@ -547,7 +535,7 @@
     old_action.sa_handler = SIG_DFL;
     ret = sigaction(signal_received, &old_action, NULL);
     if(ret == -1){
-      error_plus(0, errno, "sigaction");
+      error(0, errno, "sigaction");
     }
     raise(signal_received);
   }

=== modified file 'plugins.d/password-prompt.xml'
--- plugins.d/password-prompt.xml	2019-02-10 04:20:26 +0000
+++ plugins.d/password-prompt.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "password-prompt">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,16 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -229,7 +220,9 @@
   
   <refsect1 id="bugs">
     <title>BUGS</title>
-    <xi:include href="../bugs.xml"/>
+    <para>
+      None are known at this time.
+    </para>
   </refsect1>
   
   <refsect1 id="example">

=== modified file 'plugins.d/plymouth.c'
--- plugins.d/plymouth.c	2018-02-08 10:23:55 +0000
+++ plugins.d/plymouth.c	2011-10-05 16:00:56 +0000
@@ -2,23 +2,22 @@
 /*
  * Plymouth - Read a password from Plymouth and output it
  * 
- * Copyright © 2010-2018 Teddy Hogeborn
- * Copyright © 2010-2018 Björn Påhlsson
- * 
- * This file is part of Mandos.
- * 
- * Mandos is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- * 
- * Mandos is distributed in the hope that it will be useful, but
+ * Copyright © 2010-2011 Teddy Hogeborn
+ * Copyright © 2010-2011 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  * 
  * Contact the authors at <mandos@recompile.se>.
  */
@@ -57,11 +56,9 @@
 sig_atomic_t interrupted_by_signal = 0;
 
 /* Used by Ubuntu 11.04 (Natty Narwahl) */
-const char plymouth_old_old_pid[] = "/dev/.initramfs/plymouth.pid";
+const char plymouth_old_pid[] = "/dev/.initramfs/plymouth.pid";
 /* Used by Ubuntu 11.10 (Oneiric Ocelot) */
-const char plymouth_old_pid[] = "/run/initramfs/plymouth.pid";
-/* Used by Debian 9 (stretch) */
-const char plymouth_pid[] = "/run/plymouth/pid";
+const char plymouth_pid[] = "/run/initramfs/plymouth.pid";
 
 const char plymouth_path[] = "/bin/plymouth";
 const char plymouthd_path[] = "/sbin/plymouthd";
@@ -78,7 +75,6 @@
 }
 
 /* Function to use when printing errors */
-__attribute__((format (gnu_printf, 3, 4)))
 void error_plus(int status, int errnum, const char *formatstring,
 		...){
   va_list ap;
@@ -87,7 +83,7 @@
   
   va_start(ap, formatstring);
   ret = vasprintf(&text, formatstring, ap);
-  if(ret == -1){
+  if (ret == -1){
     fprintf(stderr, "Mandos plugin %s: ",
 	    program_invocation_short_name);
     vfprintf(stderr, formatstring, ap);
@@ -157,9 +153,8 @@
   return true;
 }
 
-__attribute__((nonnull (2, 3)))
 bool exec_and_wait(pid_t *pid_return, const char *path,
-		   const char * const *argv, bool interruptable,
+		   const char **argv, bool interruptable,
 		   bool daemonize){
   int status;
   int ret;
@@ -177,16 +172,12 @@
       }
     }
     
-    char **new_argv = malloc(sizeof(const char *));
-    if(new_argv == NULL){
-      error_plus(0, errno, "malloc");
-      _exit(EX_OSERR);
-    }
+    char **new_argv = NULL;
     char **tmp;
     int i = 0;
-    for (; argv[i] != NULL; i++){
-      tmp = realloc(new_argv, sizeof(const char *) * ((size_t)i + 2));
-      if(tmp == NULL){
+    for (; argv[i]!=NULL; i++){
+      tmp = realloc(new_argv, sizeof(const char *) * ((size_t)i + 1));
+      if (tmp == NULL){
 	error_plus(0, errno, "realloc");
 	free(new_argv);
 	_exit(EX_OSERR);
@@ -221,17 +212,16 @@
   return false;
 }
 
-__attribute__((nonnull))
 int is_plymouth(const struct dirent *proc_entry){
   int ret;
   {
-    uintmax_t proc_id;
+    uintmax_t maxvalue;
     char *tmp;
     errno = 0;
-    proc_id = strtoumax(proc_entry->d_name, &tmp, 10);
+    maxvalue = strtoumax(proc_entry->d_name, &tmp, 10);
 
     if(errno != 0 or *tmp != '\0'
-       or proc_id != (uintmax_t)((pid_t)proc_id)){
+       or maxvalue != (uintmax_t)((pid_t)maxvalue)){
       return 0;
     }
   }
@@ -272,54 +262,38 @@
 
 pid_t get_pid(void){
   int ret;
-  uintmax_t proc_id = 0;
+  uintmax_t maxvalue = 0;
   FILE *pidfile = fopen(plymouth_pid, "r");
   /* Try the new pid file location */
   if(pidfile != NULL){
-    ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
+    ret = fscanf(pidfile, "%" SCNuMAX, &maxvalue);
     if(ret != 1){
-      proc_id = 0;
+      maxvalue = 0;
     }
     fclose(pidfile);
   }
   /* Try the old pid file location */
-  if(proc_id == 0){
-    pidfile = fopen(plymouth_old_pid, "r");
-    if(pidfile != NULL){
-      ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
-      if(ret != 1){
-	proc_id = 0;
-      }
-      fclose(pidfile);
-    }
-  }
-  /* Try the old old pid file location */
-  if(proc_id == 0){
-    pidfile = fopen(plymouth_old_old_pid, "r");
-    if(pidfile != NULL){
-      ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
-      if(ret != 1){
-	proc_id = 0;
+  if(maxvalue == 0){
+    pidfile = fopen(plymouth_pid, "r");
+    if(pidfile != NULL){
+      ret = fscanf(pidfile, "%" SCNuMAX, &maxvalue);
+      if(ret != 1){
+	maxvalue = 0;
       }
       fclose(pidfile);
     }
   }
   /* Look for a plymouth process */
-  if(proc_id == 0){
+  if(maxvalue == 0){
     struct dirent **direntries = NULL;
     ret = scandir("/proc", &direntries, is_plymouth, alphasort);
-    if(ret == -1){
+    if (ret == -1){
       error_plus(0, errno, "scandir");
     }
-    if(ret > 0){
-      for(int i = ret-1; i >= 0; i--){
-	if(proc_id == 0){
-	  ret = sscanf(direntries[i]->d_name, "%" SCNuMAX, &proc_id);
-	  if(ret < 0){
-	    error_plus(0, errno, "sscanf");
-	  }
-	}
-	free(direntries[i]);
+    if (ret > 0){
+      ret = sscanf(direntries[0]->d_name, "%" SCNuMAX, &maxvalue);
+      if (ret < 0){
+	error_plus(0, errno, "sscanf");
       }
     }
     /* scandir might preallocate for this variable (man page unclear).
@@ -327,15 +301,15 @@
     free(direntries);
   }
   pid_t pid;
-  pid = (pid_t)proc_id;
-  if((uintmax_t)pid == proc_id){
+  pid = (pid_t)maxvalue;
+  if((uintmax_t)pid == maxvalue){
     return pid;
   }
   
   return 0;
 }
 
-char **getargv(pid_t pid){
+const char **getargv(pid_t pid){
   int cl_fd;
   char *cmdline_filename;
   ssize_t sret;
@@ -402,7 +376,7 @@
     return NULL;
   }
   argz_extract(cmdline, cmdline_len, argv); /* Create argv */
-  return argv;
+  return (const char **)argv;
 }
 
 int main(__attribute__((unused))int argc,
@@ -483,10 +457,11 @@
   }
   kill_and_wait(plymouth_command_pid);
   
-  char **plymouthd_argv = NULL;
+  const char **plymouthd_argv;
   pid_t pid = get_pid();
   if(pid == 0){
     error_plus(0, 0, "plymouthd pid not found");
+    plymouthd_argv = plymouthd_default_argv;
   } else {
     plymouthd_argv = getargv(pid);
   }
@@ -495,21 +470,10 @@
 		       { plymouth_path, "quit", NULL },
 		       false, false);
   if(not bret){
-    if(plymouthd_argv != NULL){
-      free(*plymouthd_argv);
-      free(plymouthd_argv);
-    }
     exit(EXIT_FAILURE);
   }
-  bret = exec_and_wait(NULL, plymouthd_path,
-		       (plymouthd_argv != NULL)
-		       ? (const char * const *)plymouthd_argv
-		       : plymouthd_default_argv,
+  bret = exec_and_wait(NULL, plymouthd_path, plymouthd_argv,
 		       false, true);
-  if(plymouthd_argv != NULL){
-    free(*plymouthd_argv);
-    free(plymouthd_argv);
-  }
   if(not bret){
     exit(EXIT_FAILURE);
   }

=== modified file 'plugins.d/plymouth.xml'
--- plugins.d/plymouth.xml	2019-02-10 04:20:26 +0000
+++ plugins.d/plymouth.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plymouth">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,14 +33,6 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -169,7 +161,7 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename class="directory">/proc</filename></term>
+	<term><filename>/proc</filename></term>
 	<listitem>
 	  <para>
 	    To find the running <citerefentry><refentrytitle
@@ -208,7 +200,6 @@
       daemon and starting a new one is ugly, but necessary as long as
       it does not support aborting a password request.
     </para>
-    <xi:include href="../bugs.xml"/>
   </refsect1>
   
   <refsect1 id="example">

=== modified file 'plugins.d/splashy.c'
--- plugins.d/splashy.c	2018-02-08 10:23:55 +0000
+++ plugins.d/splashy.c	2011-10-05 16:00:56 +0000
@@ -2,23 +2,22 @@
 /*
  * Splashy - Read a password from splashy and output it
  * 
- * Copyright © 2008-2018 Teddy Hogeborn
- * Copyright © 2008-2018 Björn Påhlsson
- * 
- * This file is part of Mandos.
- * 
- * Mandos is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- * 
- * Mandos is distributed in the hope that it will be useful, but
+ * Copyright © 2008-2011 Teddy Hogeborn
+ * Copyright © 2008-2011 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  * 
  * Contact the authors at <mandos@recompile.se>.
  */
@@ -62,7 +61,6 @@
 int signal_received;
 
 /* Function to use when printing errors */
-__attribute__((format (gnu_printf, 3, 4)))
 void error_plus(int status, int errnum, const char *formatstring,
 		...){
   va_list ap;
@@ -71,7 +69,7 @@
   
   va_start(ap, formatstring);
   ret = vasprintf(&text, formatstring, ap);
-  if(ret == -1){
+  if (ret == -1){
     fprintf(stderr, "Mandos plugin %s: ",
 	    program_invocation_short_name);
     vfprintf(stderr, formatstring, ap);

=== modified file 'plugins.d/splashy.xml'
--- plugins.d/splashy.xml	2019-02-10 04:20:26 +0000
+++ plugins.d/splashy.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "splashy">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,16 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -212,7 +203,6 @@
       is ugly, but necessary as long as it does not support aborting a
       password request.
     </para>
-    <xi:include href="../bugs.xml"/>
   </refsect1>
   
   <refsect1 id="example">

=== modified file 'plugins.d/usplash.c'
--- plugins.d/usplash.c	2018-02-08 10:23:55 +0000
+++ plugins.d/usplash.c	2011-10-05 16:00:56 +0000
@@ -2,23 +2,22 @@
 /*
  * Usplash - Read a password from usplash and output it
  * 
- * Copyright © 2008-2018 Teddy Hogeborn
- * Copyright © 2008-2018 Björn Påhlsson
- * 
- * This file is part of Mandos.
- * 
- * Mandos is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- * 
- * Mandos is distributed in the hope that it will be useful, but
+ * Copyright © 2008-2011 Teddy Hogeborn
+ * Copyright © 2008-2011 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with Mandos.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  * 
  * Contact the authors at <mandos@recompile.se>.
  */
@@ -59,7 +58,6 @@
 const char usplash_name[] = "/sbin/usplash";
 
 /* Function to use when printing errors */
-__attribute__((format (gnu_printf, 3, 4)))
 void error_plus(int status, int errnum, const char *formatstring,
 		...){
   va_list ap;
@@ -68,7 +66,7 @@
   
   va_start(ap, formatstring);
   ret = vasprintf(&text, formatstring, ap);
-  if(ret == -1){
+  if (ret == -1){
     fprintf(stderr, "Mandos plugin %s: ",
 	    program_invocation_short_name);
     vfprintf(stderr, formatstring, ap);
@@ -118,7 +116,7 @@
       ret = asprintf(&cmd_line_alloc, "%s %s", cmd, arg);
       if(ret == -1){
 	int e = errno;
-	close(*fifo_fd_r);
+	TEMP_FAILURE_RETRY(close(*fifo_fd_r));
 	errno = e;
 	return false;
       }
@@ -134,7 +132,7 @@
 		 cmd_line_len - written);
     if(sret == -1){
       int e = errno;
-      close(*fifo_fd_r);
+      TEMP_FAILURE_RETRY(close(*fifo_fd_r));
       free(cmd_line_alloc);
       errno = e;
       return false;
@@ -492,7 +490,7 @@
 	error_plus(0, errno, "read");
 	status = EX_OSERR;
       }
-      close(outfifo_fd);
+      TEMP_FAILURE_RETRY(close(outfifo_fd));
       goto failure;
     }
     if(interrupted_by_signal){
@@ -579,7 +577,7 @@
   
   /* Close FIFO */
   if(fifo_fd != -1){
-    ret = close(fifo_fd);
+    ret = (int)TEMP_FAILURE_RETRY(close(fifo_fd));
     if(ret == -1 and errno != EINTR){
       error_plus(0, errno, "close");
     }
@@ -588,7 +586,7 @@
   
   /* Close output FIFO */
   if(outfifo_fd != -1){
-    ret = close(outfifo_fd);
+    ret = (int)TEMP_FAILURE_RETRY(close(outfifo_fd));
     if(ret == -1){
       error_plus(0, errno, "close");
     }
@@ -656,7 +654,7 @@
   
   /* Close FIFO (again) */
   if(fifo_fd != -1){
-    ret = close(fifo_fd);
+    ret = (int)TEMP_FAILURE_RETRY(close(fifo_fd));
     if(ret == -1 and errno != EINTR){
       error_plus(0, errno, "close");
     }

=== modified file 'plugins.d/usplash.xml'
--- plugins.d/usplash.xml	2019-02-10 04:20:26 +0000
+++ plugins.d/usplash.xml	2011-10-05 16:00:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "usplash">
-<!ENTITY TIMESTAMP "2019-02-10">
+<!ENTITY TIMESTAMP "2011-10-03">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,16 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2010</year>
       <year>2011</year>
-      <year>2012</year>
-      <year>2013</year>
-      <year>2014</year>
-      <year>2015</year>
-      <year>2016</year>
-      <year>2017</year>
-      <year>2018</year>
-      <year>2019</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -187,7 +178,7 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename class="directory">/proc</filename></term>
+	<term><filename>/proc</filename></term>
 	<listitem>
 	  <para>
 	    To find the running <citerefentry><refentrytitle
@@ -226,7 +217,6 @@
       is ugly, but necessary as long as it does not support aborting a
       password request.
     </para>
-    <xi:include href="../bugs.xml"/>
   </refsect1>
   
   <refsect1 id="example">

=== removed file 'tmpfiles.d-mandos.conf'
--- tmpfiles.d-mandos.conf	2016-03-19 03:51:23 +0000
+++ tmpfiles.d-mandos.conf	1970-01-01 00:00:00 +0000
@@ -1,1 +0,0 @@
-d	/var/lib/mandos	700	_mandos	_mandos

